<?php

class Controller extends CController
{
    public $layout = 'layout';

    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('login', 'error', 'logout'),
                'users' => array('*'),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );

    }

    public function filters()
    {
        return array('accessControl'); // perform access control for CRUD operations

    }

    public function beforeAction($action)
    {
        //Connect database
        if(isset(Yii::app()->user->id))
            MasUsers::model()->findBypk((int) Yii::app()->user->id)->db();

        if (Yii::app()->request->isAjaxRequest) {
            $this->layout = false;
        }

        return parent::beforeAction($action);

    }

    public function actionLogin()
    {
        if (Yii::app()->user->isGuest) {
            if (!Yii::app()->request->isAjaxRequest) {
                $this->layout = "login";
            }

            $model = new LoginForm;
            if (isset($_POST['ajax']) && $_POST['ajax'] === 'login-form') {
                echo CActiveForm::validate($model);
                Yii::app()->end();
            }

            if (isset($_POST['LoginForm'])) {
                $model->attributes = $_POST['LoginForm'];
                if ($model->validate() && $model->login()) {
                    if (!Yii::app()->request->isAjaxRequest) {
                        if (isset(Yii::app()->user->is_valid_login) && !Yii::app()->user->is_valid_login) {
                            $this->redirect('index.php?r=site/payments');
                        } else {
                            // if(  isset( Yii::app()->user->access_type ) && ( Yii::app()->user->access_type == 'SU' || Yii::app()->user->access_type == 'DEV' ) )
                                $this->redirect('index.php?r=site/index');
                            // else
                            //     $this->redirect('index.php?r=voucher/salesvouchers');
                        }

                    } else {
                        $jsonData['status'] = 'success';
                        echo CJSON::encode($jsonData);
                    }
                } else {
                    if (!Yii::app()->request->isAjaxRequest) {
                        $this->redirect(array('site/login'));
                    } else {
                        $jsonData['status'] = 'error';
                        echo CJSON::encode($jsonData);
                    }
                }
            } else {
                if (!Yii::app()->request->isAjaxRequest) {
                    $this->render('login', array('model' => $model));
                } else {
                    Yii::app()->clientScript->scriptMap = array('*.js' => false, '*.min.js' => false, '*.css' => false);
                    $this->renderPartial('login', array('model' => $model), false, true);
                }
            }
        } else {

            if (!Yii::app()->request->isAjaxRequest) {
                $this->redirect(array('site/index'));
            } else {
                $jsonData['status'] = 'success';
                echo CJSON::encode($jsonData);
            }
        }
    }

    public function actionLogout()
    {
        Yii::app()->user->logout();
        $this->redirect(Yii::app()->homeUrl);
    }

    public function actionUpload($folder = null)
    {
        Yii::import("ext.EAjaxUpload.qqFileUploader");
        $folder = Yii::getPathOfAlias('webroot') . '/' . $folder . '/';
        if (!is_dir($folder)) {
            mkdir($folder, 0777, true);
        }

        $allowedExtensions = array();
        $sizeLimit = 2 * 1024 * 1024; // maximum file size in bytes
        $uploader = new qqFileUploader($allowedExtensions, $sizeLimit);
        $result = $uploader->handleUpload($folder);
        $result['fileType'] = pathinfo($folder . $result['filename'], PATHINFO_EXTENSION);
        $return = htmlspecialchars(json_encode($result), ENT_NOQUOTES);
        $fileName = $result['filename']; //GETTING FILE NAME
        echo $return; // it's array

    }

    public function get_dates_of_quarter($quarter = 'current', $year = null, $format = null)
    {
        $dateObj = new DateTime;
        if (!is_int($year)) {
            $year = $dateObj->format('Y');
        }
        $current_quarter = ceil($dateObj->format('n') / 3);
        switch (strtolower($quarter)) {
            case 'this':
            case 'current':
                $quarter = ceil($dateObj->format('n') / 3);
                break;

            case 'previous':
                $year = $dateObj->format('Y');
                if ($current_quarter == 1) {
                    $quarter = 4;
                    $year--;
                } else {
                    $quarter = $current_quarter - 1;
                }
                break;

            case 'first':
                $quarter = 1;
                break;

            case 'last':
                $quarter = 4;
                break;

            default:
                $quarter = (!is_int($quarter) || $quarter < 1 || $quarter > 4) ? $current_quarter : $quarter;
                break;
        }
        if ($quarter === 'this') {
            $quarter = ceil($dateObj->format('n') / 3);
        }
        $start = new DateTime($year . '-' . (3 * $quarter - 2) . '-1 00:00:00');
        $end = new DateTime($year . '-' . (3 * $quarter) . '-' . ($quarter == 1 || $quarter == 4 ? 31 : 30) . ' 23:59:59');

        return array(
            'start' => $format ? $start->format($format) : $start,
            'end' => $format ? $end->format($format) : $end,
        );
    }

    //     get_dates_of_quarter();
    // //return current quarter start and end dates

    // get_dates_of_quarter(2);
    // //return 2nd quarter start and end dates of current year

    // get_dates_of_quarter('first', 2010, 'Y-m-d');
    // //return start='2010-01-01' and end='2014-03-31'

    // get_dates_of_quarter('current', 2009, 'Y-m-d');
    // //Supposing today is '2014-08-22' (3rd quarter), this will return
    // //3rd quarter but of year 2009.
    // //return start='2009-07-01' and end='2009-09-30'

    // get_dates_of_quarter('previous');
    // //Supposing today is '2014-02-18' (1st quarter), this will return
    // //return start='2013-10-01' and end='2013-12-31'

}
