<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiGetAllocatablesController extends Controller {

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules() {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('Customer','Vendor'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters() {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request) {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401() {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400() {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403() {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind() {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value) {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions() {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null) {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name) {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }
  
    public function actionCustomer() {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true); 
        $ledger_id = (int) trim($variables['ledger_id']);
        //error_log($input);
        if ($userModel->id > 0) {

            $userModel->db();

            if ($ledger_id > 0) {
               
                    $ledgerModel = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $ledger_id));
                    $subledger = $ledgerModel->id;
                    $count = 0;
                    $debtor = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                    if ($ledgerModel->id > 0) {
                        $salesquery = "select id,invoice_no,invoice_date,customer_id,'Sales' AS voucher_type,grand_total from erp_sales_invoice where active='Y' AND customer_id=" . $ledger_id;
                        $salesitems = Yii::app()->db->createCommand($salesquery)->queryAll();
                    
                        if (!empty($salesitems)) {
                            foreach ($salesitems as $i => $each) {
                                $allocated = Yii::app()->db->createCommand("select SUM(A.amount) from erp_finance_voucher_head H INNER JOIN erp_finance_voucher_items I ON H.id=I.header_id INNER JOIN erp_finance_allocation_items A ON A.voucher_item_id=I.id where H.voucher_type=7 AND H.header_table_id=" . $each['id'] . " AND I.ledger_id=2 AND I.subledger_id=" . $subledger)->queryScalar();
                                $balance = $each['grand_total'] - $allocated;
                                $sales_finance_id = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=7 AND header_table_id=' . $each['id']))->id;
                                if ($sales_finance_id > 0) {
                                    $sales_finance_item_id = FinanceVoucherItems::model()->find(array('condition' => 'ledger_id=' . $debtor . ' AND header_id=' . $sales_finance_id))->id;
                                }
                                if (round($balance, 2) > 0) {
                                    $this->responseBody['data']['alloc_items'][] = array(
                                        'voucher_id' => $sales_finance_id,
                                        'voucher_item_id' => $sales_finance_item_id,
                                        'ledger_id' => $each['customer_id'],
                                        'voucher_date' => $each['invoice_date'],
                                        'voucher_type' => $each['voucher_type'],
                                        'voucher_no' => $each['invoice_no'],
                                        'voucher_value' => $this->format_currency($each['grand_total']),
                                        'allocated' => $this->format_currency($allocated),
                                        'balance' => $this->format_currency($balance),
                                    );
                                    $count++;
                                }
                            }
                        }
                        $cnquery = "select id,return_no,return_date,customer_id,'Credit Note' AS voucher_type,grand_total from erp_sales_return where active='Y' AND customer_id=" . $ledger_id;
                        $cnitems = Yii::app()->db->createCommand($cnquery)->queryAll();
                    
                        if (!empty($cnitems)) {
                            foreach ($cnitems as $i => $each) {
                                $cn_finance_id = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=10 AND header_table_id=' . $each['id']))->id;
                                if ($cn_finance_id > 0) {
                                    $cn_finance_item = FinanceVoucherItems::model()->find(array('condition' => 'ledger_id=' . $debtor . ' AND header_id=' . $cn_finance_id));
                                    $cn_finance_item_id =$cn_finance_item->id;
                                
                                    $allocated = Yii::app()->db->createCommand("select IFNULL(SUM(A.amount),0) from erp_finance_voucher_head H INNER JOIN erp_finance_voucher_items I ON H.id=I.header_id INNER JOIN erp_finance_allocation_items A ON A.voucher_item_id=I.id where H.voucher_type=10 AND H.header_table_id=" . $each['id'] . " AND I.ledger_id=2 AND I.subledger_id=" . $subledger)->queryScalar();
                                    $balance = $cn_finance_item->amount - $allocated;
                                    if (round($balance, 2) != 0) {
                                        $this->responseBody['data']['alloc_items'][] = array(
                                            'voucher_id' => $cn_finance_id,
                                            'voucher_item_id' => $cn_finance_item_id,
                                            'ledger_id' => $each['customer_id'],
                                            'voucher_date' => $each['return_date'],
                                            'voucher_type' => $each['voucher_type'],
                                            'voucher_no' => $each['return_no'],
                                            'voucher_value' => $cn_finance_item->amount,
                                            'allocated' => $this->format_currency($allocated),
                                            'balance' => $this->format_currency($balance),
                                        );
                                        $count++;
                                    }
                                }
                            }
                        }
                        //receipt
                        $voucher_type= FinanceVoucherType::model()->find(array('condition' => 'code="CA" AND outlet=' . $userModel->outlet_id))->id;//Receipt Voucher
                        $receipt_cond ='ledger_id=' . $debtor . ' AND subledger_id=' . $ledger_id.' AND parent.voucher_type="'.$voucher_type.'" ';//AND parent.is_allocated="N"
                        $modelunallocatedreceipts = FinanceVoucherItems::model()->with('parent')->findAll(array('condition' => $receipt_cond));
                        //echo '<pre>';print_r($modelunallocatedreceipts);exit;
                        
                        foreach ($modelunallocatedreceipts as $i => $each) {
                            $finance_id = $each->parent->id;
                            $finance_item_id = $each->id;
                            $allocated = Yii::app()->db->createCommand("select IFNULL(SUM(A.amount),0) from erp_finance_voucher_items I  INNER JOIN erp_finance_allocation_items A ON A.voucher_item_id=I.id where I.id=".$finance_item_id)->queryScalar();
                            $balance = $each['amount'] - $allocated;        
                            if (round($balance, 2) != 0) {
                                $voucher_no=FinanceVoucherType::getVoucherno($voucher_type, true);
                                $this->responseBody['data']['alloc_items'][] = array(
                                    'voucher_id' => $finance_id,
                                    'voucher_item_id' => $finance_item_id,
                                    'ledger_id' => "$ledger_id",
                                    'voucher_date' => $each['parent']['voucher_date'],
                                    'voucher_type' => 'Receipt',
                                    'voucher_no' => "$voucher_no" ,
                                    'voucher_value' => $each['amount'],
                                    'allocated' => $this->format_currency($allocated),
                                    'balance' => $this->format_currency($balance),
                                );
                                $count++;
                            }
                        }
                    }
                    
                 

                $this->responseBody['count'] = $count;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Ledger';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }

    
  
    public function actionVendor() {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true); 
        $ledger_id = (int) trim($variables['ledger_id']);
        //error_log($input);
        if ($userModel->id > 0) {

            $userModel->db();

            if ($ledger_id > 0) {
                
 
                    $ledgerModel = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=2 AND parent_id=' . $ledger_id));
                    $subledger = $ledgerModel->id;
                    $count = 0;
                    if ($ledgerModel->id > 0) {
                        $query = "select id,invoice_no,invoice_date,vendor_id,'Purchase' AS voucher_type,grand_total from erp_purchase_invoice where active='Y' AND vendor_id=" . $ledger_id;
                        $items = Yii::app()->db->createCommand($query)->queryAll();
                    }
                    $ledgerModel = Vendor::model()->findByPk($ledger_id);
                    if (!empty($items)) {
                        foreach ($items as $i => $each) {
                            $allocated = Yii::app()->db->createCommand("select SUM(A.amount) from erp_finance_voucher_head H INNER JOIN erp_finance_voucher_items I ON H.id=I.header_id INNER JOIN erp_finance_allocation_items A ON A.voucher_item_id=I.id where H.voucher_type=8 AND H.header_table_id=" . $each['id'] . " AND I.ledger_id=6 AND I.subledger_id=" . $subledger)->queryScalar();
                            $balance = $each['grand_total'] - abs($allocated);
                            $purchase_finance_id=FinanceVoucherHead::model()->find(array('condition'=>'voucher_type=8 AND header_table_id='.$each['id']))->id;
                            if($purchase_finance_id>0)
                            {
                            $creditors=FinanceSystemAccountsMapping::getSystemdefault('creditors');  
                            $purchase_finance_item_id=FinanceVoucherItems::model()->find(array('condition'=>'ledger_id='.$creditors.' AND header_id='.$purchase_finance_id))->id;    
                            }
                            if (round($balance, 2) > 0) {
                            $this->responseBody['data'][] = array(
                                'voucher_id' => $purchase_finance_id,
                                'voucher_item_id' => $purchase_finance_item_id,
                                'ledger_id' => $each['vendor_id'],
                                'voucher_date' => $each['invoice_date'],
                                'voucher_type' => $each['voucher_type'],
                                'voucher_no' => $each['invoice_no'],
                                'voucher_value' => $this->format_currency($each['grand_total']),
                                'allocated' => $this->format_currency($allocated),
                                'balance' => $this->format_currency($balance),
                            );
                            $count++;
                            }
                        }
                    }
                

                $this->responseBody['count'] = $count;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Ledger';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }

}