<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiGoodstransferController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array("GetList", "Save", "GetDetailssById", "Cancel", "Print"),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }




    public function actionGetList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $from = trim($variables['from']);
        $to = trim($variables['to']);
        $term = trim($variables['term']);

        $from = $this->convert_date($from);
        $to = $this->convert_date($to);
        if ($userModel->id > 0) {

            $userModel->db();

            $count = 0;

            if ($userModel->access_type == 'DEV' || $userModel->access_type == 'SU' || $userModel->access_type == 'CA' || $userModel->access_type == 'SM' || $userModel->access_type == 'OM') {
                $transfer_cnd = '1=1';
            } else {
                $transfer_cnd = 'from_user_id=' . $userModel->id;
            }
            if ($from != '' && $from != '00/00/0000') {

                $transfer_cnd .= ' AND t.transfer_date>="' . $from . '"';
            }
            if ($to != '' && $to != '00/00/0000') {

                $transfer_cnd .= ' AND t.transfer_date<="' . $to . '"';
            }

            if ($outlet > 0) {
                $transfer_cnd .= " AND t.outlet=" . $outlet;
            }

            if (isset($term) && $term != "") {
                $searchString = $term;
                $transfer_cnd .= " AND (t.transfer_no LIKE '%" . $searchString . "%' OR t.notes LIKE '%" . $searchString . "%'  OR t.notes LIKE '%" . $searchString . "%' )";
            }


            $model = GoodsTransfer::model()->findAll(array('condition' => $transfer_cnd, 'order' => 'transfer_date DESC, transfer_no DESC'));
            foreach ($model as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['voucher_no'] = $each->transfer_no;
                $this->responseBody['data'][$i]['date'] = $each->transfer_date;
                $this->responseBody['data'][$i]['from'] = $each->gt_from_user->full_name;
                $this->responseBody['data'][$i]['to'] = $each->gt_to_user->full_name;
                $this->responseBody['data'][$i]['notes'] = $each->notes;
                $this->responseBody['data'][$i]['is_posted'] = $each->posted == 'Y' ? true : false;
                $this->responseBody['data'][$i]['is_cancelable'] = $each->active == 'Y' ? true : false;
                $this->responseBody['data'][$i]['is_cancelled'] = $each->active == 'Y' ? true : false;
                /* $this->responseBody['data'][$i]['is_audited'] = $each->is_audited == 'Y' ? true : false;
              $this->responseBody['data'][$i]['is_cancelled'] = $each->is_canceled == 'Y' ? true : false; */
                $count++;
            }

            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }



    public function actionSave()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $transfer_date = trim($variables['transfer_date']);
        $id = trim($variables['id']);
        $transferItems = $variables['items'];
        $from_id = $variables['from_user_id'];
        $to_id = $variables['to_user_id'];
       
        date_default_timezone_set("Asia/Dubai");
        $valid = true;
        $groups = $other = array();
        if ($userModel->id > 0) {
            $userModel->db();
            $model = new GoodsTransfer();
            $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="GT"'));
            if((int)$from_id<=0){
                
                
            }
            if((int)$to_id<=0){
                
                  $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'To User Not Found';
                $this->_bind;
                exit;
            }
            if((int)$from_id<=0){
                
                  $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'From User  Not Found';
                $this->_bind;
                exit;
            }
            if ($id > 0) {
                $model = $model->findBypk($id);
                $scene = 'update';
                $change_type = 'UPDATED';
                 $model->updated_by = $userModel->id;
            $model->updated_time = date('Y-m-d H:i:s');
            } else {
                $model->created_by = $userModel->id;
                $model->created_time = date('Y-m-d H:i:s');
                $scene = 'new';
                $change_type = 'CREATED';
                if (empty($modelformmaster)) {
                    $model->transfer_no = 90000;
                } else
                    $model->transfer_no = $modelformmaster->next_number;
            }
            $model->from_user_id =$variables['from_user_id'];
            $model->to_user_id = $variables['to_user_id'];
            $model->notes = $variables['notes'];
            $model->outlet = $userModel->outlet_id;
            $model->transfer_date = count(explode('-', $transfer_date)) == 3 ? $transfer_date : date('Y-m-d');
            $model->posted = "N";
            $model->active = "Y";
            $model->callback_from = 'app';


            $i = 0;
            $product_ids = array();
            $items = array();
            $valid = $model->validate();
            $j = 0;
            foreach ($transferItems as $each) {
                if ($each['quantity'] > 0 && $each['product_id'] > 0) {
                    $modelItems[$j] = new GoodsTransferItems();
                    $productModel = Inventory::model()->findBypk($each['product_id']);
                    $modelItems[$j]->product_id = $each['product_id'];
                    $modelItems[$j]->description = $productModel->description;
                    $modelItems[$j]->quantity = (float) $each['quantity'];
                    $modelItems[$j]->unit = $each['unit'];
                    $modelItems[$j]->transfer_id = 0;
                    $valid = $valid && $modelItems[$j]->validate();
                    $j++;
                    // Product Grouping
                    if ($each['unit'] > 0) {
                        $unit = MasUnit::model()->findByPk($each['unit']);
                        $unit_exchange = $unit->unit_exchange_rate;
                    } else {
                        $unit_exchange = 0;
                    }
                    $key = $each['product_id'];
                    if (!array_key_exists($key, $groups)) {
                        $groups[$key] = array(
                            'id' => $each['product_id'],
                            'quantity' => $each['quantity'] * $unit_exchange,
                        );
                    } else {
                        $groups[$key]['quantity'] = $groups[$key]['quantity'] + $each['quantity'] * $unit_exchange;
                    }
                }
            }
            // QTY Checking
            if (!empty($groups)) {
                foreach ($groups as $key => $eachProduct) {
                    $product = Inventory::model()->findByPk($key);
                    if ($product->type != 'S') {
                        $condt = " AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $model->from_user_id;
                        if ($model->id > 0) {
                            $condt .= " AND header_id!=" . $model->id;
                        }
                        $in_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='IN' " . $condt)->queryScalar();
                        $out_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='OUT' " . $condt)->queryScalar();
                        $in_qty_on_date = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_date <='" . $model->transfer_date . "' AND transaction_type='IN' " . $condt)->queryScalar();
                        $out_qty_on_date = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_date <='" . $model->transfer_date . "' AND transaction_type='OUT' " . $condt)->queryScalar();
                        $current_stock = $in_qty - $out_qty;
                        $stock_on_date = $in_qty_on_date - $out_qty_on_date;
                        if ((($current_stock) < $eachProduct['quantity']) || (($stock_on_date) < $eachProduct['quantity'])) {
                            $available_stock = (($current_stock > $stock_on_date) ? $stock_on_date : $current_stock);

                            $other[] = $product->part_number . ' maximum transfer qty : ' . ($available_stock) . ' ' . $product->units->unit_name;
                            $valid = false;
                        }
                    }
                }
            }

            if ($valid && ($modelItems[0]->product_id > 0)) {
                $model->save();
                GoodsTransferItems::model()->deleteAll('transfer_id =' . $model->id);
                StockLedger::model()->deleteAll('header_id=' . $model->id . ' AND  type= "GT" AND outlet=' . $userModel->outlet_id);
                if ($scene == 'new') {
                    $next_no = $model->transfer_no + 1;
                    if (!empty($modelformmaster)) {
                        FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                    } else {
                        $modelformmaster = new FormMaster();
                        $modelformmaster->code = 'GT';
                        $modelformmaster->next_number = $next_no;
                        $modelformmaster->outlet = $userModel->outlet_id;
                        $modelformmaster->save(false);
                    }
                }

                if (!empty($modelItems)) {
                    foreach ($modelItems as $each) {
                        $each->transfer_id = $model->id;
                        $each->save();
                        $model_stockledger = new StockLedger();
                        $model_stockledger->transaction_date = $model->transfer_date;
                        $model_stockledger->transaction_type = 'OUT';
                        $model_stockledger->type = 'GT';
                        $model_stockledger->customer_vendor_id = 0;
                        $model_stockledger->header_id = $model->id;
                        $model_stockledger->item_id = $each->id;
                        $model_stockledger->product_id = $each->product_id;
                        $model_stockledger->currency = 1;
                        $model_stockledger->exchange_rate = 1;
                        $model_stockledger->quantity = $each->quantity;
                        $model_stockledger->unit = $each->unit;
                        $model_stockledger->default_product_unit = $each->gt_unit_details->base_unit_id;
                        $model_stockledger->unit_exchange_rate = $each->gt_unit_details->unit_exchange_rate;
                        $model_stockledger->created_by = $model->from_user_id;
                        $model_stockledger->created_time = date('Y-m-d H:i:s');
                        $model_stockledger->outlet = $userModel->outlet_id;
                        $model_stockledger->save(false);

                        $model_stockledger = new StockLedger();
                        $model_stockledger->transaction_date = $model->transfer_date;
                        $model_stockledger->transaction_type = 'IN';
                        $model_stockledger->type = 'GT';
                        $model_stockledger->customer_vendor_id = 0;
                        $model_stockledger->header_id = $model->id;
                        $model_stockledger->item_id = $each->id;
                        $model_stockledger->product_id = $each->product_id;
                        $model_stockledger->currency = 1;
                        $model_stockledger->exchange_rate = 1;
                        $model_stockledger->quantity = $each->quantity;
                        $model_stockledger->unit = $each->unit;
                        $model_stockledger->default_product_unit = $each->gt_unit_details->base_unit_id;
                        $model_stockledger->unit_exchange_rate = $each->gt_unit_details->unit_exchange_rate;
                        $model_stockledger->created_by = $model->to_user_id;
                        $model_stockledger->created_time = date('Y-m-d H:i:s');
                        $model_stockledger->outlet = $userModel->outlet_id;
                        $model_stockledger->save(false);
                    }
                }

                // inserting to audit log
                AuditLog::saveApiAuditLogs(
                    $userModel->outlet_id,
                    $modelformmaster->id,
                    $userModel->id,
                    $change_type,
                    " with ID - {$model->id} "
                );
                // end audit log

                $this->responseBody['id'] =  $model->id;
                $this->responseBody['status'] = 'success';
                $this->_bind;
            } else {
                $error1 = $error2 = array();
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                if ($modelItems[0]->product_id > 0)
                    $error2 = json_decode(CActiveForm::validateTabular($modelItems, '', false), true);
                else
                    $error2[] = "No Products";
                $error = array_merge_recursive($error1, $error2);
                $arr_err = $other;
                foreach ($error as $key => $errval) {
                    $arr_err[] = $errval[0];
                }
                $err_msg = implode(',', $arr_err);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }


    public function actionGetDetailssById()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $id = (int) trim($variables['id']);

        if ($userModel->id > 0) {

            $userModel->db();

            if ($id > 0) {

                $model = GoodsTransfer::model()->findBypk((int) $id);
                $this->responseBody['data']['id'] = $model->id;
                $this->responseBody['data']['transfer_no'] = $model->transfer_no;
                $this->responseBody['data']['transfer_date'] = $model->transfer_date;
                $this->responseBody['data']['from_user_id'] = $model->from_user_id;
                $this->responseBody['data']['to_user_id'] = $model->to_user_id;
                $this->responseBody['data']['from_user_name'] = $model->gt_from_user->full_name;
                $this->responseBody['data']['to_user_name'] = $model->gt_to_user->full_name;
                $this->responseBody['data']['notes'] = $model->notes;
                $this->responseBody['data']['outlet'] = $model->outlet;
                $modelItems = GoodsTransferItems::model()->findAll(array('condition' => 'transfer_id="' . $id . '"', 'order' => 'id'));
                foreach ($modelItems as $i => $each) {
                    $this->responseBody['data']['items'][$i]['id'] = $each->id;
                    $this->responseBody['data']['items'][$i]['transfer_id'] = $each->transfer_id;
                    $this->responseBody['data']['items'][$i]['product_id'] = $each->product_id;
                    $this->responseBody['data']['items'][$i]['product_name'] = $each->gt_product_details->part_number;
                    $this->responseBody['data']['items'][$i]['goods_service'] = $each->gt_product_details->type;
                    $this->responseBody['data']['items'][$i]['description'] = $each->gt_product_details->description;
                    $this->responseBody['data']['items'][$i]['quantity'] = $each->quantity;
                    $this->responseBody['data']['items'][$i]['uom'] = $each->unit;
                    $this->responseBody['data']['items'][$i]['unit_label'] = $each->gt_unit_details->unit_name;
                }

                $this->responseBody['count'] = 1;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Invoice id';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }


    public function actionCancel()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $voucher_id = (int) trim($variables['id']);

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {

            $userModel->db();

            $id = $voucher_id;
            $valid = true;
            $other = array();
            $model = GoodsTransfer::model()->findByPk($id);
            if (!empty($model)) {

                $modelItems = GoodsTransferItems::model()->findAll(array('condition' => 'transfer_id="' . $id . '"', 'order' => 'id ASC'));
                $groups = array();
                foreach ($modelItems as $each) {
                    if ($each['unit'] > 0) {
                        $unit = MasUnit::model()->findByPk($each['unit']);
                        $unit_exchange = $unit->unit_exchange_rate;
                    } else {
                        $unit_exchange = 1;
                    }
                    $key = $each['product_id'];
                    if (!array_key_exists($key, $groups)) {
                        $groups[$key] = array(
                            'id' => $each['product_id'],
                            'quantity' => $each['quantity'] * $unit_exchange,
                        );
                    } else {
                        $groups[$key]['quantity'] = $groups[$key]['quantity'] + $each['quantity'] * $unit_exchange;
                    }
                }
                if (!empty($groups)) {
                    foreach ($groups as $key => $eachProduct) {
                        $product = Inventory::model()->findByPk($key);
                        if ($product->type != 'S') {
                            $condt = " AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $model->from_user_id . " AND header_id!=" . $model->id;
                            $in_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='IN' " . $condt)->queryScalar();
                            $out_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='OUT' " . $condt)->queryScalar();
                            $in_qty_on_date = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_date <='" . $model->transfer_date . "' AND transaction_type='IN' " . $condt)->queryScalar();
                            $out_qty_on_date = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_date <='" . $model->transfer_date . "' AND transaction_type='OUT' " . $condt)->queryScalar();
                            $current_stock = $in_qty - $out_qty;
                            $stock_on_date = $in_qty_on_date - $out_qty_on_date;
                            if ($current_stock < $eachProduct['quantity']) {
                                $other[] = $product->part_number . ' available quantity mismatch';
                                $valid = false;
                            }
                            if ($stock_on_date < $eachProduct['quantity']) {
                                $product = Inventory::model()->findByPk($key);
                                $other[] = $product->part_number . ' available quantity mismatch on' . $model->transfer_date;
                                $valid = false;
                            }
                        }
                    }
                }
                $closing_date = MasClosing::getClosingDate();
                if ($closing_date >= $model->transfer_date) {
                    $other[] = 'Date should be greater than closing date (' . date('d-m-Y', strtotime($closing_date)) . ')';
                    $valid = false;
                }
            } else {

                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = ' Goods Transfer Not Found';
                $this->_bind;
                exit;
            }
            if ($valid) {
                StockLedger::model()->deleteAll('header_id=' . $model->id . ' AND  type= "GT" AND outlet=' . $userModel->outlet_id);
                $model->active = 'N';
                $model->save(false);
                $this->responseBody['status'] = 'success';
                $this->_bind;
            } else {
                $err_msg = implode(',', $other);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->responseBody['items'] = array('Negative stock is not allowed');
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }



    public function actionPrint()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);

            $model = GoodsTransfer::model()->findByPk($id);
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = 'GT_' . $model->transfer_no;

            if ($model->active == 'N') {
                $mPDF1->SetWatermarkText('CANCELLED');
                $mPDF1->showWatermarkText = true;
            }
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.goods_transfer_print', array('id' => $id, 'user_id' => $userModel->id), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
}
