<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiLedgerBalanceController extends Controller {

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules() {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('Customer','Vendor'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters() {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request) {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401() {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400() {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403() {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind() {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value) {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions() {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null) {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name) {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }
 
    
    
   
    
    public function actionCustomer() {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $ledger_id = (int) trim($variables['ledger_id']); 

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {

            $userModel->db();
            
                if($ledger_id>0)
                {
                $id = (int)FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $ledger_id . ' AND subledger_type_id=1'))->id;    
                $ledgers=Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.is_posted='Y' AND I.ledger_id=2 AND I.subledger_id=".$id)->queryScalar();
                }
                $this->responseBody['balance'] = $ledgers;
                $this->responseBody['status'] = 'success';
                $this->_bind;
                 
        } else {
            $this->_401;
        }
    }

    
    public function actionVendor() {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $ledger_id = (int) trim($variables['ledger_id']);
         

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {

            $userModel->db();
             
                if($ledger_id>0)
                {
                $id = (int)FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $ledger_id . ' AND subledger_type_id=2'))->id;    
                $ledgers=Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.is_posted='Y' AND I.ledger_id=6 AND I.subledger_id=".$id)->queryScalar();
                }
                $this->responseBody['balance'] = -1*$ledgers;
                $this->responseBody['status'] = 'success';
                $this->_bind;
            
        } else {
            $this->_401;
        }
    }

}