<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiMastersController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('Index', 'Unit', 'GetPaymentTermList', 'GetTaxTypeList'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }





    public function actionIndex()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {

            $userModel->db();

            $currencies = MasCurrency::model()->findAll(array('condition' => 'isActive="Y"'));
            $countries = MasCountry::model()->findAll(array('condition' => 'active="Y"'));
            $emirates = MasState::model()->findAll(array('condition' => 'active="Y"'));
            //$expenseTypes = VanLedgers::model()->_expense_heads;
            $productGroups = MasBrand::model()->findAll(array('condition' => 'active="Y" AND outlet=' . $userModel->outlet_id));
            $unitHeads = MasUnit::model()->findAll(array('condition' => 'active="Y" AND base_unit="Y" AND outlet=' . $userModel->outlet_id));
            $taxItems = MasTax::model()->findAll(array('condition' => 'active="Y"'));
            $genericItems = MasGeneric::model()->findAll(array('condition' => 'active="Y" AND outlet=' . $userModel->outlet_id));
            //$expenseHeads = VanLedgers::model()->get_pl_groups((int) $userModel->outlet_id);
            $paymentterms = MasPaymentTerms::model()->findAll(array('condition' => 'active="Y"'));

            foreach ($currencies as $i => $each) {
                $this->responseBody['data']['currencies'][$i]['id'] = $each->id;
                $this->responseBody['data']['currencies'][$i]['name'] = $each->currency;
                $this->responseBody['data']['currencies'][$i]['code'] = $each->currency_code;
                $this->responseBody['data']['currencies'][$i]['exchange_rate'] = $each->exchange_rate;
            }

            foreach ($countries as $i => $each) {
                $this->responseBody['data']['countries'][$i]['id'] = $each->id;
                $this->responseBody['data']['countries'][$i]['name'] = $each->country_name;
                $this->responseBody['data']['countries'][$i]['code'] = $each->country_code;
            }

            foreach ($emirates as $i => $each) {
                $this->responseBody['data']['emirates'][$i]['id'] = $each->id;
                $this->responseBody['data']['emirates'][$i]['name'] = $each->state_name;
            }

            foreach ($genericItems as $i => $each) {
                $this->responseBody['data']['generic'][$i]['id'] = $each->id;
                $this->responseBody['data']['generic'][$i]['name'] = $each->generic_name;
            }

            foreach ($productGroups as $i => $each) {
                $this->responseBody['data']['product_groups'][$i]['id'] = $each->id;
                $this->responseBody['data']['product_groups'][$i]['name'] = $each->brand_name;
            }

            foreach ($unitHeads as $i => $each) {
                $this->responseBody['data']['uom'][$i]['id'] = $each->id;
                $this->responseBody['data']['uom'][$i]['symbol'] = $each->label;
                $this->responseBody['data']['uom'][$i]['name'] = $each->unit_name;
                $this->responseBody['data']['uom'][$i]['exchange_rate'] = $each->unit_exchange_rate;
                $this->responseBody['data']['uom'][$i]['base_unit_id'] = $each->base_unit_id;
            }

            foreach ($taxItems as $i => $each) {
                $this->responseBody['data']['tax_percentage'][$i]['id'] = $each->id;
                $this->responseBody['data']['tax_percentage'][$i]['symbol'] = $each->tax_code;
                $this->responseBody['data']['tax_percentage'][$i]['percentage'] = $each->tax_percentage;
            }
            foreach ($paymentterms as $i => $each) {
                $this->responseBody['data']['payment_term'][$i]['id'] = $each->id;
                $this->responseBody['data']['payment_term'][$i]['name'] = $each->payment_term_name;
                $this->responseBody['data']['payment_term'][$i]['days'] = $each->days;
            }
            $productGroups = MasBrand::model()->findAll(array('condition' => 'active="Y"'));
            foreach ($productGroups as $i => $each) {
                $this->responseBody['data']['brand'][$i]['id'] = $each->id;
                $this->responseBody['data']['brand'][$i]['name'] = $each->brand_name;
            }

            /* foreach ($expenseHeads as $i => $each) {
              $this->responseBody['data']['expense_heads'][$i]['id'] = $each->id;
              $this->responseBody['data']['expense_heads'][$i]['name'] = $each->withparents;
              } */

            $this->_bind;
        } else {
            $this->_401;
        }
    }



    public function actionUnit()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        //error_log('ledger_details');
        //error_log($input);

        $variables = json_decode($input, true);
        $base_unit_id = trim($variables['base_unit_id']);
        if ($userModel->id > 0) {

            $userModel->db();
            $condition = 'active="Y" AND outlet=' . $userModel->outlet_id;
            if ($base_unit_id > 0) {
                $condition .= ' AND base_unit_id=' . $base_unit_id;
            }
            $unitHeads = MasUnit::model()->findAll(array('condition' => $condition));



            foreach ($unitHeads as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['symbol'] = $each->label;
                $this->responseBody['data'][$i]['unit_name'] = $each->unit_name;
                $this->responseBody['data'][$i]['unit_factor'] = $each->unit_exchange_rate;
                $this->responseBody['data'][$i]['base_unit_id'] = $each->base_unit_id;
            }

            /* foreach ($expenseHeads as $i => $each) {
              $this->responseBody['data']['expense_heads'][$i]['id'] = $each->id;
              $this->responseBody['data']['expense_heads'][$i]['name'] = $each->withparents;
              } */

            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionGetPaymentTermList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        //error_log('ledger_details');
        //error_log($input);

        $variables = json_decode($input, true);
        $base_unit_id = trim($variables['base_unit_id']);
        if ($userModel->id > 0) {
            $userModel->db();
            $MasPaymentTermslist = MasPaymentTerms::model()->findAll(array('condition' => 'active="Y"'));
            foreach ($MasPaymentTermslist as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['payment_term_name'] = $each->payment_term_name;
            }
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionGetTaxTypeList()
    {
        try {
            $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
            $userModel = MasUsers::model()->findBytoken($token);
            if ($userModel->id > 0) {
                $taxtypelist = array('E' => 'Exclude Tax', 'I' => 'Include Tax');
                if (!empty($taxtypelist)) {
                    foreach ($taxtypelist as $code => $name) {
                        $list[] = [
                            'code' => $code,
                            'payment_term_name' => $name
                        ];
                    }
                    $data = array("status" => "success", "data" => $list);
                } else {
                    $data = array("status" => "success", "data" => $list, "msg" => "Empty Records");
                }
            } else {
                header("HTTP/1.1 401 Unauthorized");
                header('Content-Type: application/json');
                $data = array('status' => "error", "msg" => "Unauthorized user");
            }
        } catch (Exception $e) {
            header("HTTP/1.1 404 not found");
            header('Content-Type: application/json');
            $data = array('status' => "error", "msg" => "Unexpected Server error");
        }
        echo json_encode($data);
    }
}
