<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiPurchaseController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array("GetList", "Save", "GetPurchaseDetailsById", "PurchaseCancel", "Print"),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }


    public function actionGetList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $from = trim($variables['from']);
        $to = trim($variables['to']);

        $from = $this->convert_date($from);
        $to = $this->convert_date($to);
        if ($userModel->id > 0) {

            $userModel->db();

            $count = 0;

            $model = PurchaseInvoice::model()->findAll(array('condition' => 'invoice_date BETWEEN "' . $from . '" AND "' . $to . '" AND created_by=' . $userModel->id, 'order' => 'invoice_date DESC, invoice_no DESC'));
            foreach ($model as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['invoice_no'] = $each->invoice_no;
                $this->responseBody['data'][$i]['prefix'] = $each->prefix;
                $this->responseBody['data'][$i]['currency_code'] = $each->purchase_cur->currency_code;
                $this->responseBody['data'][$i]['date'] = $each->invoice_date;
                $this->responseBody['data'][$i]['exchange_rate'] = $each->exchange_rate;
                $this->responseBody['data'][$i]['vendor'] = $each->vendor->vendor_name;
                $this->responseBody['data'][$i]['vendor_invoice_no'] = $each->reference_no;
                $this->responseBody['data'][$i]['total_amount'] = $this->format_currency($each->total_amount);
                $this->responseBody['data'][$i]['vat_amount'] = $this->format_currency($each->tax_total);
                $this->responseBody['data'][$i]['excise_total'] = $this->format_currency($each->excise_total);
                $this->responseBody['data'][$i]['grand_total'] = $this->format_currency($each->grand_total);
                $this->responseBody['data'][$i]['is_posted'] = $each->posted == 'Y' ? true : false;
                $model_purchase_return = StockLedger::model()->with('purchasereturn_det')->findAll(array('condition' => 'purchasereturn_det.invoice_id=' . $each->id));
                $this->responseBody['data'][$i]['have_any_active_returns'] = (empty($model_purchase_return)) ? false : true;
                $this->responseBody['data'][$i]['is_cancelable'] = (empty($model_purchase_return)) ? true : false;
                $this->responseBody['data'][$i]['is_cancelled'] = $each->active == 'Y' ? false : true;
                $posted_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=8 AND voucher_no=' . $each->invoice_no));
                $this->responseBody['data'][$i]['is_allocated'] = ($posted_voucher->is_allocated == 'Y') ? true : false;
                $this->responseBody['data'][$i]['overdue'] = $each->pay_term->days;
                $this->responseBody['data'][$i]['received'] = $each->received;
                $this->responseBody['data'][$i]['discount'] = $each->discount;
                $this->responseBody['data'][$i]['old_discount'] = $each->old_discount;
                $this->responseBody['data'][$i]['notes'] = $each->notes;
                $this->responseBody['data'][$i]['status'] = $each->active == 'Y' ? 'Active' : 'Cancel';
                $count++;
            }

            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }



    public function actionGetPurchaseDetailsById()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $id = (int) trim($variables['id']);

        if ($userModel->id > 0) {

            $userModel->db();

            if ($id > 0) {

                $model = PurchaseInvoice::model()->with('vendor')->findBypk((int) $id);
                $this->responseBody['data']['vendor'] = $model->vendor->vendor_name;
                $this->responseBody['data']['vendor_id'] = $model->vendor_id;
                $this->responseBody['data']['parent_invoice'] = $model->id;
                $this->responseBody['data']['id'] = $model->id;
                $this->responseBody['data']['invoice_no'] = $model->invoice_no;
                $this->responseBody['data']['total_amount'] = $this->format_currency($model->total_amount);
                $this->responseBody['data']['excise_total'] = $this->format_currency($model->excise_total);
                $this->responseBody['data']['tax_total'] = $this->format_currency($model->tax_total);
                $this->responseBody['data']['grand_total'] = $this->format_currency($model->grand_total);
                $this->responseBody['data']['payment_term'] = $model->payment_term;
                $this->responseBody['data']['credit_days'] = 0;
                $this->responseBody['data']['invoice_date'] = $model->invoice_date;
                $this->responseBody['data']['currency'] = $model->currency;
                $this->responseBody['data']['exchange_rate'] = $model->exchange_rate;
                $this->responseBody['data']['outlet'] = $model->outlet;
                $modelItems = PurchaseInvoiceItems::model()->findAll(array('condition' => 'invoice_id="' . $id . '"', 'order' => 'id'));
                foreach ($modelItems as $i => $each) {
                    $this->responseBody['data']['items'][$i]['id'] = $each->id;
                    $this->responseBody['data']['items'][$i]['invoice_id'] = $each->invoice_id;
                    $this->responseBody['data']['items'][$i]['ledger_id'] = $each->product_id;
                    $this->responseBody['data']['items'][$i]['product_id'] = $each->product_id;
                    $this->responseBody['data']['items'][$i]['ledger_name'] = $each->product->part_number;
                    $this->responseBody['data']['items'][$i]['goods_service'] = $each->product->type;
                    $this->responseBody['data']['items'][$i]['description'] = $each->product->description;
                    $this->responseBody['data']['items'][$i]['quantity'] = $each->quantity;
                    $this->responseBody['data']['items'][$i]['max_returnable_quantity'] = $each->quantity;
                    $this->responseBody['data']['items'][$i]['uom'] = $each->unit;
                    $this->responseBody['data']['items'][$i]['unit_label'] = $each->units->unit_name;
                    $this->responseBody['data']['items'][$i]['rate'] = $this->format_currency($each->rate);
                    $this->responseBody['data']['items'][$i]['excise_percentage'] = $each->excise_percentage;
                    $this->responseBody['data']['items'][$i]['excise_amount'] = $this->format_currency($each->excise_amount);
                    $this->responseBody['data']['items'][$i]['tax_amount'] = $this->format_currency($each->tax_amount);
                    $this->responseBody['data']['items'][$i]['tax_percentage'] = str_replace('%', '', $each->tax_percentage);
                    $this->responseBody['data']['items'][$i]['net_amount'] = $this->format_currency($each->net_amount);
                }

                $this->responseBody['count'] = 1;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Invoice id';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }




    public function actionPurchaseCancel()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $voucher_id = (int) trim($variables['id']);

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {

            $userModel->db();

            $id = $voucher_id;
            $valid = true;
            $other = array();
            $model = PurchaseInvoice::model()->findByPk($id);
            $modelItems = PurchaseInvoiceItems::model()->findAll(array('condition' => 'invoice_id="' . $id . '"', 'order' => 'id ASC'));
            $groups = array();
            foreach ($modelItems as $each) {
                if ($each['unit'] > 0) {
                    $unit = MasUnit::model()->findByPk($each['unit']);
                    $unit_exchange = $unit->unit_exchange_rate;
                } else {
                    $unit_exchange = 1;
                }
                $key = $each['product_id'];
                if (!array_key_exists($key, $groups)) {
                    $groups[$key] = array(
                        'id' => $each['product_id'],
                        'quantity' => $each['quantity'] * $unit_exchange,
                    );
                } else {
                    $groups[$key]['quantity'] = $groups[$key]['quantity'] + $each['quantity'] * $unit_exchange;
                }
            }
            if (!empty($groups)) {
                foreach ($groups as $key => $eachProduct) {
                    $product = Inventory::model()->findByPk($key);
                    if ($product->type != 'S') {
                        $in_qty = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from {{stock_ledger}} where transaction_type='IN' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND transaction_date<='" . $model->invoice_date . "'")->queryScalar();
                        $out_qty = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from {{stock_ledger}} where transaction_type='OUT' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND transaction_date<='" . $model->invoice_date . "'")->queryScalar();
                        $current_stock = $in_qty - $out_qty;
                        if ($current_stock < $eachProduct['quantity']) {
                            $product = Inventory::model()->findByPk($key);
                            $other[] = $product->part_number . ' available quantity mismatch';
                            $valid = false;
                        }
                    }
                }
            }
            $closing_date = MasClosing::getClosingDate();
            if ($closing_date >= $model->invoice_date) {
                $other[] = 'Date should be greater than closing date (' . date('d-m-Y', strtotime($closing_date)) . ')';
                $valid = false;
            }
            if ($valid) {
                $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=8 AND header_table_id=' . $model->id));
                if (!empty($fin_voucher)) {
                    FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
                    FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
                }
                StockLedger::model()->deleteAll('header_id=' . $model->id . ' AND  transaction_type="IN" AND type= "PI" AND outlet=' . $userModel->outlet_id);
                $model->active = 'N';
                $model->posted = 'N';
                $model->save(false);
                $this->responseBody['status'] = 'success';
                $this->responseBody['msg'] = 'Cancelled Successfully';
                $this->_bind;
            } else {
                $err_msg = implode(',', $other);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->responseBody['items'] = array('Negative stock is not allowed');
               $this->_bind;
            }
        } else {
            $this->_401;
        }
    }

    public function actionSave()
    {

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $this->save_request($input);
        $variables = json_decode($input, true);

        $invoice_date = trim($variables['invoice_date']);
        $id = trim($variables['id']);
        $vendor = trim($variables['vendor_id']);
        $lpo = trim($variables['lpo']);
        $currency = 1;
        $vendor_invoice = trim($variables['vendor_invoice']);

        $taxable = $variables['taxable'] == true ? true : false;
        $cash_paid = $variables['cash_paid'] == true ? true : false;
        $amount_paid = (float) $variables['amount_paid'];
        $amount_discount = (float) $variables['discount_amount'];
        $exclude_tax = $variables['exclude_tax'];
        $taxable = $variables['taxable'];
        $invoiceItems = $variables['items'];
        $other = array();
        if ($userModel->id > 0) {

            $userModel->db();
            if ($currency > 0) {
                $currency_id = $currency;
                $exchange_rate = MasCurrency::getCurrencyDetails($currency)->exchange_rate;
            } else {
                $currency_id = MasCurrency::model()->base_currency_id;
                $exchange_rate = MasCurrency::model()->base_exchange_rate;
            }
            $valid_vendor = ($vendor > 0 && Vendor::model()->findBypk((int) $vendor)->id > 0) ? true : false;
            if ($valid_vendor) {
                $model = new PurchaseInvoice;
                $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="PI"'));
                if ($id > 0) {
                    $model = $model->findBypk($id);
                    $scene = 'update';
                    $change_type = 'UPDATED';
                } else {
                    $model->created_by = $userModel->id;
                    $model->created_time = date('Y-m-d H:i:s');
                    $scene = 'new';
                    $change_type = 'CREATED';

                    $model->invoice_no = $modelformmaster->next_number;
                }
                $model->updated_by = $userModel->id;
                $model->updated_time = date('Y-m-d H:i:s');
                $model->invoice_date = count(explode('-', $invoice_date)) == 3 ? $invoice_date : date('Y-m-d');
                $model->vendor_id = $vendor;
                $model->payment_term = (int)trim($variables['payment_term']);
                $model->total_amount = 0;
                $model->reference_no = trim($variables['reference_no']);
                $model->notes = trim($variables['notes']);
                $model->posted = "Y";
                $model->callback_from = 'app';
                $model->excise_total = 0;
                $model->tax_total = 0;
                $model->grand_total = 0;
                $model->currency = $currency_id;
                $model->exchange_rate = $exchange_rate;
                $model->outlet = $userModel->outlet_id;
                $model->received = $amount_paid;
                $model->discount = $amount_discount;
                $model->exclude_tax = $exclude_tax == true ? "Y" : "N";
                $model->taxable = $taxable == true ? "Y" : "N";
                $i = 0;
                $total_vat = 0;
                $grand_total = 0;
                $total_sales = 0;
                $total_discount = 0;
                $grand_total_amount = 0;
                $total_excise_amount = 0;
                $valid = true;
                $groups = array();
                foreach ($invoiceItems as $each) {
                    if ($each['product_id'] > 0) {
                        $productModel = Inventory::model()->findBypk($each['product_id']);
                        $items[$i] = new PurchaseInvoiceItems;
                        $items[$i]->product_id = $each['product_id'];
                        $items[$i]->description = $productModel->description;
                        $items[$i]->quantity = (float) $each['quantity'];
                        $items[$i]->rate = (float) $each['price'];
                        $items[$i]->unit = $each['unit'];
                        $amount = round(((float) $each['quantity'] * (float) $each['price']), 6);
                        $items[$i]->amount = (float) $amount;
                        $items[$i]->invoice_id = 0;
                        $items[$i]->tax_percentage = $productModel->taxDetails->tax_percentage;
                        $items[$i]->tax_amount = $taxable ? round((($amount * $productModel->taxDetails->tax_percentage) / 100), 2) : 0;
                        $items[$i]->net_amount = round((float) $items[$i]->tax_amount, 3) + round((float) $amount, 3);
                        $grand_total_amount += $items[$i]->net_amount;
                        if ($each['unit'] > 0) {
                            $unit = MasUnit::model()->findByPk($each['unit']);
                            $unit_exchange = $unit->unit_exchange_rate;
                        } else {
                            $items[$i]->unit = $productModel->unit;
                            $unit_exchange = 1;
                        }

                        $key = $each['product_id'];
                        if (!array_key_exists($key, $groups)) {
                            $groups[$key] = array(
                                'id' => $each['product_id'],
                                'quantity' => $each['quantity'] * $unit_exchange,
                            );
                        } else {
                            $groups[$key]['quantity'] = $groups[$key]['quantity'] + $each['quantity'] * $unit_exchange;
                        }
                        $total_amount += $amount;
                        $grand_total_amount += $amount;
                        $total_vat += round((float) $items[$i]->tax_amount, 2);
                        $grand_total += round((float) $items[$i]->tax_amount, 2);
                        $grand_total += round((float) $amount, 3);
                        $i++;
                    }
                }
                // QTY Checking
                if (!empty($groups)) {
                    foreach ($groups as $key => $eachProduct) {
                        $product = Inventory::model()->findByPk($key);
                        if ($product->type != 'S') {
                            $qty = Yii::app()->db->createCommand("select SUM(P.quantity*U.unit_exchange_rate) from {{purchase_invoice_items}} P INNER JOIN {{master_unit}} U ON P.unit=U.id where invoice_id='" . $model->id . "' AND product_id='" . $key . "'")->queryScalar();
                            $in_qty = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from {{stock_ledger}} where transaction_type='IN' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND transaction_date<='" . $model->invoice_date . "'")->queryScalar();
                            $out_qty = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from {{stock_ledger}} where transaction_type='OUT' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND transaction_date<='" . $model->invoice_date . "'")->queryScalar();
                            $current_stock_on_date = $in_qty - $out_qty + $qty;
                            $bal = $qty - $eachProduct['quantity'];
                            if ($bal > 0 && $current_stock_on_date < $eachProduct['quantity']) {
                                $product = Inventory::model()->findByPk($key);
                                $other[] = $product->part_number . ' available quantity on ' . $model->invoice_date . ' mismatch';
                                $valid = false;
                            }
                            $in_qty1 = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from {{stock_ledger}} where transaction_type='IN' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " ")->queryScalar();
                            $out_qty1 = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from {{stock_ledger}} where transaction_type='OUT' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " ")->queryScalar();
                            $current_stock = $in_qty1 - $out_qty1 + $qty;
                            if ($bal > 0 && $current_stock < $eachProduct['quantity']) {
                                $product = Inventory::model()->findByPk($key);
                                $other[] = $product->part_number . ' available quantity mismatch';
                                $valid = false;
                            }
                        }
                    }
                }
                $grand_total = round((float) $grand_total, 2);
                $model->total_amount = $total_amount;
                $model->excise_total = $total_excise_amount;
                $model->tax_total = $total_vat;
                $model->grand_total = $grand_total;
                if (($model->received + $model->discount) > $grand_total) {
                    $other[] = 'Paid amount can not greater than invoice amount';
                    $valid = false;
                }
                /*if (($model->received + $model->discount) < $grand_total) {
                   // $model->payment_term = 2;
                } else if (($model->received + $model->discount) == $grand_total) {
                    $model->payment_term = 1;
                }*/
                $closing_date = MasClosing::getClosingDate();
                if ($closing_date >= $model->invoice_date) {
                    $other[] = 'Date should be greater than closing date (' . date('d-m-Y', strtotime($closing_date)) . ')';
                    $valid = false;
                }
                if ($valid && $model->validate() && is_array($items)) {
                    $model->save();
                    if ($scene == 'new') {
                        $next_no = $model->invoice_no + 1;
                        FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                    }

                    Yii::app()->db->createCommand("delete from {{stock_ledger}} where transaction_type='IN' AND type='PI' AND header_id='" . $model->id . "' AND outlet=" . $userModel->outlet_id)->execute();
                    Yii::app()->db->createCommand("delete from {{purchase_invoice_items}} where invoice_id='" . $model->id . "'")->execute();

                    $total_amount = 0;
                    $total_tax_amount = 0;
                    $total_excise_amount = 0;
                    $grand_total = 0;
                    foreach ($items as $eachItem) {
                        $eachItem->invoice_id = $model->id;
                        $total_amount += $eachItem->amount;
                        $total_tax_amount += $eachItem->tax_amount;
                        $total_excise_amount += $eachItem->excise_amount;
                        $eachItem->save(false);
                        $grand_total += $eachItem->net_amount;
                        $stockledger = new StockLedger;
                        $stockledger->transaction_date = $model->invoice_date;
                        $stockledger->transaction_type = 'IN';
                        $stockledger->type = 'PI';
                        $stockledger->customer_vendor_id = $model->vendor_id;
                        $stockledger->header_id = $model->id;
                        $stockledger->item_id = $eachItem->id;
                        $stockledger->product_id = $eachItem->product_id;
                        $stockledger->rate = $eachItem->rate;
                        $stockledger->currency = 1;
                        $stockledger->exchange_rate = 1;
                        $stockledger->quantity = $eachItem->quantity;
                        $stockledger->unit = $eachItem->unit;
                        $stockledger->default_product_unit = $eachItem->units->base_unit_id;
                        $stockledger->unit_exchange_rate = $eachItem->units->unit_exchange_rate;
                        $stockledger->outlet = $userModel->outlet_id;
                        $stockledger->created_by = $userModel->id;
                        $stockledger->created_time = date('Y-m-d H:i:s');
                        $stockledger->save(false);
                    }
                    // inserting to audit log
                    AuditLog::saveApiAuditLogs(
                        $userModel->outlet_id,
                        $modelformmaster->id,
                        $userModel->id,
                        $change_type,
                        " with ID - {$model->id} "
                    );
                    // end audit log
                    $model->total_amount = $total_amount;
                    $model->excise_total = $total_excise_amount;
                    $model->tax_total = $total_tax_amount;
                    $model->grand_total = $grand_total;
                    $model->save();
                    $this->purchaseInvoicePosting($model->id);
                    $this->responseBody['msg'] = "Purchase Invoice Saved Successfully";
                    $this->responseBody['id'] = $model->id;
                    $this->_bind;
                } else {
                    $this->responseBody['status'] = 'failed';
                    $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                    if (!empty($items))
                        $error2 = json_decode(CActiveForm::validateTabular($items, '', false), true);
                    else
                        $error2[] = "Please add atleast one line item";
                    $error = array_merge_recursive($error1, $error2);
                    $arr_err = $other;
                    foreach ($error as $key => $errval) {
                        $arr_err[] = $errval[0];
                    }
                    $err_msg = implode(',', $arr_err);
                    $this->responseBody['msg'] = $err_msg;
                      $this->_bind;
                }
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Vendor';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }



    function invoicePosting($invoice_id)
    {
        $mainTable = SalesInvoice::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="SI" AND outlet=' . $outlet))->id;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->invoice_no;
        $voucherHead['voucher_date'] = $mainTable->invoice_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->invoice_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->sales_man;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->invoice_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_debtor_amount = $mainTable->grand_total;
        $total_sales = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->received;
        if ($total_debtor_amount > 0) {
            //Debtor Dr
            $debit_item_sales = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1), //2-Creditors 1- debtors
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => (float) $total_debtor_amount,
                'outlet' => $mainTable->outlet
            ));
            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="CA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                $voucherHead['payment_method'] = 1;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->sales_man;
                $lastSavedid_receipt_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_receipt = $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->sales_man;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_receipt_voucher;
                $allocation_items->voucher_item_id = $debit_item_receipt;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="DA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->sales_man;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => -1 * $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => (float) $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->sales_man;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_sales > 0) {
            //Sales Accounts Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => -1 * $total_sales,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => -1 * $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }


    function purchaseInvoicePosting($invoice_id)
    {
        $mainTable = PurchaseInvoice::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="PI" AND outlet=' . $outlet))->id;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->invoice_no;
        $voucherHead['voucher_date'] = $mainTable->invoice_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->invoice_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->created_by;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->invoice_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_creditor_amount = $mainTable->grand_total;
        $total_purchase = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_excise_amount = $mainTable->excise_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->received;

        if ($total_creditor_amount > 0) {
            //Creditor Cr
            $credit_item_purchase = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => -1 * $total_creditor_amount,
                'outlet' => $mainTable->outlet
            ));
            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="PV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_payment_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $credit_item_payment = $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save();

                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_payment_voucher;
                $allocation_items->voucher_item_id = $credit_item_payment;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="JV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => -1 * $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_purchase > 0) {
            //Purchase Accounts Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => (float) $total_purchase,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => (float) $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        if ($total_excise_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('excise_tax'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => (float) $total_excise_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }

    function purchasereturnPosting($invoice_id)
    {
        $mainTable = PurchaseReturn::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="DN" AND outlet=' . $outlet))->id;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->return_no;
        $voucherHead['voucher_date'] = $mainTable->return_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->return_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->created_by;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->return_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_creditor_amount = $mainTable->grand_total;
        $total_purchase = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_excise_amount = $mainTable->excise_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->paid;

        if ($total_creditor_amount > 0) {
            //Creditor Cr
            $credit_item_purchase = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => (float) $total_creditor_amount,
                'outlet' => $mainTable->outlet
            ));
            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="CA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_payment_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $credit_item_payment = $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('total_voucher_amount' => ($total_creditor_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save();

                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_payment_voucher;
                $allocation_items->voucher_item_id = $credit_item_payment;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="JV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = 3;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => (float) $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_purchase > 0) {
            //Purchase Accounts Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => -1 * $total_purchase,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => -1 * $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        if ($total_excise_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('excise_tax'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => -1 * $total_excise_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }


    function isPostable($date)
    {
        $last_closed_year = FinanceAuditClose::lastClosed();
        $voucherDate_array = explode('-', $date);
        $voucherYear = $voucherDate_array[0];
        return ($voucherYear <= $last_closed_year) ? false : true;
    }

    public function addVoucherhead($arr_items)
    {
        $model = new FinanceVoucherHead;
        $model->voucher_type = $arr_items['voucher_type'];
        $model->voucher_no = $arr_items['voucher_no'];
        $model->voucher_date = $arr_items['voucher_date'];
        $model->related_subledger = $arr_items['related_subledger'];
        $model->emp_subledger = $arr_items['emp_subledger'];
        $model->due_date = (isset($arr_items['due_date'])) ? $arr_items['due_date'] : $model->voucher_date;
        $model->header_table_id = $arr_items['header_table_id'];
        $model->reference_no = $arr_items['reference_no'];
        $model->is_posted = (isset($arr_items['is_posted'])) ? $arr_items['is_posted'] : 'Y';
        $model->created_by = $arr_items['created_by'];
        $model->created_datetime = date('Y-m-d h:i:s');
        $model->cheque_no = $arr_items['cheque_no'];
        $model->outlet = $arr_items['outlet'];
        $model->payment_method = $arr_items['payment_method'];
        return ($model->save(false)) ? $model->id : 0; //exception
    }

    public function addVoucheritems($headerId, $arr_items)
    {
        $model = new FinanceVoucherItems;
        $model->header_id = $headerId;
        $model->ledger_id = $arr_items['ledger_id'];
        $model->brand_id = (int) $arr_items['brand_id'];
        $model->emirate_id = $arr_items['emirate_id'];
        $model->subledger_id = $arr_items['subledger_id'];
        $model->narration = $arr_items['narration'];
        $model->amount = $arr_items['amount'];
        $model->currency_id = $arr_items['currency'] > 0 ? $arr_items['currency'] : FinanceLegder::baseCurrency_id();
        $model->exchange_rate = $arr_items['exchange_rate'] > 0 ? $arr_items['exchange_rate'] : 1;
        $model->amount = $arr_items['amount'];
        $model->reconciled_date = $arr_items['reconciled_date'];
        $cleared = $arr_items['opening_cheque_cleared'];
        if ($cleared == '')
            $cleared = 'N';
        $model->opening_cheque_cleared = $cleared;
        $reconciled = $arr_items['reconciled'];
        if ($reconciled == '')
            $reconciled = 'N';
        $model->reconciled = $reconciled;
        $model->outlet = $arr_items['outlet'];
        $model->save(false);
        return $model->id > 0 ? $model->id : 0;
    }

    function getSublegder($parnet_id, $subledger_type)
    {
        return FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $parnet_id . ' AND subledger_type_id=' . (int) $subledger_type))->id;
    }




    function salesreturnPosting($invoice_id)
    {
        $mainTable = SalesReturn::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=10 AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->return_no;
        $voucherHead['voucher_date'] = $mainTable->return_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="CN" AND outlet=' . $outlet))->id;
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->return_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->created_by;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->return_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_debtor_amount = $mainTable->grand_total;
        $total_sales = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->paid;
        if ($total_debtor_amount > 0) {
            //Debtor Dr
            $debit_item_sales = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1), //2-Creditors 1- debtors
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => -1 * $total_debtor_amount,
                'outlet' => $mainTable->outlet
            ));

            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="PV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_receipt_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_receipt = $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_receipt_voucher;
                $allocation_items->voucher_item_id = $debit_item_receipt;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="DA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => (float) $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => -1 * $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_sales > 0) {
            //Sales Accounts Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_sales,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }
    
    
    

public function actionPrint()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);

            $model = PurchaseInvoice::model()->findByPk($id);
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = 'PI_' . $model->invoice_no;

            if ($model->active == 'N') {
                $mPDF1->SetWatermarkText('CANCELLED');
                $mPDF1->showWatermarkText = true;
            }
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.purchase_invoice_print', array('id' => $id, 'user_id' => $userModel->id), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    

  

}
