<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiReportsController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array(
                    'SalesReport',
                    'NetSalesReport',
                    'DailySalesReport',
                    'SalesPDFByID',
                    'PurchasePDFByID',
                    'ConsolidatedReport',
                    'BrandwiseReport',
                    'CurrentStockReport',
                    'PrintCurrentStockPos',
                    'TransactionDetailReport'
                ),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }

    //Reports function start
    public function actionSalesReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "sales";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params =  array(
                'from_date' => $variables['from_date'],
                'to_date' => $variables['to_date'],
                'customer_id' => (int) $variables['customer_id'],
                'salesman_id' => $salesman,
                'outlet_id' => $outlet,
                'view' => 'pdf_report_template_app',
                'title' => 'Sales Invoices',
                'mainDb' => $this->db_main,
            );
            $condition = 'V.invoice_date >= "' . $params['from_date'] . '" AND V.invoice_date <= "' . $params['to_date'] . '" ';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }
            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.sales_man=' . $params['salesman_id'];
            }
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_sales_invoice_list AS SELECT
                                                V.id,
                                                V.invoice_date AS date,
                                                V.invoice_no AS voucher_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                                O.name AS outlet_name,
						MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                                FROM {{sales_invoice}} V
                                            LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                            LEFT JOIN {{customer}} C ON C.id = V.customer_id
                                            LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = V.sales_man
                                            LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC')->execute();

            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_sales_report', array('title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    public function actionNetSalesReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "net_sales";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params =  array(
                'from_date' => $variables['from_date'],
                'to_date' => $variables['to_date'],
                'customer_id' => (int) $variables['customer_id'],
                'salesman_id' => $salesman,
                'outlet_id' => $outlet,
                'view' => 'pdf_report_template_app',
                'title' => 'Net Sales',
                'mainDb' => $this->db_main,
            );
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_net_sales_report', array('title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            echo "else";
            $this->_401;
        }
    }
    public function actionDailySalesReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "daily_sales_report";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params = array(
                'from_date' => $variables['from_date'],
                'to_date' => $variables['to_date'],
                'type' => $variables['report_type'],
                'customer_id' => (int) $variables['customer_id'],
                'salesman_id' => $salesman,
                'outlet_id' => $outlet,
                'view' => 'pdf_report_template_app',
                'title' => 'Daily Sales Report',
                'mainDb' => $this->db_main,
            );
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_daily_sales_report', array('title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionSalesPDFByID()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);

            $model = SalesInvoice::model()->findByPk($id);
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = 'SI_' . $model->invoice_no;

            if ($model->active == 'N') {
                $mPDF1->SetWatermarkText('CANCELLED');
                $mPDF1->showWatermarkText = true;
            }
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.sales_invoice_print', array('id' => $id, 'user_id' => $userModel->id), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    function getSublegder($parnet_id, $subledger_type)
    {
        return FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $parnet_id . ' AND subledger_type_id=' . (int) $subledger_type))->id;
    }

    public function actionPurchasePDFByID()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);

            $model = PurchaseInvoice::model()->findByPk($id);
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = 'PI_' . $model->invoice_no;

            if ($model->active == 'N') {
                $mPDF1->SetWatermarkText('CANCELLED');
                $mPDF1->showWatermarkText = true;
            }
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.api.pdf.purchase_invoice_print', array('id' => $id, 'user_id' => $userModel->id), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionConsolidatedReport()
    {
        
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "consolidated_report";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params =  array(
                'daybook_from_date' => $variables['from_date'],
                'daybook_to_date' => $variables['to_date'],
                'type' => $variables['report_type'],
                'created_user' => $salesman,
                'outlet' => $outlet,
                'view' => 'pdf_report_template_app',
                'title' => 'Consolidated Report',
                'mainDb' => $this->db_main,
                'user_id' =>$userModel->id,
            );
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            
            $mpdf->WriteHTML($this->renderPartial('application.views.front.reports.pdf.daily_report_list', $params, true));
           // $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_consolidated_report', array('title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionBrandwiseReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "brandwise";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params =  array(
                'from_date' => $variables['from_date'],
                'to_date' => $variables['to_date'],
                'type' => $variables['type'],
                'customer_id' => (int) $variables['customer_id'],
                'salesman' => (int)$variables['salesman'],
                'outlet_id' => $outlet,
                'brand' => (int) $variables['brand'],
                'payment_term' => (int) $variables['payment_term'],
                'view' => 'pdf_report_template_app',
                'title' => 'Brand wise report',
                'mainDb' => $this->db_main,
            );
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_brandwise_report', array('title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionCurrentStockReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "current_stock";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;
            $params =  array(
                'brand'     => $variables['brand'],
                'type'        => $variables['type'],
                'stock'  => (int) $variables['stock'],
                'salesman' => $salesman,
                'outlet'      => $outlet,
                'user_id'     => $userModel->id,
                'access_type' => $userModel->access_type,
                'userModel'   =>  $userModel,
                'logo'        => $userModel->companyDetails->pdf_logo,
            );
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.reports.pdf.currentstock_list', $params, true));

            //   $mpdf->WriteHTML($this->renderPartial('application.views.front.reports.pdf.currentstock_list', array('params' => $params), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
   

    public function actionPrintCurrentStockPos()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "current_stock";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;
             $access_type = $userModel->access_type;
            // -----------------------------------
            $model  = new Inventory();
            $condition      = '1=1';
            if ($outlet > 0) {
                $condition .= ' AND I.outlet=' . $outlet;
            }
            if ($type != '') {
                $condition .= ' AND I.type="' . $type . '"';
            }
            if ($brand > 0) {
                $condition .= ' AND I.brand=' . $brand;
            }
            if ($customer_id > 0) {
                $condition .= ' AND t.customer_id=' . $customer_id;
            }
            if ($salesman > 0) {
                $condition .= ' AND (S.created_by=' . $salesman . ' OR S.type="OB" )';
            }

            $query_current_stock = "SELECT * FROM (SELECT id,part_number,description,active,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,
                    unit_name,sec_unit_id,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,SUM(tax) AS tax,name,outlet_name,rate from view_current_stock group by id
                    ) A where A.qty>0";
            $arr_stock = Yii::app()->db->createCommand($query_current_stock)->queryAll();
            $sl = 1;
            if (!empty($arr_stock)) {
                $total_amount = 0;
                $list = [];
                foreach ($arr_stock as $row) {

                    $inv_type = ($row['type'] == 'I') ? "Inventory" : "Service";
                    $avg_purchase_rate = round(StockLedger::getAvgPurchaseRate($row['id'], '', $outlet), 2);
                    $cost_amount = $avg_purchase_rate * round($row['qty'], 3);
                    $total_amount += ($cost_amount);
                    $sec_unit_id = $row['sec_unit_id'];
                    $mas_unit = MasUnit::model()->findByPk($sec_unit_id);
                    $sec_qty = 0;
                    if ((int)$sec_unit_id > 0) {
                        $sec_qty = (float)$row['qty'] / (float)$mas_unit['unit_exchange_rate'];
                    }
                    
                    $cost = 0;
                    $cost_amount_pass = 0;
                    if ($access_type != 'SL' && $ctype != 'nocost') {
                        $cost = $avg_purchase_rate;
                        $cost_amount_pass = $cost_amount;
                    }
                    
                   

                    // -----------------------------------

                    $item_list[] = [
                        "part_number" => $row['part_number'],
                        "description" => $row['description'],
                        "Type" => $inv_type,
                        "brand_name" => $row['brand_name'],
                        "qty" =>  round($row['qty'], 3),
                        "unit_name" => $row['unit_name'],
                        "Sec_qty" => round($sec_qty, 2),
                        "Sec_unit_name" => $mas_unit['unit_name'],
                        "cost" => number_format($cost,2),
                        "rate" => number_format($row['rate'],2),
                        "tax" => number_format($row['tax'],2),
                        "cost_amount" => number_format($cost_amount_pass,2),
                        "outlet_name" => $row['outlet_name'],
                                              "access_type" => $access_type,
                        "user" => $userModel->full_name,
                        "user_phone" => $userModel->phone_no,
                    ];
                    $sl++;
                }
            }
              $this->responseBody['company_name'] = $userModel->outletDetails->name;
                $this->responseBody['company_adds'] = $userModel->outletDetails->address;
                $this->responseBody['company_phone'] = $userModel->outletDetails->phone_no;
                $this->responseBody['company_po_box'] = $userModel->outletDetails->po_box;
                $this->responseBody['company_trn'] = $userModel->outletDetails->trn_no;
                $this->responseBody['outlet'] = $userModel->outletDetails->name;
                $this->responseBody['user'] = $userModel->full_name;
                $this->responseBody['user_phone'] = $userModel->phone_no;
               // $this->responseBody['create_date_time'] = date('d/m/Y', strtotime($voucherModel->voucher_date)); 
                $this->responseBody['create_date_time'] = date('d/m/Y'); 
            $this->responseBody['item'] = $item_list;
            $this->responseBody['count'] = $sl;
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
     public function actionTransactionDetailReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "TransactionDetail";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;
            
            
            $params =  array(
                'from' => $variables['from_date'],
                'to'  => $variables['to_date'],
                'product_id'     => $variables['product_id'],
                'salesman' => $salesman,
                'outlet'      => $outlet,
                'user_id'     => $userModel->id,
                'access_type' => $userModel->access_type,
                'userModel'   =>  $userModel,
                'logo'        => $userModel->companyDetails->pdf_logo,
            );
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.reports.pdf.transaction_detail', $params, true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
}
