<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiReportsssController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array( 
                    'SalesReport','NetSalesReport','DailySalesReport'
                ),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }

    //Reports function start
    public function actionSalesReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
         if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "sales";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params =  array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Sales Invoices',
                    'mainDb' => $this->db_main,
             );
              $condition = 'V.invoice_date >= "' . $params['from_date'] . '" AND V.invoice_date <= "' . $params['to_date'] . '" ';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }
            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.sales_man=' . $params['salesman_id'];
            }
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }
           
            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_sales_invoice_list AS SELECT
                                                V.id,
                                                V.invoice_date AS date,
                                                V.invoice_no AS voucher_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                                O.name AS outlet_name,
						MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                                FROM {{sales_invoice}} V
                                            LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                            LEFT JOIN {{customer}} C ON C.id = V.customer_id
                                            LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = V.sales_man
                                            LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC')->execute();
            //HTML Data Generation
                        $table = '<div class="table-responsive">
                        <table class="table table-hover table-striped table-bordered ">
                            <thead>
                                <tr>
                                    <th style="width:1%">No</th>
                                    <th style="width:1%">Date</th>
                                    <th style="width:1%">Inv #</th>
                                    <th style="width:10%">Customer</th>
                                    <th style="width:5%">Salesman</th>
                                    <th class="bg-danger" style="width:4%">VAT</th>
                                    <th class="bg-danger" style="width:4%">Grand Total</th>
                                    <th style="width:1%">Currency</th>
                                    <th class="bg-success" style="width:4%">VAT(BC)</th>
                                    <th class="bg-success" style="width:4%">Grand Total(BC)</th>
                                </tr>
                            </thead>
                            <tbody>';

            $salesInvoices = Yii::app()->db->createCommand('SELECT * FROM view_report_sales_invoice_list')->queryAll();
            $total_vat = $grand_total = $total_vat_bc = $grand_total_bc = $allocated_amount =  0;
            $j = 1;

            foreach ($salesInvoices as $each) {
                if ($each['active'] == 'N') {
                    $each['total_vat'] = 0;
                    $each['grand_total'] = 0;
                    $each['total_vat_bc'] = 0;
                    $each['grand_total_bc'] = 0;
                }
                $table .= '<tr>
                                <td>' . $j++ . '</td>
                                <td>' . date('d/m/Y', strtotime($each['date'])) . '</td>
                                <td>' . $each['voucher_no'] . '</td>
                                <td>' . $each['customer_name'] . '</td>
                                <td>' . $each['sales_man'] . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat'], " ") . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total'], " ") . '</td>
                                <td>' . $each['currency'] . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat_bc'], " ") . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total_bc'], " ") . '</td>
                            </tr>';

                $total_vat += $each['total_vat'];
                $grand_total += $each['grand_total'];
                $total_vat_bc += $each['total_vat_bc'];
                $grand_total_bc += $each['grand_total_bc'];
            }

            $table .= '<tr>
                            <td colspan="8" class="boldy text-right"><b>Total</b></td>
                            <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat_bc, " ") . '</b></td>
                            <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_bc, " ") . '</b></td>
                        </tr>';

            $table .= '</tbody>
                        </table>
                    </div>';
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_report_template_app', array('title' => $params['title'], 'table'=>$table,'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
             $this->_401;
        }
    }
    public function actionNetSalesReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
         if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "net_sales";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params =  array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Net Sales',
                    'mainDb' => $this->db_main,
             );
            //HTML Data Generation
            $condition = '1=1 ';
                    $condition .= ' AND t.invoice_date>="' . $params['from_date'] . '"';
                    $condition .= ' AND t.invoice_date<="' . $params['to_date'] . '"';
                    if ($params['outlet_id'] > 0) {
                        $condition .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['customer_id'] > 0) {
                        $condition .= ' AND t.customer_id=' . $params['customer_id'];
                    }
                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND (t.sales_man=' . $params['salesman_id'] . ')';
                    }
                    $net_sales_sql = 'SELECT * FROM view_net_sale_report t 
                    WHERE ' . $condition . ' ORDER BY invoice_date asc,type DESC,invoice_no ASC';
                    $arr_invoices = Yii::app()->db->createCommand($net_sales_sql)->queryAll();
                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Inv #/CN. #</th>
                                            <th style="width:10%">Customer</th>
                                            <th style="width:5%">Salesman</th>
                                            <th>Sales Amt(BC)</th>
                                            <th>Inv.VAT(BC)</th>
                                            <th>Sales Amt with VAT(BC)</th>
                                            <th>CRN Amt(BC)</th>
                                            <th>CRN VAT(BC)</th>
                                            <th>CRN Amt with VAT(BC)</th>
                                            <th>Net Amt(AED)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $total_inv_amount_bc = $total_inv_vat_bc = $total_inv_grand_bc = 0;
                    $total_ret_amount_bc = $total_ret_vat_bc = $total_ret_grand_bc = 0;
                    $total_net_amount_bc = 0;
                    $sl = 1;
                    if (!empty($arr_invoices)) {
                        foreach ($arr_invoices as $each) {

                            if ($each['active'] == 'N') {
                                $each['total_amount'] = 0;
                                $each['tax_total'] = 0;
                                $each['grand_total'] = 0;
                            }

                            if ($each["type"] == 'I') {
                                $invoice_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $invoice_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $invoice_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                                $return_amount_bc = 0;
                                $return_vat_bc = 0;
                                $return_grandtotal_bc = 0;
                            } else {
                                $invoice_amount_bc = 0;
                                $invoice_vat_bc = 0;
                                $invoice_grandtotal_bc = 0;
                                $return_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $return_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $return_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                            }
                            $net_total = $invoice_grandtotal_bc - $return_grandtotal_bc;
                            $total_inv_amount_bc += $invoice_amount_bc;
                            $total_inv_vat_bc += $invoice_vat_bc;
                            $total_inv_grand_bc += $invoice_grandtotal_bc;
                            $total_ret_amount_bc += $return_amount_bc;
                            $total_ret_vat_bc += $return_vat_bc;
                            $total_ret_grand_bc += $return_grandtotal_bc;
                            $total_net_amount_bc += $net_total;
                            $table .= '<tr>
                                        <td>' . date('d/m/Y', strtotime($each['invoice_date'])) . '</td>
                                        <td>' . $each['invoice_no'] . '</td>
                                        <td>' . $each['customer_name'] . '</td>
                                        <td>' . $each['full_name'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_amount_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_vat_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_grandtotal_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_amount_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_vat_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_grandtotal_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($net_total, " ") . '</td>
                                    </tr>';
                            $grand_total_bc += $each['grand_total_bc'];
                            $total_vat_bc += $each['total_vat_bc'];
                            $total_disc_bc += $each['discount_bc'];

                            $total_returns_amount_bc += $each['return_total'];
                            $total_returns_vat_bc += $each['return_vat'];
                            $total_returns_disc_bc += $each['return_disount'];
                            $totalSales += $netSale;
                        }

                        $table .= '<tr>
                                    <td colspan="4" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_net_amount_bc, " ") . '</b></td>
                                </tr>';
                        $table .= '<tr>
                                <th colspan="10" class="text-right bg-info">Net Sales Without VAT</th> 
                                <th class="text-right bg-info">' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc - $total_ret_amount_bc, "") . '</th>
                             </tr>';
                    } else {
                        $table .= '<tr>
                                <th colspan="11" class="text-center bg-info">No Details Found</th>
                             </tr>';
                    }
                    $table .= '</tbody>
                                </table>
                            </div>';
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_report_template_app', array('table' => $table, 'title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            echo "else";
            $this->_401;
        }
    }
    public function actionDailySalesReport()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
         if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = "daily_sales_report";
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $params = array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Daily Sales Report',
                    'mainDb' => $this->db_main,
              );
            //HTML Data Generation
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.pdf_report_template_app', array('table' => $table, 'title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    public function generate_view($view_type, $params)
    {
 
        if ($view_type == 'sales') {

            $condition = 'V.invoice_date >= "' . $params['from_date'] . '" AND V.invoice_date <= "' . $params['to_date'] . '" ';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }
            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.sales_man=' . $params['salesman_id'];
            }
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }
            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_sales_invoice_list AS SELECT
                                                V.id,
                                                V.invoice_date AS date,
                                                V.invoice_no AS voucher_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                                O.name AS outlet_name,
						MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                                FROM {{sales_invoice}} V
                                            LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                            LEFT JOIN {{customer}} C ON C.id = V.customer_id
                                            LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = V.sales_man
                                            LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC')->execute();
        }   
    }
    public function generate_details($report_type, $conditionFormats)
            {

                $table = '';
                $params = $conditionFormats[$report_type];

                if ($report_type == 'sales') {

                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">No</th>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Inv #</th>
                                            <th style="width:10%">Customer</th>
                                            <th style="width:5%">Salesman</th>
                                            <th class="bg-danger" style="width:4%">VAT</th>
                                            <th class="bg-danger" style="width:4%">Grand Total</th>
                                            <th style="width:1%">Currency</th>
                                            <th class="bg-success" style="width:4%">VAT(BC)</th>
                                            <th class="bg-success" style="width:4%">Grand Total(BC)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $salesInvoices = Yii::app()->db->createCommand('SELECT * FROM view_report_sales_invoice_list')->queryAll();
                    $total_vat = $grand_total = $total_vat_bc = $grand_total_bc = $allocated_amount =  0;
                    $j = 1;

                    foreach ($salesInvoices as $each) {
                        if ($each['active'] == 'N') {
                            $each['total_vat'] = 0;
                            $each['grand_total'] = 0;
                            $each['total_vat_bc'] = 0;
                            $each['grand_total_bc'] = 0;
                        }
                        $table .= '<tr>
                                        <td>' . $j++ . '</td>
                                        <td>' . date('d/m/Y', strtotime($each['date'])) . '</td>
                                        <td>' . $each['voucher_no'] . '</td>
                                        <td>' . $each['customer_name'] . '</td>
                                        <td>' . $each['sales_man'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total'], " ") . '</td>
                                        <td>' . $each['currency'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat_bc'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total_bc'], " ") . '</td>
                                    </tr>';

                        $total_vat += $each['total_vat'];
                        $grand_total += $each['grand_total'];
                        $total_vat_bc += $each['total_vat_bc'];
                        $grand_total_bc += $each['grand_total_bc'];
                    }

                    $table .= '<tr>
                                    <td colspan="8" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_bc, " ") . '</b></td>
                                </tr>';

                    $table .= '</tbody>
                                </table>
                            </div>';
                }  
                else if ($report_type == 'net_sales') {
                    $condition = '1=1 ';
                    $condition .= ' AND t.invoice_date>="' . $params['from_date'] . '"';
                    $condition .= ' AND t.invoice_date<="' . $params['to_date'] . '"';
                    if ($params['outlet_id'] > 0) {
                        $condition .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['customer_id'] > 0) {
                        $condition .= ' AND t.customer_id=' . $params['customer_id'];
                    }
                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND (t.sales_man=' . $params['salesman_id'] . ')';
                    }
                    $net_sales_sql = 'SELECT * FROM view_net_sale_report t 
                    WHERE ' . $condition . ' ORDER BY invoice_date asc,type DESC,invoice_no ASC';
                    $arr_invoices = Yii::app()->db->createCommand($net_sales_sql)->queryAll();
                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Inv #/CN. #</th>
                                            <th style="width:10%">Customer</th>
                                            <th style="width:5%">Salesman</th>
                                            <th>Sales Amt(BC)</th>
                                            <th>Inv.VAT(BC)</th>
                                            <th>Sales Amt with VAT(BC)</th>
                                            <th>CRN Amt(BC)</th>
                                            <th>CRN VAT(BC)</th>
                                            <th>CRN Amt with VAT(BC)</th>
                                            <th>Net Amt(AED)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $total_inv_amount_bc = $total_inv_vat_bc = $total_inv_grand_bc = 0;
                    $total_ret_amount_bc = $total_ret_vat_bc = $total_ret_grand_bc = 0;
                    $total_net_amount_bc = 0;
                    $sl = 1;
                    if (!empty($arr_invoices)) {
                        foreach ($arr_invoices as $each) {

                            if ($each['active'] == 'N') {
                                $each['total_amount'] = 0;
                                $each['tax_total'] = 0;
                                $each['grand_total'] = 0;
                            }

                            if ($each["type"] == 'I') {
                                $invoice_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $invoice_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $invoice_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                                $return_amount_bc = 0;
                                $return_vat_bc = 0;
                                $return_grandtotal_bc = 0;
                            } else {
                                $invoice_amount_bc = 0;
                                $invoice_vat_bc = 0;
                                $invoice_grandtotal_bc = 0;
                                $return_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $return_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $return_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                            }
                            $net_total = $invoice_grandtotal_bc - $return_grandtotal_bc;
                            $total_inv_amount_bc += $invoice_amount_bc;
                            $total_inv_vat_bc += $invoice_vat_bc;
                            $total_inv_grand_bc += $invoice_grandtotal_bc;
                            $total_ret_amount_bc += $return_amount_bc;
                            $total_ret_vat_bc += $return_vat_bc;
                            $total_ret_grand_bc += $return_grandtotal_bc;
                            $total_net_amount_bc += $net_total;
                            $table .= '<tr>
                                        <td>' . date('d/m/Y', strtotime($each['invoice_date'])) . '</td>
                                        <td>' . $each['invoice_no'] . '</td>
                                        <td>' . $each['customer_name'] . '</td>
                                        <td>' . $each['full_name'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_amount_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_vat_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_grandtotal_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_amount_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_vat_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_grandtotal_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($net_total, " ") . '</td>
                                    </tr>';
                            $grand_total_bc += $each['grand_total_bc'];
                            $total_vat_bc += $each['total_vat_bc'];
                            $total_disc_bc += $each['discount_bc'];

                            $total_returns_amount_bc += $each['return_total'];
                            $total_returns_vat_bc += $each['return_vat'];
                            $total_returns_disc_bc += $each['return_disount'];
                            $totalSales += $netSale;
                        }

                        $table .= '<tr>
                                    <td colspan="4" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_net_amount_bc, " ") . '</b></td>
                                </tr>';
                        $table .= '<tr>
                                <th colspan="10" class="text-right bg-info">Net Sales Without VAT</th> 
                                <th class="text-right bg-info">' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc - $total_ret_amount_bc, "") . '</th>
                             </tr>';
                    } else {
                        $table .= '<tr>
                                <th colspan="11" class="text-center bg-info">No Details Found</th>
                             </tr>';
                    }
                    $table .= '</tbody>
                                </table>
                            </div>';
                }
                else if ($report_type == 'daily_sales_report') {

                    $from_date = $params['from_date'];
                    $daybook_date = $params['to_date'];
                    $outlet = (int) $params['outlet_id'];
                    $created_user = (int) $params['salesman_id'];
                    $customer_id = (int) $params['customer_id'];
                    $table = '<div class="table-responsive">
        <table class="table table-hover table-striped table-bordered ">
        <thead>
            <tr>
                <th style="width:3%">Sl No.</th>
                <th style="width:7%">Inv No.</th>
                <th style="width:10%">Date</th>
                <th style="width:25%">Customer</th>
                <th style="width:10%">Amount</th>
                <th style="width:10%">Receipt Amount</th>
                <th style="width:10%">Balance</th>
            </tr>
        </thead>
        <tbody>';
                    $condition_sales .= "1=1";
                    $condition_sales_return .= "1=1";

                    if ($outlet > 0) {
                        $condition_sales .= " AND outlet=" . $outlet;
                        $condition_sales_return .= " AND outlet=" . $outlet;
                    }
                    if ($created_user > 0) {
                        $condition_sales .= " AND sales_man=" . $created_user;
                        $condition_sales_return .= " AND created_by=" . $created_user;
                    }
                    if ($customer_id > 0) {
                        $condition_sales .= " AND customer_id=" . $customer_id;
                        $condition_sales_return .= " AND customer_id=" . $customer_id;
                    }
                    if ($daybook_date != '' && $daybook_date != '00/00/0000') {
                        $daybook_date = str_replace('/', '-', $daybook_date);
                        $daybook_date = date("Y-m-d", strtotime($daybook_date));
                        $condition_sales .= ' AND invoice_date="' . $daybook_date . '"';
                        $condition_sales_return .= ' AND return_date="' . $daybook_date . '"';
                    }
                    //Sales
                    $model = SalesInvoice::model()->findAll(array('condition' => $condition_sales));
                    $grand_total = 0;
                    $received_total = 0;
                    foreach ($model as $i => $row) {
                        if ($row['posted'] == 'Y') {
                            $outlet = $row->outlet;
                            $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="SI" AND outlet=' . $outlet))->id;
                            $inv = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $row->id));
                            $debtor = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                            if ($inv->id > 0) {
                                $inv_item = FinanceVoucherItems::model()->find(array('condition' => 'ledger_id=' . $debtor . ' AND header_id=' . $inv->id));
                                $inv_amount = $inv_item->amount;
                            } else {
                                $inv_amount = 0;
                            }
                            if ($inv_item->id > 0) {
                                $received_amount = Yii::app()->db->createCommand('select sum(amount) from erp_finance_allocation_items where voucher_item_id=' . $inv_item->id)->queryScalar();
                            } else {
                                $received_amount = 0;
                            }
                        }
                        $table .= '<tr>
                    <td style="text-align:center !important;">' . ($i + 1) . '</td> 
                    <td style="text-align:left !important;">' . $row->invoice_no . '</td> 
                    <td style="text-align:left !important;">' . date('d/m/Y', strtotime($row->invoice_date)) . '</td> 
                    <td style="text-align:left !important;">' . $row->customer->customer_name . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row->grand_total, "") . '</td>
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($received_amount, "") . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row->grand_total - $received_amount, "") . '</td>
                </tr>';

                        $grand_total += $row->grand_total;
                        $received_total += $received_amount;
                    }
                    $total_sales = $grand_total;
                    $table .= '<tr>
                    <td style="text-align:right !important;" colspan="4"><b>Total</b></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total, "") . '</b></td>
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($received_total, "") . '</b></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total - $received_total, "") . '</b></td>
                </tr>
        </tbody>
    </table>
    <table class="table table-hover table-striped table-bordered ">
        <thead>
            <tr>
                <th style="width:3%">Sl No.</th>
                <th style="width:7%">Cn. No.</th>
                <th style="width:10%">Cn. Date</th>
                <th style="width:25%">Customer</th>
                <th style="width:7%">Inv. No.</th>
                <th style="width:10%">Inv. Date</th>
                <th style="width:10%">Inv. Amt.</th>
                <th style="width:10%">Cn. Amt.</th>
                <th style="width:10%">Balance</th>
            </tr>
        </thead>
        <tbody>';
                    //Sales Return
                    $model = SalesReturn::model()->findAll(array('condition' => $condition_sales_return));
                    $grand_total = 0;
                    $cn_total = 0;
                    foreach ($model as $i => $row) {
                        $table .= '<tr>
                    <td style="text-align:center !important;">' . ($i + 1) . '</td> 
                    <td style="text-align:left !important;">' . $row->return_no . '</td> 
                    <td style="text-align:left !important;">' . date('d/m/Y', strtotime($row->return_date)) . '</td> 
                    <td style="text-align:left !important;">' . $row->sr_customer->customer_name . '</td> 
                    <td style="text-align:left !important;">' . $row->sr_invoice->invoice_no . '</td> 
                    <td style="text-align:left !important;">' . date('d/m/Y', strtotime($row->sr_invoice->invoice_date)) . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row->grand_total, "") . '</td>
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row->sr_invoice->grand_total, "") . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row->grand_total - $row->sr_invoice->grand_total, "") . '</td>
                </tr>';
                        $grand_total += $row->sr_invoice->grand_total;
                        $cn_total += $row->grand_total;
                    }
                    $total_sales_return = $cn_total;
                    $table .= '<tr>
                    <td style="text-align:right !important;" colspan="6"><b>Total</b></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total, "") . '</b></td>
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($cn_total, "") . '</b></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total - $cn_total, "") . '</b></td>
                </tr>
        </tbody>
    </table>
        <table class="table table-hover table-striped table-bordered ">
        <thead>
            <tr>
                <th style="width:3%">Sl No.</th>
                <th style="width:7%">Voucher No.</th>
                <th style="width:10%">Voucher Date</th>
                <th style="width:25%">Customer</th>
                <th style="width:7%">Inv No.</th>
                <th style="width:10%">Inv. Date</th>
                <th style="width:10%">Inv. Amount</th>
                <th style="width:10%">Receipt Amount</th>
                <th style="width:10%">Discount</th>
                <th style="width:10%">Balance</th>
            </tr>
        </thead>
        <tbody>';
                    $CashAccount = FinanceSystemAccountsMapping::getSystemdefault('cash_account');
                    $condition_expense = "1=1 AND H.is_posted='Y' AND H.is_posted='Y' AND amount<0 AND ledger_id=" . $CashAccount;
                    $condition_cash = "1=1 AND H.is_posted='Y' AND H.is_posted='Y' AND amount>0 AND ledger_id=" . $CashAccount;

                    if ($outlet > 0) {
                        $condition_expense .= " AND I.outlet=" . $outlet;
                        $condition_cash .= " AND I.outlet=" . $outlet;
                    }
                    if ($created_user > 0) {
                        $condition_expense .= " AND H.created_by=" . $created_user;
                        $condition_cash .= " AND H.created_by=" . $created_user;
                    }
                    if ($daybook_date != '' && $daybook_date != '00/00/0000') {
                        $daybook_date = str_replace('/', '-', $daybook_date);
                        $daybook_date = date("Y-m-d", strtotime($daybook_date));
                        $condition_expense .= ' AND H.voucher_date="' . $daybook_date . '"';
                        $condition_cash .= ' AND H.voucher_date="' . $daybook_date . '"';
                    }
                    //Expense & Cash
                    $grand_total = 0;
                    $received_total = 0;
                    $discount_tot = 0;
                    $expenses = Yii::app()->db->createCommand("SELECT H.voucher_no,H.id,I.id AS voucher_item_id,H.voucher_date,I.amount FROM {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id where $condition_cash")->queryAll();
                    foreach ($expenses as $i => $row) {
                        if ($customer_id > 0) {
                            $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $customer_id));
                        }
                        $voucher = FinanceVoucherItems::model()->find(array('condition' => 'amount<0 AND header_id=' . $row['id']));
                        if ($voucher->id > 0) {
                            $invoice = SalesInvoice::model()->findByPk($voucher->parent->header_table_id);
                            //                    $received_amount=-1*Yii::app()->db->createCommand('select sum(amount) from erp_finance_allocation_items where voucher_item_id='.$voucher->id)->queryScalar();
                        } else {
                            //                    $received_amount=0;
                        }
                        if ($customer_id > 0) {
                            if ($voucher->subledger_id == $subledger->id) {
                                if ($invoice->invoice_no > 0) {
                                    $inv_date = date('d/m/Y', strtotime($invoice->invoice_date));
                                }
                                $table .= '<tr>
                                                <td style="text-align:center !important;">' . ($i + 1) . '</td> 
                                                <td style="text-align:left !important;">' . $row['voucher_no'] . '</td> 
                                                <td style="text-align:left !important;">' . date('d/m/Y', strtotime($row['voucher_date'])) . '</td> 
                                                <td style="text-align:left !important;">' . $voucher->subledger->description . '</td> 
                                                <td style="text-align:left !important;">' . $invoice->invoice_no . '</td> 
                                                <td style="text-align:left !important;">' . $inv_date . '</td> 
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total, "") . '</td>
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row['amount'], "") . '</td> 
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->discount, "") . '</td> 
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total - $row['amount'] - $invoice->discount, "") . '</td>
                                            </tr>';
                                $grand_total += $invoice->grand_total;
                                $received_total += 1 * $row['amount'];
                                $discount_tot += $invoice->discount;
                            }
                        } else {
                            if ($voucher->subledger->description != '') {
                                if ($invoice->invoice_no > 0) {
                                    $inv_date = date('d/m/Y', strtotime($invoice->invoice_date));
                                }
                                $table .= '<tr>
                                                <td style="text-align:center !important;">' . ($i + 1) . '</td> 
                                                <td style="text-align:left !important;"><' . $row['voucher_no'] . '</td> 
                                                <td style="text-align:left !important;">' . date('d/m/Y', strtotime($row['voucher_date'])) . '</td> 
                                                <td style="text-align:left !important;">' . $voucher->subledger->description . '</td> 
                                                <td style="text-align:left !important;">' . $invoice->invoice_no . '</td> 
                                                <td style="text-align:left !important;">' . $inv_date . '</td> 
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total, "") . '</td>
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row['amount'], "") . '</td> 
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->discount, "") . '</td> 
                                                <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total - $row['amount'] - $invoice->discount, "") . '</td>
                                            </tr>';

                                $grand_total += $invoice->grand_total;
                                $received_total += 1 * $row['amount'];
                                $discount_tot += $invoice->discount;
                            }
                        }
                    }
                    $cash_collected = $received_total;
                    $table .= '<tr>
                                                <td style="text-align:right !important;" colspan="6"><b>Total</b></td> 
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total, "") . '</b></td>
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($received_total, "") . '</b></td> 
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($discount_tot, "") . '</b></td> 
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total - $received_total - $discount_tot, "") . '</b></td>
                                            </tr>
                                    </tbody>
                                </table>
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:25%">Total Sales</th>
                                            <th style="width:25%">Total Creditnote</th>
                                            <th style="width:25%">Total Cash Collected</th>
                                            <th style="width:25%">Total Discount Allowed</th>
                                        </tr>
                                    </thead>
                                    <tbody>
                                            <tr>
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_sales, "") . '</b></td>
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_sales_return, "") . '</b></td> 
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($cash_collected, "") . '</b></td>
                                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($discount_tot, "") . '</b></td>
                                            </tr>
                                    </tbody>
                                </table>
                                </div>';
                }
                return $table;
            }

}
