<?php


/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiSalesReturnController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array(
                    "GetList",
                    "Savesalesreturn",
                    "Printsalesreturn",
                    "GetData",
                    "GetInvoices",
                    "ConsolidatedSalesreturnList",
                    "SaveConsolidated",

                ),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }


    public function actionGetList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $from = trim($variables['from']);
        $to = trim($variables['to']);
        $term = trim($variables['term']);
        // $from = $this->convert_date($from);
        // $to = $this->convert_date($to);

        $from      = (count(explode('/', $from)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $from))) : date('Y-m-01');
        $to        = (count(explode('/', $to)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $to))) : date('Y-m-d');
        if ($userModel->id > 0) {

            $userModel->db();

            $count = 0;

            if (isset($term) && $term != "") {
                $searchString = $term;
                $condition = " AND (t.return_no LIKE '%" . $searchString . "%' OR t.grand_total LIKE '%" . $searchString . "%' )";
            } else {
                $condition = ""; // Ensure $condition is initialized even if $term is not set
            }

            $model = SalesReturn::model()->findAll(array(
                'condition' => 'return_date BETWEEN "' . $from . '" AND "' . $to . '" AND created_by=' . $userModel->id . $condition,
                'order' => 'return_date DESC, return_no DESC'
            ));

            foreach ($model as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['return_no'] = $each->return_no;
                $this->responseBody['data'][$i]['prefix'] = $each->prefix;
                $this->responseBody['data'][$i]['invoice_id'] = $each->invoice_id;
                $this->responseBody['data'][$i]['invoice_no'] = $each->sr_invoice->invoice_no;
                $this->responseBody['data'][$i]['date'] = $each->return_date;
                $this->responseBody['data'][$i]['customer'] = $each->sr_customer->customer_name;
                $this->responseBody['data'][$i]['currency_code'] = $each->sr_cur->currency_code;
                $this->responseBody['data'][$i]['exchange_rate'] = $each->exchange_rate;
                $this->responseBody['data'][$i]['total_amount'] = $this->format_currency($each->total_amount);
                $this->responseBody['data'][$i]['excise_total'] = $this->format_currency($each->excise_total);
                $this->responseBody['data'][$i]['vat_amount'] = $this->format_currency($each->tax_total);
                $this->responseBody['data'][$i]['grand_total'] = $this->format_currency($each->grand_total);
                $this->responseBody['data'][$i]['is_posted'] = $each->posted == 'Y' ? true : false;
                $model_sales_return = StockLedger::model()->with('salesreturn_det')->findAll(array('condition' => 'salesreturn_det.invoice_id=' . $each->sr_invoice->invoice_no));
                $this->responseBody['data'][$i]['have_any_active_returns'] = (empty($model_sales_return)) ? false : true;
                /* $this->responseBody['data'][$i]['is_cancelable'] = $each->is_cancelable;
                      $this->responseBody['data'][$i]['is_audited'] = $each->is_audited == 'Y' ? true : false;
                      $this->responseBody['data'][$i]['is_allocated'] = $each->is_allocated;
                      $this->responseBody['data'][$i]['is_returnable'] = $each->returnable; */
                $this->responseBody['data'][$i]['is_cancelled'] = $each->active == 'Y' ? false : true;
                $this->responseBody['data'][$i]['paid'] = $each->paid;
                $this->responseBody['data'][$i]['discount'] = $each->discount;
                $this->responseBody['data'][$i]['old_discount'] = $each->old_discount;
                $this->responseBody['data'][$i]['status'] = $each->active == 'Y' ? 'Active' : 'Cancel';
                $this->responseBody['data'][$i]['notes'] = $each->notes;
                $count++;
            }

            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionSavesalesreturn()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $creditnote_date = trim($variables['creditnotedate']);
        $id = (int) trim($variables['id']);
        $invoice_id = (int) trim($variables['parent_invoice_id']);
        $items_response = $variables['items'];

        $creditnote_date      = (count(explode('/', $creditnote_date)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $creditnote_date))) : date('Y-m-01');

        if ($userModel->id > 0) {
            $userModel->db();

            $invoiceModel = SalesInvoice::model()->findBypk($invoice_id);
            if ($invoiceModel->id > 0) { //Sales
                $model = new SalesReturn;
                $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="SR"'));
                if ($id > 0) {
                    $model = $model->findBypk($id);
                    $scene = 'update';
                    $change_type = 'UPDATED';
                } else {
                    $model->created_by = $userModel->id;
                    $model->created_time = date('Y-m-d H:i:s');
                    $scene = 'new';
                    $change_type = 'CREATED';

                    $model->return_no = $modelformmaster->next_number;
                }
                $model->updated_by = $userModel->id;
                $model->updated_time = date('Y-m-d H:i:s');
                $model->outlet = $userModel->outlet_id;
                $model->customer_id = $invoiceModel->customer_id;
                $model->return_date = $creditnote_date;
                $model->invoice_id = $invoiceModel->id;
                $model->invoice_no = $invoiceModel->invoice_no;
                $model->reference_no = $invoiceModel->reference_no;
                $model->posted = "N";
                $model->callback_from = 'app';
                $model->excise_total = 0;
                $model->tax_total = 0;
                $model->grand_total = 0;
                $model->currency = $invoiceModel->currency;
                $model->exchange_rate = $invoiceModel->exchange_rate;
                $model->total_amount = 0;
                $valid = $model->validate();
                $groups = $other = array();
                $sales_man = $invoiceModel->sales_man;
                if ((int) $sales_man == 0)
                    $sales_man = $userModel->id;

                $i = 0;
                if (is_array($items_response)) {
                    foreach ($items_response as $each) {
                        if ($each['item_parent_id'] > 0) {
                            $invoiceitemModel = SalesInvoiceItems::model()->findBypk((int) $each['item_parent_id']);
                            $modelItems[$i] = new SalesReturnItems();
                            $modelItems[$i]->return_id = 0;
                            $modelItems[$i]->invoice_id = $invoiceitemModel->invoice_id;
                            $modelItems[$i]->invoice_item_id = $each['item_parent_id'];
                            $modelItems[$i]->product_id = $invoiceitemModel->product_id;
                            $modelItems[$i]->description = $invoiceitemModel->description;
                            $modelItems[$i]->invoiced_qty = $invoiceitemModel->quantity;
                            $modelItems[$i]->returned_quantity = $each['quantity'];
                            $modelItems[$i]->unit = $each['unit'];
                            $modelItems[$i]->rate = $invoiceitemModel->rate;

                            $amount = round(((float) $each['quantity'] * (float) $invoiceitemModel->rate), 6);
                            $tax_amount = ($invoiceitemModel->tax_percentage > 0) ? round((($amount * $invoiceitemModel->tax_percentage) / 100), 6) : 0;
                            $modelItems[$i]->amount = (float)$amount;
                            $modelItems[$i]->tax_percentage = $invoiceitemModel->tax_percentage;
                            $modelItems[$i]->tax_amount = $tax_amount;
                            $modelItems[$i]->net_amount = $tax_amount + $amount;
                            $valid = $valid && $modelItems[$i]->validate();
                            // Product Grouping
                            if ($each['unit'] > 0) {
                                $unit = MasUnit::model()->findByPk($each['unit']);
                                $unit_exchange = $unit->unit_exchange_rate;
                            } else {
                                $unit_exchange = 0;
                            }
                            $key = $invoiceitemModel->product_id;
                            if (!array_key_exists($key, $groups)) {
                                $groups[$key] = array(
                                    'id' => $invoiceitemModel->product_id,
                                    'returned_quantity' => $each['quantity'] * $unit_exchange,
                                );
                            } else {
                                $groups[$key]['returned_quantity'] = $groups[$key]['returned_quantity'] + $each['quantity'] * $unit_exchange;
                            }
                            $i++;
                        }
                    }
                }
                // QTY Checking
                if (!empty($groups)) {
                    foreach ($groups as $key => $eachProduct) {
                        $product = Inventory::model()->findByPk($key);
                        $qty = 0;
                        if ($model->id > 0) {
                            $qty = Yii::app()->db->createCommand("select IFNULL(SUM(returned_quantity*U.unit_exchange_rate),0) from {{sales_return_items}} S INNER JOIN {{master_unit}} U ON S.unit=U.id where return_id='" . $model->id . "' AND product_id='" . $key . "'")->queryScalar();
                        }
                        $bal = $eachProduct['returned_quantity'] - $qty;
                        $invoiced_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0) from {{stock_ledger}} where type='SI' AND product_id='" . $key . "' AND header_id='" . $model->invoice_id . "'  AND outlet=" . $userModel->outlet_id . " AND created_by=" . $sales_man)->queryScalar();
                        $already_returned_qty = Yii::app()->db->createCommand("select IFNULL(SUM(S.quantity*S.unit_exchange_rate),0) from {{stock_ledger}} S LEFT JOIN {{sales_return_items}} R ON R.id=S.item_id where R.invoice_id='" . $model->invoice_id . "'  AND type='SR' AND S.product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $sales_man)->queryScalar();
                        $balance = $invoiced_qty - $already_returned_qty + $qty;
                        if ($eachProduct['returned_quantity'] > $balance) {
                            $other[] = $product->part_number . ' maximum quantities to be returned : ' . ($balance) . ' ' . $product->units->unit_name;
                            $valid = false;
                        } else if ($bal < 0 && $product->type != 'S') {
                            $in_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='IN' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $sales_man)->queryScalar();
                            $out_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='OUT' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $sales_man)->queryScalar();
                            $current_stock = $in_qty - $out_qty;
                            if ($current_stock < - ($bal)) {
                                $other[] = $product->part_number . ' have not enough quantities : ' . - ($current_stock + $bal) . ' ' . $product->units->unit_name;
                                $valid = false;
                            }
                        }
                    }
                }
                $closing_date = MasClosing::getClosingDate();


                if ($closing_date >= $model->return_date) {
                    $other[] = 'Date should be greater than closing date (' . date('d-m-Y', strtotime($closing_date)) . ')';
                    $valid = false;
                }
                if ($valid && $model->validate() && is_array($modelItems)) {
                    $model->save();
                    if ($scene == 'new') {
                        $next_no = $model->return_no + 1;
                        FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                    }
                    Yii::app()->db->createCommand("delete from {{stock_ledger}} where transaction_type='IN' AND type='SR' AND header_id='" . $model->id . "' AND outlet=" . $userModel->outlet_id)->execute();
                    Yii::app()->db->createCommand("delete from {{sales_return_items}} where return_id='" . $model->id . "'")->execute();

                    foreach ($modelItems as $eachItem) {
                        $eachItem->return_id = $model->id;
                        $total_amount += $eachItem->amount;
                        $total_tax_amount += $eachItem->tax_amount;
                        $total_excise_amount += $eachItem->excise_amount;
                        $eachItem->save(false);
                        $grand_total += $eachItem->net_amount;

                        $stockledger = new StockLedger;
                        $stockledger->transaction_date = $model->return_date;
                        $stockledger->transaction_type = 'IN';
                        $stockledger->type = 'SR';
                        $stockledger->customer_vendor_id = $model->customer_id;
                        $stockledger->header_id = $model->id;
                        $stockledger->item_id = $eachItem->id;
                        $stockledger->product_id = $eachItem->product_id;
                        $stockledger->rate = $eachItem->rate;
                        $stockledger->currency = 1;
                        $stockledger->exchange_rate = 1;
                        $stockledger->quantity = $eachItem->returned_quantity;
                        $stockledger->unit = $eachItem->unit;
                        $stockledger->default_product_unit = $eachItem->sr_units->base_unit_id;
                        $stockledger->unit_exchange_rate = $eachItem->sr_units->unit_exchange_rate;
                        $stockledger->outlet = $userModel->outlet_id;
                        $stockledger->created_by = $sales_man;
                        $stockledger->created_time = date('Y-m-d H:i:s');
                        $stockledger->save(false);
                    }
                    // inserting to audit log
                    AuditLog::saveApiAuditLogs(
                        $userModel->outlet_id,
                        $modelformmaster->id,
                        $sales_man,
                        $change_type,
                        " with ID - {$model->id} "
                    );
                    // end audit log
                    $model->total_amount = $total_amount;
                    $model->excise_total = $total_excise_amount;
                    $model->tax_total = $total_tax_amount;
                    $model->grand_total = $grand_total;
                    $model->save();
                    $this->salesreturnPosting($model->id);
                    $this->responseBody['voucher_id'] = $model->id;
                    $this->_bind;
                } else {


                    $this->responseBody['status'] = 'failed';
                    $error1 = json_decode(CActiveForm::validate($model, '', false), true);



                    if (!empty($modelItems)) {
                        $error2 = json_decode(CActiveForm::validateTabular($modelItems, '', false), true);
                    } else {
                        $error2[] = "Please add atleast one line item";
                    }



                    $error = array_merge_recursive($error1, $error2);
                    $arr_err = $other;
                    foreach ($error as $key => $errval) {
                        $arr_err[] = $errval[0];
                    }
                    $err_msg = implode(',', $arr_err);



                    $this->responseBody['msg'] = $err_msg;

                    $this->_400;
                }
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invoice is not valid';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }




    public function actionGetInvoices()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        if ($userModel->id > 0) {
            $userModel->db();
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;


            $from_date = $variables['from_date'];
            $to_date = $variables['to_date'];
            $customer_id = $variables['customer_id'];
            $created_user = $salesman;
            $condition = 'active="Y" ';
            if ($from_date != '' && $from_date != '00/00/0000') {
                $from = str_replace('/', '-', $from_date);
                $from = date("Y-m-d", strtotime($from));
                $condition .= ' AND t.invoice_date>="' . $from . '"';
            }


            if ($to_date != '' && $to_date != '00/00/0000') {
                $to = str_replace('/', '-', $to_date);
                $to = date("Y-m-d", strtotime($to));
                $condition .= ' AND t.invoice_date<="' . $to . '"';
            }
            if ($customer_id > 0) {
                $condition .= ' AND t.customer_id=' . $customer_id;
            }
            if ((int)$created_user > 0) {
                $condition .= ' AND t.sales_man=' . $created_user;
            }
            $condition .= " AND outlet=" . $outlet;
            $modelInvoices = SalesInvoice::model()->findAll(array('condition' => $condition, 'order' => 'id Desc'));



            $item_list = [];
            if (!empty($modelInvoices)) {
                foreach ($modelInvoices as $i => $each) {

                    $item_list[] = [
                        "id" => $each->id,
                        "invoice_no" =>  $each->invoice_no,
                        "invoice_date" => date("d/m/Y", strtotime($each->invoice_date)),
                        "salesman" => $each->si_salesman->full_name,
                        "grand_total" => $each->grand_total,
                        "selected" => false,

                    ];
                    $i++;
                }
            }
            $this->responseBody['data'] = $item_list;
            $this->responseBody['count'] = $i;
            $this->_bind;
        } else {
            $this->_401;
        }
    }




    public function actionGetData()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $voucher_id = (int) trim($variables['voucher_id']);
        if ($userModel->id > 0) {

            $userModel->db();


            if ($voucher_id > 0) {

                $id = $voucher_id;
                //Head
                $voucherModel = SalesReturn::model()->findbypk($id);
                $prefix = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="SR"'))->prefix;
                if ($prefix != '') {
                    $prefix = $prefix . '/';
                } else {
                    $prefix = '';
                }
                $this->responseBody['data']['voucher_type'] = 'SalesReturn';
                $this->responseBody['data']['voucher_no'] = $prefix . $voucherModel->return_no;
                $this->responseBody['data']['parent_voucher_no'] = $voucherModel->prefix;
                $this->responseBody['data']['voucher_date'] = date('d/m/Y', strtotime($voucherModel->return_date));
                $this->responseBody['data']['currency'] = $voucherModel->sr_cur->currency_code;
                $this->responseBody['data']['company_name'] = $voucherModel->sr_outlet->companyDetails->name;
                $this->responseBody['data']['company_adds'] = str_replace('<br>', ' ', $voucherModel->sr_outlet->companyDetails->billing_address);
                $this->responseBody['data']['company_phone'] = $voucherModel->sr_outlet->phone_no;
                $this->responseBody['data']['company_po_box'] = $voucherModel->sr_outlet->po_box;
                $this->responseBody['data']['company_trn'] = $voucherModel->sr_outlet->trn_no;
                $this->responseBody['data']['outlet'] = $voucherModel->sr_outlet->name;
                $this->responseBody['data']['user'] = $voucherModel->sr_user->full_name;
                $this->responseBody['data']['user_phone'] = $voucherModel->sr_user->phone_no;
                $this->responseBody['data']['create_date_time'] = date('d/m/Y', strtotime($voucherModel->return_date));
                $this->responseBody['data']['notes'] = $voucherModel->notes;
                $this->responseBody['data']['party'] = $voucherModel->sr_customer->customer_name;
                $this->responseBody['data']['party_trn'] = $voucherModel->sr_customer->trn_no;
                $this->responseBody['data']['reference_no'] = $voucherModel->reference_no;
                $totalCount = SalesReturnItems::model()->count(array('condition' => 'return_id="' . $id . '"', 'order' => 'id ASC'));
                $this->responseBody['data']['total_items'] = $totalCount;
                $this->responseBody['data']['total_vat'] = $this->format_currency($voucherModel->tax_total);
                $this->responseBody['data']['total'] = $this->format_currency($voucherModel->total_amount);
                $this->responseBody['data']['grand_total'] = $this->format_currency($voucherModel->grand_total);
                //Items
                $modelItems = SalesReturnItems::model()->findAll(array('condition' => 'return_id="' . $id . '"', 'order' => 'id ASC'));
                foreach ($modelItems as $each) {
                    $this->responseBody['data']['items'][] = array(
                        'part_number' => $each->sr_product->part_number,
                        'description' => $each->sr_product->description,
                        'quantity' => $this->format_currency($each->returned_quantity),
                        'uom' => $each->sr_units->label,
                        'rate' => $this->format_currency($each->rate),
                        'discount' => '0.00',
                        'vat' => $this->format_currency($each->tax_amount),
                        'net' => $this->format_currency($each->net_amount),
                    );
                }
                //Items
                $this->responseBody['count'] = 1;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Voucher';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }



    public function actionPrintsalesreturn()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $userModel->db();

            $input = file_get_contents('php://input');
            //error_log($input);
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');


            $model = SalesReturn::model()->findByPk($id);
            if ($model->active == 'N') {
                $mpdf->SetWatermarkText('CANCELLED');
                $mpdf->showWatermarkText = true;
            }
            $layout = 'application.views.front.api.pdf.sales_return_print';
            $print = 'SR_' . $model->return_no;


            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial($layout, array('id' => $id, 'user_id' => $userModel->id), true));
            $mpdf->Output("downloads/vouchers/$print.pdf", 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . 'downloads/vouchers/' . $print . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }




    function getSublegder($parnet_id, $subledger_type)
    {
        return FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $parnet_id . ' AND subledger_type_id=' . (int) $subledger_type))->id;
    }


    public function actionConsolidatedSalesreturnList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $from = trim($variables['from']);
        $to = trim($variables['to']);
        $term = trim($variables['term']);
        // $from = $this->convert_date($from);
        // $to = $this->convert_date($to);

        $from      = (count(explode('/', $from)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $from))) : date('Y-m-01');
        $to        = (count(explode('/', $to)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $to))) : date('Y-m-d');
        if ($userModel->id > 0) {

            $userModel->db();

            $count = 0;
            // ------------------------------------------------
            $invoice_ids = $variables['invoice_ids'];
            $customer_id = $variables['customer_id'];
            $sales_man     = $userModel->id;
            $model = new SalesReturn;
            $modelItems = array(new SalesReturnItems);
            $k = 0;
            if ($invoice_ids != '') {
                $array_invoice_ids = [];

                foreach ($invoice_ids as $ids) {
                    if ($ids['selected'] == true) {
                        $array_invoice_ids[] = $ids['id'];
                    }
                }

                $implode_invoice_ids = (!empty($array_invoice_ids)) ? implode(',', $array_invoice_ids) : "";

                $invmodelItems = SalesInvoiceItems::model()->findAll(array('condition' => 'invoice_id IN (' . $implode_invoice_ids . ')', 'order' => 't.invoice_id'));

                $arr_ids = array();
                $already_returned_qty = 0;

                foreach ($invmodelItems as $each) {
                    $sql = "select IFNULL(SUM(S.quantity*S.unit_exchange_rate),0) from {{stock_ledger}} S LEFT JOIN {{sales_return_items}} R ON R.id=S.item_id where R.invoice_id='" . $each->invoice_id . "'  AND type='SR' AND S.product_id='" . $each->product_id . "' AND outlet=" . $userModel->outlet_id;
                    $already_returned_qty = Yii::app()->db->createCommand($sql)->queryScalar();
                    $qty_returnable = $each->quantity - $already_returned_qty;
                    if ((int)$qty_returnable > 0) {

                        // ----------------------------------
                        $unit_id = $each->product->unit;
                        $arr_units = MasUnit::getSimilarUnits($unit_id, $userModel->outlet_id);
                        $unit_list = [];
                        foreach ($arr_units as $keyunit => $units) {
                            $each_unit = MasUnit::model()->findByPk($keyunit);

                            $unit_list[] = [
                                "id" => $keyunit,
                                "unit_factor" =>  $each_unit->unit_exchange_rate,
                                "unit_name" =>  $units
                            ];
                        }
                        // ----------------------------------
                        
                        $each_SalesInvoice = SalesInvoice::model()->findByPk($each->invoice_id);
                        
                        $this->responseBody['data'][$k]['product_id'] = $each->product_id;
                        $this->responseBody['data'][$k]['description'] = $each->description;
                        $this->responseBody['data'][$k]['invoice_id'] = $each->invoice_id;
                        $this->responseBody['data'][$k]['invoice_item_id'] =  $each->id;
                        $this->responseBody['data'][$k]['invoiced_qty'] =  $each->quantity;
                        $this->responseBody['data'][$k]['returned_quantity'] =  0;
                        $this->responseBody['data'][$k]['selected_uom'] = $each->unit;
                        $this->responseBody['data'][$k]['selected_uom_name'] = $each->units->unit_name;
                        $this->responseBody['data'][$k]['selected_rate'] = $each->rate;
                        $this->responseBody['data'][$k]['rate'] = $each->rate;
                        $this->responseBody['data'][$k]['amount'] = 0.00;
                        $this->responseBody['data'][$k]['excise_percentage'] = $each->excise_percentage;
                        $this->responseBody['data'][$k]['excise_amount'] = 0.00;
                        $this->responseBody['data'][$k]['tax_percentage'] = $each->tax_percentage;
                        $this->responseBody['data'][$k]['tax_amount'] = 0.00;
                        $this->responseBody['data'][$k]['net_amount'] = 0.00;
                        $this->responseBody['data'][$k]['array_units'] = $unit_list;
                        $this->responseBody['data'][$k]['exclude_tax'] = $each_SalesInvoice->exclude_tax;
                        $this->responseBody['data'][$k]['taxable'] = $each_SalesInvoice->taxable;
                        $this->responseBody['data'][$k]['invoice_type'] = $each_SalesInvoice->invoice_type;
                        if (!in_array($each->invoice->invoice_no, $arr_ids)) {
                            $arr_ids[]      = $each->invoice->invoice_no;
                        }

                        $k++;
                    }
                }
            }
            $this->responseBody['count'] = $k;


            $this->responseBody['invoice_id'] = $implode_invoice_ids;;
            $this->responseBody['customer_id'] = $customer_id;;
            $this->responseBody['invoice_no'] = (!empty($arr_ids)) ? implode(',', $arr_ids) : "";
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionSaveConsolidated()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $creditnote_date = trim($variables['creditnotedate']);
        $id = (int) trim($variables['id']);
        $items_response = $variables['items'];



        $creditnote_date      = (count(explode('/', $creditnote_date)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $creditnote_date))) : date('Y-m-01');

        if ($userModel->id > 0) {
            $userModel->db();

            $valid = true;
            $outlet_id  = $userModel->outlet_id;
            $user_id  = $userModel->id;

            // ------------------------------

            $model = new SalesReturn();
            $total_amount = 0;
            $total_tax_amount = 0;
            $total_excise_amount = 0;
            $grand_total = 0;
            $other = array();
            $model->created_by = $user_id;
            $model->created_time = date('Y-m-d H:i:s');
            $scene = 'new';
            $change_type = 'CREATED';
            $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $outlet_id . ' AND code="SR"'));
            $model->updated_by = $user_id;
            $model->updated_time = date('Y-m-d H:i:s');
            // -------------------------------------
            $model->customer_id = $variables['customer_id'];
            $model->currency = 1;
            $model->exchange_rate = 1;
            // -------------------------------------
            $model->outlet = $outlet_id;
            !empty($modelformmaster) ? $next_number = $modelformmaster->next_number : $next_number = 1;
            $model->return_no = $next_number;
            $model->total_amount = $total_amount;
            $model->excise_total = $total_excise_amount;
            $model->tax_total = $total_tax_amount;
            $model->grand_total = $grand_total;
            if ($creditnote_date != '00/00/0000') {
                $model->return_date = str_replace('/', '-', $creditnote_date);
                $model->return_date = date("Y-m-d", strtotime($model->return_date));
            }
            $model->is_multiple = 'Y';

            $groups = array();
            if ((int)$variables['salesman_id'] > 0) {
                $sales_man = (int)$variables['salesman_id'];
            } else {
                $sales_man = $user_id;
            }
            $array_invoice_id = [];
            if (is_array($items_response)) {
                foreach ($items_response as $i => $each) {
                    if ($each['product_id'] > 0) {
                        $modelItems[$i] = new SalesReturnItems();
                        // -----------------------------
                        $modelItems[$i]->product_id = $each['product_id'];
                        $modelItems[$i]->description = $each['description'];
                        $modelItems[$i]->invoice_id = $each['invoice_id'];
                        $modelItems[$i]->invoice_item_id = $each['invoice_item_id'];
                        $modelItems[$i]->invoiced_qty = $each['invoiced_qty'];
                        $modelItems[$i]->returned_quantity = $each['returned_quantity'];
                        $modelItems[$i]->unit = $each['unit'];
                        $modelItems[$i]->rate = $each['rate'];
                        $modelItems[$i]->amount = $each['amount'];
                        $modelItems[$i]->tax_percentage = $each['tax_percentage'];
                        $modelItems[$i]->tax_amount = $each['tax_amount'];
                        $modelItems[$i]->net_amount = $each['net_amount'];
                        $array_invoice_id[] = $each['invoice_id'];
                        // -----------------------------
                        $modelItems[$i]->return_id = 0;
                        $valid = $valid && $modelItems[$i]->validate();
                        // Product Grouping
                        if ($each['unit'] > 0) {
                            $unit = MasUnit::model()->findByPk($each['unit']);
                            $unit_exchange = $unit->unit_exchange_rate;
                        } else {
                            $unit_exchange = 0;
                        }
                        $key = $each['product_id'] . '_' . $each['invoice_id'];
                        if (!array_key_exists($key, $groups)) {
                            $groups[$key] = array(
                                'id' => $each['product_id'],
                                'returned_quantity' => $each['returned_quantity'] * $unit_exchange
                            );
                        } else {
                            $groups[$key]['returned_quantity'] = $groups[$key]['returned_quantity'] + $each['returned_quantity'] * $unit_exchange;
                        }
                    }
                }
            }

            $user_access_type = $userModel->access_type;

            // QTY Checking
            if (!empty($groups)) {
                foreach ($groups as $key => $eachProduct) {
                    $arr_key = explode('_', $key);
                    $pdt_id = $arr_key[0];
                    $inv_id = $arr_key[1];
                    $product = Inventory::model()->findByPk($pdt_id);
                    $bal = $eachProduct['returned_quantity'];
                    if ($user_access_type == 'CA') {
                        $invoiced_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0) from {{stock_ledger}} where type='SI' AND product_id='" . $pdt_id . "' AND header_id ='" . $inv_id . "'  AND outlet=" . $outlet_id)->queryScalar();
                    } else {
                        $invoiced_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0) from {{stock_ledger}} where type='SI' AND product_id='" . $pdt_id . "' AND header_id ='" . $inv_id . "'  AND outlet=" . $outlet_id . " AND created_by=" . $sales_man)->queryScalar();
                    }
                    $already_returned_qty = Yii::app()->db->createCommand("select IFNULL(SUM(S.quantity*S.unit_exchange_rate),0) from {{stock_ledger}} S LEFT JOIN {{sales_return_items}} R ON R.id=S.item_id where R.invoice_id='" . $inv_id . "'  AND type='SR' AND S.product_id='" . $pdt_id . "' AND outlet=" . $outlet_id)->queryScalar();
                    $balance = $invoiced_qty - $already_returned_qty;

                    if ($eachProduct['returned_quantity'] > $balance) {
                        $other[] = $product->part_number . ' maximum quantities to be returned : ' . ($balance) . ' ' . $product->units->unit_name;
                        $valid = false;
                    } else if ($bal < 0 && $product->type != 'S') {
                        $other[] = $product->part_number . ' returned quantity should be greater than zero ' . $product->units->unit_name;
                        $valid = false;
                    }
                }
            }
            
           

            $model->invoice_id = (!empty($array_invoice_id)) ? implode(',', $array_invoice_id) : "";
            

            $valid = $model->validate() && $valid;
            if ($valid && is_array($modelItems)) {
                $model->save();
                $next_no = $model->return_no + 1;
                FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                foreach ($modelItems as $eachItem) {
                    $eachItem->return_id = $model->id;
                    $total_amount += $eachItem->amount;
                    $total_tax_amount += $eachItem->tax_amount;
                    $total_excise_amount += $eachItem->excise_amount;
                    $eachItem->save(false);
                    $grand_total += $eachItem->net_amount;
                    $stockledger = new StockLedger;
                    $stockledger->transaction_date = $model->return_date;
                    $stockledger->transaction_type = 'IN';
                    $stockledger->type = 'SR';
                    $stockledger->customer_vendor_id = $model->customer_id;
                    $stockledger->header_id = $model->id;
                    $stockledger->item_id = $eachItem->id;
                    $stockledger->product_id = $eachItem->product_id;
                    $stockledger->rate = $eachItem->rate;
                    $stockledger->currency = 1;
                    $stockledger->exchange_rate = 1;
                    $stockledger->quantity = $eachItem->returned_quantity;
                    $stockledger->unit = $eachItem->unit;
                    $stockledger->default_product_unit = $eachItem->sr_units->base_unit_id;
                    $stockledger->unit_exchange_rate = $eachItem->sr_units->unit_exchange_rate;
                    $stockledger->outlet = $outlet_id;
                    $stockledger->created_by = $sales_man;
                    $stockledger->created_time = date('Y-m-d H:i:s');
                    $stockledger->save(false);
                }

                $model->total_amount = $total_amount;
                $model->excise_total = $total_excise_amount;
                $model->tax_total = $total_tax_amount;
                $model->grand_total = $grand_total;
                $model->save();
                $this->salesreturnPosting($model->id);
                $this->responseBody['id'] = $model->id;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                if (!empty($modelItems)) {
                    $error2 = json_decode(CActiveForm::validateTabular($modelItems, '', false), true);
                } else {
                    $error2[] = "Please add atleast one line item";
                }
                $error = array_merge_recursive($error1, $error2);
                $arr_err = $other;
                foreach ($error as $key => $errval) {
                    $arr_err[] = $errval[0];
                }
                $err_msg = implode(',', $arr_err);
                $this->responseBody['msg'] = $err_msg;
               $this->_bind;
            }
        } else {
            $this->_401;
        }
    }




    function salesreturnPosting($invoice_id)
    {
        $mainTable = SalesReturn::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=10 AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->return_no;
        $voucherHead['voucher_date'] = $mainTable->return_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="CN" AND outlet=' . $outlet))->id;
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->return_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->created_by;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->return_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_debtor_amount = $mainTable->grand_total;
        $total_sales = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->paid;
        if ($total_debtor_amount > 0) {
            //Debtor Dr
            $debit_item_sales = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1), //2-Creditors 1- debtors
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => -1 * $total_debtor_amount,
                'outlet' => $mainTable->outlet
            ));

            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="PV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_receipt_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_receipt = $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_receipt_voucher;
                $allocation_items->voucher_item_id = $debit_item_receipt;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="DA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => (float) $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => -1 * $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_sales > 0) {
            //Sales Accounts Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_sales,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }

    public function addVoucherhead($arr_items)
    {
        $model = new FinanceVoucherHead;
        $model->voucher_type = $arr_items['voucher_type'];
        $model->voucher_no = $arr_items['voucher_no'];
        $model->voucher_date = $arr_items['voucher_date'];
        $model->related_subledger = $arr_items['related_subledger'];
        $model->emp_subledger = $arr_items['emp_subledger'];
        $model->due_date = (isset($arr_items['due_date'])) ? $arr_items['due_date'] : $model->voucher_date;
        $model->header_table_id = $arr_items['header_table_id'];
        $model->reference_no = $arr_items['reference_no'];
        $model->is_posted = (isset($arr_items['is_posted'])) ? $arr_items['is_posted'] : 'Y';
        $model->created_by = $arr_items['created_by'];
        $model->created_datetime = date('Y-m-d h:i:s');
        $model->cheque_no = $arr_items['cheque_no'];
        $model->outlet = $arr_items['outlet'];
        $model->payment_method = $arr_items['payment_method'];
        return ($model->save(false)) ? $model->id : 0; //exception
    }

    public function addVoucheritems($headerId, $arr_items)
    {
        $model = new FinanceVoucherItems;
        $model->header_id = $headerId;
        $model->ledger_id = $arr_items['ledger_id'];
        $model->brand_id = (int) $arr_items['brand_id'];
        $model->emirate_id = $arr_items['emirate_id'];
        $model->subledger_id = $arr_items['subledger_id'];
        $model->narration = $arr_items['narration'];
        $model->amount = $arr_items['amount'];
        $model->currency_id = $arr_items['currency'] > 0 ? $arr_items['currency'] : FinanceLegder::baseCurrency_id();
        $model->exchange_rate = $arr_items['exchange_rate'] > 0 ? $arr_items['exchange_rate'] : 1;
        $model->amount = $arr_items['amount'];
        $model->reconciled_date = $arr_items['reconciled_date'];
        $cleared = $arr_items['opening_cheque_cleared'];
        if ($cleared == '')
            $cleared = 'N';
        $model->opening_cheque_cleared = $cleared;
        $reconciled = $arr_items['reconciled'];
        if ($reconciled == '')
            $reconciled = 'N';
        $model->reconciled = $reconciled;
        $model->outlet = $arr_items['outlet'];
        $model->save(false);
        return $model->id > 0 ? $model->id : 0;
    }
    
     function isPostable($date)
    {
        $last_closed_year = FinanceAuditClose::lastClosed();
        $voucherDate_array = explode('-', $date);
        $voucherYear = $voucherDate_array[0];
        return ($voucherYear <= $last_closed_year) ? false : true;
    }
}
