<?php

class ApifinancereportController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array(
                    'TrialBalanceReport','BalanceSheetReport','ProfitLossReport','CustomerVendorStatementReport','GeneralLedgerReport',
                    'ReceivablePayableAgingReport','LedgerSummaryReport'
                ),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function actionHave_session()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->with('outletDetails')->find(array('condition' => 't.token="' . $token . '" AND t.is_active="Y" AND t.access_type IN("SL", "CA","SM", "OM") AND outletDetails.is_active="Y"'));
        if ((int) $userModel->id > 0) {
            $this->responseBody['status'] = 'success';
            $this->responseBody['msg'] = 'Session is valid';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }



  public function actionTrialBalanceReport()
    {
  
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);
            
            $brand = (int) $variables['brand'];
            $outlet = (int)  $userModel->outlet_id;
            $from_date = (count(explode('/', $variables['from_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['from_date']))) : date('Y-m-01');
            $to_date = (count(explode('/', $variables['to_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['to_date']))) : date('Y-m-d');

            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = "Trialbalance_" . date("d-m-Y");
           
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.modules.finance.views.reports.pdf.trial_balance_pdf', array('id' => $id, 'user_id' => $userModel->id,'outlet'=>$outlet,'brand'=>$brand,'from_date'=>$from_date,'to_date'=>$to_date), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
  public function actionBalanceSheetReport()
    {
  
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);
            
            $brand = (int) $variables['brand'];
            $outlet = (int)  $userModel->outlet_id;
            $from_date = '2019-01-01';
            $to_date = (count(explode('/', $variables['to_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['to_date']))) : date('Y-m-d');

            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = "BalanceSheet_" . date("d-m-Y");
          
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.modules.finance.views.reports.pdf.balance_sheet_pdf', array('id' => $id, 'user_id' => $userModel->id,'outlet'=>$outlet,'brand'=>$brand,'from_date'=>$from_date,'to_date'=>$to_date), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
  public function actionProfitLossReport()
    {
  
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);
            
            $brand = (int) $variables['brand'];
            $outlet = (int)  $userModel->outlet_id;
            $from_date = (count(explode('/', $variables['from_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['from_date']))) : date('Y-m-01');
            $to_date = (count(explode('/', $variables['to_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['to_date']))) : date('Y-m-d');

            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = "ProfitLoss_" . date("d-m-Y");
          
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.modules.finance.views.reports.pdf.profit_loss_pdf', array('id' => $id, 'user_id' => $userModel->id,'outlet'=>$outlet,'brand'=>$brand,'from_date'=>$from_date,'to_date'=>$to_date), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
    public function actionCustomerVendorStatementReport()
    {
  
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $subledger_id = (int) $variables['subledger_id'];
            $currency = 1;
            $type =  $variables['type'];
            if($type=='customer'){
                  $ledger_id = (int)FinanceSystemAccountsMapping::getSystemdefault('debtors');
            }
            if($type=='vendor'){
                  $ledger_id = (int)FinanceSystemAccountsMapping::getSystemdefault('creditors');
            }

            $outlet = (int)  $userModel->outlet_id;
            $to_date = (count(explode('/', $variables['to_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['to_date']))) : date('Y-m-d');

            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            if($type=='customer'){
                $filename = "CustomerStatement_" . date("d-m-Y");
            }else{
               $filename = "VendorStatement_" . date("d-m-Y"); 
            }
          
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.modules.finance.views.reports.pdf_statement_base', array('id' => $id, 'user_id' => $userModel->id,'outlet'=>$outlet,'subledger'=>$subledger_id,'to'=>$to_date,'ledger'=>$ledger_id,'currency'=>$currency,'type'=>$type), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    public function actionGeneralLedgerReport()
    {
  
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $currency = 1;
            $ledger_id = (int) $variables['ledger_id'];
            $outlet = (int)  $userModel->outlet_id;
            $to_date = $variables['to_date'];
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = "XXX_" . date("d-m-Y");
          
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.modules.finance.views.reports.pdf_gl_report', array('id' => $id, 'user_id' => $userModel->id,'outlet'=>$outlet,'to_date'=>$to_date,'ledger'=>$ledger_id), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    public function actionReceivablePayableAgingReport()
    {
  
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            
            $salesman = (int) $variables['salesman'];
          
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $outlet = (int)  $userModel->outlet_id;
            $to_date = $variables['to_date'];
            $category = $variables['category'];
            $ledger_id = (int) $variables['ledger_id'];


            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            if($category=='RECEIVABLE'){
                $filename = "ReceivableReport_" . date("d-m-Y");
            }else{
                $filename = "PayableReport_" . date("d-m-Y");
            }
          
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.modules.finance.views.reports.pdf.aging_report', array('id' => $id,'salesman'=>$salesman,'user_id' => $userModel->id,'outlet'=>$outlet,'to_date'=>$to_date,'ledger'=>$ledger_id,''), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
    public function actionLedgerSummaryReport()
    {
  
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            
            $salesman = (int) $variables['salesman'];
          
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $outlet = (int)  $userModel->outlet_id;
            $from_date = (count(explode('/', $variables['from_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['from_date']))) : date('Y-m-d');
            $to_date = (count(explode('/', $variables['to_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['to_date']))) : date('Y-m-d');     
            $ledger = (int) $variables['ledger_id'];
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            // ------------------------------------------------------------
            $condition='';
            if ($outlet > 0) {
              $condition .= ' AND VI.outlet=' . $outlet;
            }
            
            
        

            $glSummaryquery = 'CREATE OR REPLACE VIEW {{finance_gl_summary}} AS SELECT
                                    SL.id AS subledger_id,
    								SL.account_no AS subledger_no,SL.parent_id AS customer_id,
    								IFNULL(SL.description, "No Subledger") AS subledger_name,
                                    ROUND(SUM( CASE WHEN VI.amount > 0 THEN VI.amount*VI.exchange_rate ELSE 0 END ), 4) AS debit,
                                    ROUND(SUM( CASE WHEN VI.amount < 0 THEN ABS(VI.amount*VI.exchange_rate) ELSE 0 END ), 4) AS credit,
    								ROUND(SUM(VI.amount*VI.exchange_rate), 4) AS balance,
                                    L.id AS ledger_id
    							FROM {{finance_voucher_items}} VI
    							LEFT JOIN {{finance_voucher_head}} V ON V.id = VI.header_id
    							LEFT JOIN {{finance_legder}} L ON L.id = VI.ledger_id
    							LEFT JOIN {{finance_subledger}} SL ON SL.id = VI.subledger_id
    							WHERE VI.ledger_id="' . $ledger . '" AND V.is_posted = "Y" AND V.voucher_date >= "' . $from_date . '"  AND V.voucher_date <= "' . $to_date . '" '.$condition.'
    							GROUP BY  VI.ledger_id, VI.subledger_id';
    
            Yii::app()->db->createCommand($glSummaryquery)->execute();
            // ------------------------------------------------------------

            $path = "app_reports/";
            $filename = "LedgerSummaryReport_" . date("d-m-Y");
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.modules.finance.views.reports.pdf_gl_summary_report', array('id' => $id,'salesman'=>$salesman,'user_id' => $userModel->id,'outlet'=>$outlet,'to_date'=>$to_date,'ledger'=>$ledger,''), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

   

}
