<?php

/**
 * This is the model class for table "erp_goods_transfer".
 *
 * The followings are the available columns in table 'erp_goods_transfer':
 * @property integer $id
 * @property integer $transfer_no
 * @property string $transfer_date
 * @property integer $from_user_id
 * @property integer $to_user_id
 * @property string $notes
 * @property string $active
 * @property string $printable
 * @property integer $created_by
 * @property string $created_time
 * @property integer $updated_by
 * @property string $updated_time
 * @property string $posted
 * @property integer $outlet
 */
class GoodsTransfer extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{goods_transfer}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('transfer_no, transfer_date, from_user_id, to_user_id, outlet', 'required'),
			array('transfer_no, from_user_id, to_user_id, created_by, updated_by, outlet', 'numerical', 'integerOnly'=>true),
			array('notes', 'length', 'max'=>2225),
			array('active, printable, posted', 'length', 'max'=>1),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, transfer_no, transfer_date, from_user_id, to_user_id, notes, active, printable, created_by, created_time, updated_by, updated_time, posted, outlet', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'gt_from_user'     => array(self::BELONGS_TO, 'MasUsers', 'from_user_id'),
		    'gt_to_user'     => array(self::BELONGS_TO, 'MasUsers', 'to_user_id'),
		    'gt_outlet'   => array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
		    'gt_user'     => array(self::BELONGS_TO, 'MasUsers', 'created_by'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'transfer_no' => 'Transfer No',
			'transfer_date' => 'Transfer Date',
			'from_user_id' => 'From User',
			'to_user_id' => 'To User',
			'notes' => 'Notes',
			'active' => 'Active',
			'printable' => 'Printable',
			'created_by' => 'Created By',
			'created_time' => 'Created Time',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Time',
			'posted' => 'Posted',
			'outlet' => 'Outlet',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('transfer_no',$this->transfer_no);
		$criteria->compare('transfer_date',$this->transfer_date,true);
		$criteria->compare('from_user_id',$this->from_user_id);
		$criteria->compare('to_user_id',$this->to_user_id);
		$criteria->compare('notes',$this->notes,true);
		$criteria->compare('active',$this->active,true);
		$criteria->compare('printable',$this->printable,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('updated_time',$this->updated_time,true);
		$criteria->compare('posted',$this->posted,true);
		$criteria->compare('outlet',$this->outlet);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return GoodsTransfer the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
