<?php

/**
 * This is the model class for table "erp_master_payment_terms".
 *
 * The followings are the available columns in table 'erp_master_payment_terms':
 * @property integer $id
 * @property string $payment_term_name
 * @property integer $created_by
 * @property string $created_time
 * @property integer $updated_by
 * @property string $updated_time
 * @property string $active
 */
class MasPaymentTerms extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{master_payment_terms}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('payment_term_name', 'required'),
			array('days,created_by, updated_by', 'numerical', 'integerOnly'=>true),
			array('payment_term_name', 'length', 'max'=>200),
			array('active,is_default', 'length', 'max'=>1),
			array('created_time, updated_time', 'safe'),
			array('payment_term_name', 'unique'),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, payment_term_name, created_by, created_time, updated_by, updated_time, active', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'payment_term_name' => 'Payment Term Name',
			'created_by' => 'Created By',
			'created_time' => 'Created Date',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Date',
			'active' => 'N : No, Y: Yes',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('payment_term_name',$this->payment_term_name,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('updated_time',$this->updated_time,true);
		$criteria->compare('active',$this->active,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return MasPaymentTerms the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
	public static function getActivePaymentTerms() {
        $condition = 'active="Y"';
        $model = self::model()->findAll(array('condition' => $condition,'order'=>'is_default ASC, payment_term_name ASC'));
        $returnArray =  CHtml::listData($model, 'id', 'payment_term_name');
        return $returnArray;
    }
}
