<?php

/**
 * This is the model class for table "erp_material_conversion".
 *
 * The followings are the available columns in table 'erp_material_conversion':
 * @property integer $id
 * @property integer $conversion_no
 * @property string $conversion_date
 * @property integer $from_product_id
 * @property string $from_quantity
 * @property integer $from_unit
 * @property integer $to_product_id
 * @property string $to_quantity
 * @property integer $to_unit
 * @property string $narration
 * @property integer $outlet
 * @property string $created_time
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $updated_time
 */
class MaterialConversion extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public $from_product,$to_product,$from_product_desc,$to_product_desc,$from_unit_name,$to_unit_name;
	public function tableName()
	{
		return '{{material_conversion}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('conversion_no, conversion_date, from_product_id, from_quantity, from_unit, to_product_id, to_quantity, to_unit, narration, outlet', 'required'),
			array('conversion_no, from_product_id, from_unit, to_product_id, to_unit, outlet, created_by, updated_by', 'numerical', 'integerOnly'=>true),
			array('from_quantity, to_quantity', 'length', 'max'=>15),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, conversion_no, conversion_date, from_product_id, from_quantity, from_unit, to_product_id, to_quantity, to_unit, narration, outlet, created_time, created_by, updated_by, updated_time', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'conv_frm_unit' => array(self::BELONGS_TO, 'MasUnit', 'from_unit'),
            'conv_frm_pdt' => array(self::BELONGS_TO, 'Inventory', 'from_product_id'),
            'conv_to_unit' => array(self::BELONGS_TO, 'MasUnit', 'to_unit'),
            'conv_to_pdt' => array(self::BELONGS_TO, 'Inventory', 'to_product_id'),
            'conv_outlet'   => array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
		    'conv_user'     => array(self::BELONGS_TO, 'MasUsers', 'created_by'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'conversion_no' => 'Conversion No',
			'conversion_date' => 'Conversion Date',
			'from_product_id' => 'From Product',
			'from_quantity' => 'From Quantity',
			'from_unit' => 'From Unit',
			'to_product_id' => 'To Product',
			'to_quantity' => 'To Quantity',
			'to_unit' => 'To Unit',
			'narration' => 'Narration',
			'outlet' => 'Outlet',
			'created_time' => 'Created Time',
			'created_by' => 'Created By',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Time',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('conversion_no',$this->conversion_no);
		$criteria->compare('conversion_date',$this->conversion_date,true);
		$criteria->compare('from_product_id',$this->from_product_id);
		$criteria->compare('from_quantity',$this->from_quantity,true);
		$criteria->compare('from_unit',$this->from_unit);
		$criteria->compare('to_product_id',$this->to_product_id);
		$criteria->compare('to_quantity',$this->to_quantity,true);
		$criteria->compare('to_unit',$this->to_unit);
		$criteria->compare('narration',$this->narration,true);
		$criteria->compare('outlet',$this->outlet);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('updated_time',$this->updated_time,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return MaterialConversion the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
