<?php

/**
 * This is the model class for table "erp_order_collect_item".
 *
 * The followings are the available columns in table 'erp_order_collect_item':
 * @property integer $id
 * @property integer $quotes_id
 * @property integer $product_id
 * @property string $description
 * @property string $quantity
 * @property integer $unit_id
 * @property string $base_unit_id
 * @property string $base_unit_qty
 * @property string $unit_factor
 */
class QuotesItem extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */

	public $unit;
	public $unit_price,$unit_exchange,$unit_cost;
	public function tableName()
	{
		return '{{quotes_item}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('quotes_id, product_id, description, quantity, unit_id, base_unit_id, base_unit_qty, unit_factor, rate, amount, tax_percentage, tax_amount, net_amount', 'required'),
			array('quotes_id, product_id, unit_id', 'numerical', 'integerOnly'=>true),
			array('quantity, base_unit_qty, unit_factor', 'length', 'max'=>15),
			array('base_unit_id', 'length', 'max'=>25),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, quotes_id, product_id, description, quantity, unit_id, base_unit_id, base_unit_qty, unit_factor', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'oc_unit' => array(self::BELONGS_TO, 'MasUnit', 'unit_id'),
            'oc_product' => array(self::BELONGS_TO, 'Inventory', 'product_id'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'quotes_id' => 'Quotes',
			'product_id' => 'Product',
			'description' => 'Description',
			'quantity' => 'Quantity',
			'unit_id' => 'Unit',
			'base_unit_id' => 'Base Unit',
			'base_unit_qty' => 'Base Unit Qty',
			'unit_factor' => 'Unit Factor',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('quotes_id',$this->quotes_id);
		$criteria->compare('product_id',$this->product_id);
		$criteria->compare('description',$this->description,true);
		$criteria->compare('quantity',$this->quantity,true);
		$criteria->compare('unit_id',$this->unit_id);
		$criteria->compare('base_unit_id',$this->base_unit_id,true);
		$criteria->compare('base_unit_qty',$this->base_unit_qty,true);
		$criteria->compare('unit_factor',$this->unit_factor,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return OrderCollectItem the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
