<?php
class SalesInvoiceItems extends CActiveRecord
{
	public $part_no, $unit_price, $unit_exchange, $unit_cost, $base_qty;
	public function tableName()
	{
		return '{{sales_invoice_items}}';
	}
	public function rules()
	{
		return array(
			array('invoice_id, product_id, quantity, unit, rate, amount, tax_percentage, tax_amount, net_amount', 'required'),
			array('invoice_id, product_id, unit', 'numerical', 'integerOnly' => true),
			array('description', 'length', 'max' => 255),
			array('deduction_percentage,deduction_amount', 'numerical'),

			array('quantity, rate, amount, excise_percentage, excise_amount, tax_percentage, tax_amount, net_amount', 'length', 'max' => 15),
			array(
				'quantity',
				'compare',
				'compareValue' => 0,
				'operator' => '>',
				'allowEmpty' => false,
				'message' => '{attribute} must be greater than "{compareValue}".'
			),
			// array(
			//     'rate',
			//     'compare',
			//     'compareValue' => 0,
			//     'operator' => '>',
			//     'allowEmpty' => false,
			//     'message' => '{attribute} must be greater than "{compareValue}".'
			// ),
			// array(
			//     'net_amount',
			//     'compare',
			//     'compareValue' => 0,
			//     'operator' => '>=',
			//     'allowEmpty' => false,
			//     'message' => '{attribute} must be greater than or equal to "{compareValue}".'
			// ),
			array('id, invoice_id, product_id, description, quantity,deduction_percentage,deduction_amount, unit, rate, amount, excise_percentage, excise_amount, tax_percentage, tax_amount, net_amount', 'safe', 'on' => 'search'),
		);
	}
	public function relations()
	{
		return array(
			'product' => array(self::BELONGS_TO, 'Inventory', 'product_id'),
			'invoice' => array(self::BELONGS_TO, 'SalesInvoice', 'invoice_id'),
			'units' => array(self::BELONGS_TO, 'MasUnit', 'unit'),
		);
	}
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'invoice_id' => 'Invoice',
			'product_id' => 'Product',
			'description' => 'Description',
			'quantity' => 'Quantity',
			'unit' => 'Unit',
			'rate' => 'Rate',
			'amount' => 'Amount',
			'excise_percentage' => 'Excise Tax Percentage',
			'excise_amount' => 'Excise Tax Amount',
			'tax_percentage' => 'Tax Percentage',
			'tax_amount' => 'Tax Amount',
			'net_amount' => 'Net Amount',
		);
	}

	public function search()
	{
		$criteria = new CDbCriteria;
		$criteria->compare('id', $this->id);
		$criteria->compare('invoice_id', $this->invoice_id);
		$criteria->compare('product_id', $this->product_id);
		$criteria->compare('description', $this->description, true);
		$criteria->compare('quantity', $this->quantity, true);
		$criteria->compare('unit', $this->unit);
		$criteria->compare('rate', $this->rate, true);
		$criteria->compare('amount', $this->amount, true);
		$criteria->compare('excise_percentage', $this->excise_percentage, true);
		$criteria->compare('excise_amount', $this->excise_amount, true);
		$criteria->compare('tax_percentage', $this->tax_percentage, true);
		$criteria->compare('tax_amount', $this->tax_amount, true);
		$criteria->compare('net_amount', $this->net_amount, true);

		return new CActiveDataProvider($this, array(
			'criteria' => $criteria,
		));
	}

	public static function model($className = __CLASS__)
	{
		return parent::model($className);
	}


	public static function getLastFiveSalesRates($product_id, $outlet_id = 0,$customer_id=0)
	{
		$condition = "I.product_id = '" . (int)$product_id . "' AND H.active = 'Y' ";
		if ($outlet_id == 0) {
			$condition .= ' AND H.outlet = ' . Yii::app()->user->outlet_id;
		} else {
			$condition .= ' AND H.outlet = ' . $outlet_id;
		}
		if($customer_id>0){
		    $condition .= ' AND H.customer_id = ' . $customer_id;
		}

		$sql = "SELECT I.rate,H.invoice_no,H.invoice_date  
                FROM `erp_sales_invoice_items` I
                LEFT JOIN `erp_sales_invoice` H ON I.invoice_id = H.id
                WHERE $condition 
                ORDER BY I.`id` DESC  
                LIMIT 5";

		$command = Yii::app()->db->createCommand($sql);
		$rows = $command->queryAll(); // Fetch all rows into an array


		$rates = [];
		foreach ($rows as $row) {
    		$rates[] = [
                'rate' => $row['rate'],
                'invoice_no' => $row['invoice_no'],
                'invoice_date' => $row['invoice_date']
            ];
		}

		return $rates; // Returns an array with the last 5 rates
	}
}
