<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiCostReportsController extends Controller {

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules() {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('Index'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters() {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request) {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401() {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400() {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403() {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind() {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value) {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions() {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null) {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name) {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }
  
    //Reports function start
    public function actionIndex() {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            //Outlet id
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = $variables['type'];
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;
                
            $conditionFormats = array(
                'sales' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Sales Invoices',
                    'mainDb' => $this->db_main,
                ),
                'sales_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Sales Returns',
                    'mainDb' => $this->db_main,
                ),
                'purchase' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Purchase Invoices',
                    'mainDb' => $this->db_main,
                ),
                'purchase_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Purchase Returns',
                    'mainDb' => $this->db_main,
                ),
                'net_sales' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Net Sales',
                    'mainDb' => $this->db_main,
                ),
                'net_purchase' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Net Purchase',
                    'mainDb' => $this->db_main,
                ),
                'customer' => array(
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Customers',
                    'mainDb' => $this->db_main,
                ),
                'vendor' => array(
                    'vendor_id' => (int) $variables['vendor_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Vendors',
                    'mainDb' => $this->db_main,
                ),
                'current_stock' => array(
                    'product_id' => (int) $variables['product_id'],
                    'product_group_id' => (int) $variables['product_group_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Current Stock',
                    'mainDb' => $this->db_main,
                    'access_type' => $userModel->access_type
                ),
                'transaction' => array(
                    'access_type' => $userModel->access_type,
                    'user_id' => $userModel->id,
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'product_id' => (int) $variables['product_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Transaction Details',
                    'mainDb' => $this->db_main,
                ),
                'customer_statement' => array(
                    'access_type' => $userModel->access_type,
                    'user_id' => $userModel->id,
                    'outlet_id' => $outlet,
                    'customer_id' => (int) $variables['customer_id'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Customer Statement',
                    'mainDb' => $this->db_main,
                ),
                'vendor_statement' => array(
                    'vendor_id' => (int) $variables['vendor_id'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Vendor Statement',
                    'mainDb' => $this->db_main,
                ),
                'receivable_aging' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Receivable Aging',
                    'mainDb' => $this->db_main,
                ),
                'payable_aging' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Payable Aging',
                    'mainDb' => $this->db_main,
                ),
                'day_book' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Day Book',
                    'mainDb' => $this->db_main,
                ),
                'stock_report' => array(
                    'access_type' => $userModel->access_type,
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Stock Ledger',
                    'mainDb' => $this->db_main,
                ),
                'vat_return_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Return',
                    'mainDb' => $this->db_main,
                ),
                'vat_summary_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Summary',
                    'mainDb' => $this->db_main,
                ),
                'consolidated_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Consolidated Report',
                    'mainDb' => $this->db_main,
                ),
                'daily_sales_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Daily Sales Report',
                    'mainDb' => $this->db_main,
                ),
                'cost_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'brand_id' => (int) $variables['brand_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cost Report',
                    'mainDb' => $this->db_main,
                    'access_type' => $userModel->access_type
                ),
                'cash_collection_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cash Collection Report',
                    'mainDb' => $this->db_main,
                ),
                'costofsale' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman' => (int)$salesman,
                    'outlet_id' => $outlet,
                    'brand' =>(int) $variables['brand'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cost of Sale',
                    'mainDb' => $this->db_main,
                    'access_type' => $userModel->access_type
                ),
                'brandwise' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman' => (int)$variables['salesman'],
                    'outlet_id' => $outlet,
                    'brand' =>(int) $variables['brand'],
                    'payment_term' =>(int) $variables['payment_term'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Brand wise report',
                    'mainDb' => $this->db_main,
                ),
                /*
                'tax_register' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Tax Register',
                    'mainDb' => $this->db_main,
                ),
                'vat_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Region wise',
                    'mainDb' => $this->db_main,
                ),
                'trial_balance' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Trial balance',
                    'mainDb' => $this->db_main,
                ),
                'balance_sheet' => array(
                    'to_date' => $variables['to_date'],
                    'company_id' => $userModel->company_id,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Balance Sheet',
                    'mainDb' => $this->db_main,
                ),
                'profit_and_loss' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Profit and Loss',
                    'mainDb' => $this->db_main,
                ), */
            );

            //SQL Execution
            $this->generate_view($report_type, $conditionFormats[$report_type]);

            //Negative symbol
            function getSymbol($value = 0) {
                return $value >= 0 ? '' : '(-) ';
            }

            //Space Addition
            function addSpace($level = 0) {
                $space = '';
                for ($i = 0; $i < $level * 4; $i++) {
                    $space .= '&nbsp;';
                }
                return $space;
            }

            function get_value_label($value = 0) {
                return $value > 0 ? ' Dr' : ' Cr';
            }

     
            //HTML Data Generation
            function generate_details($report_type, $conditionFormats) {

                $table = '';
                $params = $conditionFormats[$report_type];
//Start
                
       
 
$from_date = $params['from_date'];
$to_date = $params['to_date'];
$outlet = (int) $params['outlet_id'];

$condition = ' where H.active="Y" AND H.invoice_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';
if ($outlet > 0) {
    $condition .= ' AND H.outlet=' . $outlet;
     $conditions.= ' AND outlet=' . $outlet;
}
$SQL=' Select H.invoice_no,C.customer_name,I.description,I.net_amount,I.quantity,H.invoice_date,I.product_id,I.tax_amount,H.id AS invoice_id,I.id AS item_id from erp_sales_invoice H INNER JOIN erp_sales_invoice_items I ON H.id=I.invoice_id INNER JOIN erp_customer C ON H.customer_id=C.id '.$condition.' order by H.id'; 

$all=Yii::app()->db->createCommand($SQL)->queryAll();
$table = '<table class="table table-hover table-fixed table-bordered">
    <thead>
        <tr>
            <th style="width:10%">Date</th>
            <th style="width:10%">Invoice No</th>
            <th style="width:25%">Customer</th>
            <th style="width:25%">Part No.</th>
            <th style="width:5%">Net Total</th>
            <th style="width:5%">Tax</th>
            <th style="width:5%">Qty</th>
            <th style="width:5%">Cost</th>
            <th style="width:5%">Total Cost</th>
            <th style="width:5%">Profit</th>
            <th style="width:5%">Profit(%)</th>
        </tr>
    </thead>
<tbody>';
$total_cost=0;
$total_profit=0;
$total_sales=0;
$total_tax=0;
foreach ($all as $each)
{
    $qty=Yii::app()->db->createCommand('SELECT quantity*unit_exchange_rate AS qty from erp_stock_ledger where  type="SI" AND header_id='.$each["invoice_id"].' AND item_id='.$each["item_id"].' AND product_id='.$each["product_id"].' AND transaction_date <= "' . $each['invoice_date'] . '" '.$conditions)->queryScalar();
    $qty=round($qty,2);
    $current_in_rate=Yii::app()->db->createCommand('SELECT CASE
    WHEN SUM(quantity*unit_exchange_rate) > 0 THEN SUM(quantity*rate)/SUM(quantity*unit_exchange_rate) ELSE 0 END AS avg_purchase_rate 
    FROM {{stock_ledger}}  WHERE transaction_type = "IN" AND type!= "GT" AND product_id='.$each["product_id"].' AND transaction_date <= "' . $each['invoice_date'] . '" '.$conditions)->queryScalar();
    $profit=$each['net_amount']-$each['tax_amount']-($qty*$current_in_rate);
    $cost=($qty*$current_in_rate);
    if($cost>0)
    {
    $profit_percent=$profit/($cost)*100;
    }
    $table.='<tr><td>'.date("d-m-Y", strtotime($each['invoice_date'])).'</td><td>'.$each['invoice_no'].'</td><td>'.$each['customer_name'].'</td><td>'.$each["product_id"].'/'.$each['description'].'</td><td style="text-align:right !important;">'.Yii::app()->numberFormatter->formatCurrency($each['net_amount'],"").'</td><td style="text-align:right !important;">'.Yii::app()->numberFormatter->formatCurrency($each['tax_amount'],"").'</td><td>'.$qty.'</td><td style="text-align:right !important;">'.Yii::app()->numberFormatter->formatCurrency($current_in_rate, " ").'</td><td style="text-align:right !important;">'.Yii::app()->numberFormatter->formatCurrency($qty*$current_in_rate, " ").'</td><td style="text-align:right !important;">'.Yii::app()->numberFormatter->formatCurrency($profit, " ").'</td><td style="text-align:right !important;">'.Yii::app()->numberFormatter->formatCurrency($profit_percent, " ").'%</td></tr>';    
    $total_cost+=$qty*$current_in_rate;
    $total_profit+=$profit;
    $total_sales+=$each['net_amount'];
    $total_tax+=$each['tax_amount'];
}
if($total_cost>0)
{
$percent=($total_sales-$total_cost)/$total_cost*100;
}
else {
$percent=0;    
}
$table.='<tr><td colspan="4" align="right"><b>Total</b></td><td style="text-align:right !important;" align="right"><b>'.Yii::app()->numberFormatter->formatCurrency($total_sales, " ").'</b></td><td align="right" style="text-align:right !important;"><b>'.Yii::app()->numberFormatter->formatCurrency($total_tax, " ").'</b></td><td></td><td></td><td style="text-align:right !important;" align="right"><b>'.Yii::app()->numberFormatter->formatCurrency($total_cost, " ").'</b></td><td style="text-align:right !important;" align="right"><b>'.Yii::app()->numberFormatter->formatCurrency($total_profit, " ").'</b></td><td style="text-align:right !important;" align="right"><b>'.Yii::app()->numberFormatter->formatCurrency($percent, " ").'%</b></td></tr>';    
$table .= '</tbody>
            </table>';

//End
return $table;
            
            }
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.' . $conditionFormats[$report_type]['view'], array('table' => generate_details($report_type, $conditionFormats), 'title' => $conditionFormats[$report_type]['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $conditionFormats[$report_type], 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
 
    public function generate_view($view_type, $params)
    {

        /*$sales_voucher_id = 4;
        $creditnote = 5;
        $purchase_voucher_id = 1;
        $debitnote = 2;*/

        if ($view_type == 'sales') {

            $condition = 'V.invoice_date >= "'.$params['from_date'].'" AND V.invoice_date <= "'.$params['to_date'].'" ';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }
            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.sales_man=' . $params['salesman_id'];
            }
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }
            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_sales_invoice_list AS SELECT
                                                V.id,
                                                V.invoice_date AS date,
                                                V.invoice_no AS voucher_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                                O.name AS outlet_name,
						MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                                FROM erp_sales_invoice V
                                            LEFT JOIN erp_master_currency MC ON MC.id = V.currency
                                            LEFT JOIN erp_customer C ON C.id = V.customer_id
                                            LEFT JOIN ' . $this->db_main . '.erp_users U ON U.id = V.sales_man
                                            LEFT JOIN ' . $this->db_main . '.erp_outlets O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC')->execute();
        }
        else if ($view_type == 'sales_return') {

            $condition = 'V.return_date >= "' . $params['from_date'] . '" AND V.return_date <="' . $params['to_date'] . '"';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND SI.sales_man=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_creditnote_list AS SELECT
                                                V.id,
                                                V.return_date AS date,
                                                V.return_no AS voucher_no,
                                                SI.invoice_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                               	O.name AS outlet_name,
                                                MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                        FROM {{sales_return}} V
                                        LEFT JOIN {{sales_invoice}} SI ON SI.id = V.invoice_id
                                        LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                        LEFT JOIN {{customer}} C ON C.id = V.customer_id
                                        LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = SI.sales_man
                                        LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                        WHERE ' . $condition . ' ORDER BY V.return_date ASC,V.id ASC')->execute();
        } else if ($view_type == 'purchase') {
            //$params['vendor_id']=$params['ledger_id'];
            $condition = 'V.invoice_date >= "'.$params['from_date'].'" AND V.invoice_date <= "'.$params['to_date'].'" ';
            if ($params['vendor_id'] > 0) {
                $condition .= ' AND V.vendor_id=' . $params['vendor_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.created_by=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_purchase_invoice_list AS SELECT
                                            V.id,
                                            V.invoice_date AS date,
                                            V.invoice_no AS voucher_no,
                                            C.vendor_name AS vendor_name,
                                            U.full_name AS sales_man,
                                            O.name AS outlet_name,
                                            MC.currency_code AS currency,
                                            V.tax_total AS total_vat,
                                            V.grand_total,
                                            (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                            (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                            V.active
                                            FROM {{purchase_invoice}} V
                                            LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                            LEFT JOIN {{vendor}} C ON C.id = V.vendor_id                                                
                                            LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = V.created_by
                                            LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC ')->execute();
        } else if ($view_type == 'purchase_return') {
            $condition = 'V.return_date >= "' . $params['from_date'] . '" AND V.return_date <="' . $params['to_date'] . '"';
            if ($params['vendor_id'] > 0) {
                $condition .= ' AND V.vendor_id=' . $params['vendor_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND PI.created_by=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }            
            
            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_debitnote_list AS SELECT
                                                V.id,
                                                V.return_date AS date,
                                                V.return_no AS voucher_no,
                                                PI.invoice_no,
                                                C.vendor_name AS vendor_name,
                                                U.full_name AS sales_man,
                                               	O.name AS outlet_name,
                                                MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active                                                
                                        FROM {{purchase_return}} V
                                        LEFT JOIN {{purchase_invoice}} PI ON PI.id = V.invoice_id
                                        LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                        LEFT JOIN {{vendor}} C ON C.id = V.vendor_id
                                        LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = PI.created_by
                                        LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                        WHERE ' . $condition . ' ORDER BY V.return_date ASC,V.id ASC')->execute();
        }else if ($view_type == 'net_purchase') {

            /*Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_net_purchase_report AS (SELECT "I" as type,I.id,I.vendor_id,I.created_by,I.invoice_no,I.invoice_date,V.vendor_name,U.full_name,I.exchange_rate,I.total_amount,I.grand_total,I.tax_total,I.outlet,I.active
              FROM  {{purchase_invoice}} I
              LEFT JOIN  {{vendor}} V
              ON V.id=I.vendor_id
              LEFT JOIN   bluesell_new_main.erp_users U
              ON U.id=I.created_by
              )
              UNION ALL
              (SELECT "CN" as type,CN.id,I.vendor_id,I.created_by,CN.return_no AS invoice_no,CN.return_date AS invoice_date,V.vendor_name,U.full_name,I.exchange_rate,CN.total_amount,CN.grand_total ,CN.tax_total,CN.outlet,CN.active 
              FROM  {{purchase_return}} CN
              LEFT JOIN  {{vendor}} V
              ON V.id=CN.vendor_id
              LEFT JOIN {{purchase_invoice}} I
              ON I.id=CN.invoice_id
              LEFT JOIN   bluesell_new_main.erp_users U
              ON U.id=I.created_by)')->execute();*/
               
        } /* else if ($view_type == 'aging') {

            $ledger_id = (int) $params['ledger_id'];
            $outlet = (int) $params['outlet_id'];
            $to_date = $params['to_date'];
            $type = $params['type'] == 'payable' ? 'payable' : 'receivable';

            $dates_array = array(
                '0_to_30' => array('from' => date('Y-m-d', strtotime($to_date . ' -30 days')), 'to' => $to_date),
                '31_to_60' => array('from' => date('Y-m-d', strtotime($to_date . ' -60 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -31 days'))),
                '61_to_90' => array('from' => date('Y-m-d', strtotime($to_date . ' -90 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -61 days'))),
                '91_to_120' => array('from' => date('Y-m-d', strtotime($to_date . ' -120 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -91 days'))),
                'greaterthan_120' => array('from' => '2010-01-01', 'to' => date('Y-m-d', strtotime($to_date . ' -121 days'))),
            );

            foreach ($dates_array as $key => $each_date_pair) {

                $condition = 'V.date BETWEEN "' . $each_date_pair['from'] . '" AND "' . $each_date_pair['to'] . '" AND V.is_posted="Y"';
                if ($ledger_id > 0) {
                    $condition .= ' AND T.ledger_id = ' . (int) $ledger;
                }

                if ($outlet > 0) {
                    $condition .= ' AND L.outlet_id=' . (int) $outlet;
                }

                if ($type == 'receivable') {
                    $condition .= '  AND L.classification="' . VanLedgers::CUSTOMER . '"';
                } else {
                    $condition .= '  AND L.classification="' . VanLedgers::VENDOR . '"';
                }

                Yii::app()->db->createCommand('CREATE OR REPLACE VIEW {{ageing_' . $key . '}} AS SELECT
                                                    L.id AS ledger_id,
                                                    L.name AS ledger_name,
                                                    IFNULL(ROUND(SUM(T.amount*T.exchange_rate), 6), 0) AS voucher_value
                                                FROM {{transactions}} T
                                                LEFT JOIN {{vouchers}} V ON V.id = T.voucher_id
                                                LEFT JOIN {{ledgers}} L ON L.id = T.ledger_id
                                                LEFT JOIN {{allocation}} A ON A.child_voucher_id = V.id
                                                WHERE ' . $condition . '
                                                GROUP BY L.id')->execute();
            }
        } else if ($view_type == 'trial_balance') {

            $condition = 'V.date  BETWEEN "' . $params['from_date'] . '" AND "' . $params['to_date'] . '" AND  V.is_posted="Y"';
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet_id=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW trial_balance AS
                                            SELECT
                                                L.outlet_id,
                                                L.is_expandable,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND((SUM(CASE WHEN TR.amount > 0 THEN TR.amount*TR.exchange_rate ELSE 0 END)), 6) AS debit,
                                                ABS(ROUND((SUM(CASE WHEN TR.amount < 0 THEN TR.amount*TR.exchange_rate ELSE 0 END)), 6)) AS credit,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        } else if ($view_type == 'balance_sheet') {

            $condition = 'V.date <= "' . $params['to_date'] . '" AND L.type IN ("A", "L") AND V.is_posted="Y"';
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet_id=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW balance_sheet AS
                                            SELECT
                                                L.outlet_id,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        } else if ($view_type == 'profit_and_loss') {

            $from_date = $params['from_date'];
            $to_date = $params['to_date'];
            $outlet = $params['outlet_id'];

            $condition = 'V.date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND L.type IN ("I", "E") AND V.is_posted="Y"';
            if ($outlet > 0) {
                $condition .= ' AND V.outlet_id=' . $outlet;
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW profit_and_loss AS
                                            SELECT
                                                L.outlet_id,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        }*/
    }
}