<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiCustomerController extends Controller {

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules() {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('GetCustomerList','Ledger_details','BlockLedger','Savecustomer'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters() {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request) {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401() {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400() {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403() {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind() {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value) {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions() {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null) {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name) {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }
 
    //changed
    public function actionGetCustomerList() {

        $input = file_get_contents('php://input');
        //error_log('ledgers');
        //error_log($input);
        $variables = json_decode($input, true); 
        $term = $variables['term']; 
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $items = array();
        $userModel = MasUsers::model()->findBytoken($token);

        //Common or all
        $count = 0;
        $items = array();
        $factor = 1;

        if ($userModel->id > 0) {

            $userModel->db();

            
            $condition = 'active="Y" ';
             
            if (!empty($term)) { 
                $term = '%' . $term . '%';
                $condition .= ' AND (customer_name LIKE "' . $term . '" OR phone LIKE "' . $term . '")';
            } 
             
            if ($userModel->access_type!="SL") {
                $condition .= ' AND outlet=' . $userModel->outlet_id;
            } else {
                $condition .= 'AND (t.created_by = ' . $userModel->id . ' OR FIND_IN_SET(' . $userModel->id . ',accessed_by))';
            }
            $items = Customer::model()->findAll(array('condition' => $condition, 'order' => 'customer_name ASC'));
            $modmasCurrency     = new MasCurrency();
            
            $base_currency_id   = (int)$modmasCurrency->base_currency_id();
            $modmasCurrency     = $modmasCurrency->findByPK($base_currency_id);
            

            if (!empty($items)) {
                foreach ($items as $i => $each) {
                 
                        $this->responseBody['data'][$i]['id'] = $each->id;
                        $this->responseBody['data'][$i]['name'] = $each->customer_name;
                        $this->responseBody['data'][$i]['alias'] = $each->customer_no;
                        $this->responseBody['data'][$i]['editable'] = true;
                        $this->responseBody['data'][$i]['credit_limit'] = $each->credit_limit;
                        $this->responseBody['data'][$i]['credit_days'] = $each->credit_days;
                        $this->responseBody['data'][$i]['phone_no'] = $each->phone;
                        $this->responseBody['data'][$i]['mobile'] =(!empty( $each->mobile))? $each->mobile :'';
                        $this->responseBody['data'][$i]['email'] = (!empty($each->email))?$each->email:'';
                        $customer_subledger = FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $each->id . ' AND subledger_type_id=1'))->id;
                        $customer_ledger = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                        $current_credit = Yii::app()->db->createCommand("select SUM(I.amount) from erp_finance_voucher_items I INNER JOIN erp_finance_voucher_head H ON H.id=I.header_id where H.is_posted='Y' AND ledger_id='$customer_ledger' AND subledger_id='$customer_subledger'")->queryScalar();
                        $this->responseBody['data'][$i]['current_balance'] = number_format($current_credit, 2);
                        $this->responseBody['data'][$i]['currency_code'] = $modmasCurrency->currency_code;
                        $this->responseBody['data'][$i]['currency_name'] = $modmasCurrency->currency;
                        $this->responseBody['data'][$i]['currency_id'] = $base_currency_id;
                        $this->responseBody['data'][$i]['currency_exchange_rate'] = $modmasCurrency->exchange_rate;
                        $this->responseBody['data'][$i]['payment_term'] = $each->payment_term;
                        $this->responseBody['data'][$i]['emirate'] = $each->state_detail->state_name;
                        $this->responseBody['data'][$i]['taxable'] = 'N';
                        $this->responseBody['data'][$i]['country'] = $each->country_detail->country_name;
                        $this->responseBody['data'][$i]['trn_no'] = $each->trn_no;
                        $this->responseBody['data'][$i]['billing_address'] = $each->address;
                        $this->responseBody['data'][$i]['location'] = (!empty($each->location))?$each->location:'';
                        $this->responseBody['data'][$i]['contact_person'] = (!empty($each->contact_person))?$each->contact_person:'';
                        $this->responseBody['data'][$i]['latitude'] = $each->latitude;
                        $this->responseBody['data'][$i]['longitude'] = $each->longitude;
                        $this->responseBody['data'][$i]['tax_type'] = $each->tax_type;
                        $this->responseBody['data'][$i]['payment_term_name'] = (!empty($each->paymentterm_detail->payment_term_name))?$each->paymentterm_detail->payment_term_name:'';

                    $count++;
                }
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    
    public function actionLedger_details() {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        //error_log('ledger_details');
        //error_log($input);
        
        $variables = json_decode($input, true);
        $ledger_id = trim($variables['ledger_id']); 

        if ($userModel->id > 0) {

            $userModel->db();

            if ($ledger_id > 0) {
                //Model
                
                    $model = Customer::model()->findBypk($ledger_id);
                    $this->responseBody['data']['id'] = $model->id;
                    $this->responseBody['data']['name'] = $model->customer_name;
                    $this->responseBody['data']['alias'] = $model->customer_no;
                    $this->responseBody['data']['phone_no'] = $model->phone;
                    $this->responseBody['data']['mobile'] = $model->mobile;
                    $this->responseBody['data']['email'] = $model->email;
                    $this->responseBody['data']['currency_id'] = 1;
                    $this->responseBody['data']['currency_code'] = 'AED';
                    $this->responseBody['data']['opening_balance'] = 0.00;
                    $this->responseBody['data']['opening_date'] = '2019-01-01';
                    $this->responseBody['data']['emirate_id'] = $model->state;
                    $this->responseBody['data']['emirate_name'] = $model->state_detail->state_name;
                    $this->responseBody['data']['country_id'] = $model->country;
                    $this->responseBody['data']['country_name'] = $model->country_detail->country_name;
                    $this->responseBody['data']['trn_no'] = $model->trn_no;
                    $this->responseBody['data']['billing_address'] = $model->address;
                    $this->responseBody['data']['credit_limit'] = $model->credit_limit;
                    $this->responseBody['data']['phone_no'] = $model->phone;
                    $this->responseBody['data']['payment_term'] = $model->payment_term;
                    $this->responseBody['data']['credit_days'] = $model->credit_days;
                    $this->responseBody['data']['location'] = $model->location;
                    $this->responseBody['data']['contact_person'] = $model->contact_person;
                    $this->responseBody['data']['latitude'] = $model->latitude;
                    $this->responseBody['data']['longitude'] = $model->longitude;
                    $this->responseBody['data']['tax_type'] = $model->tax_type;
                    $customer_subledger=$this->getSublegder($ledger_id, 1);
                    $customer_ledger=FinanceSystemAccountsMapping::getSystemdefault('debtors');
                    $current_credit=Yii::app()->db->createCommand("select SUM(I.amount) from erp_finance_voucher_items I INNER JOIN erp_finance_voucher_head H ON H.id=I.header_id where H.is_posted='Y' AND ledger_id='$customer_ledger' AND subledger_id='$customer_subledger'")->queryScalar();
                    $this->responseBody['data']['outstanding_amount'] = (float)$current_credit;
                
                 
                $this->responseBody['count'] = 1;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'ledger_id undefined';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }

    public function actionBlockLedger() {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $ledger_id = (int) trim($variables['ledger_id']);
        

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {

            $userModel->db();
            
                $id = $ledger_id;
                $model = Customer::model()->findByPk($id);
                $model->active = 'N';
                $model->save(false);
                $this->responseBody['id'] = $ledger_id;
                $this->responseBody['status'] = 'success';
                $this->_bind;
           
        } else {
            $this->_401;
        }
    }
    public function actionSavecustomer25_10_2024() {
        $input = file_get_contents('php://input');
        $ledgerData = json_decode($input, true);
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        
        $other = array();
        if ($userModel->id > 0) {

            $userModel->db();
            $id = $ledgerData['id'];
            $other = array();
            $currency = $ledgerData['currency'];
            $country = $ledgerData['country'];
            $payment_term = (int)$ledgerData['payment_term'];
            $emirate = $ledgerData['emirate'];
            $phone_no = trim($ledgerData['phone_no']);
            $trn_no = trim($ledgerData['trn_no']);
            $billing_address = trim($ledgerData['billing_address']);
            $is_barcode_visible = $ledgerData['is_barcode_visible'];
            //$credit_limit = (float) $ledgerData['credit_limit'];
         
            $credit_days = $ledgerData['credit_days'];
            $location = $ledgerData['location'];
            $latitude = $ledgerData['latitude'];
            $longitude = $ledgerData['longitude'];
            $tax_type = $ledgerData['tax_type'];
            $model = new Customer();
            $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="C"'));

            if ($id > 0) {
                $model = $model->findByPK($id);
                $scene = 'update';
                $change_type = 'UPDATED';
            } else {
                $model->created_by = $userModel->id;
                $model->created_time = date('Y-m-d H:i:s');
                $scene = 'new';
                $change_type = 'CREATED';
                $model->customer_no = $modelformmaster->next_number;
                $model->accessed_by = $userModel->id;
                
            }
            $model->latitude = $latitude;
            $model->longitude = $longitude;
            $model->customer_name = trim($ledgerData['name']);
            $model->state = $emirate;
            $model->address = $billing_address;
            $model->country = $country;
            $model->location = $location;
            $model->payment_term = $payment_term;
            $model->trn_no = $trn_no;
            $model->phone = $phone_no;
            $model->credit_days = $credit_days;
             // ---------added by indhu ------------------------
             $outlet_id = $userModel->outlet_id;
             if ((int)$outlet_id > 0) {
                 $modelMasOutlets            = new MasOutlets();
                 $modelMasOutlets_det        = $modelMasOutlets->findByPK($outlet_id);
                 $credit_limit_edit_salesman = $modelMasOutlets_det->credit_limit_edit_salesman;
                 $add_edit_customer          = $modelMasOutlets_det->add_edit_customer;
                 if ($credit_limit_edit_salesman == 'Y') {
                     $model->credit_limit = (float) $ledgerData['credit_limit'];
                 }

                 if ($add_edit_customer == 'N') {
                     $message = ($scene == 'update') ? 'Editing for Customers Is Blocked' : 'Adding Customers Are Blocked';
                     $other[] = $message;
                     $valid   = false;
                 }
             }
             // ---------------------------------
            //$model->credit_limit = $credit_limit;
            $model->updated_by = $userModel->id;
            $model->updated_time = date('Y-m-d H:i:s');
            $model->outlet = $userModel->outlet_id;
            $model->active = 'Y';
            $model->tax_type=$tax_type;
            $valid_currency = MasCurrency::model()->findBypk((int) $ledgerData['currency'])->id > 0 ? true : false;
            if ($valid_currency == false)
                $other[] = 'Invalid currency';
            /* $validationResponse['errors']['country'] = 'Invalid Country';
              $validationResponse['errors']['emirate'] = 'Invalid Emirate';
              $validationResponse['errors']['name'] = 'Name already exists';
              $validationResponse['errors']['credit_limit'] = 'Invalid credit limit';
              $validationResponse['errors']['payment_term'] = 'Invalid Credit days'; */
            $valid = $model->validate();
            if ($valid) {
                $model->save();
                
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $model->id));
                if (empty($subledger)) {
                    $newItem = new FinanceSubledger;
                } else {
                    $newItem = FinanceSubledger::model()->findByPk($subledger->id);
                }
                $newItem->description = $model->customer_name;
                $newItem->account_no = $model->customer_no;
                if ($model->address != '') {
                    $newItem->address = $model->address;
                }
                if ($model->phone != '') {
                    $newItem->phone = $model->phone;
                }
                $newItem->subledger_type_id = 1;
                $newItem->parent_id = $model->id;
                $newItem->created_by = $model->created_by;
                $newItem->outlet = $userModel->outlet_id;
                $newItem->save(false);
                /////////////////////////////////////////////////////////////
                if ($scene == 'new') {
                    $next_no = $model->customer_no + 1;
                    FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                }
                // inserting to audit log
                AuditLog::saveApiAuditLogs(
                        $userModel->outlet_id, $modelformmaster->id, $userModel->id, $change_type, " with ID - {$model->id} "
                );
                // end audit log
                $this->responseBody['status'] = 'success';
                $this->responseBody['data'] = array('ledger_id' => $model->id);
                $this->_bind;
            } else {
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                $error = $error1;
                $arr_err=$other;
                foreach ($error as $key => $errval) {
                    $arr_err[]=$errval[0];
                }
                $err_msg = implode(',', $arr_err);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }
    
    
    
       public function actionSavecustomer() {
       
           $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
           $modelMasUsers = new MasUsers();
        $userModel = $modelMasUsers->findBytoken($token);
       
         $input = file_get_contents('php://input');
        $ledgerData = json_decode($input, true);
        
    
        
        $other = array();
        if ((int)$userModel->id > 0) {

             $userModel->db();
            $id = $ledgerData['id'];
            $other = array();
            
            $customer_name   = $ledgerData['customer_name'];
            $contact_person  = $ledgerData['contact_person'];
            $phone  = $ledgerData['phone'];
            $mobile  = $ledgerData['mobile'];
            $email  = $ledgerData['email'];
            $trn_no  = $ledgerData['trn_no'];
            $payment_term  = $ledgerData['payment_term'];
            $credit_days  = $ledgerData['credit_days'];
            $credit_limit  = $ledgerData['credit_limit'];
            $location  = $ledgerData['location'];
            $state  = $ledgerData['state'];
            $country  = $ledgerData['country'];
            $invoice_barcode_show  = $ledgerData['invoice_barcode_show'];
            $latitude  = $ledgerData['latitude'];
            $longitude  = $ledgerData['longitude'];
            $tax_type  = $ledgerData['tax_type'];
            $address  = $ledgerData['address'];
            $accessed_by  = $ledgerData['accessed_by'];



            $currency = $ledgerData['currency'];
            $emirate = $ledgerData['emirate'];
            $billing_address = trim($ledgerData['billing_address']);
            $is_barcode_visible = $ledgerData['is_barcode_visible'];
            $model = new Customer(); 
                
            $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="C"'));

            if ($id > 0) {
                $model = $model->findByPK($id);
                $scene = 'update';
                $change_type = 'UPDATED';
                $model->updated_by = $userModel->id;
                $model->updated_time = date('Y-m-d H:i:s');
            } else {
                $model->created_by = $userModel->id;
                $model->created_time = date('Y-m-d H:i:s');
                $scene = 'new';
                $change_type = 'CREATED';
                $model->customer_no = $modelformmaster->next_number;
                $model->accessed_by = $userModel->id;
                
            }
            
            
               $model->customer_name = trim($customer_name); 
               $model->contact_person = $contact_person;
               $model->phone = $phone;
               $model->mobile = $mobile;
               $model->email = $email;
               $model->trn_no = $trn_no;
               $model->payment_term = $payment_term;
               $model->credit_days = $credit_days;
            //   $model->credit_limit = $credit_limit;
               $model->location = $location;
               $model->state = $state;
               $model->country = $country;
               $model->invoice_barcode_show = $invoice_barcode_show;
               $model->latitude = $latitude;
               $model->longitude = $longitude;
               $model->tax_type = $tax_type;
               $model->address = $address;
               
             // ---------added by indhu ------------------------
             $outlet_id = $userModel->outlet_id;
             if ((int)$outlet_id > 0) {
                 
              
                 $modelMasOutlets            = new MasOutlets();
                 $modelMasOutlets_det        = $modelMasOutlets->findByPK($outlet_id);
                 $credit_limit_edit_salesman = $modelMasOutlets_det->credit_limit_edit_salesman;
                 $add_edit_customer          = $modelMasOutlets_det->add_edit_customer;
                 if ($credit_limit_edit_salesman == 'Y') {
                     $model->credit_limit = (float) $credit_limit;
                 }

                 if ($add_edit_customer == 'N') {
                     $message = ($scene == 'update') ? 'Editing for Customers Is Blocked' : 'Adding Customers Are Blocked';
                     $other[] = $message;
                     $valid   = false;
                 }
             }
             // ---------------------------------
             
             
            
         
            $model->outlet = $userModel->outlet_id;
            $model->accessed_by =  $userModel->id;
            $model->active = 'Y';
            
            /* $valid_currency = MasCurrency::model()->findBypk((int) $ledgerData['currency'])->id > 0 ? true : false;
            if ($valid_currency == false)
                $other[] = 'Invalid currency';
            $validationResponse['errors']['country'] = 'Invalid Country';
              $validationResponse['errors']['emirate'] = 'Invalid Emirate';
              $validationResponse['errors']['name'] = 'Name already exists';
              $validationResponse['errors']['credit_limit'] = 'Invalid credit limit';
              $validationResponse['errors']['payment_term'] = 'Invalid Credit days'; */
            
            $valid = $model->validate();
            if ($valid) {
                
                 
                  
                $model->save();
                
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $model->id));
                if (empty($subledger)) {
                    $newItem = new FinanceSubledger;
                } else {
                    $newItem = FinanceSubledger::model()->findByPk($subledger->id);
                }
                $newItem->description = $model->customer_name;
                $newItem->account_no = $model->customer_no;
                if ($model->address != '') {
                    $newItem->address = $model->address;
                }
                if ($model->phone != '') {
                    $newItem->phone = $model->phone;
                }
                $newItem->subledger_type_id = 1;
                $newItem->parent_id = $model->id;
                $newItem->created_by = $model->created_by;
                $newItem->outlet = $userModel->outlet_id;
                $newItem->save(false);
                /////////////////////////////////////////////////////////////
                if ($scene == 'new') {
                    $next_no = $model->customer_no + 1;
                    FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                }
                // inserting to audit log
                AuditLog::saveApiAuditLogs(
                        $userModel->outlet_id, $modelformmaster->id, $userModel->id, $change_type, " with ID - {$model->id} "
                );
                // end audit log
                  $this->responseBody['msg'] = 'Customer created successfully';
                $this->responseBody['status'] = 'success';
                $this->responseBody['data'] = array('ledger_id' => $model->id);
                $this->_bind;
            } else {
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                $error = $error1;
                $arr_err=$other;
                foreach ($error as $key => $errval) {
                    $arr_err[]=$errval[0];
                }
                $err_msg = implode(',', $arr_err);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                 $this->_bind;
            }
        } else {
            $this->_401;
        }
    }


}