<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiDayBookReportsController extends Controller {

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules() {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('Index'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters() {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request) {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401() {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400() {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403() {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind() {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value) {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions() {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null) {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name) {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }
  
    //Reports function start
    public function actionIndex() {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            //Outlet id
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = $variables['type'];
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;
                
            $conditionFormats = array(
                'sales' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Sales Invoices',
                    'mainDb' => $this->db_main,
                ),
                'sales_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Sales Returns',
                    'mainDb' => $this->db_main,
                ),
                'purchase' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Purchase Invoices',
                    'mainDb' => $this->db_main,
                ),
                'purchase_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Purchase Returns',
                    'mainDb' => $this->db_main,
                ),
                'net_sales' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Net Sales',
                    'mainDb' => $this->db_main,
                ),
                'net_purchase' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Net Purchase',
                    'mainDb' => $this->db_main,
                ),
                'customer' => array(
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Customers',
                    'mainDb' => $this->db_main,
                ),
                'vendor' => array(
                    'vendor_id' => (int) $variables['vendor_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Vendors',
                    'mainDb' => $this->db_main,
                ),
                'current_stock' => array(
                    'product_id' => (int) $variables['product_id'],
                    'product_group_id' => (int) $variables['product_group_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Current Stock',
                    'mainDb' => $this->db_main,
                    'access_type' => $userModel->access_type
                ),
                'transaction' => array(
                    'access_type' => $userModel->access_type,
                    'user_id' => $userModel->id,
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'product_id' => (int) $variables['product_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Transaction Details',
                    'mainDb' => $this->db_main,
                ),
                'customer_statement' => array(
                    'access_type' => $userModel->access_type,
                    'user_id' => $userModel->id,
                    'outlet_id' => $outlet,
                    'customer_id' => (int) $variables['customer_id'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Customer Statement',
                    'mainDb' => $this->db_main,
                ),
                'vendor_statement' => array(
                    'vendor_id' => (int) $variables['vendor_id'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Vendor Statement',
                    'mainDb' => $this->db_main,
                ),
                'receivable_aging' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Receivable Aging',
                    'mainDb' => $this->db_main,
                ),
                'payable_aging' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Payable Aging',
                    'mainDb' => $this->db_main,
                ),
                'day_book' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Day Book',
                    'mainDb' => $this->db_main,
                ),
                'stock_report' => array(
                    'access_type' => $userModel->access_type,
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Stock Ledger',
                    'mainDb' => $this->db_main,
                ),
                'vat_return_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Return',
                    'mainDb' => $this->db_main,
                ),
                'vat_summary_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Summary',
                    'mainDb' => $this->db_main,
                ),
                'consolidated_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Consolidated Report',
                    'mainDb' => $this->db_main,
                ),
                'daily_sales_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Daily Sales Report',
                    'mainDb' => $this->db_main,
                ),
                'cost_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'brand_id' => (int) $variables['brand_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cost Report',
                    'mainDb' => $this->db_main,
                    'access_type' => $userModel->access_type
                ),
                'cash_collection_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cash Collection Report',
                    'mainDb' => $this->db_main,
                ),
                'costofsale' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman' => (int)$salesman,
                    'outlet_id' => $outlet,
                    'brand' =>(int) $variables['brand'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cost of Sale',
                    'mainDb' => $this->db_main,
                    'access_type' => $userModel->access_type
                ),
                'brandwise' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman' => (int)$variables['salesman'],
                    'outlet_id' => $outlet,
                    'brand' =>(int) $variables['brand'],
                    'payment_term' =>(int) $variables['payment_term'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Brand wise report',
                    'mainDb' => $this->db_main,
                ),
                /*
                'tax_register' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Tax Register',
                    'mainDb' => $this->db_main,
                ),
                'vat_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Region wise',
                    'mainDb' => $this->db_main,
                ),
                'trial_balance' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Trial balance',
                    'mainDb' => $this->db_main,
                ),
                'balance_sheet' => array(
                    'to_date' => $variables['to_date'],
                    'company_id' => $userModel->company_id,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Balance Sheet',
                    'mainDb' => $this->db_main,
                ),
                'profit_and_loss' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Profit and Loss',
                    'mainDb' => $this->db_main,
                ), */
            );

            //SQL Execution
            $this->generate_view($report_type, $conditionFormats[$report_type]);

            //Negative symbol
            function getSymbol($value = 0) {
                return $value >= 0 ? '' : '(-) ';
            }

            //Space Addition
            function addSpace($level = 0) {
                $space = '';
                for ($i = 0; $i < $level * 4; $i++) {
                    $space .= '&nbsp;';
                }
                return $space;
            }

            function get_value_label($value = 0) {
                return $value > 0 ? ' Dr' : ' Cr';
            }

     
            //HTML Data Generation
            function generate_details($report_type, $conditionFormats) {

                $table = '';
                $params = $conditionFormats[$report_type];
//Start
                
       
$table = '<div class="table-responsive">';
$ledgerItems = array();

//        $ledger_id = (int) $_POST['ledger_id'];
//        $subledger = (int) $_POST['subledger_id'];
//        $brand = (int) $_POST['brand'];
//        $currency = 1;
//        $user = (int) $_POST['created_user'];
$cur=MasCurrency::model()->findByPk(1);
$exchange_rate=$cur->exchange_rate;
$currency_code=$cur->currency_code;
if(!empty($params['to_date']))
{
$from_date =  $params['to_date'];
$to_date =  $params['to_date'];
}
else {
$from_date = date('Y-m-d');
$to_date = date('Y-m-d');
}
$condition = 'parent.is_posted = "Y" AND parent.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';

//        if ($ledger_id > 0) {
//            $condition .= ' AND t.ledger_id IN(' . $ledger_id . ')';
//        }
//
//        if ($subledger > 0) {
//            $condition .= ' AND t.subledger_id=' . $subledger;
//        }
//
//        if ($brand > 0) {
//            $condition .= ' AND t.brand_id=' . $brand;
//        }

if ($outlet > 0) {
$condition .= ' AND t.outlet=' . $outlet;
}

if ($params['salesman_id'] > 0) {
$condition .= ' AND parent.created_by=' . $params['salesman_id'];
}

$items = FinanceVoucherItems::model()->with('parent','parent.vtype')->findAll(array('condition' => $condition, 'order' => 'parent.voucher_date DESC,vtype.code ASC,parent.voucher_no ASC'));
$previousBalance = $openingBalance;
$closingBalance = $openingBalance;
$symbol = ($previousBalance > 0) ? 'Dr' : 'Cr';

$total_debit = 0;
$total_credit = 0;
$discount_allowed=FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'); 
$table = '<table class="table table-hover table-fixed table-bordered" id="gl_table">
<thead>
<tr>
<th style="width:7%" >Salesman</th>
<th style="width:7%" >Date</th>
<th style="width:5%" >Type</th>
<th style="width:5%" >VO #</th>
<th style="width:10%" >Ledger</th>
<th style="width:20%" >Subledger</th>
<th style="width:10%" >PT</th>
<th style="width:20%" >Ref</th>
<th style="width:40%">Narration</th>
<th style="width:12%;text-align:right !important;" class="text-right">Value</th>
</tr>
</thead>
<tbody>';
$parent = 1;
$sales_total=0;
$purchase_total=0;
$creditnote_total=0;
$debitnote_total=0;
$receipt_total=0;
$payment_total=0;
$journal_total=0;
$discount_total=0;
$receipt_discount_total=0;
foreach ($items as $each) {

$debit = ($each->amount > 0) ? round(($each->amount * $each->exchange_rate), 4) : '';
$credit = ($each->amount < 0) ? round(abs($each->amount * $each->exchange_rate), 4) : '';

$debit= ($currency==$each->currency_id) ? $debit/$each->exchange_rate : $debit/$exchange_rate;
$credit= ($currency==$each->currency_id) ? $credit/$each->exchange_rate : $credit/$exchange_rate;

$running = $debit - $credit + $previousBalance;
$previousBalance = $running;
$symbol = ($running > 0) ? 'Dr' : 'Cr';
$closingBalance += ($debit - $credit);

$total_debit += $debit;
$total_credit += $credit;

$subledgerName = ($each->subledger_id > 0) ? $each->subledger->description : '';
$vouchreformat =  $each->parent->fullvoucher;
if($each->parent->vtype->code=="SI" && $debit>0)
{
if($each->parent->header_table_id>0)   
{
$SalesInvoice=SalesInvoice::model()->findByPk($each->parent->header_table_id);
switch($SalesInvoice->payment_term)
{
case 1:
$payTerms='Cash';
break;
case 2:
$payTerms='Credit';
if($SalesInvoice->customer->credit_days>0)
{
$payTerms.= ' - '.$SalesInvoice->customer->credit_days.' Days ';    
}
break;
case 3:
$payTerms='Credit 60 PDC';
break;
case 4:
$payTerms='Credit 90 PDC';
break;
case 5:
$payTerms='Credit 30 PDC';
break;
case 6:
$payTerms='Credit 15 PDC';
break;
}
}
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> SALES </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td>' . $payTerms . '</td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
</tr>';
$sales_total+=$debit;
}
else if($each->parent->vtype->code=="PI" && $credit>0)
{
if($each->parent->header_table_id>0)   
{
$PurchaseInvoice=PurchaseInvoice::model()->findByPk($each->parent->header_table_id);
switch($PurchaseInvoice->payment_term)
{
case 1:
$payTerms='Cash';
break;
case 2:
$payTerms='Credit';
break;
case 3:
$payTerms='Credit 60 PDC';
break;
case 4:
$payTerms='Credit 90 PDC';
break;
case 5:
$payTerms='Credit 30 PDC';
break;
case 6:
$payTerms='Credit 15 PDC';
break;
}
}
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> PURCHASE </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td>' . $payTerms . '</td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
</tr>';
$purchase_total+=$credit;
}
else if($each->parent->vtype->code=="CN" && $credit>0)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> CREDIT NOTE </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
</tr>';
$creditnote_total+=$credit;
}
else if($each->parent->vtype->code=="DN" && $debit>0)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> DEBIT NOTE </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
</tr>';
$debitnote_total+=$debit;
}
else if($each->parent->vtype->code=="PV" && $debit>0)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> PAYMENT </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
</tr>';
$payment_total+=$debit;
}
else if($each->parent->vtype->code=="CA" && $credit>0)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> RECEIPT </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
</tr>';
$receipt_total+=$credit;
}

else if($each->parent->vtype->code=="CA" && $each->ledger_id==$discount_allowed)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> DISCOUNT RECEIPT </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
</tr>';
$receipt_discount_total+=$debit;
}
else if($each->parent->vtype->code=="CHQ" && $credit>0)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> RECEIPT </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
</tr>';
$receipt_total+=$credit;
}
else if($each->parent->vtype->code=="DA" && $credit>0)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> DISCOUNT </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
</tr>';
$discount_total+=$credit;
}
else if($each->parent->vtype->code=="JV" && $debit>0)
{
$table .= ' <tr  data-id="' . $parent . '" data-parent="">
<td>' . $each->parent->created->full_name . '</td>
<td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
<td> JOURNAL </td>
<td>' . $vouchreformat . '</td>
<td>' . $each->ledger->description . '</td>
<td>' . $subledgerName . '</td>
<td></td>
<td>' . $each->parent->reference_no . '</td>
<td><small>' . $each->narration . '</small></td>
<td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
</tr>';
$journal_total+=$debit;
}
$parent = $child;
}
$running = $debit - $credit + $previousBalance;
$symbol = ($closingBalance > 0) ? 'Dr' : 'Cr';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL PURCHASE </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($purchase_total), " ") . '</b></td>
</tr>';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL DEBIT NOTE </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($debitnote_total), " ") . '</b></td>
</tr>';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL SALES </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($sales_total), " ") . '</b></td>
</tr>';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL CREDIT NOTE </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($creditnote_total), " ") . '</b></td>
</tr>';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL RECEIPT </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($receipt_total), " ") . '</b></td>
</tr>';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL PAYMENT </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($payment_total), " ") . '</b></td>
</tr>';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL JOURNAL </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($journal_total), " ") . '</b></td>
</tr>';
$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL SALES DISCOUNT </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($discount_total), " "). '</b></td>
</tr>';

$table .= '<tr>
<td colspan="9" style="text-align:right !important;"><b>TOTAL RECEIPT DISCOUNT </b></td>
<td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($receipt_discount_total), " "). '</b></td>
</tr>';

$table .= '</tbody>
</table>';
$table .= '</div>';
 

//End
return $table;
            
            }
            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.' . $conditionFormats[$report_type]['view'], array('table' => generate_details($report_type, $conditionFormats), 'title' => $conditionFormats[$report_type]['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $conditionFormats[$report_type], 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
 
    public function generate_view($view_type, $params)
    {

        /*$sales_voucher_id = 4;
        $creditnote = 5;
        $purchase_voucher_id = 1;
        $debitnote = 2;*/

        if ($view_type == 'sales') {

            $condition = 'V.invoice_date >= "'.$params['from_date'].'" AND V.invoice_date <= "'.$params['to_date'].'" ';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }
            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.sales_man=' . $params['salesman_id'];
            }
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }
            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_sales_invoice_list AS SELECT
                                                V.id,
                                                V.invoice_date AS date,
                                                V.invoice_no AS voucher_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                                O.name AS outlet_name,
						MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                                FROM erp_sales_invoice V
                                            LEFT JOIN erp_master_currency MC ON MC.id = V.currency
                                            LEFT JOIN erp_customer C ON C.id = V.customer_id
                                            LEFT JOIN ' . $this->db_main . '.erp_users U ON U.id = V.sales_man
                                            LEFT JOIN ' . $this->db_main . '.erp_outlets O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC')->execute();
        }
        else if ($view_type == 'sales_return') {

            $condition = 'V.return_date >= "' . $params['from_date'] . '" AND V.return_date <="' . $params['to_date'] . '"';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND SI.sales_man=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_creditnote_list AS SELECT
                                                V.id,
                                                V.return_date AS date,
                                                V.return_no AS voucher_no,
                                                SI.invoice_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                               	O.name AS outlet_name,
                                                MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                        FROM {{sales_return}} V
                                        LEFT JOIN {{sales_invoice}} SI ON SI.id = V.invoice_id
                                        LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                        LEFT JOIN {{customer}} C ON C.id = V.customer_id
                                        LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = SI.sales_man
                                        LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                        WHERE ' . $condition . ' ORDER BY V.return_date ASC,V.id ASC')->execute();
        } else if ($view_type == 'purchase') {
            //$params['vendor_id']=$params['ledger_id'];
            $condition = 'V.invoice_date >= "'.$params['from_date'].'" AND V.invoice_date <= "'.$params['to_date'].'" ';
            if ($params['vendor_id'] > 0) {
                $condition .= ' AND V.vendor_id=' . $params['vendor_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.created_by=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_purchase_invoice_list AS SELECT
                                            V.id,
                                            V.invoice_date AS date,
                                            V.invoice_no AS voucher_no,
                                            C.vendor_name AS vendor_name,
                                            U.full_name AS sales_man,
                                            O.name AS outlet_name,
                                            MC.currency_code AS currency,
                                            V.tax_total AS total_vat,
                                            V.grand_total,
                                            (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                            (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                            V.active
                                            FROM {{purchase_invoice}} V
                                            LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                            LEFT JOIN {{vendor}} C ON C.id = V.vendor_id                                                
                                            LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = V.created_by
                                            LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC ')->execute();
        } else if ($view_type == 'purchase_return') {
            $condition = 'V.return_date >= "' . $params['from_date'] . '" AND V.return_date <="' . $params['to_date'] . '"';
            if ($params['vendor_id'] > 0) {
                $condition .= ' AND V.vendor_id=' . $params['vendor_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND PI.created_by=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }            
            
            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_debitnote_list AS SELECT
                                                V.id,
                                                V.return_date AS date,
                                                V.return_no AS voucher_no,
                                                PI.invoice_no,
                                                C.vendor_name AS vendor_name,
                                                U.full_name AS sales_man,
                                               	O.name AS outlet_name,
                                                MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active                                                
                                        FROM {{purchase_return}} V
                                        LEFT JOIN {{purchase_invoice}} PI ON PI.id = V.invoice_id
                                        LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                        LEFT JOIN {{vendor}} C ON C.id = V.vendor_id
                                        LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = PI.created_by
                                        LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                        WHERE ' . $condition . ' ORDER BY V.return_date ASC,V.id ASC')->execute();
        }else if ($view_type == 'net_purchase') {

            /*Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_net_purchase_report AS (SELECT "I" as type,I.id,I.vendor_id,I.created_by,I.invoice_no,I.invoice_date,V.vendor_name,U.full_name,I.exchange_rate,I.total_amount,I.grand_total,I.tax_total,I.outlet,I.active
              FROM  {{purchase_invoice}} I
              LEFT JOIN  {{vendor}} V
              ON V.id=I.vendor_id
              LEFT JOIN   bluesell_new_main.erp_users U
              ON U.id=I.created_by
              )
              UNION ALL
              (SELECT "CN" as type,CN.id,I.vendor_id,I.created_by,CN.return_no AS invoice_no,CN.return_date AS invoice_date,V.vendor_name,U.full_name,I.exchange_rate,CN.total_amount,CN.grand_total ,CN.tax_total,CN.outlet,CN.active 
              FROM  {{purchase_return}} CN
              LEFT JOIN  {{vendor}} V
              ON V.id=CN.vendor_id
              LEFT JOIN {{purchase_invoice}} I
              ON I.id=CN.invoice_id
              LEFT JOIN   bluesell_new_main.erp_users U
              ON U.id=I.created_by)')->execute();*/
               
        } /* else if ($view_type == 'aging') {

            $ledger_id = (int) $params['ledger_id'];
            $outlet = (int) $params['outlet_id'];
            $to_date = $params['to_date'];
            $type = $params['type'] == 'payable' ? 'payable' : 'receivable';

            $dates_array = array(
                '0_to_30' => array('from' => date('Y-m-d', strtotime($to_date . ' -30 days')), 'to' => $to_date),
                '31_to_60' => array('from' => date('Y-m-d', strtotime($to_date . ' -60 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -31 days'))),
                '61_to_90' => array('from' => date('Y-m-d', strtotime($to_date . ' -90 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -61 days'))),
                '91_to_120' => array('from' => date('Y-m-d', strtotime($to_date . ' -120 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -91 days'))),
                'greaterthan_120' => array('from' => '2010-01-01', 'to' => date('Y-m-d', strtotime($to_date . ' -121 days'))),
            );

            foreach ($dates_array as $key => $each_date_pair) {

                $condition = 'V.date BETWEEN "' . $each_date_pair['from'] . '" AND "' . $each_date_pair['to'] . '" AND V.is_posted="Y"';
                if ($ledger_id > 0) {
                    $condition .= ' AND T.ledger_id = ' . (int) $ledger;
                }

                if ($outlet > 0) {
                    $condition .= ' AND L.outlet_id=' . (int) $outlet;
                }

                if ($type == 'receivable') {
                    $condition .= '  AND L.classification="' . VanLedgers::CUSTOMER . '"';
                } else {
                    $condition .= '  AND L.classification="' . VanLedgers::VENDOR . '"';
                }

                Yii::app()->db->createCommand('CREATE OR REPLACE VIEW {{ageing_' . $key . '}} AS SELECT
                                                    L.id AS ledger_id,
                                                    L.name AS ledger_name,
                                                    IFNULL(ROUND(SUM(T.amount*T.exchange_rate), 6), 0) AS voucher_value
                                                FROM {{transactions}} T
                                                LEFT JOIN {{vouchers}} V ON V.id = T.voucher_id
                                                LEFT JOIN {{ledgers}} L ON L.id = T.ledger_id
                                                LEFT JOIN {{allocation}} A ON A.child_voucher_id = V.id
                                                WHERE ' . $condition . '
                                                GROUP BY L.id')->execute();
            }
        } else if ($view_type == 'trial_balance') {

            $condition = 'V.date  BETWEEN "' . $params['from_date'] . '" AND "' . $params['to_date'] . '" AND  V.is_posted="Y"';
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet_id=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW trial_balance AS
                                            SELECT
                                                L.outlet_id,
                                                L.is_expandable,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND((SUM(CASE WHEN TR.amount > 0 THEN TR.amount*TR.exchange_rate ELSE 0 END)), 6) AS debit,
                                                ABS(ROUND((SUM(CASE WHEN TR.amount < 0 THEN TR.amount*TR.exchange_rate ELSE 0 END)), 6)) AS credit,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        } else if ($view_type == 'balance_sheet') {

            $condition = 'V.date <= "' . $params['to_date'] . '" AND L.type IN ("A", "L") AND V.is_posted="Y"';
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet_id=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW balance_sheet AS
                                            SELECT
                                                L.outlet_id,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        } else if ($view_type == 'profit_and_loss') {

            $from_date = $params['from_date'];
            $to_date = $params['to_date'];
            $outlet = $params['outlet_id'];

            $condition = 'V.date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND L.type IN ("I", "E") AND V.is_posted="Y"';
            if ($outlet > 0) {
                $condition .= ' AND V.outlet_id=' . $outlet;
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW profit_and_loss AS
                                            SELECT
                                                L.outlet_id,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        }*/
    }
}