<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiGetReportDataController extends Controller {

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules() {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('StockReport','StockLedgerReport'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters() {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request) {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401() {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400() {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403() {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind() {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value) {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions() {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null) {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name) {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }
 
    
    
   

    public function actionStockReport() {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true); 
        $from_date = trim($variables['from_date']);
        $to_date = trim($variables['to_date']);
        $salesman = trim($variables['salesman']);
        if ($userModel->id > 0) {
        $userModel->db();
            
        
            
        $condition      = '1=1 ';
        if($userModel->outlet_id>0)
        {
        $condition.=' AND I.outlet='.$userModel->outlet_id;    
        } 
        $salesman=$userModel->id;
        if($salesman >0)
        {
        $condition.=' AND (S.created_by='.$salesman.' OR S.type="OB" )';    
        }
//        else if($userModel->user_access_type == 'SL')
//        {
//        $condition.=' AND (S.created_by='.Yii::app()->user->id.' OR S.type="OB" )';    
//        }
        
        $stockcondition ='1=1';
        $SQL="CREATE OR REPLACE VIEW erp_inventory_list AS
        (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,SUM(S.quantity*S.unit_exchange_rate) AS in_qty,
        0 AS out_qty,U.unit_name,CASE
        WHEN SUM(S.quantity*S.unit_exchange_rate) > 0 THEN SUM(S.quantity*S.rate)/SUM(S.quantity*S.unit_exchange_rate) ELSE 0 END AS avg_purchase_rate,0 AS avg_sales_rate,tax,O.name,O.outlet_name,I.rate 
        FROM erp_inventory I 
        LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
        LEFT JOIN erp_master_brand B ON I.brand=B.id 
        LEFT JOIN erp_master_unit U ON I.unit=U.id 
        LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='IN' OR S.type='OB') group by I.id)
        UNION
        (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,0 AS in_qty,SUM(S.quantity*S.unit_exchange_rate) AS out_qty,
        U.unit_name,0 AS avg_purchase_rate,CASE
        WHEN SUM(S.quantity*S.unit_exchange_rate) > 0 THEN SUM(S.quantity*S.rate)/SUM(S.quantity*S.unit_exchange_rate) ELSE 0 END AS avg_sales_rate,0 AS tax,O.name,O.outlet_name,I.rate 
        FROM erp_inventory I 
        LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
        LEFT JOIN erp_master_brand B ON I.brand=B.id 
        LEFT JOIN erp_master_unit U ON I.unit=U.id 
        LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='OUT' OR S.type='OB')  group by I.id)";
        Yii::app()->db->createCommand($SQL)->execute();
        $model=Yii::app()->db->createCommand("SELECT * FROM(SELECT id,part_number,description,active,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,
        unit_name,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,SUM(tax) AS tax,name,outlet_name,rate from erp_inventory_list group by id )A WHERE ".$stockcondition)->queryAll();
        $counts=Yii::app()->db->createCommand("SELECT * FROM(SELECT id,part_number,description,active,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,
        unit_name,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,SUM(tax) AS tax,name,outlet_name,rate from erp_inventory_list group by id )A WHERE ".$stockcondition)->queryAll();
        $count          = count($counts);
        $rowData                = array();
        $recordsTotal   = $count;
        $recordsFiltered        = $count;
        $data['draw']   = $draw;
        $data['recordsTotal']   = $recordsTotal;
        $data['recordsFiltered']= $recordsFiltered;
        $rowData                = array();
        $z=0;
        $this->responseBody['data']['company_name'] =$userModel->outletDetails->name;
        $this->responseBody['data']['company_adds'] = str_replace('<br>', ' ', $userModel->outletDetails->address);
        $this->responseBody['data']['company_phone'] = $userModel->outletDetails->phone_no;
        $this->responseBody['data']['company_po_box'] = $userModel->outletDetails->po_box;
        $this->responseBody['data']['company_trn'] = $userModel->outletDetails->trn_no;
        $this->responseBody['data']['outlet'] = $userModel->outletDetails->name;
        $this->responseBody['data']['user'] = $userModel->full_name;
        $this->responseBody['data']['user_phone'] = $userModel->phone_no;
        $this->responseBody['data']['create_date_time'] = date('d/m/Y');
        
        foreach ($model as $i=>$row)
        {
            
            $inv_type=($row['type']=='I')? "Inventory" : "Service";
            $avg_purchase_rate = StockLedger::getAvgPurchaseRate($row['id'],'',$userModel->outlet_id);
            $count=strlen($row['description']);
            $description=$row['description'];
            if($count<17)
            {
            switch($count)
            {
                case 1:
                $description.="________________";
                break;
                case 2:
                $description.="_______________";
                break;
                case 3:
                $description.="______________";
                break;
                case 4:
                $description.="_____________";
                break;
                case 5:
                $description.="____________";
                break;
                case 6:
                $description.="___________";
                break;
                case 7:
                $description.="__________";
                break;
                case 8:
                $description.="_________";
                break;
                case 9:
                $description.="________";
                break;
                case 10:
                $description.="_______";
                break;
                case 11:
                $description.="______";
                break;
                case 12:
                $description.="_____";
                break;
                case 13:
                $description.="____";
                break;
                case 14:
                $description.="___";
                break;
                case 15:
                $description.="__";
                break;
                case 16:
                $description.="_";
                break;
            }
            }
            if($this->format_currency($row['qty'])>0)
            {
             $this->responseBody['data']['items'][] = array(
                            'part_number' => $row['part_number'],
                            'description' => substr($description, 0, 17),
                            'type' => $inv_type,
                            'brand' => $row['brand_name'],
                            'quantity' => $this->format_currency($row['qty']),
                            'uom' => $row['unit_name'],
                            'cost' => $this->format_currency($avg_purchase_rate),
                            'rate' => $this->format_currency($row['rate'])
                        );
            }
             $z++;
        } 
                    $this->responseBody['count'] = $z; 
                    $this->_bind;
                
        } else {
            $this->_401;
        }
    }


    public function actionStockLedgerReport() {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true); 
        $from_date = trim($variables['from_date']);
        $to_date = trim($variables['to_date']);
        $salesman = trim($variables['salesman']);
        if ($userModel->id > 0) {
        $userModel->db();
            
        
        $condition      = '1=1 '; 
        $outlet=$userModel->outlet_id;
       $model  = new Inventory();
            $condition      = '1=1';
            if($outlet>0){
                $condition.=' AND I.outlet='.$outlet;
            } 
            if($brand>0)
            {
                $condition.=' AND I.brand='.$brand;    
            }
            if($customer_id>0)
            {
                $condition.=' AND t.customer_id='.$customer_id;    
            }
//            if($salesman >0)
//            {
//                $condition.=' AND (S.created_by='.$salesman.' OR S.type="OB" )'; 
//            }
        if($userModel->outlet_id>0)
        {
        $condition.=' AND I.outlet='.$userModel->outlet_id;    
        }    
        if($salesman >0)
        {
        $condition.=' AND (S.created_by='.$salesman.' OR S.type="OB" )';    
        }
//        else if($userModel->user_access_type == 'SL')
//        {
//        $condition.=' AND (S.created_by='.Yii::app()->user->id.' OR S.type="OB" )';    
//        }
        
            if(!empty($from_date))
            {
            $from_date = date("Y-m-d", strtotime($from_date));
            }
            else {
            $from_date = date("Y-m-d");    
            }
            if(!empty($to_date))
            {
            $to_date = date("Y-m-d", strtotime($to_date));
            }
            else {
            $to_date = date("Y-m-d");    
            }
            $query_current_stock="SELECT id,part_number,description,active,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,
                    unit_name,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,tax,name,outlet_name,rate from view_current_stock  where transaction_date>='".$from_date."' AND  transaction_date<='".$to_date."'  group by id";

            $SQL="CREATE OR REPLACE VIEW view_current_stock AS
                (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,S.quantity*S.unit_exchange_rate AS in_qty,
                0 AS out_qty,U.unit_name,0 AS avg_purchase_rate,0 AS avg_sales_rate,tax,
                O.name,O.outlet_name,I.rate,S.transaction_date
                FROM erp_inventory I 
                LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                LEFT JOIN erp_master_brand B ON I.brand=B.id 
                LEFT JOIN erp_master_unit U ON I.unit=U.id 
                LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='IN' OR S.type='OB'))
                UNION
                (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,0 AS in_qty,S.quantity*S.unit_exchange_rate AS out_qty,
                U.unit_name,0 AS avg_purchase_rate,0 AS avg_sales_rate,tax,O.name,O.outlet_name,I.rate,S.transaction_date
                FROM erp_inventory I 
                LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                LEFT JOIN erp_master_brand B ON I.brand=B.id 
                LEFT JOIN erp_master_unit U ON I.unit=U.id 
                LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='OUT' OR S.type='OB'))";
            
                Yii::app()->db->createCommand($SQL)->execute();
            $arr_stock = Yii::app()->db->createCommand($query_current_stock)->queryAll();
        $z=0;
        $this->responseBody['data']['company_name'] =$userModel->outletDetails->name;
        $this->responseBody['data']['company_adds'] = str_replace('<br>', ' ', $userModel->outletDetails->address);
        $this->responseBody['data']['company_phone'] = $userModel->outletDetails->phone_no;
        $this->responseBody['data']['company_po_box'] = $userModel->outletDetails->po_box;
        $this->responseBody['data']['company_trn'] = $userModel->outletDetails->trn_no;
        $this->responseBody['data']['outlet'] = $userModel->outletDetails->name;
        $this->responseBody['data']['user'] = $userModel->full_name;
        $this->responseBody['data']['user_phone'] = $userModel->phone_no;
        $this->responseBody['data']['create_date_time'] = date('d/m/Y');
        
                foreach ($arr_stock as $row)
                { 
                    $inv_type=($row['type']=='I')? "Inventory" : "Service";
//                    $avg_purchase_rate = StockLedger::getAvgPurchaseRate($row['id'],'',$outlet);
                    $avg_purchase_rate = 0;
                    $product_id=$row['id'];
                    
                    $open_in_qty=Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from erp_stock_ledger where product_id=$product_id AND transaction_type='IN' AND transaction_date<'".$from_date."'")->queryScalar();
                    $open_out_qty=Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from erp_stock_ledger where product_id=$product_id AND transaction_type='OUT' AND transaction_date<'".$from_date."'")->queryScalar();
                    if(($open_in_qty-$open_out_qty)>0 || $row['in_qty']>0 || $row['out_qty']>0)
                    {
                     $this->responseBody['data']['items'][] = array(
                            'part_number' => $row['part_number'],
//                            'description' => substr($row['description'], 0, 8),
                            'description' => $row['part_number'],
                            'type' => $inv_type,
                            'brand' => $row['brand_name'],
                            'opening_quantity' => round(($open_in_qty-$open_out_qty),3),
                            'in_quantity' => round(($row['in_qty']),3),
                            'out_quantity' => round(($row['out_qty']),3),
                            'total_quantity' => round((($open_in_qty-$open_out_qty+$row['qty'])),3),
                            'uom' => $row['unit_name'],
                            'cost' => $this->format_currency($avg_purchase_rate),
                            'rate' => $this->format_currency($row['rate'])
                        );
                    $z++;
                    }
                } 
                    $this->responseBody['count'] = $z;
                    $this->_bind;
                 
        } else {
            $this->_401;
        }
    }

}