<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiGoodsIssueController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array("GetList", "Save", "GetDetailssById", "Cancel", "Print"),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }


    public function actionGetList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $from = trim($variables['from']);
        $to = trim($variables['to']);
        $term = trim($variables['term']);

        $from = $this->convert_date($from);
        $to = $this->convert_date($to);
        if ($userModel->id > 0) {

            $userModel->db();

            $count = 0;

            if ($userModel->access_type == 'DEV' || $userModel->access_type == 'SU' || $userModel->access_type == 'CA' || $userModel->access_type == 'SM' || $userModel->access_type == 'OM') {
                $issue_cnd = '1=1';
            } else {
                $issue_cnd = 'sales_man=' . $userModel->id;
            }

            if ($from != '' && $from != '00/00/0000') {

                $issue_cnd .= ' AND t.issue_date>="' . $from . '"';
            }
            if ($to != '' && $to != '00/00/0000') {

                $issue_cnd .= ' AND t.issue_date<="' . $to . '"';
            }

            if ($outlet > 0) {
                $issue_cnd .= " AND t.outlet=" . $outlet;
            }

            if (isset($term) && $term != "") {
                $searchString = $term;
                $issue_cnd .= " AND (t.issue_no LIKE '%" . $searchString . "%' OR t.reference LIKE '%" . $searchString . "%'  OR t.notes LIKE '%" . $searchString . "%' )";
            }

            $model = GoodsIssue::model()->findAll(array('condition' => $issue_cnd, 'order' => 'issue_date DESC, issue_no DESC'));
            foreach ($model as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['voucher_no'] = $each->issue_no;
                $this->responseBody['data'][$i]['date'] = $each->issue_date;
                $this->responseBody['data'][$i]['narration'] = $each->reference;
                $this->responseBody['data'][$i]['notes'] = $each->notes;
                $this->responseBody['data'][$i]['is_posted'] = $each->posted == 'Y' ? true : false;
                $this->responseBody['data'][$i]['is_cancelable'] = $each->active == 'Y' ? true : false;
                $this->responseBody['data'][$i]['is_cancelled'] = $each->active == 'Y' ? true : false;
                /* $this->responseBody['data'][$i]['is_cancelled'] = $each->active == 'Y' ? false : true;
                      $this->responseBody['data'][$i]['is_audited'] = $each->is_audited == 'Y' ? true : false;
                      $this->responseBody['data'][$i]['is_cancelled'] = $each->is_canceled == 'Y' ? true : false; */
                $count++;
            }

            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }




    public function actionGetDetailssByIdpos()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $voucher_id = (int) trim($variables['id']);
        if ($userModel->id > 0) {

            $userModel->db();


            if ($voucher_id > 0) {

                $id = $voucher_id;
                $voucherModel = GoodsIssue::model()->findbypk($id);
                if (!empty($voucherModel)) {
                    $this->responseBody['data']['voucher_type'] = 'Goods Issue';
                    $this->responseBody['data']['voucher_no'] = $voucherModel->issue_no;
                    $this->responseBody['data']['parent_voucher_no'] = '';
                    $this->responseBody['data']['voucher_date'] = date('d/m/Y', strtotime($voucherModel->issue_date));
                    //$this->responseBody['data']['currency'] = $voucherModel->purchase_cur->currency_code;
                    $this->responseBody['data']['company_name'] = $voucherModel->gi_outlet->name;
                    $this->responseBody['data']['company_adds'] = str_replace('<br>', ' ', $voucherModel->gi_outlet->address);
                    $this->responseBody['data']['company_phone'] = $voucherModel->gi_outlet->phone_no;
                    $this->responseBody['data']['company_po_box'] = $voucherModel->gi_outlet->po_box;
                    $this->responseBody['data']['company_trn'] = $voucherModel->gi_outlet->trn_no;
                    $this->responseBody['data']['outlet'] = $voucherModel->gi_outlet->name;
                    $this->responseBody['data']['user'] = $voucherModel->gi_salesman->full_name;
                    $this->responseBody['data']['user_phone'] = $voucherModel->gi_salesman->phone_no;
                    $this->responseBody['data']['create_date_time'] = date('d/m/Y', strtotime($voucherModel->issue_date));
                    $this->responseBody['data']['notes'] = $voucherModel->notes;
                    $this->responseBody['data']['party'] = '';
                    $this->responseBody['data']['reference_no'] = $voucherModel->reference;
                    $this->responseBody['data']['total_vat'] = '';
                    $this->responseBody['data']['total'] = '';
                    $this->responseBody['data']['grand_total'] = '';
                    //Items
                    $modelItems = GoodsIssueItems::model()->findAll(array('condition' => 'issue_id="' . $id . '"', 'order' => 'id ASC'));
                    foreach ($modelItems as $each) {

                        $this->responseBody['data']['items'][] = array(
                            'part_number' => $each->gi_product_details->part_number,
                            'description' => $each->gi_product_details->description,
                            'quantity' => $this->format_currency($each->quantity),
                            'uom' => $each->gi_unit_details->label,
                            'unit_label' => $each->gi_unit_details->unit_name,
                            'rate' => '',
                            'discount' => '0.00',
                            'vat' => '',
                            'net' => '',
                        );
                    }
                    //Items
                    $this->responseBody['count'] = 1;
                    $this->_bind;
                } else {
                    $this->responseBody['status'] = 'failed';
                    $this->responseBody['msg'] = 'Invalid Goods Issue';
                    $this->_bind;
                }
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Goods Issue';
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }



    public function actionSave()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $issue_date = trim($variables['issue_date']);
        $id = trim($variables['id']);
        $sales_man = trim($variables['salesman']); //new
        //$expense_account = trim($variables['expense_account']);
        $issueItems = $variables['items'];

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        date_default_timezone_set("Asia/Dubai");
        $valid = true;
        $groups = $other = array();
        if ($userModel->id > 0) {

            $userModel->db();

            $model = new GoodsIssue;
            $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="GI"'));
            if ($id > 0) {
                $model = $model->findBypk($id);
                $scene = 'update';
                $change_type = 'UPDATED';
            } else {
                $model->created_by = $userModel->id;
                $model->created_time = date('Y-m-d H:i:s');
                $scene = 'new';
                $change_type = 'CREATED';
                if ($scene == 'new') {
                    $model->issue_no = $modelformmaster->next_number;
                }
            }
            $model->callback_from = 'app';
            $model->updated_by = $userModel->id;
            $model->updated_time = date('Y-m-d H:i:s');
            $model->outlet = $userModel->outlet_id;
            $model->issue_date = count(explode('-', $issue_date)) == 3 ? $issue_date : date('Y-m-d');
            $model->reference = trim($variables['reference']);
            $model->notes = trim($variables['notes']);
            $model->posted = "N";
            $model->active = "Y";
            $model->sales_man = ($sales_man > 0) ? $sales_man : $userModel->id;

            $i = 0;
            $product_ids = array();
            $items = array();
            $valid = $model->validate();
            $j = 0;
            foreach ($issueItems as $each) {
                if ($each['product_id'] > 0) {
                    $modelItems[$j] = new GoodsIssueItems();
                    $productModel = Inventory::model()->findBypk($each['product_id']);
                    $modelItems[$j]->product_id = $each['product_id'];
                    $modelItems[$j]->description = $productModel['description'];
                    $modelItems[$j]->quantity = (float) $each['quantity'];
                    $modelItems[$j]->unit = $each['unit'];
                    $modelItems[$j]->issue_id = 0;
                    $valid = $valid && $modelItems[$j]->validate();
                    $j++;
                    // Product Grouping
                    if ($each['unit'] > 0) {
                        $unit = MasUnit::model()->findByPk($each['unit']);
                        $unit_exchange = $unit->unit_exchange_rate;
                    } else {
                        $unit = MasUnit::model()->findByPk($productModel->unit);
                        $unit_exchange = $unit->unit_exchange_rate;
                    }
                    $key = $each['product_id'];
                    if (!array_key_exists($key, $groups)) {
                        $groups[$key] = array(
                            'id' => $each['product_id'],
                            'quantity' => $each['quantity'] * $unit_exchange,
                        );
                    } else {
                        $groups[$key]['quantity'] = $groups[$key]['quantity'] + $each['quantity'] * $unit_exchange;
                    }
                }
            }
            if (!empty($groups)) {
                foreach ($groups as $key => $eachProduct) {
                    $product = Inventory::model()->findByPk($key);
                    if ($product->type != 'S') {
                        $qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*U.unit_exchange_rate),0) from {{goods_issue_items}} GI INNER JOIN {{master_unit}} U ON GI.unit=U.id where issue_id='" . $model->id . "' AND product_id='" . $key . "'")->queryScalar();
                        $bal = $eachProduct['quantity'] - $qty;
                        $in_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='IN' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $model->sales_man)->queryScalar();
                        $out_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='OUT' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $model->sales_man)->queryScalar();
                        $in_qty_on_date = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_date <='" . $model->issue_date . "' AND transaction_type='IN' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $model->sales_man)->queryScalar();
                        $out_qty_on_date = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_date <='" . $model->issue_date . "' AND transaction_type='OUT' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $model->sales_man)->queryScalar();
                        $current_stock = $in_qty - $out_qty;
                        $stock_on_date = $in_qty_on_date - $out_qty_on_date;
                        if ((($current_stock + $qty) < $eachProduct['quantity']) || (($stock_on_date + $qty) < $eachProduct['quantity'])) {
                            $available_stock = (($current_stock > $stock_on_date) ? $stock_on_date : $current_stock) + $qty;
                            $other[] = $product->part_number . ' maximum sellable qty : ' . ($available_stock) . ' ' . $product->units->unit_name;
                            $valid = false;
                        }
                    }
                }
            }

            $closing_date = MasClosing::getClosingDate();
            if ($closing_date >= $model->issue_date) {
                $other[] = 'Date should be greater than closing date (' . date('d-m-Y', strtotime($closing_date)) . ')';
                $valid = false;
            }
            if ($valid && ($modelItems[0]->product_id > 0)) {
                $model->save();
                GoodsIssueItems::model()->deleteAll('issue_id =' . $model->id);
                StockLedger::model()->deleteAll('header_id=' . $model->id . ' AND  transaction_type="OUT" AND type= "GI" AND outlet=' . $userModel->outlet_id);
                if ($scene == 'new') {
                    $next_no = $model->issue_no + 1;
                    FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                }
                foreach ($modelItems as $each) {
                    $each->issue_id = $model->id;
                    $each->save();
                    $model_stockledger = new StockLedger();
                    $model_stockledger->transaction_date = date('Y-m-d H:i:s');
                    $model_stockledger->transaction_type = 'OUT';
                    $model_stockledger->type = 'GI';
                    $model_stockledger->customer_vendor_id = 0;
                    $model_stockledger->header_id = $model->id;
                    $model_stockledger->item_id = $each->id;
                    $model_stockledger->product_id = $each->product_id;
                    $model_stockledger->currency = 1;
                    $model_stockledger->exchange_rate = 1;
                    $model_stockledger->quantity = $each->quantity;
                    $model_stockledger->unit = $each->unit;
                    $model_stockledger->default_product_unit = $each->gi_unit_details->base_unit_id;
                    $model_stockledger->unit_exchange_rate = $each->gi_unit_details->unit_exchange_rate;
                    $model_stockledger->created_by = $model->sales_man;
                    $model_stockledger->created_time = date('Y-m-d H:i:s');
                    $model_stockledger->outlet = $userModel->outlet_id;
                    $model_stockledger->save(false);
                }

                // inserting to audit log
                AuditLog::saveApiAuditLogs(
                    $userModel->outlet_id,
                    $modelformmaster->id,
                    $userModel->id,
                    $change_type,
                    " with ID - {$model->id} "
                );
                // end audit log
                $this->responseBody['status'] = 'success';
                $this->responseBody['voucher_id'] = $model->id;
                $this->responseBody['type'] = 'goodsissue';
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $error1 = $error2 = array();
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                if ($modelItems[0]->product_id > 0)
                    $error2 = json_decode(CActiveForm::validateTabular($modelItems, '', false), true);
                else
                    $error2[] = "No Products";
                $error = array_merge_recursive($error1, $error2);
                $arr_err = $other;
                foreach ($error as $key => $errval) {
                    $arr_err[] = $errval[0];
                }
                $err_msg = implode(',', $arr_err);
                $this->responseBody['msg'] = $err_msg;
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }


    public function actionGetDetailssById()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $id = (int) trim($variables['id']);

        if ($userModel->id > 0) {

            $userModel->db();

            if ($id > 0) {

                $model = GoodsIssue::model()->findBypk((int) $id);
                $this->responseBody['data']['id'] = $model->id;
                $this->responseBody['data']['issue_no'] = $model->issue_no;
                $this->responseBody['data']['issue_date'] = $model->issue_date;
                $this->responseBody['data']['reference'] = $model->reference;
                $this->responseBody['data']['notes'] = $model->notes;
                $this->responseBody['data']['outlet'] = $model->outlet;
                $this->responseBody['data']['salesman'] = $model->sales_man;
                $modelItems = GoodsIssueItems::model()->findAll(array('condition' => 'issue_id="' . $id . '"', 'order' => 'id'));
                foreach ($modelItems as $i => $each) {
                    $this->responseBody['data']['items'][$i]['id'] = $each->id;
                    $this->responseBody['data']['items'][$i]['issue_id'] = $each->issue_id;
                    $this->responseBody['data']['items'][$i]['product_id'] = $each->product_id;
                    $this->responseBody['data']['items'][$i]['product_name'] = $each->gi_product_details->part_number;
                    $this->responseBody['data']['items'][$i]['goods_service'] = $each->gi_product_details->type;
                    $this->responseBody['data']['items'][$i]['description'] = $each->gi_product_details->description;
                    $this->responseBody['data']['items'][$i]['quantity'] = $each->quantity;
                    $this->responseBody['data']['items'][$i]['uom'] = $each->unit;
                    $this->responseBody['data']['items'][$i]['unit_label'] = $each->gi_unit_details->unit_name;
                }

                $this->responseBody['count'] = 1;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Invoice id';
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }



    public function actionCancel()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $voucher_id = (int) trim($variables['id']);

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {

            $userModel->db();

            $id = $voucher_id;
            $valid = true;
            $model = GoodsIssue::model()->findByPk($id);
            if (!empty($model)) {
                $closing_date = MasClosing::getClosingDate();
                if ($closing_date >= $model->issue_date) {
                    $other[] = 'Date should be greater than closing date (' . date('d-m-Y', strtotime($closing_date)) . ')';
                    $valid = false;
                }
                if ($valid) {
                    StockLedger::model()->deleteAll('header_id=' . $model->id . ' AND  transaction_type="OUT" AND type= "GI" AND outlet=' . $userModel->outlet_id);
                    $model->active = 'N';
                    $model->save(false);
                    $this->responseBody['status'] = 'success';
                    $this->responseBody['msg'] = 'Goods Issue Cancelled';
                    $this->_bind;
                } else {
                    $err_msg = implode(',', $other);
                    $this->responseBody['msg'] = $err_msg;
                    $this->responseBody['status'] = 'failed';
                    $this->responseBody['items'] = array();
                    $this->_bind;
                }
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Issue Not Found';
                $this->_bind;
                exit;
            }
        } else {
            $this->_401;
        }
    }




    public function actionPrint()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);

            $model = GoodsIssue::model()->findByPk($id);
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = 'GR_' . $model->issue_no;

            if ($model->active == 'N') {
                $mPDF1->SetWatermarkText('CANCELLED');
                $mPDF1->showWatermarkText = true;
            }
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.goods_issue_print', array('id' => $id, 'user_id' => $userModel->id), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
}
