<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiInventoryController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('SaveInventory', 'InventoryList'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }

    //changed



    public function actionSaveInventory()
    {

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $part_number = trim($variables['part_number']);
        $description = trim($variables['description']);
        $type        = trim($variables['type']);
        $brand       = (int)$variables['brand_id'];
        $generic_id  = (int)$variables['generic_id'];
        $unit        = (int)$variables['unit_id'];
        $rate        = (float)$variables['rate'];
        $tax_code    = trim($variables['tax_code_id']);
        $excise_tax  = trim($variables['excise_tax']);
        $barcode     = trim($variables['barcode']);
        $sec_unit_id = (int)$variables['sec_unit_id'];
        $id          = (int) trim($variables['id']);

        if ($userModel->id > 0) {
            $userModel->db();
            $model = new Inventory();
            if ($id > 0) {
                $model = $model->findBypk($id);
                $model->updated_time = date('Y-m-d h:i:s');
                $scene = 'update';
            } else {
                $model->created_by = $userModel->id;
                $model->created_time = date('Y-m-d h:i:s');
                $scene = 'new';
            }

            $model->part_number = $part_number;
            $model->description = $description;
            $model->type        = $type;
            $model->brand       = $brand;
            $model->generic_id  = $generic_id;
            $model->unit        = $unit;
            $model->rate        = $rate;
            $model->tax_code    = $tax_code;
            $model->excise_tax  = $excise_tax;
            $model->barcode     = $barcode;
            $model->sec_unit_id = $sec_unit_id;
            if ($model->barcode == '') {
                $model->barcode = $model->part_number;
            }
            $model->outlet = $userModel->outlet_id;
            $valid = $model->validate();
            if ($valid) {
                $model->tax = MasTax::model()->findByPk($model->tax_code)->tax_percentage;
                $model->save();
                StockLedger::model()->deleteAll(array('condition' => 'product_id=' . $model->id . ' AND type="OB" AND transaction_type="IN" AND outlet=' . $userModel->outlet_id));
                $stockledger = new StockLedger;
                $stockledger->transaction_date = date('Y-m-d');
                $stockledger->transaction_type = 'IN';
                $stockledger->type = 'OB';
                $stockledger->customer_vendor_id = 0;
                $stockledger->header_id = 0;
                $stockledger->item_id = 0;
                $stockledger->product_id = $model->id;
                $stockledger->rate = 0;
                $stockledger->currency = 1;
                $stockledger->exchange_rate = 1;
                $stockledger->quantity = 0;
                $stockledger->unit = $model->unit;
                $stockledger->default_product_unit = $model->units->base_unit_id;
                $stockledger->unit_exchange_rate = $model->units->unit_exchange_rate;
                $stockledger->outlet = $userModel->outlet_id;
                $stockledger->created_by = Yii::app()->user->id;
                $stockledger->created_time = date('Y-m-d H:i:s');
                $stockledger->save(false);
                $this->responseBody['inventory_id'] = $model->id;
                $this->_bind;
            } else {

                $this->responseBody['status'] = 'failed';
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);

                $error = array_merge_recursive($error1);
                $arr_err = $other;
                foreach ($error as $key => $errval) {
                    $arr_err[] = $errval[0];
                }
                $err_msg = implode(',', $arr_err);

                $this->responseBody['msg'] = $err_msg;
                $this->_bind;
             
            }
        } else {
            $this->_401;
        }
    }





    public function actionInventoryList()
    {
       

        $input = file_get_contents('php://input');
        //error_log('ledgers');
        //error_log($input);
        $variables = json_decode($input, true);
        // $party = (int) $variables['party'];
        // $salesman = (int) $variables['salesman'];
        $customer_id  = (int) $variables['customer_id'];
        $vendor_id  = (int) $variables['vendor_id'];
        $type =  trim($variables['type']);
        $term_type =  trim($variables['term_type']); 
        $term =  trim($variables['term']);
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $items = array();
        $userModel = MasUsers::model()->findBytoken($token);

        //Common or all
        $count = 0;
        $items = array();
        $factor = 1;

        if ($userModel->id > 0) {

            $userModel->db();
            
            
            $sales_price_type = 0;
            if ((int)$customer_id > 0) {
                $sales_price_type = Customer::model()->findByPk($customer_id)->sales_price_type;
            }
        

            $condition = 'I.active ="Y"';
            $condition .= ' AND I.outlet=' . $userModel->outlet_id;
            if ($type != "") {
                $condition .= ' AND (I.type = "' . $type . '")';
            }
            if ($term != "") {
                if($term_type=='B'){
                     $condition .= ' AND (I.barcode LIKE "%' . $term . '%")';
                }else{
                    $condition .= ' AND (I.part_number LIKE "%' . $term . '%" OR I.description LIKE "%' . $term . '%")';
                }
            }
            
               
            // if($salesman>0)
            // {
            // $condition .= ' AND (S.created_by = ' . $salesman . ' OR S.type="OB" )';    
            // }
            // else 
            // {
             // $condition .= ' AND (S.created_by = ' . $userModel->id . ' OR S.type="OB" )';
            //}
             
             
            $condition .= ' AND (S.created_by = ' . $userModel->id . ' OR S.type="OB" )';

        
            
            $SQL = "CREATE OR REPLACE VIEW erp_inventory_list_app AS
                    (SELECT I.id,I.part_number,I.barcode,I.description,I.active,I.excise_tax,B.brand_name,I.type,SUM(S.quantity*S.unit_exchange_rate) AS in_qty,
                        0 AS out_qty,U.unit_name,U.id as unit_id,U.unit_exchange_rate,U.label,G.generic_name,
                        CASE
                        WHEN SUM(S.quantity*S.unit_exchange_rate) > 0 THEN SUM(S.quantity*S.rate)/SUM(S.quantity*S.unit_exchange_rate) ELSE 0 END AS avg_purchase_rate,0 AS avg_sales_rate,tax,O.name,O.outlet_name,
                        I.rate,I.rate_a,I.rate_b 
                        FROM erp_inventory I 
                        LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                        LEFT JOIN erp_master_brand B ON I.brand=B.id 
                        LEFT JOIN erp_master_generic G ON I.generic_id=G.id 
                        LEFT JOIN erp_master_unit U ON I.unit=U.id 
                        LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='IN' OR S.type='OB') group by I.id)
                        UNION
                        (SELECT I.id,I.part_number,I.barcode,I.description,I.active,I.excise_tax,B.brand_name,I.type,0 AS in_qty,SUM(S.quantity*S.unit_exchange_rate) AS out_qty,
                        U.unit_name,U.id as unit_id,U.unit_exchange_rate,U.label,G.generic_name,0 AS avg_purchase_rate,CASE
                        WHEN SUM(S.quantity*S.unit_exchange_rate) > 0 THEN SUM(S.quantity*S.rate)/SUM(S.quantity*S.unit_exchange_rate) ELSE 0 END AS avg_sales_rate,0 AS tax,O.name,O.outlet_name,
                        I.rate,I.rate_a,I.rate_b
                        FROM erp_inventory I 
                        LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                        LEFT JOIN erp_master_brand B ON I.brand=B.id 
                        LEFT JOIN erp_master_generic G ON I.generic_id=G.id 
                        LEFT JOIN erp_master_unit U ON I.unit=U.id 
                        LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='OUT' OR S.type='OB')  group by I.id)";
            Yii::app()->db->createCommand($SQL)->execute();
            $items = Yii::app()->db->createCommand("SELECT id,barcode,part_number,description,active,excise_tax,generic_name,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,unit_id,
                unit_name,unit_exchange_rate,label as symbol,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,SUM(tax) AS tax,name,outlet_name,rate,rate_a,rate_b from erp_inventory_list_app group by id order BY part_number ASC")->queryAll();


            if (!empty($items)) {
                foreach ($items as $i => $each) {

                    $this->responseBody['data'][$i]['id'] = $each['id'];
                    $this->responseBody['data'][$i]['generic_name'] = isset($each['generic_name']) ? $each['generic_name'] : '';
                    $this->responseBody['data'][$i]['part_number'] = $each['part_number'];
                    $this->responseBody['data'][$i]['barcode'] = $each['barcode'];
                    $this->responseBody['data'][$i]['description'] = $each['description'];
                    $inv_type = ($each['type'] == 'I') ? "Inventory" : "Service";
                    $this->responseBody['data'][$i]['type'] = $each['type']; //$inv_type;
                    $this->responseBody['data'][$i]['brand_name'] = isset($each['brand_name']) ? $each['brand_name'] : '';
                    $avg_purchase_rate = StockLedger::getAvgPurchaseRate($each['id'], '', $userModel->outlet_id);
                    $this->responseBody['data'][$i]['cost_rate'] = round($avg_purchase_rate, 2);
                    $this->responseBody['data'][$i]['quantity'] =  ($each['type'] == 'I') ? round($each['qty'], 2) : 1000;
                    $this->responseBody['data'][$i]['unit_id'] = $each['unit_id'];
                    $this->responseBody['data'][$i]['unit_name'] = $each['unit_name'];
                    $this->responseBody['data'][$i]['unit_factor'] = $each['unit_exchange_rate'];
                    $this->responseBody['data'][$i]['symbol'] = $each['symbol'];
                    $consider_previous_sprice = MasOutlets::model()->findByPk($userModel->outlet_id)->consider_previous_sprice;
                    if ($consider_previous_sprice == 'Y') {
                       // $mod_pre_his = StockLedger::model()->with('sales_det')->find(array('condition' => 't.type ="SI"  AND product_id =' . $each['id'], 'order' => 't.id DESC'));
                        $mod_pre_his = StockLedger::model()->with('sales_det')->find(array('condition' => 't.type ="SI"  AND sales_det.customer_id =' . $customer_id . '  AND product_id =' . $each['id'], 'order' => 't.id DESC'));
                       
                        if (empty($mod_pre_his)) {
                            if ($sales_price_type == 1) {
                                $pre_rate = $each['rate'];
                            } else if ($sales_price_type == 2) {
                                $pre_rate = $each['rate_a'];
                            } else if ($sales_price_type == 3) {
                                $pre_rate = $each['rate_b'];
                            } else {
                                $pre_rate = $each['rate'];
                            }
                        } else {
                         
                           $qty = ($mod_pre_his->quantity * $mod_pre_his->unit_exchange_rate); 
                            if ($qty > 0) {
                                $pre_rate = $mod_pre_his->rate / $qty;
                            } else {
                                if ($sales_price_type == 1) {
                                    $pre_rate = $each['rate'];
                                } else if ($sales_price_type == 2) {
                                    $pre_rate = $each['rate_a'];
                                } else if ($sales_price_type == 3) {
                                    $pre_rate = $each['rate_b'];
                                } else {
                                    $pre_rate = $each['rate'];
                                }
                            }
                        }
                    } else {
                        if ($sales_price_type == 1) {
                            $pre_rate = $each['rate'];
                        } else if ($sales_price_type == 2) {
                            $pre_rate = $each['rate_a'];
                        } else if ($sales_price_type == 3) {
                            $pre_rate = $each['rate_b'];
                        } else {
                            $pre_rate = $each['rate'];
                        }
                    }
                    //Purchase Price
                    if ($consider_previous_sprice == 'Y') {
                        $mod_pre_his_pur = StockLedger::model()->with('pur_det')->find(array('condition' => 't.type ="PI"  AND pur_det.vendor_id =' . $vendor_id . ' AND product_id =' . $each['id'], 'order' => 't.id DESC'));
                        if (empty($mod_pre_his_pur)) {
                            $pre_rate_pur = 0;
                        } else {
                            $qty = ($mod_pre_his_pur->quantity * $mod_pre_his_pur->unit_exchange_rate);
                            if ($qty > 0) {
                                $pre_rate_pur = $mod_pre_his_pur->rate / $qty;
                            } else {
                                $pre_rate_pur = 0;
                            }
                        }
                    } else {
                        $pre_rate_pur = 0;
                    }
                    
                    $consider_sp_not_less_cp = MasOutlets::model()->findByPk($userModel->outlet_id)->consider_sp_not_less_cp;
                    $min_sales_rate = ($consider_sp_not_less_cp=='Y')?(float) $each['rate']:0;

                    $this->responseBody['data'][$i]['default_sales_rate'] = round($pre_rate, 3);
                    $this->responseBody['data'][$i]['default_purchase_rate'] = round($pre_rate_pur, 3);
                    $this->responseBody['data'][$i]['min_sales_rate'] = $min_sales_rate;
                    $this->responseBody['data'][$i]['discount_percentage'] = 0;
                    $this->responseBody['data'][$i]['tax_percentage'] = $each['tax'];
                    $this->responseBody['data'][$i]['is_taxable'] = $each['tax'] > 0 ? true : false;
                    $this->responseBody['data'][$i]['excise_tax'] = $each['excise_tax'];
                    $this->responseBody['data'][$i]['consider_previous_price'] = $consider_previous_sprice;

                    $count++;
                }
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }
}
