<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiOrderController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('GetOrderList', 'GetOrderDetailsId', 'SaveOrder', 'SalesOrderPrint','GetOrderDetailsIdNew'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }



    public function actionGetOrderList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $date = trim($variables['date']);
        if ($date == '') {
            $date = date('Y-m-d');
        }
        if ($userModel->id > 0) {
            $userModel->db();
            $condition = 't.is_invoiced="N" AND t.active="Y" AND t.order_date="' . $date . '" AND t.salesman_id = "' . $userModel->id . '" ';
            $modelorder = new OrderCollect();
            $model = $modelorder->with('oc_customer_det', 'oc_salesman_det')->findAll(array('condition' => $condition, 'order' => 't.id desc'));
            $count = 0;
            foreach ($model as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['order_date'] = date("d/m/Y", strtotime($each['order_date']));
                $this->responseBody['data'][$i]['salesman'] = $each['oc_salesman_det']['full_name'];
                $this->responseBody['data'][$i]['salesman_id'] = $each->salesman_id;
                $this->responseBody['data'][$i]['customer'] = $each['oc_customer_det']['customer_name'];
                $this->responseBody['data'][$i]['customer_id'] = $each->customer_id;
                $this->responseBody['data'][$i]['notes'] = $each->notes;
                $this->responseBody['data'][$i]['outlet'] = $each->outlet;
                $this->responseBody['data'][$i]['outlet_name'] = $each['oc_outlet_det']['outlet_name'];
                $this->responseBody['data'][$i]['is_invoiced'] = $each->is_invoiced;
                $count++;
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionGetOrderDetailsId()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $id = trim($variables['id']);
        if ($userModel->id > 0) {
            $userModel->db();
            $model = OrderCollect::model()->with('oc_customer_det', 'oc_salesman_det')->findByPk($id);
            if (!empty($model)) {
                $this->responseBody['data']['customer'] = $model->oc_customer_det->customer_name;
                $this->responseBody['data']['customer_id'] = $model->customer_id;
                $this->responseBody['data']['id'] = $model->id;
                $this->responseBody['data']['notes'] = $model->notes;
                $this->responseBody['data']['outlet'] = $model->outlet;
                $this->responseBody['data']['outlet_name'] =  $model['oc_outlet_det']['outlet_name'];
                $this->responseBody['data']['is_invoiced'] = $model->is_invoiced;
                $this->responseBody['data']['salesman_id'] = $model->salesman_id;
                $this->responseBody['data']['salesman'] = $model->oc_salesman_det->full_name;
                $this->responseBody['data']['order_date'] = $model->order_date;
                $modelItems = OrderCollectItem::model()->findAll(array('condition' => 'order_id="' . $id . '"', 'order' => 'id'));
                foreach ($modelItems as $i => $each) {
                    $this->responseBody['data']['items'][$i]['id'] = $each->id;
                    $this->responseBody['data']['items'][$i]['order_id'] = $each->order_id;
                    $this->responseBody['data']['items'][$i]['product_id'] = $each->product_id;
                    $this->responseBody['data']['items'][$i]['part_number'] = $each->oc_product->part_number;
                    $this->responseBody['data']['items'][$i]['description'] = $each->description;
                    $this->responseBody['data']['items'][$i]['brand_name'] = ((int) $each->oc_product->brand>0)? $each->oc_product->brands->brand_name: '';
                    $this->responseBody['data']['items'][$i]['generic_name'] = ((int) $each->oc_product->generic_id>0)? $each->oc_product->generic->generic_name: '';
                    $this->responseBody['data']['items'][$i]['type'] = $each->oc_product->type;
                    $this->responseBody['data']['items'][$i]['quantity'] = $this->format_currency($each->quantity); 
                    $this->responseBody['data']['items'][$i]['unit_id'] = $each->unit_id;
                    $this->responseBody['data']['items'][$i]['unit_name'] = $each->oc_unit->label;
                    $this->responseBody['data']['items'][$i]['base_unit_id'] = $each->base_unit_id;
                    $this->responseBody['data']['items'][$i]['base_unit_qty'] = $this->format_currency($each->base_unit_qty);
                    $this->responseBody['data']['items'][$i]['unit_factor'] = $each->unit_factor;
                    $this->responseBody['data']['items'][$i]['rate'] = $this->format_currency($each->rate); 
                    $this->responseBody['data']['items'][$i]['amount'] = $this->format_currency($each->amount); 
                    $this->responseBody['data']['items'][$i]['tax_percentage'] = $this->format_currency($each->tax_percentage); 
                    $this->responseBody['data']['items'][$i]['tax_amount'] = $this->format_currency($each->tax_amount); 
                    $this->responseBody['data']['items'][$i]['net_amount'] =$this->format_currency($each->net_amount);  
                    $this->responseBody['data']['items'][$i]['selected_quantity'] = $this->format_currency($each->quantity); 
                    $this->responseBody['data']['items'][$i]['selected_rate'] = $this->format_currency($each->rate); 
                    $this->responseBody['data']['items'][$i]['selected_uom_id'] = $each->unit_id;
                    $this->responseBody['data']['items'][$i]['selected_uom_unit_name'] = $each->oc_unit->label;
                    $this->responseBody['data']['items'][$i]['selected_uom_unit_factor'] = $each->unit_factor;
                }
                $this->responseBody['count'] = 1;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Order id';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }

    public function actionSaveOrder()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        //error_log('ordercollect');
        //error_log($input);
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $userModel->db();
            $order_date = trim($variables['order_date']);
            $id = trim($variables['id']);
            $salesman_id = trim($variables['salesman_id']);
            $customer_id = trim($variables['customer_id']);
            $notes = trim($variables['notes']);
            $orderItems = $variables['items'];
            $orderItems = $variables['items'];
            $taxable = $variables['taxable'] == true ? true : false;
            $others =[];
            if ($customer_id > 0) {
                $model = new OrderCollect();
                //$modelformmaster=FormMaster::model()->find(array('condition'=>'outlet='.$userModel->outlet_id.' AND code="OC"'));
                if ($id > 0) {
                    $model = $model->findBypk($id);
                    $scene = 'update';
                    $change_type = 'UPDATED';
                     $model->updated_date_time = date('Y-m-d H:i:s');
                    $model->updated_by = $userModel->id;
                   
                } else {
                    $scene = 'new';
                    $change_type = 'CREATED';
                    $model->created_date_time = date('Y-m-d H:i:s');
                    $model->created_by = $userModel->id;
                }



                $model->order_date = count(explode('-', $order_date)) == 3 ? $order_date : date('Y-m-d');
                $model->customer_id = $customer_id;
                $model->outlet = $userModel->outlet_id;
                $model->salesman_id = ($salesman_id > 0) ? $salesman_id : $userModel->id;
                $model->notes = $notes;
                $model->invoice_type = $taxable == true ? "1" : "2"; // here we consider  $taxable as  invoice_type
                $model->is_invoiced = 'N';

                $valid = true;
                $i = 0;
                $arr_pdts =[];
                    $reversedorderItems = array_reverse($orderItems);


                foreach ($reversedorderItems as $each) {
                    if ($each['product_id'] > 0) {
                        $productModel = Inventory::model()->findBypk($each['product_id']);
                        $items[$i] = new OrderCollectItem;
                        $items[$i]->order_id = 0;
                        $items[$i]->product_id = $each['product_id'];
                        $items[$i]->description = $productModel->description;
                        $items[$i]->quantity = (float) $each['quantity'];
                        $items[$i]->unit_id = $each['unit'];
                        $items[$i]->rate = $each['rate'];
                        $amount = round(((float) $each['quantity'] * (float) $each['rate']), 6);
                        $items[$i]->amount = round($amount, 2);
                       // $items[$i]->tax_percentage = $productModel->taxDetails->tax_percentage;
                       //$items[$i]->tax_amount = round((($amount * $productModel->taxDetails->tax_percentage) / 100), 2);
                        $items[$i]->tax_percentage =  $each['tax_percentage'] ; 
                        $items[$i]->tax_amount = $taxable ? round((($amount * $each['tax_percentage']) / 100), 6) : 0;
                        $items[$i]->net_amount = $items[$i]->tax_amount + round($amount, 6);
                        if ((int)$each['unit'] > 0) {
                            $unit = MasUnit::model()->findByPk((int)$each['unit']);
                            $items[$i]->unit_factor = $unit->unit_exchange_rate;
                            $items[$i]->base_unit_id = $unit->base_unit_id;
                            $items[$i]->base_unit_qty = (float) ($each['quantity'] * $unit->unit_exchange_rate);
                        }
                        
                         // --------------------------------------------------
                        $unit = MasUnit::model()->findByPk($each['unit']);
                         // Construct the composite key using product_id and unit
                            $compositeKey = $each['product_id'] . '-' . $each['unit'];
                            
                            // Check if the composite key already exists
                            if (in_array($compositeKey, $arr_pdts)) {
                                $valid = false; // Mark validation as failed
                                $others[] = 'Product ' . $productModel->part_number . ' with Unit ' . $unit->label . ' is duplicated ,Line No:-'.($i+1);
                            } else {
                                // Add the composite key to the array
                                $arr_pdts[] = $compositeKey;
                            }
                        // --------------------------------------------------
                        
                        $valid = $items[$i]->validate() && $valid;
                        $i++;
                    }
                }
                if ($valid && $model->validate() && is_array($items)) {
                    $model->save();
                    if ($scene == 'new') {
                        //$next_no=$model->order_no+1;
                        //FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                    }

                    Yii::app()->db->createCommand("delete from {{order_collect_item}} where order_id='" . $model->id . "'")->execute();
                    foreach ($items as $eachItem) {
                        $eachItem->order_id = $model->id;
                        $eachItem->save(false);
                    }
                    // inserting to audit log
                    /*    AuditLog::saveApiAuditLogs(
                      $userModel->outlet_id,$modelformmaster->id,
                      $userModel->id,
                      $change_type,
                      " with ID - {$model->id} "
                      ); */
                    // end audit log
                    $this->responseBody['status'] = 'success';
                    $this->responseBody['msg'] = 'Order created successfully';
                    $this->responseBody['order_id'] = $model->id;
                    $this->_bind;
                } else {
                    $this->responseBody['status'] = 'failed';
                    $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                    if (!empty($items))
                        $error2 = json_decode(CActiveForm::validateTabular($items, '', false), true);
                    else
                        $error2[] = "Please add atleast one line item";
                    $error = array_merge_recursive($error1, $error2,$others);
                    $arr_err = $other;
                    foreach ($error as $key => $errval) {
                        $arr_err[] = $errval[0];
                    }
                    $err_msg = implode(',', $arr_err);
                    $this->responseBody['msg'] = $err_msg;

                    $this->_bind;
                }
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Customer';
                $this->_bind;
            }
        } else { //usermodel
            $this->_401;
        }
    }


    public function actionSalesOrderPrint()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $id = (int) trim($variables['id']);
            $model = OrderCollect::model()->findByPk($id);
            $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');

            $path = "app_reports/";
            $filename = 'SO_' . $model->id;
            $mPDF1->WriteHTML($stylesheet, 1);
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.order_collect_print', array('id' => $id, 'user_id' => $userModel->id), true));
            $mPDF1->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
      public function actionGetOrderDetailsIdNew()
    {

      //  try {
            $data = [];
            $other = [];
            $valid = true;
            $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
            $userModel = MasUsers::model()->findBytoken($token);
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            if ($userModel->id > 0) {
                $order_id = $variables['id'];
                $userModel->db();


                if ((int) $order_id > 0) {
                    $modelInvoice = new OrderCollect();
                    $modelInvoice = $modelInvoice->find(array('condition' => 'id=' . (int) $order_id));
                    if (!empty($modelInvoice)) {

                        $modelInvoiceItem = new OrderCollectItem();
                        $modelInvoiceItem = $modelInvoiceItem->model()->findAll(array('condition' => 'order_id="' . (int) $modelInvoice['id'] . '"'));
                        
                        $company_name    = $modelInvoice->oc_outlet_det->companyDetails->name; 
                        $company_address = trim($modelInvoice->oc_outlet_det->companyDetails->billing_address); 
                        $company_phone   = $modelInvoice->oc_outlet_det->companyDetails->phone; 


                        $item_list = [];
                        $total_discount = 0;
                        $total_wo_vat = 0;
                        $total_amount = $grand_total = 0;
                        $tax_total = 0;
                        $item_count =0;
                        foreach ($modelInvoiceItem as $eachItem) {
                            
                            $total_amount += $eachItem['amount'];
                            $tax_total    += $eachItem['tax_amount'];
                            $grand_total    += $eachItem['net_amount'];
                            $item_count ++;

                            $item_list[] = [
                                "invoice_item_id"   => $eachItem['id'],
                                "product_id"        => $eachItem['product_id'],
                                "part_number"       => $eachItem['oc_product']['part_number'],
                                "description"       => $eachItem['description'],
                                "bar_code"          => $eachItem['oc_product']['barcode'],
                                "quantity"          => $eachItem['quantity'],
                                "rate"              => $eachItem['rate'],
                                "amount"            => $eachItem['amount'],
                                "excise_percentage" => 0,
                                "excise_amount"     => 0,
                                "tax_percentage"    => number_format((float)$eachItem['tax_percentage'], 2, '.', '') ,
                                "tax_amount"        => number_format((float)$eachItem['tax_amount'], 2, '.', '') ,
                                "net_amount"        => number_format((float)$eachItem['net_amount'], 2, '.', '') ,
                                "unit"              => $eachItem['unit_id'],
                                "unit_name"         => $eachItem['oc_unit']['unit_name'],
                            ];
                            
                            
                        }



                        if ($valid) {

                            $data['status']            = "success";
                            $data['company_name']      = $company_name;
                            $data['company_address']   = $company_address;
                            $data['company_phone']     = $company_phone;
                            $data['id']                = $modelInvoice['id'];
                            $data['invoice_no']        = $modelInvoice['id'];
                            $data['invoice_date']      = date("d/m/Y", strtotime($modelInvoice['order_date']));
                            $data['customer_name']     = $modelInvoice['oc_customer_det']['customer_name'];
                            $data['customer_address']  = $modelInvoice['oc_customer_det']['address'];
                            $data['customer_trn_no']   = $modelInvoice['oc_customer_det']['trn_no'];
                            $data['customer_phone_no'] = $modelInvoice['oc_customer_det']['phone'];
                            $data['sales_man']         = $modelInvoice['oc_salesman_det']['full_name'];
                            $data['salesman_phone']    = $modelInvoice['oc_salesman_det']['phone_no'];
                            $data['payment_term']      = '';
                            $data['payment_term']      = '';
                            $data['total_amount']      = number_format((float)$total_amount, 2, '.', '');
                            $data['excise_total']      = number_format((float)0, 2, '.', '') ;
                            $data['tax_total']         = number_format((float)$tax_total, 2, '.', '') ;
                            $data['grand_total']       = number_format((float)$grand_total, 2, '.', ''); 
                            $data['posted']            = '';
                            $data['invoice_type']      = ($modelInvoice['invoice_type'] == 1) ? 'Tax Order' : 'Export Order';
                            $data['created_datetime']  =  date("d/m/Y", strtotime($modelInvoice['created_date_time']));
                            $data['invoice_item_list'] = $item_list;
                            $data['total_items']       = $item_count;
                        } else {
                            $err_msg = implode(',', $other);
                            $data = ['status' => "failed", "msg" => $err_msg];
                        }
                    } else {
                        $data = array('status' => "failed", "msg" => "Invoice not Found");
                    }
                } else {
                    $data = array('status' => "failed", "msg" => "Invoice ID not Given");
                }
            } else {
                $this->_401;
            }
       /* } catch (Exception $e) {
            header("HTTP/1.1 404 not found");
            header('Content-Type: application/json');
            $data = array('status' => "failed", "msg" => "Unexpected Server error");
        }
*/

        echo json_encode($data);
    }
    
}
