<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiPurchaseReturnController extends Controller {

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules() {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array("GetList","Save","Edit","Cancel","Print","GetData"),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters() {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request) {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401() {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400() {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403() {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind() {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value) {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions() {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null) {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }
 
    public function connectMainDB($company_db_name) {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }
 
    
    

public function actionGetList() {
    $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
    $userModel = MasUsers::model()->findBytoken($token);

    $input = file_get_contents('php://input');
    $variables = json_decode($input, true);
    
    $from = trim($variables['from']);
    $to = trim($variables['to']);

    $from = $this->convert_date($from);
    $to = $this->convert_date($to);
    if ($userModel->id > 0) {

        $userModel->db();

        $count = 0;
      
        $model = PurchaseReturn::model()->findAll(array('condition' => 'return_date BETWEEN "' . $from . '" AND "' . $to . '" AND created_by=' . $userModel->id, 'order' => 'return_date DESC, return_no DESC'));
        foreach ($model as $i => $each) {
            $this->responseBody['data'][$i]['id'] = $each->id;
            $this->responseBody['data'][$i]['prefix'] = $each->prefix;
            $this->responseBody['data'][$i]['voucher_no'] = $each->return_no;
            $this->responseBody['data'][$i]['invoice_id'] = $each->invoice_id;
            $this->responseBody['data'][$i]['invoice_no'] = $each->pr_invoice->invoice_no;
            $this->responseBody['data'][$i]['date'] = $each->return_date;
            $model_purchase_return = StockLedger::model()->with('purchasereturn_det')->findAll(array('condition' => 'purchasereturn_det.invoice_id=' . $each->pr_invoice->invoice_no));
            $this->responseBody['data'][$i]['have_any_active_returns'] = (empty($model_purchase_return)) ? false : true;
            $this->responseBody['data'][$i]['currency_code'] = $each->pr_cur->currency_code;
            $this->responseBody['data'][$i]['exchange_rate'] = $each->exchange_rate;
            $this->responseBody['data'][$i]['vendor'] = $each->pr_vendor->vendor_name;
            $this->responseBody['data'][$i]['total_amount'] = $this->format_currency($each->total_amount);
            $this->responseBody['data'][$i]['excise_total'] = $this->format_currency($each->excise_total);
            $this->responseBody['data'][$i]['vat_amount'] = $this->format_currency($each->tax_total);
            $this->responseBody['data'][$i]['grand_total'] = $this->format_currency($each->grand_total);
            $this->responseBody['data'][$i]['is_posted'] = $each->posted == 'Y' ? true : false;
            $this->responseBody['data'][$i]['paid'] = $each->paid;
            $this->responseBody['data'][$i]['discount'] = $each->discount;
            $this->responseBody['data'][$i]['old_discount'] = $each->old_discount;
            $this->responseBody['data'][$i]['notes'] = $each->notes;
            /* $this->responseBody['data'][$i]['is_cancelable'] = $each->is_cancelable;
              $this->responseBody['data'][$i]['is_audited'] = $each->is_audited == 'Y' ? true : false;
              $this->responseBody['data'][$i]['is_allocated'] = $each->is_allocated;
              $this->responseBody['data'][$i]['is_returnable'] = $each->returnable; */
            $this->responseBody['data'][$i]['status'] = $each->active == 'Y' ? 'Active' : 'Cancel';
            $this->responseBody['data'][$i]['is_cancelled'] = $each->active == 'Y' ? false : true;
            $count++;
        }
       
        $this->responseBody['count'] = $count;
        $this->_bind;
    } else {
        $this->_401;
    }
 
}


public function actionGetData() {
    $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
    $userModel = MasUsers::model()->findBytoken($token);

    $input = file_get_contents('php://input');
    $variables = json_decode($input, true);
    $voucher_id = (int) trim($variables['voucher_id']); 
    if ($userModel->id > 0) {

        $userModel->db();


        if ($voucher_id > 0) {

            $id = $voucher_id;
            //Head
            $voucherModel = PurchaseReturn::model()->findbypk($id);
            $this->responseBody['data']['voucher_type'] = 'PurchaseReturn';
            $this->responseBody['data']['voucher_no'] = $voucherModel->return_no;
            $this->responseBody['data']['parent_voucher_no'] = $voucherModel->prefix;
            $this->responseBody['data']['voucher_date'] = date('d/m/Y', strtotime($voucherModel->return_date));
            $this->responseBody['data']['currency'] = $voucherModel->pr_cur->currency_code;
            $this->responseBody['data']['company_name'] =$voucherModel->pr_outlet->name;
            $this->responseBody['data']['company_adds'] = str_replace('<br>', ' ', $voucherModel->pr_outlet->address);
            $this->responseBody['data']['company_phone'] = $voucherModel->pr_outlet->phone_no;
            $this->responseBody['data']['company_po_box'] = $voucherModel->pr_outlet->po_box;
            $this->responseBody['data']['company_trn'] = $voucherModel->pr_outlet->trn_no;
            $this->responseBody['data']['outlet'] = $voucherModel->pr_outlet->name;
            $this->responseBody['data']['user'] = $voucherModel->pr_user->full_name;
            $this->responseBody['data']['user_phone'] = $voucherModel->pr_user->phone_no;
            $this->responseBody['data']['create_date_time'] = date('d/m/Y', strtotime($voucherModel->return_date));
            $this->responseBody['data']['notes'] = $voucherModel->notes;
            $this->responseBody['data']['party'] = $voucherModel->pr_vendor->vendor_name;
            $this->responseBody['data']['party_trn'] = $voucherModel->pr_vendor->trn_no;
            $this->responseBody['data']['reference_no'] = $voucherModel->reference_no;
            $totalCount = PurchaseReturnItems::model()->count(array('condition' => 'return_id="' . $id . '"', 'order' => 'id ASC'));
            $this->responseBody['data']['total_items'] = $totalCount;
            $this->responseBody['data']['total_vat'] = $this->format_currency($voucherModel->tax_total);
            $this->responseBody['data']['total'] = $this->format_currency($voucherModel->total_amount);
            $this->responseBody['data']['grand_total'] = $this->format_currency($voucherModel->grand_total);
            //Items
            $modelItems = PurchaseReturnItems::model()->findAll(array('condition' => 'return_id="' . $id . '"', 'order' => 'id ASC'));
            foreach ($modelItems as $each) {
                $this->responseBody['data']['items'][] = array(
                    'part_number' => $each->pr_product->part_number,
                    'description' => $each->pr_product->description,
                    'quantity' => $this->format_currency($each->returned_quantity),
                    'uom' => $each->pr_units->label,
                    'rate' => $this->format_currency($each->rate),
                    'discount' => '0.00',
                    'vat' => $this->format_currency($each->tax_amount),
                    'net' => $this->format_currency($each->net_amount),
                );
            }
            //Items
            $this->responseBody['count'] = 1;
            $this->_bind;

            
        } else {
            $this->responseBody['status'] = 'failed';
            $this->responseBody['msg'] = 'Invalid Voucher';
            $this->_400;
        }
    } else {
        $this->_401;
    }
}

public function actionSave() {
    $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
    $userModel = MasUsers::model()->findBytoken($token);

    $input = file_get_contents('php://input');
    $this->save_request($input);
    $variables = json_decode($input, true);
    $debitnote_date = trim($variables['debitnotedate']);
    $id = (int) trim($variables['id']);
    $invoice_id = (int) trim($variables['parent_invoice']);
    $items_response = $variables['items'];
    $total_tax_amount = $total_amount = $total_excise_amount = $grand_total = 0;
    if ($userModel->id > 0) {

        $userModel->db();

        $invoiceModel = PurchaseInvoice::model()->findBypk($invoice_id);
        if ($invoiceModel->id > 0) { //Purchase
            $model = new PurchaseReturn();
            $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="PR"'));
            if ($id > 0) {
                $model = $model->findBypk($id);
                $scene = 'update';
                $change_type = 'UPDATED';
            } else {
                $model->created_by = $userModel->id;
                $model->created_time = date('Y-m-d H:i:s');
                $scene = 'new';
                $change_type = 'CREATED';

                $model->return_no = $modelformmaster->next_number;
            }
            $model->invoice_no = $invoiceModel->invoice_no;
            $model->invoice_id = $invoiceModel->id;
            $model->vendor_id = $invoiceModel->vendor_id;
            $model->currency = $invoiceModel->currency;
            $model->exchange_rate = $invoiceModel->exchange_rate;
            $model->updated_by = $userModel->id;
            $model->updated_time = date('Y-m-d H:i:s');
            $model->return_date = $debitnote_date;
            $model->outlet = $userModel->outlet_id;
            $model->total_amount = $total_amount;
            $model->excise_total = $total_excise_amount;
            $model->tax_total = $total_tax_amount;
            $model->grand_total = $grand_total;
            $valid = $model->validate();
            $groups = array();
            $i = 0;
            foreach ($items_response as $each) {
                if ($each['quantity'] > 0) {
                    $invoiceitemModel = PurchaseInvoiceItems::model()->findBypk((int) $each['item_parent_id']);
                    $modelItems[$i] = new PurchaseReturnItems();
                    $modelItems[$i]->return_id = 0;
                    $modelItems[$i]->invoice_id = $invoiceModel->id;
                    $modelItems[$i]->invoice_item_id = $invoiceitemModel->id;
                    $modelItems[$i]->product_id = $invoiceitemModel->product_id;
                    $modelItems[$i]->description = $invoiceitemModel->description;
                    $modelItems[$i]->returned_quantity = $each['quantity'];
                    $modelItems[$i]->invoiced_qty = $invoiceitemModel->quantity;
                    $modelItems[$i]->unit = $each['unit'];
                    $modelItems[$i]->rate = $invoiceitemModel->rate;
                    $amount = round(((float) $each['quantity'] * (float) $invoiceitemModel->rate), 6);
                    $modelItems[$i]->amount = $amount;
                    $modelItems[$i]->tax_percentage = $invoiceitemModel->tax_percentage;
                    $tax_amount = $amount * ($invoiceitemModel->tax_percentage / 100);
                    $modelItems[$i]->tax_amount = $tax_amount;
                    $modelItems[$i]->net_amount = $tax_amount + $amount;
                    $valid = $valid && $modelItems[$i]->validate();
                    // Product Grouping
                    if ($each['unit'] > 0) {
                        $unit = MasUnit::model()->findByPk($each['unit']);
                        $unit_exchange = $unit->unit_exchange_rate;
                    } else {
                        $unit_exchange = 0;
                    }
                    $key = $invoiceitemModel->product_id;
                    if (!array_key_exists($key, $groups)) {
                        $groups[$key] = array(
                            'id' => $invoiceitemModel->product_id,
                            'returned_quantity' => $each['quantity'] * $unit_exchange,
                        );
                    } else {
                        $groups[$key]['returned_quantity'] = $groups[$key]['returned_quantity'] + $each['quantity'] * $unit_exchange;
                    }
                    $i++;
                }
            }
            // QTY Checking
            if (!empty($groups)) {
                foreach ($groups as $key => $eachProduct) {
                    $qty = 0;
                    if ($model->id > 0) {
                        $qty = Yii::app()->db->createCommand("select IFNULL(SUM(returned_quantity*U.unit_exchange_rate),0) from {{purchase_return_items}} S INNER JOIN {{master_unit}} U ON S.unit=U.id where return_id='" . $model->id . "' AND product_id='" . $key . "'")->queryScalar();
                    }
                    $bal = $eachProduct['returned_quantity'] - $qty;
                    $invoiced_qty = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from {{stock_ledger}} where transaction_type='IN' AND type='PI' AND product_id='" . $key . "' AND header_id='" . $model->invoice_id . "'  AND outlet=" . $userModel->outlet_id . " AND created_by=" . $userModel->id)->queryScalar();
                    $already_returned_qty = Yii::app()->db->createCommand("select IFNULL(SUM(S.quantity*S.unit_exchange_rate),0) from {{stock_ledger}} S LEFT JOIN {{purchase_return_items}} R ON R.id=S.item_id where R.invoice_id='" . $model->invoice_id . "'  AND  transaction_type='OUT' AND type='PR' AND S.product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $userModel->id)->queryScalar();
                    $balance = $invoiced_qty - $already_returned_qty + $qty;
                    $product = Inventory::model()->findByPk($key);
                    if ($eachProduct['returned_quantity'] > $balance) {
                        $other[] = $product->part_number . ' maximum quantities to be returned : ' . ($balance) . ' ' . $product->units->unit_name;
                        $valid = false;
                    } else if ($bal > 0 && $product->type != 'S') {
                        $in_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='IN' AND product_id='" . $key . "' AND transaction_date<='" . $model->return_date . "' AND outlet=" . $userModel->outlet_id)->queryScalar();
                        $out_qty = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='OUT' AND product_id='" . $key . "' AND transaction_date<='" . $model->return_date . "' AND outlet=" . $userModel->outlet_id)->queryScalar();
                        $current_stock_till_date = $in_qty - $out_qty;
                        if ($current_stock_till_date < $bal) {
                            $other[] = $product->part_number . ' have only ' . $current_stock_till_date . ' ' . $product->units->unit_name . ' quantities on ' . $model->return_date;
                            $valid = false;
                        }
                        $in_qty1 = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='IN' AND product_id='" . $key . "'  AND outlet=" . $userModel->outlet_id . " AND created_by=" . $userModel->id)->queryScalar();
                        $out_qty1 = Yii::app()->db->createCommand("select IFNULL(SUM(quantity*unit_exchange_rate),0)  from {{stock_ledger}} where transaction_type='OUT' AND product_id='" . $key . "' AND outlet=" . $userModel->outlet_id . " AND created_by=" . $userModel->id)->queryScalar();
                        $current_stock = $in_qty1 - $out_qty1;
                        if ($current_stock < $bal) {
                            $product = Inventory::model()->findByPk($key);
                            $other[] = $product->part_number . ' have only ' . $current_stock . ' ' . $product->units->unit_name . ' quantities in current stock';
                            $valid = false;
                        }
                    }
                }
            }
    $closing_date=MasClosing::getClosingDate();
    if($closing_date>=$model->return_date)
    {
    $other[]='Date should be greater than closing date ('.date('d-m-Y',strtotime($closing_date)).')'; 
    $valid=false;    
    }
            if ($valid && $model->validate() && is_array($modelItems)) {
                $model->save();
                if ($scene == 'new') {
                    $next_no = $model->return_no + 1;
                    FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                }
                Yii::app()->db->createCommand("delete from {{stock_ledger}} where transaction_type='OUT' AND type='PR' AND header_id='" . $model->id . "' AND outlet=" . $userModel->outlet_id)->execute();
                Yii::app()->db->createCommand("delete from {{purchase_return_items}} where return_id='" . $model->id . "'")->execute();

                foreach ($modelItems as $eachItem) {
                    $eachItem->return_id = $model->id;
                    $total_amount+=$eachItem->amount;
                    $total_tax_amount+=$eachItem->tax_amount;
                    $total_excise_amount+=$eachItem->excise_amount;
                    $eachItem->save(false);
                    $grand_total+=$eachItem->net_amount;

                    $stockledger = new StockLedger;
                    $stockledger->transaction_date = $model->return_date;
                    $stockledger->transaction_type = 'OUT';
                    $stockledger->type = 'PR';
                    $stockledger->customer_vendor_id = $model->vendor_id;
                    $stockledger->header_id = $model->id;
                    $stockledger->item_id = $eachItem->id;
                    $stockledger->product_id = $eachItem->product_id;
                    $stockledger->rate = $eachItem->rate;
                    $stockledger->currency = 1;
                    $stockledger->exchange_rate = 1;
                    $stockledger->quantity = $eachItem->returned_quantity;
                    $stockledger->unit = $eachItem->unit;
                    $stockledger->default_product_unit = $eachItem->pr_units->base_unit_id;
                    $stockledger->unit_exchange_rate = $eachItem->pr_units->unit_exchange_rate;
                    $stockledger->outlet = $userModel->outlet_id;
                    $stockledger->created_by = $userModel->id;
                    $stockledger->created_time = date('Y-m-d H:i:s');
                    $stockledger->save(false);
                }
                // inserting to audit log
                AuditLog::saveApiAuditLogs(
                        $userModel->outlet_id, $modelformmaster->id, $userModel->id, $change_type, " with ID - {$model->id} "
                );
                // end audit log
                $model->total_amount = $total_amount;
                $model->excise_total = $total_excise_amount;
                $model->tax_total = $total_tax_amount;
                $model->grand_total = $grand_total;
                $model->save();
                $this->purchasereturnPosting($model->id);
                $this->responseBody['status'] = 'success';
                $this->responseBody['voucher_id'] = $model->id;
                $this->responseBody['type'] = 'purchasereturn';
                $this->_bind;
            } else {
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                if (!empty($modelItems))
                    $error2 = json_decode(CActiveForm::validateTabular($modelItems, '', false), true);
                else
                    $error2[] = "Please add atleast one line item";
                $error = array_merge_recursive($error1, $error2);
                $arr_err=$other;
                foreach ($error as $key => $errval) {
                    $arr_err[]=$errval[0];
                }
                $err_msg = implode(',', $arr_err);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->_400;
            }
        } else {
            $this->responseBody['status'] = 'failed';
            $this->responseBody['msg'] = 'Invoice is not valid';
            $this->_400;
        }
    } else {
        $this->_401;
    }
}


    
    
public function actionEdit() {
    $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
    $userModel = MasUsers::model()->findBytoken($token);

    $input = file_get_contents('php://input');
    $variables = json_decode($input, true); 
    $id = (int) trim($variables['id']);

    if ($userModel->id > 0) {

        $userModel->db();

        if ($id > 0) {
            
            $model = PurchaseReturn::model()->with('pr_vendor')->findBypk((int) $id);
                $this->responseBody['data']['vendor'] = $model->pr_vendor->vendor_name;
                $this->responseBody['data']['vendor_id'] = $model->vendor_id;
                $this->responseBody['data']['parent_invoice'] = $model->invoice_id;
                $this->responseBody['data']['id'] = $model->id;
                $this->responseBody['data']['invoice_no'] = $model->pr_invoice->invoice_no;
                $this->responseBody['data']['return_no'] = $model->return_no;
                $this->responseBody['data']['total_amount'] = $this->format_currency($model->total_amount);
                $this->responseBody['data']['excise_total'] = $this->format_currency($model->excise_total);
                $this->responseBody['data']['tax_total'] = $this->format_currency($model->tax_total);
                $this->responseBody['data']['grand_total'] = $this->format_currency($model->grand_total);
                $this->responseBody['data']['payment_term'] = $model->pr_invoice->payment_term;
                $this->responseBody['data']['return_date'] = $model->return_date;
                $this->responseBody['data']['currency'] = $model->currency;
                $this->responseBody['data']['exchange_rate'] = $model->exchange_rate;
                $this->responseBody['data']['outlet'] = $model->outlet;
                $modelItems = PurchaseReturnItems::model()->findAll(array('condition' => 'return_id="' . $id . '"', 'order' => 'id'));
                foreach ($modelItems as $i => $each) {
                    $this->responseBody['data']['items'][$i]['id'] = $each->id;
                    $this->responseBody['data']['items'][$i]['return_id'] = $each->return_id;
                    $this->responseBody['data']['items'][$i]['invoice_id'] = $each->invoice_id;
                    $this->responseBody['data']['items'][$i]['item_parent_id']=$each->invoice_item_id;
                    $this->responseBody['data']['items'][$i]['ledger_id'] = $each->product_id;
                    $this->responseBody['data']['items'][$i]['product_id'] = $each->product_id;
                    $this->responseBody['data']['items'][$i]['ledger_name'] = $each->pr_product->part_number;
                    $this->responseBody['data']['items'][$i]['goods_service'] = $each->pr_product->type;
                    $this->responseBody['data']['items'][$i]['description'] = $each->pr_product->description;
                    $this->responseBody['data']['items'][$i]['quantity'] = $each->returned_quantity;
                    $this->responseBody['data']['items'][$i]['max_returnable_quantity'] = $each->invoiced_qty;
                    $this->responseBody['data']['items'][$i]['uom'] = $each->unit;
                    $this->responseBody['data']['items'][$i]['unit_label'] = $each->pr_units->unit_name;
                    $this->responseBody['data']['items'][$i]['rate'] = $this->format_currency($each->rate);
                    $this->responseBody['data']['items'][$i]['excise_percentage'] = $each->excise_percentage;
                    $this->responseBody['data']['items'][$i]['excise_amount'] = $this->format_currency($each->excise_amount);
                    $this->responseBody['data']['items'][$i]['tax_amount'] = $this->format_currency($each->tax_amount);
                    $this->responseBody['data']['items'][$i]['tax_percentage'] = str_replace('%', '', $each->tax_percentage);
                    $this->responseBody['data']['items'][$i]['net_amount'] = $this->format_currency($each->net_amount);
                }
           
            $this->responseBody['count'] = 1;
            $this->_bind;
        } else {
            $this->responseBody['status'] = 'failed';
            $this->responseBody['msg'] = 'Invalid Invoice id';
            $this->_400;
        }
    } else {
        $this->_401;
    }
}



public function actionCancel() {
    $input = file_get_contents('php://input');
    $variables = json_decode($input, true);
    $voucher_id = (int) trim($variables['voucher_id']); 

    $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
    $userModel = MasUsers::model()->findBytoken($token);

    if ($userModel->id > 0) {

        $userModel->db();
        
        $id = $voucher_id;
        $model = PurchaseReturn::model()->findByPk($id);
        $valid=true;
        $closing_date=MasClosing::getClosingDate();
        if($closing_date>=$model->return_date)
        {
        $other[]='Date should be greater than closing date ('.date('d-m-Y',strtotime($closing_date)).')'; 
        $valid=false;       
        }
        if ($valid) {
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=9 AND header_table_id=' . $model->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        StockLedger::model()->deleteAll('header_id=' . $model->id . ' AND  transaction_type="OUT" AND type= "PR" AND outlet=' . $userModel->outlet_id);
        $model->active = 'N';
        $model->posted = 'N';
        $model->save(false);
        $this->responseBody['status'] = 'success';
        $this->_bind;
        } else {
            $err_msg = implode(',', $other);
            $this->responseBody['msg'] = $err_msg;
            $this->responseBody['status'] = 'failed';
            $this->responseBody['items'] = array();
            $this->_400;
        }


    } else {
        $this->_401;
    }
}



public function actionPrint() {
    $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
    $userModel = MasUsers::model()->findBytoken($token);
    if ($userModel->id > 0) {
    $userModel->db();

    $input = file_get_contents('php://input');
    //error_log($input);
    $variables = json_decode($input, true);
    $id = (int) trim($variables['id']); 
    $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
    $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
    
    
    $model = PurchaseReturn::model()->findByPk($id);
    if ($model->active == 'N') {
        $mpdf->SetWatermarkText('CANCELLED');
        $mpdf->showWatermarkText = true;
    }
    $layout = 'application.views.front.api.pdf.purchase_return_print';
    $print = 'PR_' . $model->return_no;

    
    $mpdf->WriteHTML($stylesheet, 1);
    $mpdf->WriteHTML($this->renderPartial($layout, array('id' => $id, 'user_id' => $userModel->id), true));
    $mpdf->Output("downloads/vouchers/$print.pdf", 'F');
    $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . 'downloads/vouchers/' . $print . '.pdf';
    $this->_bind;
    } else {
        $this->_401;
    }
}



function invoicePosting($invoice_id) {
    $mainTable = SalesInvoice::model()->findByPk((int) $invoice_id);
    $outlet=$mainTable->outlet;
    $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="SI" AND outlet='.$outlet))->id;
    $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type='.$vouchertype.' AND header_table_id=' . $mainTable->id));
    if (!empty($fin_voucher)) {
        FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
        FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
    }
    $voucherHead['voucher_no'] = $mainTable->invoice_no;
    $voucherHead['voucher_date'] = $mainTable->invoice_date;
    $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
    $voucherHead['voucher_type'] = $vouchertype;
    // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
    $voucherHead['reference_no'] = $mainTable->reference_no;
    $voucherHead['header_table_id'] = $mainTable->id;
    $voucherHead['due_date'] = $mainTable->invoice_date;
    $voucherHead['outlet'] = $mainTable->outlet;
    $voucherHead['created_by'] = $mainTable->sales_man;
    // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
    $valid = $this->isPostable($mainTable->invoice_date);

    $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

    $total_debtor_amount = $mainTable->grand_total;
    $total_sales = $mainTable->total_amount;
    $total_tax_amount = $mainTable->tax_total;
    $total_discount_amount = $mainTable->discount;
    $total_received_amount = $mainTable->received;
    if ($total_debtor_amount > 0) {
        //Debtor Dr
        $debit_item_sales = $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => $this->getSublegder($mainTable->customer_id, 1), //2-Creditors 1- debtors
            'narration' => 'Sales #' . $mainTable->invoice_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->customer->state,
            'amount' => (float) $total_debtor_amount,
            'outlet' => $mainTable->outlet
        ));
        if ($total_received_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="CA" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->invoice_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
            $voucherHead['voucher_type'] = $vouchertype;
            $voucherHead['payment_method'] = 1;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->invoice_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->sales_man;
            $lastSavedid_receipt_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $debit_item_receipt = $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => -1 * $total_received_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => (float) $total_received_amount,
                'outlet' => $mainTable->outlet
            ));
            FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->invoice_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
            $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
            $allocation->amount = (float) $total_received_amount;
            $allocation->created_by = $mainTable->sales_man;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save(false);
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $debit_item_sales;
            $allocation_items->amount = (float) $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_receipt_voucher;
            $allocation_items->voucher_item_id = $debit_item_receipt;
            $allocation_items->amount = -1 * $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('is_allocated' => 'Y'));
        }
        // Discount Start
        if ($total_discount_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="DA" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->invoice_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
            $voucherHead['voucher_type'] = $vouchertype;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->invoice_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->sales_man;
            $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => -1 * $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'),
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => (float) $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->invoice_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
            $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
            $allocation->amount = (float) $total_discount_amount;
            $allocation->created_by = $mainTable->sales_man;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save(false);
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $debit_item_sales;
            $allocation_items->amount = (float) $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_discount_voucher;
            $allocation_items->voucher_item_id = $debit_item_discount;
            $allocation_items->amount = -1 * $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
        }
        // END Discount
    }

    if ($total_sales > 0) {
        //Sales Accounts Cr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales'),
            // 'brand_id' => $mainTable->order_id,
            // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
            'narration' => 'Sales #' . $mainTable->invoice_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->customer->state,
            'amount' => -1 * $total_sales,
            'outlet' => $mainTable->outlet
        ));
    }
    if ($total_tax_amount > 0) {
        //VAT Cr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales_vat'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => 0,
            'narration' => 'Sales #' . $mainTable->invoice_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->customer->state,
            'amount' => -1 * $total_tax_amount,
            'outlet' => $mainTable->outlet
        ));
    }

    FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
    $mainTable->posted = 'Y';
    $mainTable->save();
}


function purchaseInvoicePosting($invoice_id) {
    $mainTable = PurchaseInvoice::model()->findByPk((int) $invoice_id);
    $outlet=$mainTable->outlet;
    $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="PI" AND outlet='.$outlet))->id;
    $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type='.$vouchertype.' AND header_table_id=' . $mainTable->id));
    if (!empty($fin_voucher)) {
        FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
        FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
    }
    $voucherHead['voucher_no'] = $mainTable->invoice_no;
    $voucherHead['voucher_date'] = $mainTable->invoice_date;
    $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
    $voucherHead['voucher_type'] = $vouchertype;
    // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
    $voucherHead['reference_no'] = $mainTable->reference_no;
    $voucherHead['header_table_id'] = $mainTable->id;
    $voucherHead['due_date'] = $mainTable->invoice_date;
    $voucherHead['outlet'] = $mainTable->outlet;
    $voucherHead['created_by'] = $mainTable->created_by;
    // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
    $valid = $this->isPostable($mainTable->invoice_date);

    $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

    $total_creditor_amount = $mainTable->grand_total;
    $total_purchase = $mainTable->total_amount;
    $total_tax_amount = $mainTable->tax_total;
    $total_excise_amount = $mainTable->excise_total;
    $total_discount_amount = $mainTable->discount;
    $total_received_amount = $mainTable->received;

    if ($total_creditor_amount > 0) {
        //Creditor Cr
        $credit_item_purchase = $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
            'narration' => 'Purchase #' . $mainTable->invoice_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->vendor->state,
            'amount' => -1 * $total_creditor_amount,
            'outlet' => $mainTable->outlet
        ));
        if ($total_received_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="PV" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->invoice_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
            $voucherHead['voucher_type'] = $vouchertype;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->invoice_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->created_by;
            $lastSavedid_payment_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $credit_item_payment = $this->addVoucheritems($lastSavedid_payment_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => (float) $total_received_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_payment_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => -1 * $total_received_amount,
                'outlet' => $mainTable->outlet
            ));
            FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->invoice_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
            $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
            $allocation->amount = (float) $total_received_amount;
            $allocation->created_by = $mainTable->created_by;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save();

            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $credit_item_purchase;
            $allocation_items->amount = -1 * $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save();
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_payment_voucher;
            $allocation_items->voucher_item_id = $credit_item_payment;
            $allocation_items->amount = (float) $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save();
            FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('is_allocated' => 'Y'));
        }
        // Discount Start
        if ($total_discount_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="JV" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->invoice_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
            $voucherHead['voucher_type'] = $vouchertype;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->invoice_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->created_by;
            $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount'),
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => -1 * $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->invoice_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
            $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
            $allocation->amount = (float) $total_discount_amount;
            $allocation->created_by = $mainTable->created_by;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save(false);
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $credit_item_purchase;
            $allocation_items->amount = -1 * $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_discount_voucher;
            $allocation_items->voucher_item_id = $debit_item_discount;
            $allocation_items->amount = (float) $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
        }
        // END Discount
    }

    if ($total_purchase > 0) {
        //Purchase Accounts Dr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase'),
            // 'brand_id' => $mainTable->order_id,
            // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
            'narration' => 'Purchase #' . $mainTable->invoice_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->vendor->state,
            'amount' => (float) $total_purchase,
            'outlet' => $mainTable->outlet
        ));
    }
    if ($total_tax_amount > 0) {
        //VAT Dr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase_vat'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => 0,
            'narration' => 'Purchase #' . $mainTable->invoice_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->vendor->state,
            'amount' => (float) $total_tax_amount,
            'outlet' => $mainTable->outlet
        ));
    }

    if ($total_excise_amount > 0) {
        //VAT Dr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('excise_tax'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => 0,
            'narration' => 'Purchase #' . $mainTable->invoice_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->vendor->state,
            'amount' => (float) $total_excise_amount,
            'outlet' => $mainTable->outlet
        ));
    }

    FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
    $mainTable->posted = 'Y';
    $mainTable->save();
}

function purchasereturnPosting($invoice_id) {
    $mainTable = PurchaseReturn::model()->findByPk((int) $invoice_id);
    $outlet=$mainTable->outlet;
    $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="DN" AND outlet='.$outlet))->id;
    $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type='.$vouchertype.' AND header_table_id=' . $mainTable->id));
    if (!empty($fin_voucher)) {
        FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
        FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
    }
    $voucherHead['voucher_no'] = $mainTable->return_no;
    $voucherHead['voucher_date'] = $mainTable->return_date;
    $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
    $voucherHead['voucher_type'] = $vouchertype;
    // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
    $voucherHead['reference_no'] = $mainTable->reference_no;
    $voucherHead['header_table_id'] = $mainTable->id;
    $voucherHead['due_date'] = $mainTable->return_date;
    $voucherHead['outlet'] = $mainTable->outlet;
    $voucherHead['created_by'] = $mainTable->created_by;
    // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
    $valid = $this->isPostable($mainTable->return_date);

    $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

    $total_creditor_amount = $mainTable->grand_total;
    $total_purchase = $mainTable->total_amount;
    $total_tax_amount = $mainTable->tax_total;
    $total_excise_amount = $mainTable->excise_total;
    $total_discount_amount = $mainTable->discount;
    $total_received_amount = $mainTable->paid;

    if ($total_creditor_amount > 0) {
        //Creditor Cr
        $credit_item_purchase = $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
            'narration' => 'Purchase Return #' . $mainTable->return_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->pr_vendor->state,
            'amount' => (float) $total_creditor_amount,
            'outlet' => $mainTable->outlet
        ));
        if ($total_received_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="CA" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->return_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
            $voucherHead['voucher_type'] = $vouchertype;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->return_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->created_by;
            $lastSavedid_payment_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $credit_item_payment = $this->addVoucheritems($lastSavedid_payment_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => -1 * $total_received_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_payment_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => (float) $total_received_amount,
                'outlet' => $mainTable->outlet
            ));
            FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('total_voucher_amount' => ($total_creditor_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->return_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
            $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
            $allocation->amount = (float) $total_received_amount;
            $allocation->created_by = $mainTable->created_by;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save();

            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $credit_item_purchase;
            $allocation_items->amount = (float) $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save();
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_payment_voucher;
            $allocation_items->voucher_item_id = $credit_item_payment;
            $allocation_items->amount = -1 * $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save();
            FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('is_allocated' => 'Y'));
        }
        // Discount Start
        if ($total_discount_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="JV" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->return_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
            $voucherHead['voucher_type'] = 3;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->return_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->created_by;
            $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount'),
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => (float) $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->return_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
            $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
            $allocation->amount = (float) $total_discount_amount;
            $allocation->created_by = $mainTable->created_by;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save(false);
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $credit_item_purchase;
            $allocation_items->amount = (float) $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_discount_voucher;
            $allocation_items->voucher_item_id = $debit_item_discount;
            $allocation_items->amount = -1 * $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
        }
        // END Discount
    }

    if ($total_purchase > 0) {
        //Purchase Accounts Dr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase'),
            // 'brand_id' => $mainTable->order_id,
            // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
            'narration' => 'Purchase Return #' . $mainTable->return_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->pr_vendor->state,
            'amount' => -1 * $total_purchase,
            'outlet' => $mainTable->outlet
        ));
    }
    if ($total_tax_amount > 0) {
        //VAT Dr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase_vat'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => 0,
            'narration' => 'Purchase Return #' . $mainTable->return_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->pr_vendor->state,
            'amount' => -1 * $total_tax_amount,
            'outlet' => $mainTable->outlet
        ));
    }

    if ($total_excise_amount > 0) {
        //VAT Dr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('excise_tax'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => 0,
            'narration' => 'Purchase Return #' . $mainTable->return_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->pr_vendor->state,
            'amount' => -1 * $total_excise_amount,
            'outlet' => $mainTable->outlet
        ));
    }

    FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
    $mainTable->posted = 'Y';
    $mainTable->save();
}


function isPostable($date) {
    $last_closed_year = FinanceAuditClose::lastClosed();
    $voucherDate_array = explode('-', $date);
    $voucherYear = $voucherDate_array[0];
    return ($voucherYear <= $last_closed_year) ? false : true;
}

public function addVoucherhead($arr_items) {
    $model = new FinanceVoucherHead;
    $model->voucher_type = $arr_items['voucher_type'];
    $model->voucher_no = $arr_items['voucher_no'];
    $model->voucher_date = $arr_items['voucher_date'];
    $model->related_subledger = $arr_items['related_subledger'];
    $model->emp_subledger = $arr_items['emp_subledger'];
    $model->due_date = (isset($arr_items['due_date'])) ? $arr_items['due_date'] : $model->voucher_date;
    $model->header_table_id = $arr_items['header_table_id'];
    $model->reference_no = $arr_items['reference_no'];
    $model->is_posted = (isset($arr_items['is_posted'])) ? $arr_items['is_posted'] : 'Y';
    $model->created_by = $arr_items['created_by'];
    $model->created_datetime = date('Y-m-d h:i:s');
    $model->cheque_no = $arr_items['cheque_no'];
    $model->outlet = $arr_items['outlet'];
    $model->payment_method = $arr_items['payment_method'];
    return ($model->save(false)) ? $model->id : 0; //exception
}

public function addVoucheritems($headerId, $arr_items) {
    $model = new FinanceVoucherItems;
    $model->header_id = $headerId;
    $model->ledger_id = $arr_items['ledger_id'];
    $model->brand_id = (int) $arr_items['brand_id'];
    $model->emirate_id = $arr_items['emirate_id'];
    $model->subledger_id = $arr_items['subledger_id'];
    $model->narration = $arr_items['narration'];
    $model->amount = $arr_items['amount'];
    $model->currency_id = $arr_items['currency'] > 0 ? $arr_items['currency'] : FinanceLegder::baseCurrency_id();
    $model->exchange_rate = $arr_items['exchange_rate'] > 0 ? $arr_items['exchange_rate'] : 1;
    $model->amount = $arr_items['amount'];
    $model->reconciled_date = $arr_items['reconciled_date'];
    $cleared = $arr_items['opening_cheque_cleared'];
    if ($cleared == '')
        $cleared = 'N';
    $model->opening_cheque_cleared = $cleared;
    $reconciled = $arr_items['reconciled'];
    if ($reconciled == '')
        $reconciled = 'N';
    $model->reconciled = $reconciled;
    $model->outlet = $arr_items['outlet'];
    $model->save(false);
    return $model->id > 0 ? $model->id : 0;
}

function getSublegder($parnet_id, $subledger_type) {
    return FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $parnet_id . ' AND subledger_type_id=' . (int) $subledger_type))->id;
}




function salesreturnPosting($invoice_id) {
    $mainTable = SalesReturn::model()->findByPk((int) $invoice_id);
    $outlet=$mainTable->outlet;
    $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=10 AND header_table_id=' . $mainTable->id));
    if (!empty($fin_voucher)) {
        FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
        FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
    }
    $voucherHead['voucher_no'] = $mainTable->return_no;
    $voucherHead['voucher_date'] = $mainTable->return_date;
    $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
    $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="CN" AND outlet='.$outlet))->id;
    $voucherHead['voucher_type'] = $vouchertype;
    // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
    $voucherHead['reference_no'] = $mainTable->reference_no;
    $voucherHead['header_table_id'] = $mainTable->id;
    $voucherHead['due_date'] = $mainTable->return_date;
    $voucherHead['outlet'] = $mainTable->outlet;
    $voucherHead['created_by'] = $mainTable->created_by;
    // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
    $valid = $this->isPostable($mainTable->return_date);

    $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

    $total_debtor_amount = $mainTable->grand_total;
    $total_sales = $mainTable->total_amount;
    $total_tax_amount = $mainTable->tax_total;
    $total_discount_amount = $mainTable->discount;
    $total_received_amount = $mainTable->paid;
    if ($total_debtor_amount > 0) {
        //Debtor Dr
        $debit_item_sales = $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => $this->getSublegder($mainTable->customer_id, 1), //2-Creditors 1- debtors
            'narration' => 'Sales Return #' . $mainTable->return_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->sr_customer->state,
            'amount' => -1 * $total_debtor_amount,
            'outlet' => $mainTable->outlet
        ));

        if ($total_received_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="PV" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->return_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
            $voucherHead['voucher_type'] = $vouchertype;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->return_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->created_by;
            $lastSavedid_receipt_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $debit_item_receipt = $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_received_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => -1 * $total_received_amount,
                'outlet' => $mainTable->outlet
            ));
            FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->return_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
            $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
            $allocation->amount = (float) $total_received_amount;
            $allocation->created_by = $mainTable->created_by;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save(false);
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $debit_item_sales;
            $allocation_items->amount = (float) $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_receipt_voucher;
            $allocation_items->voucher_item_id = $debit_item_receipt;
            $allocation_items->amount = -1 * $total_received_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('is_allocated' => 'Y'));
        }
        // Discount Start
        if ($total_discount_amount > 0) {
            $vouchertype=FinanceVoucherType::model()->find(array('condition'=>'code="DA" AND outlet='.$outlet))->id;
            $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
            $voucherHead['voucher_date'] = $mainTable->return_date;
            $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
            $voucherHead['voucher_type'] = $vouchertype;
            // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
            $voucherHead['reference_no'] = $mainTable->reference_no;
            $voucherHead['header_table_id'] = $mainTable->id;
            $voucherHead['due_date'] = $mainTable->return_date;
            $voucherHead['outlet'] = $mainTable->outlet;
            $voucherHead['created_by'] = $mainTable->created_by;
            $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

            $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            $this->addVoucheritems($lastSavedid_discount_voucher, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'),
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => -1 * $total_discount_amount,
                'outlet' => $mainTable->outlet
            ));

            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
            //Auto Allocation
            $allocation = new FinanceAllocation;
            $allocation->allocation_date = $mainTable->return_date;
            $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
            $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
            $allocation->amount = (float) $total_discount_amount;
            $allocation->created_by = $mainTable->created_by;
            $allocation->created_date_time = date('Y-m-d h:i:s');
            $allocation->outlet = $mainTable->outlet;
            $allocation->save(false);
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid;
            $allocation_items->voucher_item_id = $debit_item_sales;
            $allocation_items->amount = -1 * $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
            $allocation_items = new FinanceAllocationItems;
            $allocation_items->header_id = $allocation->id;
            $allocation_items->voucher_id = $lastSavedid_discount_voucher;
            $allocation_items->voucher_item_id = $debit_item_discount;
            $allocation_items->amount = (float) $total_discount_amount;
            $allocation_items->outlet = $mainTable->outlet;
            $allocation_items->save(false);
            FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
        }
        // END Discount
    }

    if ($total_sales > 0) {
        //Sales Accounts Cr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales'),
            // 'brand_id' => $mainTable->order_id,
            // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
            'narration' => 'Sales Return #' . $mainTable->return_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->sr_customer->state,
            'amount' => (float) $total_sales,
            'outlet' => $mainTable->outlet
        ));
    }
    if ($total_tax_amount > 0) {
        //VAT Cr
        $this->addVoucheritems($lastSavedid, array(
            'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales_vat'),
            // 'brand_id' => $mainTable->order_id,
            'subledger_id' => 0,
            'narration' => 'Sales Return #' . $mainTable->return_no,
            'currency' => $mainTable->currency,
            'exchange_rate' => $mainTable->exchange_rate,
            'emirate_id' => (int) $mainTable->sr_customer->state,
            'amount' => (float) $total_tax_amount,
            'outlet' => $mainTable->outlet
        ));
    }

    FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
    $mainTable->posted = 'Y';
    $mainTable->save();
}

}