<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiVendorController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('GetList', 'VendordetailsById', 'BlockUnblockVendor', 'SaveVendor'),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }


    //changed
    public function actionGetList()
    {

        $input = file_get_contents('php://input');
        //error_log('ledgers');
        //error_log($input);
        $variables = json_decode($input, true);
        $party = (int) $variables['party'];
        $salesman = (int) $variables['salesman'];
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $items = array();
        $userModel = MasUsers::model()->findBytoken($token);

        //Common or all
        $count = 0;
        $items = array();
        $factor = 1;

        if ($userModel->id > 0) {

            $userModel->db();


            $condition = '1=1 ';
            if ($userModel->show_all_vendors) {
                $condition .= ' AND outlet=' . $userModel->outlet_id;
            } else {
                $condition .= 'AND (t.created_by = ' . $userModel->id . ')';
            }
            $items = Vendor::model()->findAll(array('condition' => $condition, 'order' => 'id DESC'));


            if (!empty($items)) {
                foreach ($items as $i => $each) {

                    $this->responseBody['data'][$i]['id'] = $each->id;
                    $this->responseBody['data'][$i]['name'] = $each->vendor_name;
                    $this->responseBody['data'][$i]['vendor_no'] = $each->vendor_no;
                    $this->responseBody['data'][$i]['credit_limit'] = $each->credit_limit;
                    $this->responseBody['data'][$i]['phone_no'] = $each->phone;
                    $this->responseBody['data'][$i]['mobile'] = $each->mobile;
                    $this->responseBody['data'][$i]['email'] = $each->email;
                    //current_balance
                    $current_credit = 0;
                    $vendor_subledger = FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $each->id . ' AND subledger_type_id=2'))->id;
                    $vendor_ledger = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                    if ($vendor_subledger > 0) {
                        $current_credit = Yii::app()->db->createCommand("select SUM(I.amount) from erp_finance_voucher_items I INNER JOIN erp_finance_voucher_head H ON H.id=I.header_id where H.is_posted='Y' AND ledger_id='$vendor_ledger' AND subledger_id='$vendor_subledger'")->queryScalar();
                    }
                    $this->responseBody['data'][$i]['current_balance'] = number_format((-1 * $current_credit), 2);
                    $this->responseBody['data'][$i]['currency'] = 'AED';
                    $this->responseBody['data'][$i]['payment_term'] = $each->payment_term;
                    $this->responseBody['data'][$i]['emirate'] = $each->state_detail->state_name;
                    $this->responseBody['data'][$i]['country'] = $each->country_detail->country_name;
                    $this->responseBody['data'][$i]['trn_no'] = $each->trn_no;
                    $this->responseBody['data'][$i]['billing_address'] = $each->address;
                    $this->responseBody['data'][$i]['location'] = $each->location;
                    $this->responseBody['data'][$i]['contact_person'] = $each->contact_person;

                    $count++;
                }
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionVendordetailsById()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        //error_log('ledger_details');
        //error_log($input);

        $variables = json_decode($input, true);
        $ledger_id = trim($variables['id']);

        if ($userModel->id > 0) {

            $userModel->db();

            if ($ledger_id > 0) {               //Model

                $model = Vendor::model()->findBypk($ledger_id);
                $this->responseBody['data']['id'] = $model->id;
                $this->responseBody['data']['name'] = $model->vendor_name;
                $this->responseBody['data']['alias'] = $model->vendor_no;
                $this->responseBody['data']['phone_no'] = $model->phone;
                $this->responseBody['data']['mobile'] = $model->mobile;
                $this->responseBody['data']['email'] = $model->email;
                $this->responseBody['data']['currency_id'] = 1;
                $this->responseBody['data']['currency_code'] = 'AED';
                $this->responseBody['data']['opening_balance'] = 0.00;
                $this->responseBody['data']['opening_date'] = '2019-01-01';
                $this->responseBody['data']['emirate_id'] = $model->state;
                $this->responseBody['data']['emirate_name'] = $model->state_detail->state_name;
                $this->responseBody['data']['country_id'] = $model->country;
                $this->responseBody['data']['country_name'] = $model->country_detail->country_name;
                $this->responseBody['data']['trn_no'] = $model->trn_no;
                $this->responseBody['data']['billing_address'] = $model->address;
                $this->responseBody['data']['credit_limit'] = $model->credit_limit;
                //$this->responseBody['data']['credit_days'] = $model->credit_days;
                $this->responseBody['data']['payment_term'] = $model->payment_term;
                $this->responseBody['data']['location'] = $model->location;
                $this->responseBody['data']['contact_person'] = $model->contact_person;


                $this->responseBody['count'] = 1;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Vendor undefined';
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }



    public function actionBlockUnblockVendor()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $ledger_id = (int) trim($variables['id']);
        $type =  trim($variables['type']);


        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {
            if ($ledger_id > 0 && !empty($type)) {
                $userModel->db();

                $id = $ledger_id;
                $model = Vendor::model()->findByPk($id);
                if ($type == 'block') {
                    $model->active = 'N';
                } else if ($type == 'unblock') {
                    $model->active = 'Y';
                }

                $model->save(false);
                $this->responseBody['id'] = $ledger_id;
                $this->responseBody['status'] = 'success';
                $this->responseBody['msg'] = ($type == 'block') ? 'Vendor Blocked Successfully' : 'Vendor Unblocked Successfully';
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Vendor undefined';
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }
    public function actionSaveVendor()
    {

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        //$ledgerData = $variables['data'];
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $other = array();
        if ($userModel->id > 0) {
            $userModel->db();
            $id = $variables['id'];
            $other = array();
            $currency = 1;
            $is_barcode_visible = $variables['is_barcode_visible'];
            $vendor_name = $variables['name'];
            $model = new Vendor();
            $modelformmaster = FormMaster::model()->find(array('condition' => 'outlet=' . $userModel->outlet_id . ' AND code="V"'));

            if ($id > 0) {
                $model = $model->findByPK($id);
                $scene = 'update';
                $change_type = 'UPDATED';
                $model->updated_by = $userModel->id;
                $model->updated_time = date('Y-m-d H:i:s');
            } else {
                $model->created_by = $userModel->id;
                $model->created_time = date('Y-m-d H:i:s');
                $scene = 'new';
                $change_type = 'CREATED';
                $model->vendor_no = $modelformmaster->next_number;
            }
            $model->vendor_name = trim($variables['vendor_name']);
            $model->contact_person = trim($variables['contact_person']);
            $model->address = trim($variables['address']);
            $model->country = $variables['country'];
            $model->state = $variables['state'];
            $model->location = $variables['location'];
            $model->payment_term =  $variables['payment_term'];
            $model->trn_no =  trim($variables['trn_no']);
            $model->phone = trim($variables['phone']);
            $model->mobile = trim($variables['mobile']);
            $model->email = trim($variables['email']);
            $model->credit_limit = (float) $variables['credit_limit'];
            $model->outlet = $userModel->outlet_id;
            $model->active = 'Y';
            $valid_currency = MasCurrency::model()->findBypk((int)$currency)->id > 0 ? true : false;
            if ($valid_currency == false)
                $other[] = 'Invalid currency';
            $valid = $model->validate();
            if ($valid) {
                $model->save();
                /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=2 AND parent_id=' . $model->id));
                if (empty($subledger)) {
                    $newItem = new FinanceSubledger;
                } else {
                    $newItem = FinanceSubledger::model()->findByPk($subledger->id);
                }
                $newItem->description = $model->vendor_name;
                $newItem->account_no = $model->vendor_no;
                if ($model->address != '') {
                    $newItem->address = $model->address;
                }
                if ($model->phone != '') {
                    $newItem->phone = $model->phone;
                }
                $newItem->subledger_type_id = 2;
                $newItem->parent_id = $model->id;
                $newItem->created_by = $model->created_by;
                $newItem->outlet = $userModel->outlet_id;
                $newItem->save(false);
                /////////////////////////////////////////////////////////////
                if ($scene == 'new') {
                    $next_no = $model->vendor_no + 1;
                    FormMaster::model()->updateByPk($modelformmaster->id, array('next_number' => $next_no));
                }
                // inserting to audit log
                AuditLog::saveApiAuditLogs(
                    $userModel->outlet_id,
                    $modelformmaster->id,
                    $userModel->id,
                    $change_type,
                    " with ID - {$model->id} "
                );
                // end audit log
                $this->responseBody['status'] = 'success';
                $this->responseBody['vendor_id'] = $model->id;
                $this->_bind;
            } else {
                $error1 = json_decode(CActiveForm::validate($model, '', false), true);
                $error = array_merge_recursive($error1);
                $arr_err = $other;
                foreach ($error as $key => $errval) {
                    $arr_err[] = $errval[0];
                }
                $err_msg = implode(',', $arr_err);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }
}
