<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApifinanceController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array(
                    'GetVoucherList',
                    'GetVoucherDetails',
                    'Savereceipt',
                    'subledgerlist',
                    'Paymentmethodlist',
                    'GetAllocationDataForReceipt',
                    'VoucherPrint',
                    'GetReceiptVoucherList',
                    'GetVouchersSummary',
                    'SaveExpenseVoucher',
                    'GetExpenseAccounts',
                    'Updatevoucher',
                    'Deleteallocation',
                    'Savepayment',
                    'GetallocationdataForPayment',
                    'SaveCashTransfer',
                   
                ),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function actionHave_session()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->with('outletDetails')->find(array('condition' => 't.token="' . $token . '" AND t.is_active="Y" AND t.access_type IN("SL", "CA","SM", "OM") AND outletDetails.is_active="Y"'));
        if ((int) $userModel->id > 0) {
            $this->responseBody['status'] = 'success';
            $this->responseBody['msg'] = 'Session is valid';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }




    public function actionGetVoucherDetails()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $voucher_id = (int) trim($variables['voucher_id']);
        if ($userModel->id > 0) {

            $userModel->db();


            if ($voucher_id > 0) {

                $id = $voucher_id;
                $voucherModel = FinanceVoucherHead::model()->findbypk($id);
                $debtor = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $vat_on_expense = FinanceSystemAccountsMapping::getSystemdefault('vat_on_expense');
                $discount = FinanceSystemAccountsMapping::getSystemdefault('discount_allowed');
                $subledger = FinanceVoucherItems::model()->find(array('condition' => 'header_id=' . $id . ' AND ledger_id=' . $debtor))->subledger->description;
                $vat = FinanceVoucherItems::model()->find(array('condition' => 'header_id=' . $id . ' AND ledger_id=' . $vat_on_expense))->amount;
                $discount_amount = FinanceVoucherItems::model()->find(array('condition' => 'header_id=' . $id . ' AND ledger_id=' . $discount))->amount;
                $prefix = $userModel->outletDetails->prefix;
                if ($prefix != '') {
                    $prefix = $prefix . '/';
                } else {
                    $prefix = '';
                }

                // ------------------------------
                $list = [];
                foreach ($voucherModel->items as $eachItem) {
                    $item_list[] = [
                        "voucher_item_list" => $eachItem['id'],
                        "ledger_id"      => $eachItem['ledger_id'],
                        "ledger_name"    => $eachItem['ledger']['description'],
                        "subledger_id"   => $eachItem['subledger_id'],
                        "subledger_name" => ((int)$eachItem['subledger_id'] > 0) ? $eachItem['subledger']['description'] : '',
                        "narration"      => $eachItem['narration'],
                        "amount"         =>  $this->format_currency($eachItem['amount']),
                        "debit"          => ($eachItem['amount'] > 0) ? $this->format_currency($eachItem['amount']) : 0,
                        "credit"         => ($eachItem['amount'] < 0) ? $this->format_currency($eachItem['amount']) : 0,
                    ];
                }

                $allocation_items = [];
                $allocations = FinanceAllocationItems::model()->with('parent')->findAll(array('condition' => 't.voucher_id=' . $voucherModel['id'], 'group' => 'header_id'));
                foreach ($allocations as $allocation) {
                    $allocation_items[] = [
                        "header_id" => $allocation['header_id'],
                        "allocation_item_id" => $allocation['id'],
                        "allocation_date" => date('d/m/Y', strtotime($allocation->parent->allocation_date)),
                        "total_allocation_amount" => number_format(round($allocation->parent->amount, 2), 2),
                        "allocated_amount" => number_format(round(abs($allocation->amount), 2), 2),
                    ];
                }

                // --------------------------------
                $this->responseBody['data']['voucher_type'] = $voucherModel->vtype->description;
                $this->responseBody['data']['payment_mode'] = $voucherModel->paymentmode->mode;
                $this->responseBody['data']['received_by'] = '';
                $this->responseBody['data']['voucher_no'] = $prefix . $voucherModel->vtype->code . '/' . $voucherModel->voucher_no;
                $this->responseBody['data']['parent_voucher_no'] = $voucherModel->voucher_no;
                $this->responseBody['data']['voucher_date'] = date('d/m/Y', strtotime($voucherModel->voucher_date));
                $this->responseBody['data']['company_name'] = $userModel->outletDetails->name;
                $this->responseBody['data']['company_adds'] = $userModel->outletDetails->address;
                $this->responseBody['data']['company_phone'] = $userModel->outletDetails->phone_no;
                $this->responseBody['data']['company_po_box'] = $userModel->outletDetails->po_box;
                $this->responseBody['data']['company_trn'] = $userModel->outletDetails->trn_no;
                $this->responseBody['data']['outlet'] = $userModel->outletDetails->name;
                $this->responseBody['data']['user'] = $userModel->full_name;
                $this->responseBody['data']['user_phone'] = $userModel->phone_no;
                $this->responseBody['data']['create_date_time'] = date('d/m/Y H:i:s', strtotime($voucherModel->voucher_date));
                $this->responseBody['data']['notes'] = $voucherModel->general_narration;
                $this->responseBody['data']['party'] = $subledger;
                $this->responseBody['data']['reference_no'] = $voucherModel->reference_no;
                $this->responseBody['data']['total'] = $this->format_currency($voucherModel->total_voucher_amount - abs($vat) - abs($discount_amount));
                $this->responseBody['data']['total_vat'] = $this->format_currency(abs($vat));
                $this->responseBody['data']['total_discount'] = $this->format_currency(abs($discount_amount));
                $this->responseBody['data']['grand_total'] = $voucherModel->total_voucher_amount;
                $this->responseBody['data']['print_receipt'] = $userModel->outletDetails->invoice_with_receipt_print == "Y" ? "true" : "false";
                $this->responseBody['count'] = 1;
                $this->responseBody['items'] = $item_list;
                $this->responseBody['allocation_items'] = $allocation_items;
                $this->_bind;
            } else {
                $this->responseBody['status'] = 'failed';
                $this->responseBody['msg'] = 'Invalid Voucher';
                $this->_bind;
            }
        } else {
            $this->_401;
        }
    }






    function invoicePosting($invoice_id)
    {
        $mainTable = SalesInvoice::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="SI" AND outlet=' . $outlet))->id;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->invoice_no;
        $voucherHead['voucher_date'] = $mainTable->invoice_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->invoice_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->sales_man;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->invoice_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_debtor_amount = $mainTable->grand_total;
        $total_sales = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->received;
        if ($total_debtor_amount > 0) {
            //Debtor Dr
            $debit_item_sales = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1), //2-Creditors 1- debtors
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => (float) $total_debtor_amount,
                'outlet' => $mainTable->outlet
            ));
            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="CA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                $voucherHead['payment_method'] = 1;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->sales_man;
                $lastSavedid_receipt_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_receipt = $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->sales_man;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_receipt_voucher;
                $allocation_items->voucher_item_id = $debit_item_receipt;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="DA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->sales_man;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => -1 * $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->customer->state,
                    'amount' => (float) $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->sales_man;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_sales > 0) {
            //Sales Accounts Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => -1 * $total_sales,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Sales #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->customer->state,
                'amount' => -1 * $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }


    function purchaseInvoicePosting($invoice_id)
    {
        $mainTable = PurchaseInvoice::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="PI" AND outlet=' . $outlet))->id;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->invoice_no;
        $voucherHead['voucher_date'] = $mainTable->invoice_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->invoice_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->created_by;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->invoice_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_creditor_amount = $mainTable->grand_total;
        $total_purchase = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_excise_amount = $mainTable->excise_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->received;

        if ($total_creditor_amount > 0) {
            //Creditor Cr
            $credit_item_purchase = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => -1 * $total_creditor_amount,
                'outlet' => $mainTable->outlet
            ));
            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="PV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_payment_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $credit_item_payment = $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save();

                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_payment_voucher;
                $allocation_items->voucher_item_id = $credit_item_payment;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="JV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->invoice_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->invoice_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase #' . $mainTable->invoice_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->vendor->state,
                    'amount' => -1 * $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->invoice_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_purchase > 0) {
            //Purchase Accounts Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => (float) $total_purchase,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => (float) $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        if ($total_excise_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('excise_tax'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase #' . $mainTable->invoice_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->vendor->state,
                'amount' => (float) $total_excise_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }

    function purchasereturnPosting($invoice_id)
    {
        $mainTable = PurchaseReturn::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="DN" AND outlet=' . $outlet))->id;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->return_no;
        $voucherHead['voucher_date'] = $mainTable->return_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->return_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->created_by;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->return_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_creditor_amount = $mainTable->grand_total;
        $total_purchase = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_excise_amount = $mainTable->excise_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->paid;

        if ($total_creditor_amount > 0) {
            //Creditor Cr
            $credit_item_purchase = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => (float) $total_creditor_amount,
                'outlet' => $mainTable->outlet
            ));
            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="CA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_payment_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $credit_item_payment = $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2), //2-Creditors 1- debtors
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_payment_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('total_voucher_amount' => ($total_creditor_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save();

                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_payment_voucher;
                $allocation_items->voucher_item_id = $credit_item_payment;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save();
                FinanceVoucherHead::model()->updatebypk($lastSavedid_payment_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="JV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->vendor_id, 2);
                $voucherHead['voucher_type'] = 3;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('creditors'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount'),
                    'subledger_id' => $this->getSublegder($mainTable->vendor_id, 2),
                    'narration' => 'Purchase Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->pr_vendor->state,
                    'amount' => (float) $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $allocation->subledger_id = $this->getSublegder($mainTable->vendor_id, 2);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $credit_item_purchase;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_purchase > 0) {
            //Purchase Accounts Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => -1 * $total_purchase,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('purchase_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => -1 * $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        if ($total_excise_amount > 0) {
            //VAT Dr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('excise_tax'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Purchase Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->pr_vendor->state,
                'amount' => -1 * $total_excise_amount,
                'outlet' => $mainTable->outlet
            ));
        }

        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }


    function isPostable($date)
    {
        $last_closed_year = FinanceAuditClose::lastClosed();
        $voucherDate_array = explode('-', $date);
        $voucherYear = $voucherDate_array[0];
        return ($voucherYear <= $last_closed_year) ? false : true;
    }

    public function addVoucherhead($arr_items)
    {
        $model = new FinanceVoucherHead;
        $model->voucher_type = $arr_items['voucher_type'];
        $model->voucher_no = $arr_items['voucher_no'];
        $model->voucher_date = $arr_items['voucher_date'];
        $model->related_subledger = $arr_items['related_subledger'];
        $model->emp_subledger = $arr_items['emp_subledger'];
        $model->due_date = (isset($arr_items['due_date'])) ? $arr_items['due_date'] : $model->voucher_date;
        $model->header_table_id = $arr_items['header_table_id'];
        $model->reference_no = $arr_items['reference_no'];
        $model->is_posted = (isset($arr_items['is_posted'])) ? $arr_items['is_posted'] : 'Y';
        $model->created_by = $arr_items['created_by'];
        $model->created_datetime = date('Y-m-d h:i:s');
        $model->cheque_no = $arr_items['cheque_no'];
        $model->outlet = $arr_items['outlet'];
        $model->payment_method = $arr_items['payment_method'];
        return ($model->save(false)) ? $model->id : 0; //exception
    }

    public function addVoucheritems($headerId, $arr_items)
    {
        $model = new FinanceVoucherItems;
        $model->header_id = $headerId;
        $model->ledger_id = $arr_items['ledger_id'];
        $model->brand_id = (int) $arr_items['brand_id'];
        $model->emirate_id = $arr_items['emirate_id'];
        $model->subledger_id = $arr_items['subledger_id'];
        $model->narration = $arr_items['narration'];
        $model->amount = $arr_items['amount'];
        $model->currency_id = $arr_items['currency'] > 0 ? $arr_items['currency'] : FinanceLegder::baseCurrency_id();
        $model->exchange_rate = $arr_items['exchange_rate'] > 0 ? $arr_items['exchange_rate'] : 1;
        $model->amount = $arr_items['amount'];
        $model->reconciled_date = $arr_items['reconciled_date'];
        $cleared = $arr_items['opening_cheque_cleared'];
        if ($cleared == '')
            $cleared = 'N';
        $model->opening_cheque_cleared = $cleared;
        $reconciled = $arr_items['reconciled'];
        if ($reconciled == '')
            $reconciled = 'N';
        $model->reconciled = $reconciled;
        $model->outlet = $arr_items['outlet'];
        $model->save(false);
        return $model->id > 0 ? $model->id : 0;
    }

    function getSublegder($parnet_id, $subledger_type)
    {
        return FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $parnet_id . ' AND subledger_type_id=' . (int) $subledger_type))->id;
    }

    function salesreturnPosting($invoice_id)
    {
        $mainTable = SalesReturn::model()->findByPk((int) $invoice_id);
        $outlet = $mainTable->outlet;
        $fin_voucher = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=10 AND header_table_id=' . $mainTable->id));
        if (!empty($fin_voucher)) {
            FinanceVoucherItems::model()->deleteAll(array('condition' => 'header_id=' . $fin_voucher->id));
            FinanceVoucherHead::model()->deleteByPk($fin_voucher->id);
        }
        $voucherHead['voucher_no'] = $mainTable->return_no;
        $voucherHead['voucher_date'] = $mainTable->return_date;
        $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
        $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="CN" AND outlet=' . $outlet))->id;
        $voucherHead['voucher_type'] = $vouchertype;
        // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
        $voucherHead['reference_no'] = $mainTable->reference_no;
        $voucherHead['header_table_id'] = $mainTable->id;
        $voucherHead['due_date'] = $mainTable->return_date;
        $voucherHead['outlet'] = $mainTable->outlet;
        $voucherHead['created_by'] = $mainTable->created_by;
        // $voucherHead['due_date'] = date('Y-m-d', strtotime($mainTable->invoice_date . ' + ' . (int) $mainTable->paymentTerm->no_of_days . ' days'));
        $valid = $this->isPostable($mainTable->return_date);

        $lastSavedid = $valid ? $this->addVoucherhead($voucherHead) : 0;

        $total_debtor_amount = $mainTable->grand_total;
        $total_sales = $mainTable->total_amount;
        $total_tax_amount = $mainTable->tax_total;
        $total_discount_amount = $mainTable->discount;
        $total_received_amount = $mainTable->paid;
        if ($total_debtor_amount > 0) {
            //Debtor Dr
            $debit_item_sales = $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => $this->getSublegder($mainTable->customer_id, 1), //2-Creditors 1- debtors
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => -1 * $total_debtor_amount,
                'outlet' => $mainTable->outlet
            ));

            if ($total_received_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="PV" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_receipt_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_receipt = $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => (float) $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_receipt_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('cash_account'),
                    'subledger_id' => MasUsers::model()->findByPk($mainTable->created_by)->subledger_id,
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => -1 * $total_received_amount,
                    'outlet' => $mainTable->outlet
                ));
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('total_voucher_amount' => ($total_received_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_received_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = (float) $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_receipt_voucher;
                $allocation_items->voucher_item_id = $debit_item_receipt;
                $allocation_items->amount = -1 * $total_received_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_receipt_voucher, array('is_allocated' => 'Y'));
            }
            // Discount Start
            if ($total_discount_amount > 0) {
                $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="DA" AND outlet=' . $outlet))->id;
                $voucherHead['voucher_no'] = FinanceVoucherType::getVoucherno($vouchertype, true);
                $voucherHead['voucher_date'] = $mainTable->return_date;
                $voucherHead['related_subledger'] = $this->getSublegder($mainTable->customer_id, 1);
                $voucherHead['voucher_type'] = $vouchertype;
                // $voucherHead['emp_subledger'] = getSublegder($mainTable->lead_owner_id, 3);
                $voucherHead['reference_no'] = $mainTable->reference_no;
                $voucherHead['header_table_id'] = $mainTable->id;
                $voucherHead['due_date'] = $mainTable->return_date;
                $voucherHead['outlet'] = $mainTable->outlet;
                $voucherHead['created_by'] = $mainTable->created_by;
                $lastSavedid_discount_voucher = $valid ? $this->addVoucherhead($voucherHead) : 0;

                $debit_item_discount = $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('debtors'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => (float) $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                $this->addVoucheritems($lastSavedid_discount_voucher, array(
                    'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('discount_allowed'),
                    'subledger_id' => $this->getSublegder($mainTable->customer_id, 1),
                    'narration' => 'Sales Return #' . $mainTable->return_no,
                    'currency' => $mainTable->currency,
                    'exchange_rate' => $mainTable->exchange_rate,
                    'emirate_id' => (int) $mainTable->sr_customer->state,
                    'amount' => -1 * $total_discount_amount,
                    'outlet' => $mainTable->outlet
                ));

                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('total_voucher_amount' => ($total_discount_amount), 'cheque_cleared' => 'Y')); //$netTotal
                //Auto Allocation
                $allocation = new FinanceAllocation;
                $allocation->allocation_date = $mainTable->return_date;
                $allocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $allocation->subledger_id = $this->getSublegder($mainTable->customer_id, 1);
                $allocation->amount = (float) $total_discount_amount;
                $allocation->created_by = $mainTable->created_by;
                $allocation->created_date_time = date('Y-m-d h:i:s');
                $allocation->outlet = $mainTable->outlet;
                $allocation->save(false);
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid;
                $allocation_items->voucher_item_id = $debit_item_sales;
                $allocation_items->amount = -1 * $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid, array('is_allocated' => 'Y'));
                $allocation_items = new FinanceAllocationItems;
                $allocation_items->header_id = $allocation->id;
                $allocation_items->voucher_id = $lastSavedid_discount_voucher;
                $allocation_items->voucher_item_id = $debit_item_discount;
                $allocation_items->amount = (float) $total_discount_amount;
                $allocation_items->outlet = $mainTable->outlet;
                $allocation_items->save(false);
                FinanceVoucherHead::model()->updatebypk($lastSavedid_discount_voucher, array('is_allocated' => 'Y'));
            }
            // END Discount
        }

        if ($total_sales > 0) {
            //Sales Accounts Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales'),
                // 'brand_id' => $mainTable->order_id,
                // 'subledger_id' => $this->getSublegder($mainTable->lead_owner_id, 3),
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_sales,
                'outlet' => $mainTable->outlet
            ));
        }
        if ($total_tax_amount > 0) {
            //VAT Cr
            $this->addVoucheritems($lastSavedid, array(
                'ledger_id' => FinanceSystemAccountsMapping::getSystemdefault('sales_vat'),
                // 'brand_id' => $mainTable->order_id,
                'subledger_id' => 0,
                'narration' => 'Sales Return #' . $mainTable->return_no,
                'currency' => $mainTable->currency,
                'exchange_rate' => $mainTable->exchange_rate,
                'emirate_id' => (int) $mainTable->sr_customer->state,
                'amount' => (float) $total_tax_amount,
                'outlet' => $mainTable->outlet
            ));
        }


        FinanceVoucherHead::model()->updatebypk($lastSavedid, array('total_voucher_amount' => ($mainTable->grand_total), 'cheque_cleared' => 'Y')); //$netTotal
        $mainTable->posted = 'Y';
        $mainTable->save();
    }



    public function actionGetVoucherList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $voucher_type =  trim($variables['voucher_type']);

        $from_date = ($variables['from_date'] != "") ? $variables['from_date'] : date('Y-m-01');
        $to_date = ($variables['to_date'] != "") ? $variables['to_date'] : date('Y-m-t');
        $term = trim($variables['term']);
        $from_type =  trim($variables['from_type']);
        if ($userModel->id > 0) {
            $userModel->db();
            $count = 0;
            $condition = "1=1";

            if ($from_type == 'receipt') {
                $modelFinanceVoucherType = new FinanceVoucherType();
                $voucherType_1 =  FinanceVoucherType::getApiVoucherId("CHQ", $userModel->outlet_id);
                $voucherType_2 =  FinanceVoucherType::getApiVoucherId("CA", $userModel->outlet_id);
                $voucher_type_id = $modelFinanceVoucherType->find(array('condition' => 'code="' . $voucher_type . '"'))->id;
                $condition .= " AND voucher_type_id IN(" . $voucherType_1 . "," . $voucherType_2 . ")";
            } else  if ($from_type == 'expense') {
                $voucherType =  FinanceVoucherType::getApiVoucherId("JV", $userModel->outlet_id);
                $condition .= " AND voucher_type_id IN(" . $voucherType . ")";
            } else {
                $condition .= " AND 1=2";
            }

            if ((int)$userModel->outlet_id > 0) {
                $condition .= " AND outlet =" . $userModel->outlet_id;
            }


            $condition .= ' AND code != "YC" AND voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';


            if ($term != "") {
                $condition .= ' AND ( reference_no LIKE "%' . $term . '%" OR 
                            voucher_no LIKE "%' . $term . '%" OR
                             narration LIKE "%' . $term . '%" OR 
                             subledger LIKE "%' . $term . '%" OR 
                             ledger LIKE "%' . $term . '%" )';
            }

            $model = Yii::app()->db->createCommand('SELECT H.*, O.outlet_name, U.full_name 
                FROM {{finance_voucher_head_list}} H 
                INNER JOIN bluesell_new_main.{{outlets}} O ON H.outlet = O.id 
                INNER JOIN bluesell_new_main.erp_users U ON H.created_by = U.id 
                WHERE ' . $condition . ' 
                ORDER BY H.id DESC')->queryAll();

            foreach ($model as $i => $each) {
                $this->responseBody['data'][$i]['voucher_id'] = $each['id'];
                $this->responseBody['data'][$i]['voucher_no'] = $each['voucher_no'];
                $this->responseBody['data'][$i]['voucher_date'] = date('d/m/Y', strtotime($each['voucher_date']));
                $this->responseBody['data'][$i]['ledger'] = $each['ledger'];
                $this->responseBody['data'][$i]['subledger'] = $each['subledger'];
                $this->responseBody['data'][$i]['reference_no'] = $each['reference_no'];
                $this->responseBody['data'][$i]['narration'] = $each['narration'];
                $this->responseBody['data'][$i]['total_voucher_amount'] = Yii::app()->numberFormatter->formatCurrency($each['total_voucher_amount'], " ");
                $this->responseBody['data'][$i]['outlet_name'] = $each['outlet_name'];
                $this->responseBody['data'][$i]['full_name'] = $each['full_name'];
                //$this->responseBody['count'] = 1;

                $count++;
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }





    public function actionsubledgerlist()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $term = trim($variables['term']);
        if ($userModel->id > 0) {
            $userModel->db();
            $ledger = FinanceSystemAccountsMapping::getSystemdefault('debtors');
            $outlet = (int)$userModel->outlet_id;
            $count = 0;
            if ($ledger > 0) {
                $legderModel = FinanceLegder::model()->findBypk($ledger);
                $sub_types =  $legderModel->subledger_types;
                if ($sub_types != '') {
                    if (!empty($term)) {
                        $subledgers = FinanceSubledger::model()->findAll(array('select' => 'account_no, description, id', 'condition' => '( account_no LIKE "%' . $term . '%" OR description LIKE "%' . $term . '%" ) AND outlet=' . $outlet . ' AND subledger_type_id IN (' . $sub_types . ')', 'limit' => 10));
                    } else {
                        $subledgers = FinanceSubledger::model()->findAll(array('select' => 'account_no, description, id', 'condition' => 'outlet=' . $outlet . ' AND subledger_type_id IN (' . $sub_types . ')', 'limit' => 10));
                    }

                    foreach ($subledgers as $i => $each) {
                        $ledgers = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.is_posted='Y' AND I.ledger_id=" . $ledger . " AND I.subledger_id=" . $each['id'])->queryScalar();
                        $this->responseBody['data'][$i]['label'] = $each['account_no'] . ' : ' . $each['description'] . ' : AED ' . (float)$ledgers;
                        $this->responseBody['data'][$i]['value'] = $each['id'];
                        $count++;
                    }
                }
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionPaymentmethodlist()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $term = trim($variables['term']);
        $id   = trim($variables['id']);
        if ($userModel->id > 0) {
            $userModel->db();
            $outlet = (int)$userModel->outlet_id;
            $count = 0;
            $model = new FinancePaymentMode();

            $condition    = 'isActive="Y"';
            if ((int)$id > 0) {
                $condition    .= ' AND id="' . $id . '"';
            }
            $model = $model->findAll(array('condition' => $condition));
            $count = 0;
            foreach ($model as $i => $each) {
                $ledger_name = '';
                if ($each['ledger_id'] > 0) {
                    $ledger_name = FinanceLegder::model()->findBypk($each['ledger_id'])->description;
                }
                $this->responseBody['data'][$count]['id']          = $each['id'];
                $this->responseBody['data'][$count]['mode']        = $each['mode'];
                $this->responseBody['data'][$count]['code']        = $each['code'];
                $this->responseBody['data'][$count]['ledger_id']   = $each['ledger_id'];
                $this->responseBody['data'][$count]['ledger_name'] = $ledger_name;
                $this->responseBody['data'][$count]['bank_name']   = $each['bank_name'];
                $this->responseBody['data'][$count]['cheque_no']   = $each['cheque_no'];
                $this->responseBody['data'][$count]['card_no']     = $each['card_no'];
                $this->responseBody['data'][$count]['cheque_date'] = $each['cheque_date'];
                $count++;
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }




    public function actionGetAllocationDataForReceipt()
    {


        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        if ($userModel->id > 0) {
            $userModel->db();
            $outlet = (int)$userModel->outlet_id;
            $count = 0;
            $ledger = FinanceSystemAccountsMapping::getSystemdefault('debtors');
            $subledger = (int) $variables['subledger_id'];
            $allocationDebit = 'CREATE OR REPLACE VIEW erp_finance_allocated_debit AS SELECT
								V.id AS voucher_id,
								VI.id AS voucher_item_id,
								V.voucher_no, VI.amount,
								IFNULL(ABS(SUM(AI.amount)), 0) AS allocated,
								(VI.amount*VI.exchange_rate) - IFNULL(ABS(SUM(AI.amount)), 0) AS balance
							FROM erp_finance_voucher_items VI
							LEFT JOIN erp_finance_voucher_head V ON V.id = VI.header_id
							LEFT JOIN erp_finance_allocation_items AI ON AI.voucher_item_id = VI.id
							WHERE V.is_posted ="Y" AND VI.amount > 0 AND VI.ledger_id = ' . $ledger . ' AND VI.subledger_id = ' . $subledger . '
							GROUP BY VI.id';

            Yii::app()->db->createCommand($allocationDebit)->execute();

            $debitAllocatables = Yii::app()->db->createCommand('SELECT GROUP_CONCAT(voucher_item_id) AS value FROM erp_finance_allocated_debit WHERE balance > 0')->queryAll();

            $count = 0;
            if ($debitAllocatables[0]['value'] != '') {
                $allItems = FinanceVoucherItems::model()->findAll(array('condition' => 'id IN (' . $debitAllocatables[0]['value'] . ')'));

                foreach ($allItems as $i => $each) {
                    $balance = Yii::app()->db->createCommand('SELECT balance AS balance FROM erp_finance_allocated_debit WHERE voucher_item_id=' . $each->id)->queryScalar();


                    $this->responseBody['data'][$count]['voucher_no'] =  $each->parent->fullvoucher;
                    $this->responseBody['data'][$count]['type'] = $each->parent->vtype->description;
                    $this->responseBody['data'][$count]['date'] = date('d/m/Y', strtotime($each->parent->voucher_date));
                    $this->responseBody['data'][$count]['balance'] =  number_format((float)$balance, 2, '.', ''); 
                    $this->responseBody['data'][$count]['voucher_item_id'] = $each->id;
                    $this->responseBody['data'][$count]['voucher_id'] = $each->header_id;
                    $this->responseBody['data'][$count]['selected_amount'] = 0;
                    $this->responseBody['data'][$count]['selected_discount_amount'] = 0;
                    $this->responseBody['data'][$count]['selected'] =  false;
                    $count++;
                }
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionSavereceipt()
    {


        $other = array();
        $currency = 1;
        $exchange_rate = 1;
        function savableDate($date)
        {
            $last_closed_year = FinanceAuditClose::lastClosed();
            $voucherDate_array = explode('-', $date);
            $voucherYear = $voucherDate_array[0];
            return ($voucherYear <= $last_closed_year) ? false : true;
        }
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);


        //        $myfile = fopen("payment_file.txt", "w") or die("Unable to open file!");
        //        $txt = "json data $variables\n";
        //        fwrite($myfile, $txt);
        //        fclose($myfile);






        $receipt_date = (count(explode('/', trim($variables['receipt_date']))) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', trim($variables['receipt_date'])))) : date('Y-m-d');
        $receipt_ledger = (int) trim($variables['receipt_ledger']);
        //        $mode_of_payment = trim($variables['mode']);
        $mode_of_payment = 'ca';
        //  $customer = (int) trim($variables['ledger_id']); //customer_subleger
        $subledger_id = (int) trim($variables['customer_subledger_id']); //customer_subleger
        $id = (int) trim($variables['id']);
        $cheque_no = trim($variables['cheque_no']);
        $cheque_date = trim($variables['cheque_date']);
        $receipt_bank = trim($variables['receipt_bank']);
        $narration = trim($variables['narration']);
        $reference_no = trim($variables['reference_no']);
        $payment_method = (int)$variables['payment_method'];
        $receipt_account_ledger = $variables['receipt_account_ledger'];
        $items = $variables['items'];
        if (trim($receipt_date) == '') {
            $other[] = 'Please choose receipt date ';
        }
        if ($userModel->id > 0) {

            $userModel->db();

            $invoice = new SalesInvoice;
            if ($id > 0) {
                $invoice = $invoice->findBypk($id);
            }
            $total_amount = 0;
            $total_received = 0;
            $total_discount = 0;
            foreach ($items as $each) {
                if ($each['selected_amount'] != 0 && $each['selected'] == true) {
                    $total_received += $each['selected_amount'];
                    $total_discount += $each['selected_discount_amount'];
                }
            }
            $total_amount = $total_received ;
            if ((float)$total_amount == 0) {
                $other[] = 'Please check the amounts';
            }
            /*if ((int)$customer == 0) {
                $other[] = 'Please choose customer';
            }*/
            if ((int)$subledger_id == 0) {
                $other[] = 'Please choose Customer';
            }
            if ((int)$subledger_id == 0) {
                $other[] = 'Please choose customer';
            }
            if ((int)$payment_method == 0) {
                $other[] = 'Please choose Mode of Receipt';
            }

            $modelFinancePaymentMode = FinancePaymentMode::model()->findBypk((int)$payment_method);


            $model = new FinanceVoucherHead;
            $model->general_narration = $narration;
            $model->is_posted = 'Y';
            $model->created_by = $userModel->id;
            $model->created_datetime = date('Y-m-d h:i:s');
            $model->outlet = $userModel->outlet_id;
            $model->voucher_date = $receipt_date;
            $model->due_date = $receipt_date;
            $model->reference_no = $reference_no;
            $model->payment_method = $payment_method;
            $savableVoucher = savableDate($model->voucher_date);

            /*if ($modelFinancePaymentMode->code == 'CHQ' || $modelFinancePaymentMode->code == 'BANK' || $modelFinancePaymentMode->code == 'CH') {
                $voucherType =  FinanceVoucherType::getApiVoucherId("CHQ", $userModel->outlet_id);
            } else {
                $voucherType =  FinanceVoucherType::getApiVoucherId("CA", $userModel->outlet_id);
            }*/
            
            $voucherType = FinanceVoucherType::model()->find(array('condition' => 'code="CA" AND outlet=' . $userModel->outlet_id))->id;

            $model->total_voucher_amount = $total_amount;
            $voucherModel = FinanceVoucherType::model()->findBypk((int) $voucherType);
            $model->voucher_type = $voucherType;
            $model->voucher_no = $voucherModel->last_voucher_no + 1;
            // ------------------------------------------------


            /* if($modelFinancePaymentMode->bank_name=='Y'){
                 $other[] = 'Please add Bank Name';
            }
            if($modelFinancePaymentMode->cheque_no=='Y'){
                 $other[] = 'Please add cheque number';
            }
            if($modelFinancePaymentMode->card_no=='Y'){
                 $other[] = 'Please choose Card number ';
            }
            if($modelFinancePaymentMode->cheque_date=='Y'){
                  $other[] = 'Please choose cheque date ';
            }*/

            // ------------------------------------------------

            if ($modelFinancePaymentMode->code == 'CA') //card
                $model->scenario = 'card';
            else if ($modelFinancePaymentMode->code == 'CHQ') { //cheque
                $model->scenario = 'chequepaid';
                $model->realisation_date = (count(explode('/', $cheque_date)) == 3) ? date('Y-m-d', strtotime(str_replace('/', '-', $cheque_date))) : date('Y-m-d');
                $model->cheque_date = (count(explode('/', $cheque_date)) == 3) ? date('Y-m-d', strtotime(str_replace('/', '-', $cheque_date))) : date('Y-m-d');
                $model->cheque_cleared = 'N';
            } else if ($modelFinancePaymentMode->code == 'BANK') {
                //Bank
                $model->scenario = 'bank';
            } else { //Cash
                $model->scenario = 'cash';
            }
            $model->total_voucher_amount = $total_amount;
            if ($savableVoucher != true) {
                $other[] = 'Year Closed';
            }
            if (empty($other)) {

                $paymentmode_subledger_id = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=3 AND parent_id=' . $userModel->id))->id;



                $model->is_allocated = 'Y';
                $model->save(false);
                $voucherModel->last_voucher_no += 1;
                $voucherModel->save();
                $modelItems = new FinanceVoucherItems;
                $modelItems->outlet = $userModel->outlet_id;
                $modelItems->header_id = $model->id;
                $modelItems->ledger_id = $modelFinancePaymentMode->ledger_id;
                $modelItems->subledger_id = $paymentmode_subledger_id;
                $modelItems->narration = $narration;
                $modelItems->amount = $total_received;
                $modelItems->currency_id = $currency;
                $modelItems->exchange_rate = $exchange_rate;
                $modelItems->save(false);

                //  $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $customer))->id;


                $subledger = $subledger_id;

                $modelItemsDiscount = new FinanceVoucherItems;
                $modelItemsDiscount->outlet = $userModel->outlet_id;
                $modelItemsDiscount->header_id = $model->id;
                $modelItemsDiscount->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('discount_allowed');
                $modelItemsDiscount->narration = $narration;
                $modelItemsDiscount->amount = $total_discount;
                $modelItemsDiscount->currency_id = $currency;
                $modelItemsDiscount->exchange_rate = $exchange_rate;
                $modelItemsDiscount->save(false);

                $modelItemsCreditor = new FinanceVoucherItems;
                $modelItemsCreditor->outlet = $userModel->outlet_id;
                $modelItemsCreditor->header_id = $model->id;
                $modelItemsCreditor->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $modelItemsCreditor->scenario = 'havesubledger';
                $modelItemsCreditor->subledger_id = $subledger;
                $modelItemsCreditor->narration = $narration;
                $modelItemsCreditor->amount = -1 * $total_amount;
                $modelItemsCreditor->currency_id = $currency;
                $modelItemsCreditor->exchange_rate = $exchange_rate;
                $modelItemsCreditor->save(false);

                $modelAllocation = new FinanceAllocation;
                $modelAllocation->allocation_date = date('Y-m-d');
                $modelAllocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $modelAllocation->subledger_id = $subledger;
                $modelAllocation->amount = abs($total_amount);
                $modelAllocation->outlet = $userModel->outlet_id;
                $modelAllocation->created_by = $userModel->id;
                $modelAllocation->created_date_time = date('Y-m-d h:i:s');
                $modelAllocation->save();
                foreach ($items as $each) {
                    if ($each['selected_amount'] != 0 && $each['selected'] == true) {
                        $debitAllocation = new FinanceAllocationItems;
                        $debitAllocation->header_id = $modelAllocation->id;
                        $debitAllocation->voucher_id = $each['voucher_id'];
                        $debitAllocation->voucher_item_id = $each['voucher_item_id'];
                        $debitAllocation->amount = ($each['selected_amount'] + $each['selected_discount_amount']);
                        $debitAllocation->outlet = $userModel->outlet_id;
                        $debitAllocation->save();
                    }
                }
                $creditAllocation = new FinanceAllocationItems;
                $creditAllocation->header_id = $modelAllocation->id;
                $creditAllocation->voucher_id = $model->id;
                $creditAllocation->voucher_item_id = $modelItemsCreditor->id;
                $creditAllocation->amount = -1 * $total_amount;
                $creditAllocation->outlet = $userModel->outlet_id;
                $creditAllocation->save();
                $this->responseBody['status'] = 'success';
                $this->responseBody['voucher_id'] = $model->id;
                $this->responseBody['type'] = 'receipt';
                $this->_bind;
            } else {
                //$error = CJSON::encode(array_merge_recursive($other));
                $err_msg = implode(',', $other);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }



    public function actionVoucherPrint()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);

            $id = (int) $variables['id'];
            $model = FinanceVoucherHead::model()->findBypk($id);
            $outlet = (int) $userModel->outlet_id;


            $path = "app_reports/";
            $report_type = $model->vtype->pdf_layout;
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);

            $mpdf->WriteHTML(
                $this->renderPartial(
                    'application.modules.finance.views.core.' . $model->vtype->pdf_layout,
                    array(
                        'title' => $params['title'],
                        'logo' => $userModel->companyDetails->pdf_logo,
                        'model' => $model,
                        'outlet' => $outlet,
                        'id' => $id,
                        'userModel' => $userModel,
                        'user_id' => $userModel->id
                    ),
                    true
                )
            );


            // $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.$model->vtype->pdf_layout', array('title' => $params['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $params, 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionGetReceiptVoucherList()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $voucher_type =  trim($variables['voucher_type']);

        $from_date = ($variables['from_date'] != "") ? $variables['from_date'] : date('Y-m-01');
        $to_date = ($variables['to_date'] != "") ? $variables['to_date'] : date('Y-m-t');
        $term = trim($variables['term']);
        if ($userModel->id > 0) {
            $userModel->db();
            $count = 0;
            $condition = "1=1";

            $modelFinanceVoucherType = new FinanceVoucherType();
            $voucherType_1 =  FinanceVoucherType::getApiVoucherId("CHQ", $userModel->outlet_id);
            $voucherType_2 =  FinanceVoucherType::getApiVoucherId("CA", $userModel->outlet_id);
            /*  
            if (!empty($voucher_type)) {

                $voucher_type_id = $modelFinanceVoucherType->find(array('condition' => 'code="' . $voucher_type . '"'))->id;
            }
*/

            //  if ((int)$voucher_type_id > 0) {
            $condition .= " AND voucher_type_id IN(" . $voucherType_1 . "," . $voucherType_2 . ")";

            // }

            if ((int)$userModel->outlet_id > 0) {
                $condition .= " AND outlet =" . $userModel->outlet_id;
            }


            $condition .= ' AND code != "YC" AND voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';


            if ($term != "") {
                $condition .= ' AND ( reference_no LIKE "%' . $term . '%" OR 
                            voucher_no LIKE "%' . $term . '%" OR
                             narration LIKE "%' . $term . '%" OR 
                             subledger LIKE "%' . $term . '%" OR 
                             ledger LIKE "%' . $term . '%" )';
            }

            $model = Yii::app()->db->createCommand('SELECT H.*, O.outlet_name, U.full_name 
                FROM {{finance_voucher_head_list}} H 
                INNER JOIN bluesell_new_main.{{outlets}} O ON H.outlet = O.id 
                INNER JOIN bluesell_new_main.erp_users U ON H.created_by = U.id 
                WHERE ' . $condition . ' 
                ORDER BY H.id DESC')->queryAll();

            foreach ($model as $i => $each) {
                $this->responseBody['data'][$i]['voucher_id'] = $each['id'];
                $this->responseBody['data'][$i]['voucher_no'] = $each['voucher_no'];
                $this->responseBody['data'][$i]['voucher_date'] = date('d/m/Y', strtotime($each['voucher_date']));
                $this->responseBody['data'][$i]['ledger'] = $each['ledger'];
                $this->responseBody['data'][$i]['subledger'] = $each['subledger'];
                $this->responseBody['data'][$i]['reference_no'] = $each['reference_no'];
                $this->responseBody['data'][$i]['narration'] = $each['narration'];
                $this->responseBody['data'][$i]['total_voucher_amount'] = Yii::app()->numberFormatter->formatCurrency($each['total_voucher_amount'], " ");
                $this->responseBody['data'][$i]['outlet_name'] = $each['outlet_name'];
                $this->responseBody['data'][$i]['full_name'] = $each['full_name'];
                //$this->responseBody['count'] = 1;

                $count++;
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionGetVouchersSummary()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        $voucher_type =  trim($variables['voucher_type']);

        $from_date = ($variables['from_date'] != "") ? $variables['from_date'] : date('Y-m-01');
        $to_date = ($variables['to_date'] != "") ? $variables['to_date'] : date('Y-m-t');
        $term = trim($variables['term']);
        if ($userModel->id > 0) {
            $userModel->db();

            // ----------------------------
            $totalCash = 0;
            $totalBank = 0;
            $totalReceivable = 0;
            $totalPayable = 0;
            $sales_invoices = 0;
            $date = date('Y-m-d');
            if ($userModel->access_type == 'CA') {
                $bank = FinanceSystemAccountsMapping::getSystemdefault('bank');
                $totalBank = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.voucher_date<='" . $date . "' AND H.is_posted='Y' AND I.ledger_id=" . $bank . " AND H.outlet=" . $userModel->outlet_id)->queryScalar();
                $debtors = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $totalReceivable = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.voucher_date<='" . $date . "' AND H.is_posted='Y' AND I.ledger_id=" . $debtors . " AND H.outlet=" . $userModel->outlet_id)->queryScalar();
                $creditors = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $totalPayable = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.voucher_date<='" . $date . "' AND H.is_posted='Y' AND I.ledger_id=" . $creditors . " AND H.outlet=" . $userModel->outlet_id)->queryScalar();
            }
            $cash = FinanceSystemAccountsMapping::getSystemdefault('cash_account');
            $totalCash = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.is_posted='Y' AND I.ledger_id=" . $cash . " AND H.outlet=" . $userModel->outlet_id . " AND H.created_by=" . $userModel->id)->queryScalar();
            $current_date          = date('Y-m-d H:i:s');
            $subscription_end_date = $userModel->outletDetails->subscription_end_date;
            $notify_days           = $userModel->outletDetails->notify_days;
            $first_date = strtotime($current_date);
            $second_date = strtotime($subscription_end_date);
            $datediff = $second_date - $first_date;
            $diff = round($datediff / (60 * 60 * 24));
            // ----------------------------


            $this->responseBody['data'] = array(
                'server_date_time'      => $current_date,
                'subscription_end_date' => $subscription_end_date,
                'reminder_days'         => $notify_days,
                'date_difference'       => $diff,
                'cash_in_hand' => $totalCash > 0 ? $this->format_currency($totalCash) : 0,
                'cash_in_bank' => $totalBank > 0 ? $this->format_currency($totalBank) : 0,
                'total_receivable' => $totalReceivable > 0 ? $this->format_currency($totalReceivable) : 0
            );
            if ($userModel->access_type == 'CA') {
                $this->responseBody['data']['total_payable'] = $totalPayable < 0 ? $this->format_currency(abs($totalPayable)) : 0;
            } else {
                $this->responseBody['data']['no_of_sales'] = (int) $sales_invoices;
            }
            $this->_bind;
        } else {
            $this->_401;
        }
    }


    public function actionGetExpenseAccounts()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        if ($userModel->id > 0) {
            $userModel->db();
            $term = trim($variables['term']);
            $condition = 'main_account_type="E" AND isParent="N"';

            if ($term != "") {
                $condition .= ' AND ( description LIKE "%' . $term . '%" OR description LIKE "%' . $term . '%" )';
            }


            $expense_accounts = FinanceLegder::model()->findAll(array('condition' => $condition, 'order' => 'description'));
            foreach ($expense_accounts as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['name'] = $each->description;
            }
            $this->responseBody['status'] = 'success';
            $this->responseBody['count'] = $i + 1;
            $this->_bind;
        } else {
            $this->responseBody['msg'] = 'Unauthorized user';
            $this->responseBody['status'] = 'failed';
            $this->_400;
        }
    }




    public function actionSaveExpenseVoucher()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $userModel->db();
            $created_by = $userModel->id;
            $outlet = $userModel->outlet_id;
            $cash = FinanceSystemAccountsMapping::getSystemdefault('cash_account');
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $voucher_date = trim($variables['voucher_date']);
            $amount = (float) trim($variables['amount']);
            $vat_amount = (float) trim($variables['vat_amount']);
            $to_account = trim($variables['expense_account']);
            $narration = trim($variables['narration']);
            $payment_method = (int)trim($variables['payment_method']);
            if (isset($to_account) && isset($voucher_date) && isset($amount) && ($amount > $vat_amount) && $payment_method > 0) {
                function savableDate($date)
                {
                    $last_closed_year = FinanceAuditClose::lastClosed();
                    $voucherDate_array = explode('-', $date);
                    $voucherYear = $voucherDate_array[0];
                    return ($voucherYear <= $last_closed_year) ? false : true;
                }

                $modelFinancePaymentMode = FinancePaymentMode::model()->findBypk((int)$payment_method);

                $model = new FinanceVoucherHead;
                $model->voucher_type = FinanceVoucherType::getApiVoucherId("JV", $userModel->outlet_id);
                $model->voucher_no = FinanceVoucherType::getVoucherno($model->voucher_type, true);
                $model->voucher_date = (count(explode('/', $voucher_date)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $voucher_date))) : date('Y-m-d');
                $model->due_date = (count(explode('/', $voucher_date)) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $voucher_date))) : date('Y-m-d');
                $model->total_voucher_amount = $amount;
                $model->general_narration = $narration;
                $model->is_posted = 'Y';
                $model->created_by = $created_by;
                $model->created_datetime = date('Y-m-d h:i:s');
                $model->outlet = $outlet;
                $model->payment_method = $payment_method;
                $savableVoucher = savableDate($model->voucher_date);
                if ($savableVoucher) {
                    $model->save();

                    $paymentmode_subledger_id = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=3 AND parent_id=' . $userModel->id))->id;

                    $modelItems = new FinanceVoucherItems;
                    $modelItems->header_id = $model->id;
                    $modelItems->ledger_id = $modelFinancePaymentMode->ledger_id;
                    $modelItems->subledger_id = $paymentmode_subledger_id;
                    $modelItems->narration = $narration;
                    $modelItems->amount = - ($amount);
                    $modelItems->currency_code = 1;
                    $modelItems->currency_id = 1;
                    $modelItems->exchange_rate = 1;
                    $modelItems->outlet = $outlet;
                    $modelItems->save(false);
                    // Debit Side
                    $modelItems = new FinanceVoucherItems;
                    $modelItems->header_id = $model->id;
                    $modelItems->ledger_id = $to_account;
                    $modelItems->narration = $narration;
                    $modelItems->amount = $amount - $vat_amount;
                    $modelItems->currency_code = 1;
                    $modelItems->currency_id = 1;
                    $modelItems->exchange_rate = 1;
                    $modelItems->outlet = $outlet;
                    $modelItems->save(false);
                    // VAT
                    if ($vat_amount > 0) {
                        $modelItems = new FinanceVoucherItems;
                        $modelItems->header_id = $model->id;
                        $modelItems->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('vat_on_expense');
                        $modelItems->narration = $narration;
                        $modelItems->amount = $vat_amount;
                        $modelItems->currency_code = 1;
                        $modelItems->currency_id = 1;
                        $modelItems->exchange_rate = 1;
                        $modelItems->outlet = $outlet;
                        $modelItems->save(false);
                    }
                    $this->responseBody['status'] = 'success';
                    $this->responseBody['voucher_id'] = $model->id;
                    $this->responseBody['type'] = 'payment';
                    $this->_bind;
                } else {
                    $this->responseBody['msg'] = 'Year closed';
                    $this->responseBody['status'] = 'failed';
                    $this->_bind;
                }
            } else {
                if ($vat_amount < $amount) {
                    $this->responseBody['msg'] = 'Amount must be greater than VAT';
                    $this->responseBody['status'] = 'failed';
                    $this->_bind;
                } else {
                    $this->responseBody['msg'] = 'Please fill all mandatory fields';
                    $this->responseBody['status'] = 'failed';
                    $this->_bind;
                }
            }
        } else {
            $this->responseBody['msg'] = 'Unauthorized user';
            $this->responseBody['status'] = 'failed';
            $this->_400;
        }
    }



    public function actionUpdatevoucher()
    {

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $userModel->db();
            function isPostable($date)
            {
                $last_closed_year = FinanceAuditClose::lastClosed();
                $voucherDate_array = explode('-', $date);
                $voucherYear = $voucherDate_array[0];
                return ($voucherYear <= $last_closed_year) ? false : true;
            }
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);

            $id = (int) $variables['id'];
            $type = $variables['type'];

            if ($id > 0) {
                $voucherModel = FinanceVoucherHead::model()->findBypk($id);
                if (empty($voucherModel)) {
                    $this->responseBody['msg'] = 'Voucher Not Found!';
                    $this->responseBody['status'] = 'failed';
                    $this->_bind;
                    exit;
                }

                if (isPostable($voucherModel->voucher_date)) {
                    if ($type == 'unpost') {
                        //unpost

                        if ($voucherModel->is_allocated == 'N' && !$voucherModel->haveReconcilation()) {
                            $voucherModel->is_posted = 'N';
                            $voucherModel->save(false);
                            $this->responseBody['status'] = 'success';
                            $this->responseBody['voucher_id'] = $voucherModel->id;
                            $this->_bind;
                            exit;
                        } else {
                            $this->responseBody['msg'] = 'Remove Allocation and Reconciliation first!';
                            $this->responseBody['status'] = 'failed';
                            $this->_bind;
                            exit;
                        }
                    } else if ($type == 'delete') {
                        //delete
                        if ($voucherModel->is_allocated == 'N') {
                            FinanceVoucherHead::model()->deleteBypk($id);
                            FinanceVoucherItems::model()->deleteAll('header_id=:refer_id', array(':refer_id' => $id));
                            $this->responseBody['status'] = 'success';
                            $this->responseBody['voucher_id'] = $id;
                            $this->_bind;
                            exit;
                        } else {
                            $this->responseBody['msg'] = 'Remove allocation first!';
                            $this->responseBody['status'] = 'failed';
                            $this->_bind;
                            exit;
                        }
                    } else if ($type == 'post') {
                        //post
                        if ($voucherModel->is_postable_voucher()) {
                            $voucherModel->is_posted = 'Y';
                            $voucherModel->save(false);

                            $this->responseBody['status'] = 'success';
                            $this->responseBody['voucher_id'] =  $voucherModel->id;
                            $this->_bind;
                            exit;
                        } else {
                            $this->responseBody['msg'] = 'There is an error in your voucher!';
                            $this->responseBody['status'] = 'failed';
                            $this->_bind;
                            exit;
                        }
                    } else {
                        $this->responseBody['msg'] = 'Something Went Wrong!';
                        $this->responseBody['status'] = 'failed';
                        $this->_bind;
                        exit;
                    }
                } else {
                    $this->responseBody['msg'] = 'Audited!';
                    $this->responseBody['status'] = 'failed';
                    $this->_bind;
                }
            } else {
                $this->responseBody['msg'] = 'Something Went Wrong!';
                $this->responseBody['status'] = 'failed';
                $this->_bind;
            }
        } else {
            $this->responseBody['msg'] = 'Unauthorized user';
            $this->responseBody['status'] = 'failed';
            $this->_400;
        }
    }

    public function actionDeleteallocation()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        if ($userModel->id > 0) {
            $userModel->db();
            $id = (int) $variables['header_id'];
            if ($id > 0) {
                FinanceAllocation::model()->deletebypk($id);
                $modelItem = FinanceAllocationItems::model()->findAll(array('condition' => 'header_id=' . $id));
                FinanceAllocationItems::model()->deleteAll('header_id=:refer_id', array(':refer_id' => $id));
                foreach ($modelItem as $each):
                    $allocation = FinanceAllocationItems::model()->findAll(array('condition' => 'voucher_id=' . $each->voucher_id));
                    if (empty($allocation)) {
                        FinanceVoucherHead::model()->updateBypk($each->voucher_id, array('is_allocated' => 'N'));
                    }
                endforeach;

                $this->responseBody['status'] = 'success';
                $this->responseBody['msg'] = 'Allocation Deleted Successfully';
                $this->_bind;
            } else {
                $this->responseBody['msg'] = 'Failed to delete!';
                $this->responseBody['status'] = 'failed';
                $this->_bind;
            }
        } else {
            $this->responseBody['msg'] = 'Unauthorized user';
            $this->responseBody['status'] = 'failed';
            $this->_400;
        }
    }


    public function actionSavepayment()
    {

        $other = array();
        $currency = 1;
        $exchange_rate = 1;
        function savableDate($date)
        {
            $last_closed_year = FinanceAuditClose::lastClosed();
            $voucherDate_array = explode('-', $date);
            $voucherYear = $voucherDate_array[0];
            return ($voucherYear <= $last_closed_year) ? false : true;
        }
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);


        //        $myfile = fopen("payment_file.txt", "w") or die("Unable to open file!");
        //        $txt = "json data $variables\n";
        //        fwrite($myfile, $txt);
        //        fclose($myfile);


        $payment_date = trim($variables['payment_date']);
        $payment_ledger = (int) trim($variables['payment_ledger']);
        //        $mode_of_payment = trim($variables['mode']);
        $mode_of_payment = 'ca';
        $vendor = (int) trim($variables['vendor_id']);
        $id = (int) trim($variables['id']);
        $cheque_no = trim($variables['cheque_no']);
        $issued_date = trim($variables['issued_date']);
        $narration = trim($variables['narration']);
        $payment_method = (int)$variables['payment_method'];
        $items = $variables['items'];
        if (trim($payment_date) == '') {
            $other[] = 'Please choose payment date ';
        }
        if ($userModel->id > 0) {

            $userModel->db();

            $invoice = new PurchaseInvoice; 
            if ($id > 0) {
                $invoice = $invoice->findBypk($id);
            }
            $total_amount = 0;
            $total_received = 0;
            $total_discount = 0;
            foreach ($items as $each) {
                 if ($each['selected_amount'] != 0 && $each['selected'] == true) {
                    $total_received += $each['selected_amount'];
                    $total_discount += $each['selected_discount_amount'];
                }          
                
            }
            $total_amount = $total_received + $total_discount;
            if ((float)$total_amount == 0) {
                $other[] = 'Please check the amounts';
            }
            if ((int)$vendor == 0) {
                $other[] = 'Please choose supplier';
            }
            $model = new FinanceVoucherHead;
            $model->general_narration = $narration;
            $model->is_posted = 'Y';
            $model->created_by = $userModel->id;
            $model->created_datetime = date('Y-m-d h:i:s');
            $model->outlet = $userModel->outlet_id;
            $model->voucher_date = $payment_date;
            $model->due_date = $payment_date;
            $savableVoucher = savableDate($model->voucher_date);
            $voucherType =  FinanceVoucherType::getApiVoucherId("PV", $userModel->outlet_id);
            $model->total_voucher_amount = $total_amount;
            $voucherModel = FinanceVoucherType::model()->findBypk((int) $voucherType);
            $model->voucher_type = $voucherType;
            $model->voucher_no = $voucherModel->last_voucher_no + 1;
            
             if ((int)$payment_method == 0) {
                $other[] = 'Please choose Payment method';
            }
            
            $modelFinancePaymentMode = FinancePaymentMode::model()->findBypk((int)$payment_method);
          

        
            
            if ($modelFinancePaymentMode->code == 'CA') //card
                $model->scenario = 'card';
            else if ($modelFinancePaymentMode->code == 'CHQ') { //cheque
                $model->scenario = 'chequepaid';
                $model->realisation_date = (count(explode('/', $issued_date)) == 3) ? date('Y-m-d', strtotime(str_replace('/', '-', $issued_date))) : date('Y-m-d');
                $model->cheque_date = (count(explode('/', $issued_date)) == 3) ? date('Y-m-d', strtotime(str_replace('/', '-', $issued_date))) : date('Y-m-d');
                $model->cheque_cleared = 'N';
            } else if ($modelFinancePaymentMode->code == 'BANK') {
                //Bank
                $model->scenario = 'bank';
            } else { //Cash
                $model->scenario = 'cash';
            }
            $model->total_voucher_amount = $total_amount;
            if ($savableVoucher != true) {
                $other[] = 'Year Closed';
            }
            if (empty($other)) {
                
                $paymentmode_subledger_id = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=3 AND parent_id=' . $userModel->id))->id;
                
                $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=2 AND parent_id=' . $vendor))->id;


                $model->is_allocated = 'Y';
                $model->save(false);
                $voucherModel->last_voucher_no += 1;
                $voucherModel->save();
                $modelItems = new FinanceVoucherItems;
                $modelItems->outlet = $userModel->outlet_id;
                $modelItems->header_id = $model->id;
                $modelItems->ledger_id = $modelFinancePaymentMode->ledger_id;
                $modelItems->subledger_id = $paymentmode_subledger_id;
                $modelItems->narration = $narration;
                $modelItems->amount = -1 * $total_received;
                $modelItems->currency_id = $currency;
                $modelItems->exchange_rate = $exchange_rate;
                $modelItems->save(false);

                $modelItemsDiscount = new FinanceVoucherItems;
                $modelItemsDiscount->outlet = $userModel->outlet_id;
                $modelItemsDiscount->header_id = $model->id;
                $modelItemsDiscount->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('discount_allowed');
                $modelItemsDiscount->narration = $narration;
                $modelItemsDiscount->amount = -1 * $total_discount;
                $modelItemsDiscount->currency_id = $currency;
                $modelItemsDiscount->exchange_rate = $exchange_rate;
                $modelItemsDiscount->save(false);

                $modelItemsCreditor = new FinanceVoucherItems;
                $modelItemsCreditor->outlet = $userModel->outlet_id;
                $modelItemsCreditor->header_id = $model->id;
                $modelItemsCreditor->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $modelItemsCreditor->scenario = 'havesubledger';
                $modelItemsCreditor->subledger_id = $subledger;
                $modelItemsCreditor->narration = $narration;
                $modelItemsCreditor->amount = $total_amount;
                $modelItemsCreditor->currency_id = $currency;
                $modelItemsCreditor->exchange_rate = $exchange_rate;
                $modelItemsCreditor->save(false);

                $modelAllocation = new FinanceAllocation;
                $modelAllocation->allocation_date = date('Y-m-d');
                $modelAllocation->ledger_id = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $modelAllocation->subledger_id = $subledger;
                $modelAllocation->amount = abs($total_amount);
                $modelAllocation->outlet = $userModel->outlet_id;
                $modelAllocation->created_by = $userModel->id;
                $modelAllocation->created_date_time = date('Y-m-d h:i:s');
                $modelAllocation->save();
                foreach ($items as $each) {
                     if ($each['selected_amount'] != 0 && $each['selected'] == true) {
                        $debitAllocation = new FinanceAllocationItems;
                        $debitAllocation->header_id = $modelAllocation->id;
                        $debitAllocation->voucher_id = $each['voucher_id'];
                        $debitAllocation->voucher_item_id = $each['voucher_item_id'];
                        $debitAllocation->amount =  -1 * ($each['selected_amount'] + $each['selected_discount_amount']); 
                        $debitAllocation->outlet = $userModel->outlet_id;
                        $debitAllocation->save();
                    }
                }
                $creditAllocation = new FinanceAllocationItems;
                $creditAllocation->header_id = $modelAllocation->id;
                $creditAllocation->voucher_id = $model->id;
                $creditAllocation->voucher_item_id = $modelItemsCreditor->id;
                $creditAllocation->amount = $total_amount;
                $creditAllocation->outlet = $userModel->outlet_id;
                $creditAllocation->save();
                $this->responseBody['status'] = 'success';
                $this->responseBody['voucher_id'] = $model->id;
                $this->responseBody['type'] = 'payment';
                $this->_bind;
            } else {
                //$error = CJSON::encode(array_merge_recursive($other));
                $err_msg = implode(',', $other);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                $this->_400;
            }
        } else {
            $this->_401;
        }
    }
    public function actionGetallocationdataForPayment()
    {


        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);

        if ($userModel->id > 0) {
            $userModel->db();
            $outlet = (int)$userModel->outlet_id;
            $count = 0;
            $ledger = (int) $variables['ledger'];
            $subledger = (int) $variables['subledger'];
            $allocationCredit = 'CREATE OR REPLACE VIEW erp_finance_allocated_credit AS SELECT
								V.id AS voucher_id,
								VI.id AS voucher_item_id,
								V.voucher_no, VI.amount,
								IFNULL(ABS(SUM(AI.amount)), 0) AS allocated,
								(VI.amount*VI.exchange_rate) - IFNULL((SUM(AI.amount)), 0) AS balance
							FROM erp_finance_voucher_items VI
							LEFT JOIN erp_finance_voucher_head V ON V.id = VI.header_id
							LEFT JOIN erp_finance_allocation_items AI ON AI.voucher_item_id = VI.id
							WHERE V.is_posted ="Y" AND VI.amount < 0 AND VI.ledger_id = ' . $ledger . ' AND VI.subledger_id = ' . $subledger . '
							GROUP BY VI.id';

            Yii::app()->db->createCommand($allocationCredit)->execute();

            $creditAllocatables = Yii::app()->db->createCommand('SELECT GROUP_CONCAT(voucher_item_id) AS value FROM erp_finance_allocated_credit WHERE balance < 0')->queryAll();



            $count = 0;
            if ($creditAllocatables[0]['value'] != '') {
                $allItems = FinanceVoucherItems::model()->findAll(array('condition' => 'id IN (' . $creditAllocatables[0]['value'] . ')'));
                foreach ($allItems as $i => $each) {
                    $balance = Yii::app()->db->createCommand('SELECT balance AS balance FROM erp_finance_allocated_credit WHERE voucher_item_id=' . $each->id)->queryScalar();
                    $this->responseBody['data'][$count]['voucher_no'] =  $each->parent->fullvoucher;
                    $this->responseBody['data'][$count]['type'] = $each->parent->vtype->description;
                    $this->responseBody['data'][$count]['date'] = date('d/m/Y', strtotime($each->parent->voucher_date));
                    $this->responseBody['data'][$count]['balance'] = -1 * $balance;
                    $this->responseBody['data'][$count]['voucher_item_id'] = $each->id;
                    $this->responseBody['data'][$count]['voucher_id'] = $each->header_id;
                    $this->responseBody['data'][$count]['selected_amount'] = 0;
                    $this->responseBody['data'][$count]['selected_discount_amount'] = 0;
                    $this->responseBody['data'][$count]['selected'] =  false;
                    $count++;
                }
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }
    
    
        public function actionSaveCashTransfer()
    {
       
        $other = array();
        $currency = 1;
        $exchange_rate = 1;
        function savableDate($date)
        {
            $last_closed_year = FinanceAuditClose::lastClosed();
            $voucherDate_array = explode('-', $date);
            $voucherYear = $voucherDate_array[0];
            return ($voucherYear <= $last_closed_year) ? false : true;
        }
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);


        $receipt_date = (count(explode('/', trim($variables['vocuher_date']))) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', trim($variables['vocuher_date'])))) : date('Y-m-d');
     
        $amount = (float)$variables['amount'];
        $cash_from = (int)$variables['cash_from'];
        $cash_to = (int)$variables['cash_to'];
        $narration = $variables['narration'];
        $reference_no = $variables['reference_no'];

      
        if (trim($receipt_date) == '') {
            $other[] = 'Please choose receipt date ';
        }
        if ($userModel->id > 0) {

            $userModel->db();

            
            if ((float)$amount == 0) {
                $other[] = 'Please check the amounts';
            }
            
        
            if ((int)$cash_to == 0) {
                $other[] = 'Please choose Cash To';
            }
            if (empty($narration)) {
                $other[] = 'Narraction Cannot be Null';
            }



            $model = new FinanceVoucherHead;
            $model->general_narration = $narration;
            $model->is_posted = 'Y';
            $model->created_by = $userModel->id;
            $model->created_datetime = date('Y-m-d h:i:s');
            $model->outlet = $userModel->outlet_id;
            $model->voucher_date = $receipt_date;
            $model->due_date = $receipt_date;
            $model->reference_no = $reference_no;
            $model->payment_method = 1;
            $savableVoucher = savableDate($model->voucher_date);
          
            

            $voucherType =  FinanceVoucherType::getApiVoucherId("CAT", $userModel->outlet_id);
          

            $model->total_voucher_amount = $total_amount;
            $voucherModel = FinanceVoucherType::model()->findBypk((int) $voucherType);
            $model->voucher_type = $voucherType;
            $model->voucher_no = $voucherModel->last_voucher_no + 1;

            // ------------------------------------------------

           
            $model->total_voucher_amount = $amount;
            if ($savableVoucher != true) {
                $other[] = 'Year Closed';
            }
            if (empty($other)) {


                $model->is_allocated = 'Y';
                $model->save(false);
                $voucherModel->last_voucher_no += 1;
                $voucherModel->save();
                
                $cash_from_subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=3 AND parent_id=' . $userModel->id))->id;
                $cash_to_subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=3 AND parent_id=' . $cash_to))->id;
                $cash_ledger = FinanceSystemAccountsMapping::getSystemdefault('cash_account');
                
                $modelItems_cash_from = new FinanceVoucherItems;
                $modelItems_cash_from->outlet = $userModel->outlet_id;
                $modelItems_cash_from->header_id = $model->id;
                $modelItems_cash_from->ledger_id = $cash_ledger;
                $modelItems_cash_from->subledger_id = $cash_from_subledger;
                $modelItems_cash_from->narration = $narration;
                $modelItems_cash_from->amount =  (-1 * abs($amount));;
                $modelItems_cash_from->currency_id = $currency;
                $modelItems_cash_from->exchange_rate = $exchange_rate;
                $modelItems_cash_from->save(false);
                
                
                $modelItems_cash_to = new FinanceVoucherItems;
                $modelItems_cash_to->outlet = $userModel->outlet_id;
                $modelItems_cash_to->header_id = $model->id;
                $modelItems_cash_to->ledger_id = (int)$cash_ledger;
                $modelItems_cash_to->subledger_id = $cash_to_subledger;
                $modelItems_cash_to->narration = $narration;
                $modelItems_cash_to->amount =   abs($amount);
                $modelItems_cash_to->currency_id = $currency;
                $modelItems_cash_to->exchange_rate = $exchange_rate;
                $modelItems_cash_to->save(false);
      
                $this->responseBody['status'] = 'success';
                $this->responseBody['voucher_id'] = $model->id;
                $this->responseBody['type'] = 'cashtransfer';
                $this->_bind;
            } else {
                //$error = CJSON::encode(array_merge_recursive($other));
                $err_msg = implode(',', $other);
                $this->responseBody['msg'] = $err_msg;
                $this->responseBody['status'] = 'failed';
                  $this->_bind;
            }
        } else {
            $this->_401;
        }
    }
    
    
 

}
