<?php

class QuotationController extends Controller
{

    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('login', 'error', 'logout'),
                'users' => array('*'),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array('Index', 'GetList', 'Create', 'Save','GetSimilarUnits', 'DeleteItem', 'PrintQuotesReportPDF', 'PrintOrderCollectPDF', 'OrderList',
                    'refreshOrderList'),
                'expression' => '$user->isMember()',
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function actionIndex()
    {
        $this->render('index');
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function actionGetList()
    {
        //for login salesman
        $login_user = Yii::app()->user->id;
        $salesman_id = $_POST['salesman'];

        $modelquotes = new Quotes();
        $draw = $_POST['draw'];
        $offset = $_POST['start'];
        $limit = $_POST['length'];
        $columns = array(
            // datatable column index  => database column name
            0 => 'id',
            1 => 'oc_customer_det.customer_name',
            2 => 'quote_date',
            3 => 'oc_salesman_det.full_name',
            4 => 't.id',
        );

        $filterColumn = $columns[$_POST['order'][0]['column']];
        $filterOrder = $_POST['order'][0]['dir'];
        $orderBy = $filterColumn . " " . $filterOrder;
        $date = count(explode('/', $_POST['date'])) == 3 ? date('Y-m-d', strtotime(str_replace('/', '-', $_POST['date']))) : date('Y-m-d');
        $condition = 't.is_ordered="N" AND t.quote_date="' . $date . '"';

        if ($salesman_id>0) {
            $condition .= ' AND t.salesman_id = "' . $salesman_id . '" ';
        }

        if (isset($_POST['search']['value']) && $_POST['search']['value'] != "") {
            $searchString = $_POST['search']['value'];
            $condition = "t.id LIKE '%" . $searchString . "%' OR oc_customer_det.customer_name LIKE '%" . $searchString . "%' OR oc_salesman_det.full_name LIKE '%" . $searchString . "%' OR t.quote_date LIKE '%" . $searchString . "%'";
        }

        $count = $modelquotes->with('oc_customer_det', 'oc_salesman_det')->count(array('condition' => $condition));
        $modelquotes = $modelquotes->with('oc_customer_det', 'oc_salesman_det')->findAll(array('condition' => $condition, 'limit' => $limit, 'offset' => $offset, 'order' => 't.id desc'));
        $recordsTotal = $count;
        $recordsFiltered = $count;
        $data['draw'] = $draw;
        $data['recordsTotal'] = $recordsTotal;
        $data['recordsFiltered'] = $recordsFiltered;
        $rowData = array();

        foreach ($modelquotes as $i => $row) {

            $div = '';
            $div .= '<div class="text-center">';
            if ($row['is_ordered'] == 'N') {
                $div .= '<button class="btn btn-primary btn-xs" title="Edit Quotation" onclick="loadPopupModal(\'quotation/create&id=' . $row['id'] . '\',\' Edit Quotation\',\'xl\')"><i class="fa fa-pencil"></i></button> ';
                $div .= ' <button class="btn btn-primary btn-xs" title="Convert to Order" onclick="loadPopupModal(\'ordercollect/Convertfromquotes&quotes_id=' . $row['id'] . '\',\' New Order \',\'xl\')"><i class="fa fa-retweet"></i></button> ';
            }
            $div .= ' <a class="btn btn-primary btn-xs" title="Print PDF" target="_blank" href="?r=quotation/PrintQuotesReportPDF&id=' . $row['id'] . '"><i class="fa fa-file-pdf-o"></i></a>';
            $div .= ' </div>';
            $rowData[] = array($row['id'], $row['oc_customer_det']['customer_name'], date("d/m/Y", strtotime($row['quote_date'])), $row['oc_salesman_det']['full_name'], $div);
        }
        $data['data'] = $rowData;
        echo json_encode($data);
    }

    public function actionCreate()
    {
        $model = new Quotes();
        $id = Yii::app()->request->getParam('id');
        $post = Yii::app()->request->getParam('type');
        $modelItems = array(new QuotesItem);
        if ($id > 0) {
            $model = $model->findByPk($id);
            $modelItems = QuotesItem::model()->findAll(array('condition' => 'quotes_id="' . $id . '"'));
            if (empty($modelItems)) {
                $modelItems = array(new QuotesItem());
            }
        }
        Yii::app()->clientScript->scriptMap = array('*jquery.js' => false, '*jquery.min.js' => false, 'jquery.min.js' => false, '*jquery-ui.js' => false, '*jquery-ui.min.js' => false, 'jquery-2.1.4.min.js' => false);
        $this->render('create', array('model' => $model, 'modelItems' => $modelItems));
    }

    public function actionSave()
    {
        if (isset($_POST['Quotes'])) {
            $modelOrder = new Quotes;
            $others = array();
            if ((int) $_POST['Quotes']['id'] > 0) {
                $modelOrder = $modelOrder->findByPK($_POST['Quotes']['id']);
                $modelOrder->updated_by = Yii::app()->user->id;
                $modelOrder->updated_date_time = date('Y-m-d H:i:s');
            } else {
                $modelOrder->created_by = Yii::app()->user->id;
                $modelOrder->created_date_time = date('Y-m-d H:i:s');
            }
            $modelOrder->attributes = $_POST['Quotes'];
            if ($_POST['Quotes']['quote_date'] != '') {
                $modelOrder->quote_date = str_replace('/', '-', $_POST['Quotes']['quote_date']);
                $modelOrder->quote_date = date("Y-m-d", strtotime($modelOrder->quote_date));
            } else {
                $modelOrder->quote_date = date("Y-m-d");
            }
            $modelOrder->salesman_id = $_POST['Quotes']['salesman_id'];
            $modelOrder->notes = $_POST['Quotes']['notes'];
            $modelOrder->outlet_id =(int) Yii::app()->user->outlet_id;
            $valid = $modelOrder->validate();
            $products_id = array();

            if (is_array($_POST['QuotesItem'])) {
                foreach ($_POST['QuotesItem'] as $i => $item) {
                    if ($item['product_id'] != '') {
                        $modelOrderItems[$i] = new QuotesItem;
                        $modelOrderItems[$i]->attributes = $item;
                        $modelOrderItems[$i]->quotes_id = 0;
                        $modelOrderItems[$i]->description = $item['description'];
                        $modelOrderItems[$i]->part_number = $item['part_number'];
                        $modelOrderItems[$i]->unit_id = $item['unit_id'];
                        if( $item['unit_id']>0){
                            $unit=MasUnit::model()->findByPk($item['unit_id']);
                            $modelOrderItems[$i]->unit_factor=$unit->unit_exchange_rate;
                            $modelOrderItems[$i]->base_unit_id=$unit->base_unit_id;
                            $modelOrderItems[$i]->base_unit_qty=$unit->unit_exchange_rate * $item['quantity'];
                        }

                        /*                         * **************** END Code added by jisha on 17/06/2019 for secondary unit calculation ******************** */

                        $valid = $modelOrderItems[$i]->validate() && $valid;
                    }
                }
            }
            
            if($modelOrder->is_ordered=='Y'){
             $others[]='This order has already been converted to an invoice.'; 
             $valid=false; 
            }
            
            
            
            if ($valid && !empty($modelOrderItems)) {
                $modelOrder->save();

                QuotesItem::model()->deleteAll('quotes_id=:refer_id', array(':refer_id' => $modelOrder->id));
                if (!empty($modelOrderItems)) {
                    foreach ($modelOrderItems as $eachItem) {
                        $eachItem->quotes_id = $modelOrder->id;
                        $eachItem->save();
                    }
                }

                $jsonData['status'] = 'success';

                echo CJSON::encode($jsonData);
            } else {
                $error1 = $error2 = $error3 = $error4 = array();
                $error1 = json_decode(CActiveForm::validate($modelOrder, '', false), true);
                if (!empty($modelOrderItems[0]->product_id)) {
                    $error2 = json_decode(CActiveForm::validateTabular($modelOrderItems, '', false), true);
                } else {
                    $error2[] = "Please add atleast one line item";
                }

                $error = CJSON::encode(array_merge_recursive($error1, $error2,$others));

                if ($error != '[]') {
                    echo $error;
                }

            }
        } else {
            echo 'error';
        }
    }
    public function actionGetSimilarUnits(){
        $unit_id=$_POST['unit'];
        $i=$_POST['row_field'];
        $arr_units=MasUnit::getSimilarUnits($unit_id);
        $modelItems=new QuotesItem();
        $modelItems->unit_id=$unit_id;
        $data['unit']=CHtml::activeDropDownList($modelItems, "[" . $i . "]unit_id",$arr_units, array("class" =>"go cell-center text-center",'onchange'=>'getPriceOnUnitChange(this.id)'));
        echo CJSON::encode($data);
    }
    /*public function actionDeleteItem()
    {
        $id = $_POST['id'];
        if ($id > 0) {
            QuotesItem::model()->deleteByPk($id);
            $jsonData['status'] = 'success';
            echo CJSON::encode($jsonData);
        }
    }*/

    public function actionPrintQuotesReportPDF()
    {
        $id = Yii::app()->request->getParam('id');
        $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
        $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
        $mPDF1->WriteHTML($stylesheet, 1);
        $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.quotation_print', array('model' => $model, 'id' => $id), true));
        $mPDF1->Output($htmlTemplate, 'I');
    }

    public function actionPrintOrderCollectPDF()
    {
        $frm = Yii::app()->request->getParam('frm');
        $salesman_id = (int) Yii::app()->request->getParam('salesman_id');
         $quote_date = Yii::app()->request->getParam('quote_date');

        //$created_date_time= Yii::app()->request->getParam('created_date_time');
        if (!empty($quote_date)) {
            $str_quote_date = str_replace('/', '-', $quote_date);
            $str_quote_date = date("Y-m-d ", strtotime($str_quote_date));
        }
        $mPDF1 = Yii::app()->ePdf->mpdf('', 'A4');
        $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
        $mPDF1->WriteHTML($stylesheet, 1);
        if ($frm == 'salesman') {
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.print_order_collect_detailed', array('arr_list' => $arr_list, 'frm' => $frm, 'quote_date' => $quote_date, 'salesman_id' => $salesman_id, 'str_quote_date'=> $str_quote_date), true));
        } else {
            $sql="SELECT O.id AS quotes_id,O.quote_date,O.salesman_id,O.customer_id,O.outlet_id,OI.product_id,I.part_number,OI.description,SUM(OI.base_unit_qty)/U.unit_exchange_rate AS quantity,OI.unit_id,OI.base_unit_id,SUM(OI.base_unit_qty) AS base_qty,OI.unit_factor,BU.unit_name as base_unit,U.unit_name as ordered_unit,U.unit_exchange_rate
                    FROM erp_order_collect_item OI
                    LEFT JOIN erp_order_collect O ON O.id=OI.quotes_id
                    LEFT JOIN erp_inventory I ON I.id=OI.product_id
                    LEFT JOIN erp_master_unit U ON U.id=I.sec_unit_id
                    LEFT JOIN erp_master_unit BU ON BU.id=OI.base_unit_id
                    LEFT JOIN bluesell_new_main.erp_users MU ON MU.id=O.salesman_id
                    WHERE O.quote_date= '".$str_quote_date."' AND O.is_ordered='N' AND O.outlet_id =".(int) Yii::app()->user->outlet_id."
                    GROUP BY I.id,O.quote_date"   ; 
            
            //$SQL = "SELECT SUM(primary_unit_qty) as quantity,part_no,primary_unit_id as unit,salesman_name,description,salesman_id,SUM((primary_unit_qty/sec_factor)) AS secondary_qty,secondary_unit_symbol as secondary_unit, generic_name FROM view_order_collect WHERE is_ordered='N' AND quote_date='" . $str_quote_date . "' GROUP BY product_id";
            $arr_list = Yii::app()->db->createCommand($sql)->queryAll();
            $mPDF1->WriteHTML($this->renderPartial('application.views.front.pdf.print_order_collect_datewise', array('arr_list' => $arr_list, 'frm' => $frm, 'quote_date' => $quote_date), true));
        }
        $mPDF1->Output($htmlTemplate, 'I');
    }

    public function actionOrderList()
    {
        $this->render('orderlist');
    }
    public function actionrefreshOrderList()
    {
        $salesman_id = $_POST['salesman_id'];
        $quote_date = $_POST['quote_date'];
        $this->renderPartial('orderlisttbody', array('salesman_id' => $salesman_id, 'quote_date' => $quote_date));

    }
}
