<?php

/**
 * This is the model class for table "erp_audit_log".
 *
 * The followings are the available columns in table 'erp_audit_log':
 * @property integer $id
 * @property integer $form_master_id
 * @property integer $created_by
 * @property string $created_time
 * @property string $changes
 */
class AuditLog extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'erp_audit_log';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('form_master_id, created_by, created_time, changes', 'required'),
			array('form_master_id, created_by', 'numerical', 'integerOnly'=>true),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, form_master_id, created_by, created_time, changes', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'form_master_id' => 'Form Master',
			'created_by' => 'Created By',
			'created_time' => 'Created Time',
			'changes' => 'Changes',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('form_master_id',$this->form_master_id);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('changes',$this->changes,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return AuditLog the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
	public static function saveAuditLogs($form_master_id, $user, $type, $changes)
    {
        try {
            date_default_timezone_set("Asia/Dubai");
            $model_audit_log = new AuditLog();
            $model_audit_log->form_master_id = $form_master_id;
            $model_audit_log->created_by = $user;
            $model_audit_log->created_time = date('Y-m-d h:i:s');
            $model_audit_log->outlet=Yii::app()->user->outlet_id;
            switch ($type) {
                case 'CREATED':
                    $change_type = 'CTD';
                    break;
                case 'UPDATED':
                    $change_type = 'UTD';
                    break;
                case 'DELETED':
                    $change_type = 'DLTD';
                    break;
                default:
                    $change_type = 'UTD';
                    break;
            // end audit log for vendor
            }
            $changes = " {$change_type} {$changes} ";
            $model_audit_log->changes = $changes;
            $model_audit_log->save(false);
        } catch (\yii\db\Exception $e) {
            echo 'An error occured. Please try again';
        }
    }
    public static function saveApiAuditLogs($outlet_id,$form_master_id, $user, $type, $changes)
    {
        try {
            date_default_timezone_set("Asia/Dubai");
            $model_audit_log = new AuditLog();
            $model_audit_log->form_master_id = $form_master_id;
            $model_audit_log->created_by = $user;
            $model_audit_log->created_time = date('Y-m-d h:i:s');
            $model_audit_log->outlet=$outlet_id;
            switch ($type) {
                case 'CREATED':
                    $change_type = 'CTD';
                    break;
                case 'UPDATED':
                    $change_type = 'UTD';
                    break;
                case 'DELETED':
                    $change_type = 'DLTD';
                    break;
                default:
                    $change_type = 'UTD';
                    break;
            // end audit log for vendor
            }
            $changes = " {$change_type} {$changes} ";
            $model_audit_log->changes = $changes.' API';
            $model_audit_log->save(false);
        } catch (\yii\db\Exception $e) {
            echo 'An error occured. Please try again';
        }
    }
}
