<?php

/**
 * This is the model class for table "erp_customer".
 *
 * The followings are the available columns in table 'erp_customer':
 * @property integer $id
 * @property integer $customer_no
 * @property string $customer_name
 * @property string $address
 * @property string $location
 * @property integer $state
 * @property integer $country
 * @property string $credit_limit
 * @property string $phone
 * @property string $email
 * @property string $contact_person
 * @property string $mobile
 * @property integer $payment_term
 * @property string $trn_nostring
 * @property string $gps_coordinates
 * @property integer $created_by
 * @property string $accessed_by
 * @property string $created_time
 * @property integer $updated_by
 * @property string $updated_time
 * @property string $active
 * @property integer $outlet
 */
class Customer extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{customer}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('customer_name, state, country,payment_term,trn_no,phone, credit_limit, outlet,invoice_barcode_show', 'required'),
			array('customer_no, state, country, payment_term, created_by, updated_by, outlet,credit_days,sales_price_type', 'numerical', 'integerOnly' => true),
			array('customer_name, contact_person, trn_no', 'length', 'max' => 150),
			array('address, gps_coordinates,accessed_by', 'length', 'max' => 255),
			array('location, phone, email, mobile', 'length', 'max' => 100),
			array('credit_limit,latitude,longitude', 'length', 'max' => 15),
			array('active,invoice_barcode_show', 'length', 'max' => 1),
			array('created_time, updated_time', 'safe'),
			array('customer_name', 'checkUniqueness'),
			// array('mobile', 'checkUniquenessMobile'),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, customer_no, customer_name, address, location, state, country, credit_limit, phone, email, contact_person, mobile, payment_term, trn_no, gps_coordinates, created_by,accessed_by, created_time, updated_by, updated_time, active, outlet', 'safe', 'on' => 'search'),
		);
	}

	public function checkUniqueness($attribute, $params)
	{
		if ($this->id > 0) {
			$model = Customer::model()->find('customer_name = "' . $this->customer_name . '" AND id!=' . $this->id . ' AND outlet=' . $this->outlet);
		} else {
			$model = Customer::model()->find('customer_name = "' . $this->customer_name . '" AND outlet=' . $this->outlet);
		}
		if (count($model) > 0) {
			$this->addError('id', 'This Customer already exist');
		}
	}


	public function checkUniquenessMobile($attribute, $params)
	{
		if($this->mobile!=''){
			if ($this->id > 0) {
				$model = Customer::model()->find('mobile = "' . $this->mobile . '" AND id!=' . $this->id . ' AND outlet=' . $this->outlet);
			} else {
				$model = Customer::model()->find('mobile = "' . $this->mobile . '" AND outlet=' . $this->outlet);
			}
			if (count($model) > 0) {
				$this->addError('id', 'This Mobile No already exist');
			}
		}
		
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
			'outlet_detail' => array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
			'state_detail' => array(self::BELONGS_TO, 'MasState', 'state'),
			'country_detail' => array(self::BELONGS_TO, 'MasCountry', 'country'),
			'paymentterm_detail' => array(self::BELONGS_TO, 'MasPaymentTerms', 'payment_term'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'customer_no' => 'Customer No',
			'customer_name' => 'Customer Name',
			'address' => 'Address',
			'location' => 'Location',
			'state' => 'State',
			'country' => 'Country',
			'credit_limit' => 'Credit Limit',
			'phone' => 'Phone',
			'email' => 'Email',
			'contact_person' => 'Contact Person',
			'mobile' => 'Mobile',
			'payment_term' => 'Payment Term',
			'trn_no' => 'Trn No',
			'gps_coordinates' => 'Gps Coordinates',
			'created_by' => 'Created By',
			'accessed_by' => 'Accessed By',
			'created_time' => 'Created Time',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Time',
			'active' => 'N: No, Y: Yes',
			'outlet' => 'Outlet',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria = new CDbCriteria;

		$criteria->compare('id', $this->id);
		$criteria->compare('customer_no', $this->customer_no);
		$criteria->compare('customer_name', $this->customer_name, true);
		$criteria->compare('address', $this->address, true);
		$criteria->compare('location', $this->location, true);
		$criteria->compare('state', $this->state);
		$criteria->compare('country', $this->country);
		$criteria->compare('credit_limit', $this->credit_limit, true);
		$criteria->compare('phone', $this->phone, true);
		$criteria->compare('email', $this->email, true);
		$criteria->compare('contact_person', $this->contact_person, true);
		$criteria->compare('mobile', $this->mobile, true);
		$criteria->compare('payment_term', $this->payment_term);
		$criteria->compare('trn_no', $this->trn_no, true);
		$criteria->compare('gps_coordinates', $this->gps_coordinates, true);
		$criteria->compare('created_by', $this->created_by);
		$criteria->compare('accessed_by', $this->accessed_by);
		$criteria->compare('created_time', $this->created_time, true);
		$criteria->compare('updated_by', $this->updated_by);
		$criteria->compare('updated_time', $this->updated_time, true);
		$criteria->compare('active', $this->active, true);
		$criteria->compare('outlet', $this->outlet);

		return new CActiveDataProvider($this, array(
			'criteria' => $criteria,
		));
	}


	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return Customer the static model class
	 */
	public static function model($className = __CLASS__)
	{
		return parent::model($className);
	}
	public static function getCustomer()
	{
		$condition = 'active="Y"';
		$model = self::model()->findAll(array('condition' => $condition, 'order' => 'customer_name ASC'));
		$returnArray =  CHtml::listData($model, 'id', 'customer_name');
		return $returnArray;
	}
}
