<?php

/**
 * This is the model class for table "erp_goods_issue_items".
 *
 * The followings are the available columns in table 'erp_goods_issue_items':
 * @property integer $id
 * @property integer $issue_id
 * @property integer $product_id
 * @property integer $description
 * @property string $quantity
 * @property integer $unit
 */
class GoodsIssueItems extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public $part_no;
	public function tableName()
	{
		return '{{goods_issue_items}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('issue_id, product_id, description, quantity, unit', 'required'),
			array('issue_id, product_id,  unit', 'numerical', 'integerOnly'=>true),
			array('quantity', 'length', 'max'=>15),
			array('description', 'length', 'max'=>5000),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, issue_id, product_id, description, quantity, unit', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'gi_product_details'     => array(self::BELONGS_TO, 'Inventory', 'product_id'),
            'gi_unit_details'        => array(self::BELONGS_TO, 'MasUnit', 'unit'),
        );
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'issue_id' => 'Issue',
			'product_id' => 'Product',
			'description' => 'Description',
			'quantity' => 'Quantity',
			'unit' => 'Unit',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('issue_id',$this->issue_id);
		$criteria->compare('product_id',$this->product_id);
		$criteria->compare('description',$this->description);
		$criteria->compare('quantity',$this->quantity,true);
		$criteria->compare('unit',$this->unit);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return GoodsIssueItems the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
