<?php

/**
 * This is the model class for table "erp_goods_receipt".
 *
 * The followings are the available columns in table 'erp_goods_receipt':
 * @property integer $id
 * @property integer $receipt_no
 * @property string $receipt_date
 * @property string $reference
 * @property string $notes
 * @property string $active
 * @property integer $created_by
 * @property string $created_time
 * @property integer $updated_by
 * @property string $updated_time
 * @property string $posted
 * @property integer $outlet
 */
class GoodsReceipt extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{goods_receipt}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('receipt_no, receipt_date, reference,  outlet,sales_man', 'required'),
			array('receipt_no, created_by, updated_by, outlet', 'numerical', 'integerOnly'=>true),
			array('reference', 'length', 'max'=>255),
			array('notes', 'length', 'max'=>2225),
			array('active, posted', 'length', 'max'=>1),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, receipt_no, receipt_date, reference, notes, active, created_by, created_time, updated_by, updated_time, posted, outlet', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'gr_outlet'   => array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
		    'gr_user'     => array(self::BELONGS_TO, 'MasUsers', 'created_by'),
            'gr_salesman' => array(self::BELONGS_TO, 'MasUsers', 'sales_man'),
        );
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'receipt_no' => 'Receipt No',
			'sales_man'  => 'Sales Man',
			'receipt_date' => 'Receipt Date',
			'reference' => 'Reference',
			'notes' => 'Notes',
			'active' => 'Active',
			'created_by' => 'Created By',
			'created_time' => 'Created Time',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Time',
			'posted' => 'Posted',
			'outlet' => 'Outlet',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('receipt_no',$this->receipt_no);
		$criteria->compare('receipt_date',$this->receipt_date,true);
		$criteria->compare('reference',$this->reference,true);
		$criteria->compare('notes',$this->notes,true);
		$criteria->compare('active',$this->active,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('updated_time',$this->updated_time,true);
		$criteria->compare('posted',$this->posted,true);
		$criteria->compare('outlet',$this->outlet);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return GoodsReceipt the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
