<?php

/**
 * This is the model class for table "vs_outlets".
 *
 * The followings are the available columns in table 'vs_outlets':
 * @property integer $id
 * @property integer $company_id
 * @property string $name
 * @property string $is_active
 */
class MasOutlets extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
    public $company_name;
	public function tableName()
	{
		return MasCompany::MAIN_DB.'.{{outlets}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('company_id, name,outlet_name, mode', 'required'),
			array('company_id', 'numerical', 'integerOnly'=>true),
			array('name', 'length', 'max'=>255),
			array('name', 'unique'),
			array('is_active,consider_previous_sprice,consider_sp_not_less_cp', 'length', 'max'=>1),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, company_id, name, is_active', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
			'companyDetails' => array(self::BELONGS_TO, 'MasCompany', 'company_id'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'outlet_name'=>'Outlet Name',
			'company_id' => 'Company',
			'name' => 'Code',
			'is_active' => 'Is Active',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('company_id',$this->company_id);
		$criteria->compare('name',$this->name,true);
		$criteria->compare('is_active',$this->is_active,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return MasOutlets the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	public static function get_outlets(){
		$model = self::model()->findAll( array('condition' => 'company_id='.(int) Yii::app()->user->company_id, 'order' => 'name' ) );
        return CHtml::listdata($model, 'id', 'name');
	}

	public function get_company_outlets($id =0){
		$model = self::model()->findAll( array('condition' => 'company_id='.$id, 'order' => 'name' ) );
        return CHtml::listdata($model, 'id', 'name');
	}

	public function allusers($id=0){
		$model = MasUsers::model()->findAll( array( 'condition' => 'outlet_id='.(int) $id ) );
		return CHtml::listdata($model, 'id', 'full_name');
	}

	public static function salesmans(){
		$model = MasUsers::model()->findAll( array( 'condition' => 'outlet_id='.(int) Yii::app()->user->outlet_id.' AND access_type IN("SL", "CA")' ) );
		return CHtml::listdata($model, 'id', 'full_name');
	}

	public static  function salesmans_without_user( $user_id =0){
		$model = MasUsers::model()->findAll( array( 'condition' => 'id != '.(int) $user_id.' AND outlet_id='.(int) Yii::app()->user->outlet_id.' AND access_type IN("SL", "CA")' ) );
		return CHtml::listdata($model, 'id', 'full_name');
	}

	public function vat_during_period($from, $to){
		$query = 'SELECT
					ROUND(SUM(T.amount*T.exchange_rate), 2) AS vat_value
				FROM {{transactions}} T
				LEFT JOIN {{vouchers}} V ON V.id = T.voucher_id
				LEFT JOIN {{ledgers}} L ON T.ledger_id = L.id
				WHERE V.is_posted="Y" AND V.outlet_id='.(int) $this->id.' AND L.classification IN ("'.MasLedgers::PURCHASE_VAT.'", "'.MasLedgers::SALES_VAT.'", "'.MasLedgers::EXPENSE_VAT.'") AND V.date BETWEEN "' . $from . '" AND "' . $to . '" ';
		return -1 * (float) Yii::app()->db->createCommand($query)->queryScalar();
	}

	public function generate_activation_code(){
		$this->activation_code = 'ES-'.rand(1000,9999).'-AE'.rand(10,99).'-'.rand(1000,9999);
	}

	public function generate_related_party(){

		$partyHead = new MasLedgers;
		$partyHead->parent_id = MasLedgers::model()->get_parent_id(MasLedgers::RELATED_PARTIES);
		$partyHead->company_id = Yii::app()->user->company_id;
		$partyHead->currency = MasCurrency::getBase_currency();
		$partyHead->name = $this->name;
		$partyHead->system_generated="Y";
		$partyHead->is_parent = 'Y';
		$partyHead->inherit_parent();
		$partyHead->save(false);
		$partyHead->reset_levels();

		if( $partyHead->id > 0 ){
			$partyItem = new MasLedgers;
			$partyItem->parent_id = $partyHead->id;
			$partyItem->company_id = Yii::app()->user->company_id;
			$partyItem->classification = MasLedgers::SPONSOR_FEE;
			$partyItem->name = 'Sponsor Fee Receivable';
			$partyItem->currency = MasCurrency::getBase_currency();
			$partyItem->system_generated="Y";
			$partyItem->inherit_parent();
			$partyItem->save(false);
			$partyItem->reset_levels();

			$partyItem = new MasLedgers;
			$partyItem->parent_id = $partyHead->id;
			$partyItem->company_id = Yii::app()->user->company_id;
			$partyItem->classification = MasLedgers::SALIK_FINES;
			$partyItem->name = 'Salik & Fines Receivable';
			$partyItem->currency = MasCurrency::getBase_currency();
			$partyItem->system_generated="Y";
			$partyItem->inherit_parent();
			$partyItem->save(false);
			$partyItem->reset_levels();
		}

		$this->ledger_id = $partyHead->id;
		$this->update();
	}

	public function generate_users(){
		if( $this->mode == 'S' ||  $this->mode == 'M'  ){
			MasUsers::create_user('admin@'.strtolower(str_replace(' ','', $this->name)), '1234', 'CA', $this->id);
		} else {
			MasUsers::create_user('admin@'.strtolower(str_replace(' ','', $this->name)), '1234', 'CA', $this->id);
			MasUsers::create_user('sales01@'.strtolower(str_replace(' ','', $this->name)), '1234', 'SL', $this->id);
		}
	}

	public function getIs_valid_login(){
		return ( strtotime($this->subscription_end_date) > strtotime(date('Y-m-d')) ) ? true : false;
	}

	public function remaining_days(){
		$diff = date_diff(date_create(date('Y-m-d')), date_create(date($this->subscription_end_date)));
        return $diff->format("%a days");
	}

	public static function getAllOutlets(){
		if(Yii::app()->user->user_access_type == 'DEV' || Yii::app()->user->user_access_type == 'SU')
		{
		$model = self::model()->findAll( array('condition' => 'company_id='.(int) Yii::app()->user->company_id, 'order' => 'outlet_name' ) );
		return CHtml::listdata($model, 'id', 'outlet_name');
		}
		else if(Yii::app()->user->user_access_type == 'CA' ||Yii::app()->user->user_access_type == 'PM' || Yii::app()->user->user_access_type == 'SL' || Yii::app()->user->user_access_type == 'SM' || Yii::app()->user->user_access_type == 'OM')
		{
		$model = self::model()->findAll( array('condition' => 'id='.(int) Yii::app()->user->outlet_id.' AND company_id='.(int) Yii::app()->user->company_id, 'order' => 'outlet_name' ) );
		return CHtml::listdata($model, 'id', 'outlet_name');	
		}
	}
}
