<?php
class MasUnit extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{master_unit}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('unit_name,outlet,base_unit_id,unit_exchange_rate,base_unit,label', 'required'),
			array('created_by, updated_by, outlet, base_unit_id', 'numerical', 'integerOnly'=>true),
			array('unit_exchange_rate', 'numerical'),
			array('unit_name,label', 'length', 'max'=>50),
			array('active,is_default,base_unit', 'length', 'max'=>1),
			array('created_time, updated_time', 'safe'),
			array('unit_name', 'checkUniqueness'),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, unit_name, created_by, created_time, updated_by, updated_time, active, outlet', 'safe', 'on'=>'search'),
		);
	}
	public function checkUniqueness($attribute,$params)
    {
			if($this->id>0)
			{
			$model = MasUnit::model()->find('unit_name = "'.$this->unit_name.'" AND id!='.$this->id.' AND outlet='.$this->outlet);
			}
            else {
			$model = MasUnit::model()->find('unit_name = "'.$this->unit_name.'" AND outlet='.$this->outlet);	
			}
			if(count($model)>0)	
			{
			$this->addError('id','This Unit already exist'); 
			}
    }
	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'unit_outlet'=>array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'unit_name' => 'Unit Name',
			'created_by' => 'Created By',
			'created_time' => 'Created Date',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Date',
			'active' => 'N : No, Y : Yes',
			'outlet' => 'Outlet',
			'unit_exchange_rate' => 'Exchange Quantity',
			'label' => 'Print Label'
		);
	}

	public function search()
	{

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('unit_name',$this->brand_name,true);
		$criteria->compare('base_unit',$this->base_unit,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('updated_time',$this->updated_time,true);
		$criteria->compare('active',$this->active,true);
		$criteria->compare('outlet',$this->outlet);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
	public static function getAllUnits(){
	return CHtml::listData(self::model()->findAll( array( 'condition' => 'active="Y" AND outlet='.Yii::app()->user->outlet_id, 'order' => 'is_default ASC, unit_name ASC' ) ), 'id', 'unit_name');
	}
	public static function getAllBaseUnits($id=""){
	if($id>0)
	{
	return CHtml::listData(self::model()->findAll( array( 'condition' => 'active="Y" AND base_unit="Y" AND outlet='.Yii::app()->user->outlet_id.' AND id!='.$id, 'order' => 'is_default ASC, unit_name ASC' ) ), 'id', 'unit_name');
	}
	else {
	return CHtml::listData(self::model()->findAll( array( 'condition' => 'active="Y" AND base_unit="Y" AND outlet='.Yii::app()->user->outlet_id, 'order' => 'is_default ASC, unit_name ASC' ) ), 'id', 'unit_name');
	}
	}
	public static function getSimilarUnits($id="",$oulet=0){
	    if($oulet>0){
	       $oulet_id =$oulet;
	   }else{
	       $oulet_id =Yii::app()->user->outlet_id;
	   }
	    if($id>0){
	        $base_unit=self::model()->findByPK($id)->base_unit_id;
	        $arr_units =self::model()->findAll( array( 'condition' => 'active="Y"  AND outlet='.$oulet_id.' AND (base_unit_id='.$base_unit.' OR id='.$base_unit.')', 'order' => 'is_default ASC, unit_name ASC' ) );
	    }else{
	      $arr_units =self::model()->findAll( array( 'condition' => 'active="Y" AND outlet='.$oulet_id, 'order' => 'is_default ASC, unit_name ASC' ) );
	    }
	    return CHtml::listData($arr_units, 'id', 'unit_name');
	}
	public function getBaseUnit(){
	    if($this->base_unit=='N'){
	        $base_unit = self::model()->findByPk($this->base_unit_id)->unit_name;
	    }else{
	        $base_unit = $this->unit_name;
	    }
	    return $base_unit;
	}
}
