<?php
class MasUsers extends CActiveRecord
{

    public $show_all_customers = false;

    public $show_all_stock = true;

    public $show_all_vendors = false;



    //Database connection

    public $db_host;

    public $db_main;

    public $db_name;

    public $db_username;

    public $db_password;

    public $prefix;

    public $for_main = false;



    public function tableName()

    {

       

        return MasCompany::MAIN_DB.'.{{users}}';

    }


    public function rules()

    {


        return array(

            array('company_id, phone_no, username, access_type, outlet_id, full_name', 'required'),

            array('password', 'required', 'on' => 'new_user'),

            array('username', 'is_unique'),

            array('company_id, outlet_id', 'numerical', 'integerOnly' => true),

            array('is_active', 'length', 'max' => 1),
            
            array('username', 'length', 'max' => 20),
            
            array('token, phone_no', 'length', 'max' => 255),

            array('id, company_id, outlet_id, token, username, password, is_active', 'safe', 'on' => 'search'),

        );

    }



    public function is_unique($attribute, $params)

    {

        $count_of_existing = 0;



        if (trim($this->username) != '') {

            $condition = 'username="' . trim($this->username) . '"';

            if ($this->id > 0)

                $condition .= ' AND id !=' . $this->id;

            $count_of_existing = self::model()->count(array('condition' => $condition));

        }



        if ($count_of_existing > 0)

            $this->addError('username', 'Username already exist!');

    }




    public function relations()

    {

        return array(

            'outletDetails' => array(self::BELONGS_TO, 'MasOutlets', 'outlet_id'),

            'companyDetails' => array(self::BELONGS_TO, 'MasCompany', 'company_id'),

        );

    }



    public function attributeLabels()

    {

        return array(

            'id' => 'ID',

            'company_id' => 'Company',

            'outlet_id' => 'Outlet',

            'token' => 'Token',

            'username' => 'Username',

            'password' => 'Password',

            'is_active' => 'Is Active',

        );

    }


    public function search()

    {

        $criteria = new CDbCriteria;



        $criteria->compare('id', $this->id);

        $criteria->compare('company_id', $this->company_id);

        $criteria->compare('outlet_id', $this->outlet_id);

        $criteria->compare('token', $this->token, true);

        $criteria->compare('username', $this->username, true);

        $criteria->compare('password', $this->password, true);

        $criteria->compare('is_active', $this->is_active, true);



        return new CActiveDataProvider($this, array(

            'criteria' => $criteria,

        ));

    }


    public static function model($className = __CLASS__)

    {

        return parent::model($className);

    }



    public function get_generate_token()

    {

        $this->token = $this->id . '-' . time();

        $this->update();

    }



    public function validate_token($client_token)

    {

        return $this->token == $client_token && $this->id > 0 ? true : false;

    }



    public function authenticate($attribute, $params)

    {

        if (!$this->hasErrors()) {

            $this->_identity = new UserIdentity($this->username, $this->password);

            if (!$this->_identity->authenticate()) {

                $this->addError('password', 'Incorrect username or password.');

            }

        }

    }



    public function login()

    {

        if ($this->_identity === null) {

            $this->_identity = new UserIdentity($this->username, $this->password);

            $this->_identity->authenticate();

        }

        if ($this->_identity->errorCode === UserIdentity::ERROR_NONE) {

            $duration = $this->rememberMe ? 3600 * 24 * 30 : 0; // 30 days

            Yii::app()->user->login($this->_identity, $duration);

            return true;

        } else {

            return false;

        }

    }

    public function get_outlet_users($outlet)

    {

        $response = array();

        if ($outlet > 0) {

            $response = CHtml::listdata(self::model()->findAll(array('condition' => 'outlet_id=' . $outlet.' AND is_active="Y"')), 'id', 'full_name');

        }

        return $response;

    }



    public function get_outlet_salesmen($outlet)

    {

        $response = array();

        if ($outlet > 0) {

            $response = CHtml::listdata(self::model()->findAll(array('condition' => 'outlet_id=' . $outlet.' AND is_active="Y" AND access_type="SL"')), 'id', 'full_name');

        }

        return $response;

    }
    
    public static function get_outlet_admin($outlet)
    {

        $response = array();

        if ($outlet > 0) {

            $response = self::model()->find(array('condition' => 'outlet_id=' . $outlet.' AND is_active="Y" AND access_type="CA"'))->id;

        }

        return $response;

    }


    public function create_user($username, $password, $access, $outlet)

    {

        $model = new self;

        $model->company_id = Yii::app()->user->company_id;

        $model->outlet_id = $outlet;

        $model->username = $username;

        $model->password = crypt($password, $password);

        $model->full_name = $username;

        $model->phone_no = '050';

        $model->access_type = $access;

        $model->is_active = 'Y';

        $model->save(false);

        $model->get_generate_token();

    }



    public function findBytoken($token = NULL)

    {

        $model = new self;

        if (trim($token) != NULL) {

            $model = self::model()->with('outletDetails')->find(array('condition' => 'outletDetails.is_active="Y" AND t.token="' . trim($token) . '" AND t.is_active="Y" AND CURDATE() <= DATE(outletDetails.subscription_end_date)'));

        }

        return $model;

    }

    public function connect(){
        //Database

        $database = $this->for_main ? $this->db_main : $this->db_name;



        Yii::app()->db->setActive(false);

        Yii::app()->db->connectionString = 'mysql:host=' . $this->db_host . ';dbname=' . $database;

        Yii::app()->db->username = $this->db_username;

        Yii::app()->db->password = $this->db_password;

        Yii::app()->db->tablePrefix = $this->prefix;

        Yii::app()->db->setActive(true);

    }

    public function db($main = false)

    {

        $main_db_arr = explode('=', Yii::app()->db->connectionString);

        $host_array = explode(';', $main_db_arr[1]);



        $this->db_host = $host_array[0];

        $this->db_main = $main_db_arr[2];

        $this->db_name = MasCompany::model()->findBypk((int) $this->company_id)->db;

        $this->db_username =  Yii::app()->db->username;

        $this->db_password =  Yii::app()->db->password;

        $this->prefix =  Yii::app()->db->tablePrefix;

        $this->for_main = $main ? $main : false;



        $this->connect();

    }

    public function auth(){

        return self::model()->findbypk((int) Yii::app()->user->id);

    }
    public static function get_outlet_salesandadmin($outlet,$id=0)

    {

        $response = array();

        if ($outlet > 0) {
            $condtn = 'outlet_id=' . $outlet.' AND is_active="Y"  AND access_type IN("SL", "CA")';
            if( Yii::app()->user->user_access_type == 'SL')
            {
                $condtn .=' AND id ='.(int)Yii::app()->user->id;
            }
           else if(Yii::app()->user->user_access_type == 'SM' || Yii::app()->user->user_access_type == 'OM') {
            $condtn .=' AND access_type !="CA"';    
            }
            else if($id>0){
                $condtn .=' AND id ='.(int)$id;
            }
            $response = CHtml::listdata(self::model()->findAll(array('condition' => $condtn)), 'id', 'full_name');

        }else{
             $response = CHtml::listdata(self::model()->findAll(array('condition' => 'is_active="Y"  AND access_type IN("SL", "CA")')), 'id', 'full_name');
        }
        return $response;

    }

}

