<?php

/**
 * This is the model class for table "erp_order_collect".
 *
 * The followings are the available columns in table '{{order_collect}}':
 * @property string $id
 * @property string $customer_id
 * @property string $order_date
 * @property string $notes
 * @property integer $salesman_id
 * @property integer $outlet
 * @property string $is_invoiced
 * @property integer $created_by
 * @property integer $updated_by
 * @property string $created_date_time
 * @property string $updated_date_time
 */
class OrderCollect extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public $customer_name; 
	public function tableName()
	{
		return '{{order_collect}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('customer_id, order_date,salesman_id,invoice_type, outlet', 'required'),
			array('salesman_id, outlet, created_by, updated_by,invoice_type', 'numerical', 'integerOnly'=>true),
			array('customer_id', 'length', 'max'=>20),
			array('is_invoiced', 'length', 'max'=>1),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, customer_id, order_date, notes, salesman_id, outlet, is_invoiced, created_by, updated_by, created_date_time, updated_date_time', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'oc_customer_det' => array(self::BELONGS_TO, 'Customer', 'customer_id'),
			'oc_outlet_det' => array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
			'oc_salesman_det'=> array(self::BELONGS_TO, 'MasUsers', 'salesman_id'),
			'oc_user'=> array(self::BELONGS_TO, 'MasUsers', 'created_by'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'customer_id' => 'Customer',
			'order_date' => 'Order Date',
			'notes' => 'Notes',
			'salesman_id' => 'Salesman',
			'outlet' => 'Outlet',
			'is_invoiced' => 'Is Invoiced',
			'created_by' => 'Created By',
			'updated_by' => 'Updated By',
			'created_date_time' => 'Created Date Time',
			'updated_date_time' => 'Updated Date Time',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id,true);
		$criteria->compare('customer_id',$this->customer_id,true);
		$criteria->compare('order_date',$this->order_date,true);
		$criteria->compare('notes',$this->notes,true);
		$criteria->compare('salesman_id',$this->salesman_id);
		$criteria->compare('outlet',$this->outlet);
		$criteria->compare('is_invoiced',$this->is_invoiced,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('created_date_time',$this->created_date_time,true);
		$criteria->compare('updated_date_time',$this->updated_date_time,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}
	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return OrderCollect the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
