<?php

/**
 * This is the model class for table "erp_purchase_return".
 *
 * The followings are the available columns in table 'erp_purchase_return':
 * @property integer $id
 * @property string $prefix
 * @property integer $invoice_id
 * @property integer $return_no
 * @property string $reference_no
 * @property string $return_date
 * @property integer $vendor_id
 * @property string $total_amount
 * @property string $excise_total
 * @property string $tax_total
 * @property string $grand_total
 * @property integer $currency
 * @property string $exchange_rate
 * @property integer $created_by
 * @property string $created_time
 * @property integer $updated_by
 * @property string $updated_time
 * @property string $active
 * @property string $posted
 * @property integer $outlet
 * @property string $notes
 */
class PurchaseReturn extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public $vendor_name,$invoice_no;
	public function tableName()
	{
		return '{{purchase_return}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('invoice_id, return_no, vendor_id, total_amount, excise_total, tax_total, grand_total, currency, exchange_rate, outlet', 'required'),
			array('invoice_id, return_no, vendor_id, currency, created_by, updated_by, outlet', 'numerical', 'integerOnly'=>true),
			array('prefix, reference_no', 'length', 'max'=>50),
			array('total_amount, excise_total, tax_total, grand_total, exchange_rate,paid,discount,old_discount', 'numerical'),
			array('active, posted', 'length', 'max'=>1),
			array('notes', 'length', 'max'=>2225),
			array('return_date, created_time, updated_time', 'safe'),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, prefix, invoice_id, return_no, reference_no, return_date, vendor_id, total_amount, excise_total, tax_total, grand_total, currency, exchange_rate, created_by, created_time, updated_by, updated_time, active, posted, outlet, notes', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'pr_cur' => array(self::BELONGS_TO, 'MasCurrency', 'currency'),
            'pr_invoice' => array(self::BELONGS_TO, 'PurchaseInvoice', 'invoice_id'),
		    'pr_outlet' => array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
            'pr_vendor' => array(self::BELONGS_TO, 'Vendor', 'vendor_id'),
            'pr_user' => array(self::BELONGS_TO, 'MasUsers', 'created_by'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'prefix' => 'Prefix',
			'invoice_id' => 'Invoice',
			'return_no' => 'Return No',
			'reference_no' => 'Customer Invoice No',
			'return_date' => 'Return Date',
			'vendor_id' => 'Vendor',
			'total_amount' => 'W/O Tax',
			'excise_total' => 'Excise Total',
			'tax_total' => 'Tax Total',
			'grand_total' => 'Grand Total',
			'currency' => 'Currency',
			'exchange_rate' => 'Exchange Rate',
			'created_by' => 'Created By',
			'created_time' => 'Created Time',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Time',
			'active' => 'N : No, Y : Yes',
			'posted' => 'N : No, Y : Yes',
			'outlet' => 'Outlet',
			'notes' => 'Notes',
                        'paid' => 'Received'
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('prefix',$this->prefix,true);
		$criteria->compare('invoice_id',$this->invoice_id);
		$criteria->compare('return_no',$this->return_no);
		$criteria->compare('reference_no',$this->reference_no,true);
		$criteria->compare('return_date',$this->return_date,true);
		$criteria->compare('vendor_id',$this->vendor_id);
		$criteria->compare('total_amount',$this->total_amount,true);
		$criteria->compare('excise_total',$this->excise_total,true);
		$criteria->compare('tax_total',$this->tax_total,true);
		$criteria->compare('grand_total',$this->grand_total,true);
		$criteria->compare('currency',$this->currency);
		$criteria->compare('exchange_rate',$this->exchange_rate,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('updated_time',$this->updated_time,true);
		$criteria->compare('active',$this->active,true);
		$criteria->compare('posted',$this->posted,true);
		$criteria->compare('outlet',$this->outlet);
		$criteria->compare('notes',$this->notes,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	
	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return PurchaseReturn the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
