<?php

/**
 * This is the model class for table "erp_purchase_return_items".
 *
 * The followings are the available columns in table 'erp_purchase_return_items':
 * @property integer $id
 * @property integer $return_id
 * @property integer $invoice_id
 * @property integer $invoice_item_id
 * @property integer $product_id
 * @property string $returned_quantity
 * @property string $description
 * @property string $invoiced_qty
 * @property integer $unit
 * @property string $rate
 * @property string $amount
 * @property string $excise_percentage
 * @property string $excise_amount
 * @property string $tax_percentage
 * @property string $tax_amount
 * @property string $net_amount
 */
class PurchaseReturnItems extends CActiveRecord
{
	public $part_no,$unit_price,$unit_exchange;
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{purchase_return_items}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('return_id, invoice_id, invoice_item_id, product_id, returned_quantity, invoiced_qty, unit, rate, amount, tax_percentage, tax_amount, net_amount', 'required'),
			array('return_id, invoice_id, invoice_item_id, product_id, unit', 'numerical', 'integerOnly'=>true),
			array('returned_quantity, invoiced_qty, rate, amount, excise_percentage, excise_amount, tax_percentage, tax_amount, net_amount', 'length', 'max'=>15),
			array('description', 'length', 'max'=>255),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, return_id, invoice_id, invoice_item_id, product_id, returned_quantity, description, invoiced_qty, unit, rate, amount, excise_percentage, excise_amount, tax_percentage, tax_amount, net_amount', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'pr_product' => array(self::BELONGS_TO, 'Inventory', 'product_id'),
		    'pr_invoice' => array(self::BELONGS_TO, 'PurchaseInvoice', 'invoice_id'),
		    'pr_return' => array(self::BELONGS_TO, 'PurchaseReturn', 'return_id'),
		    'pr_units' => array(self::BELONGS_TO, 'MasUnit', 'unit'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'return_id' => 'Return',
			'invoice_id' => 'Invoice',
			'invoice_item_id' => 'Invoice Item',
			'product_id' => 'Product',
			'returned_quantity' => 'Returned Quantity',
			'description' => 'Description',
			'invoiced_qty' => 'Invoiced Qty',
			'unit' => 'Unit',
			'rate' => 'Rate',
			'amount' => 'Amount',
			'excise_percentage' => 'Excise Percentage',
			'excise_amount' => 'Excise Amount',
			'tax_percentage' => 'Tax Percentage',
			'tax_amount' => 'Tax Amount',
			'net_amount' => 'Net Amount',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('return_id',$this->return_id);
		$criteria->compare('invoice_id',$this->invoice_id);
		$criteria->compare('invoice_item_id',$this->invoice_item_id);
		$criteria->compare('product_id',$this->product_id);
		$criteria->compare('returned_quantity',$this->returned_quantity,true);
		$criteria->compare('description',$this->description,true);
		$criteria->compare('invoiced_qty',$this->invoiced_qty,true);
		$criteria->compare('unit',$this->unit);
		$criteria->compare('rate',$this->rate,true);
		$criteria->compare('amount',$this->amount,true);
		$criteria->compare('excise_percentage',$this->excise_percentage,true);
		$criteria->compare('excise_amount',$this->excise_amount,true);
		$criteria->compare('tax_percentage',$this->tax_percentage,true);
		$criteria->compare('tax_amount',$this->tax_amount,true);
		$criteria->compare('net_amount',$this->net_amount,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return PurchaseReturnItems the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
}
