<?php

/**
 * This is the model class for table "erp_vendor".
 *
 * The followings are the available columns in table 'erp_vendor':
 * @property integer $id
 * @property string $vendor_name
 * @property string $address
 * @property string $location
 * @property integer $state
 * @property integer $country
 * @property string $credit_limit
 * @property string $phone
 * @property string $email
 * @property string $contact_person
 * @property string $mobile
 * @property integer $payment_term
 * @property string $trn_no
 * @property string $gps_coordinates
 * @property integer $created_by
 * @property string $created_time
 * @property integer $updated_by
 * @property string $updated_time
 * @property string $active
 * @property integer $outlet
 */
class Vendor extends CActiveRecord
{
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{vendor}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('vendor_name, state, country,payment_term,trn_no, phone, credit_limit, outlet', 'required'),
			array('vendor_no,state, country, payment_term, created_by, updated_by, outlet', 'numerical', 'integerOnly'=>true),
			array('vendor_name, contact_person, trn_no', 'length', 'max'=>150),
			array('address, gps_coordinates', 'length', 'max'=>255),
			array('location, phone, email, mobile', 'length', 'max'=>100),
			array('credit_limit', 'length', 'max'=>15),
			array('active', 'length', 'max'=>1),
			array('created_time, updated_time', 'safe'),
			array('vendor_name', 'checkUniqueness'),
			// The following rule is used by search().
			// @todo Please remove those attributes that should not be searched.
			array('id, vendor_name, address, location, state, country, credit_limit, phone, email, contact_person, mobile, payment_term, trn_no, gps_coordinates, created_by, created_time, updated_by, updated_time, active, outlet', 'safe', 'on'=>'search'),
		);
	}

	public function checkUniqueness($attribute,$params)
    {
			if($this->id>0)
			{
			$model = Vendor::model()->find('vendor_name = "'.$this->vendor_name.'" AND id!='.$this->id.' AND outlet='.$this->outlet);
			}
            else {
			$model = Vendor::model()->find('vendor_name = "'.$this->vendor_name.'" AND outlet='.$this->outlet);	
			}
			if(count($model)>0)	
			{
			$this->addError('id','This  Vendor already exist'); 
			}
    }
	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		    'outlet_detail' => array(self::BELONGS_TO, 'MasOutlets', 'outlet'),
            'state_detail' => array(self::BELONGS_TO, 'MasState', 'state'),
            'country_detail' => array(self::BELONGS_TO, 'MasCountry', 'country'),
            'paymentterm_detail' => array(self::BELONGS_TO, 'MasPaymentTerms', 'payment_term'),
		);
	}
    
    /**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => 'ID',
			'vendor_name' => 'Vendor Name',
			'address' => 'Address',
			'location' => 'Location',
			'state' => 'State',
			'country' => 'Country',
			'credit_limit' => 'Credit Limit',
			'phone' => 'Phone',
			'email' => 'Email',
			'contact_person' => 'Contact Person',
			'mobile' => 'Mobile',
			'payment_term' => 'Payment Term',
			'trn_no' => 'Trn No',
			'gps_coordinates' => 'Gps Coordinates',
			'created_by' => 'Created By',
			'created_time' => 'Created Time',
			'updated_by' => 'Updated By',
			'updated_time' => 'Updated Time',
			'active' => 'N: No, Y: Yes',
			'outlet' => 'Outlet',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		// @todo Please modify the following code to remove attributes that should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id);
		$criteria->compare('vendor_name',$this->vendor_name,true);
		$criteria->compare('address',$this->address,true);
		$criteria->compare('location',$this->location,true);
		$criteria->compare('state',$this->state);
		$criteria->compare('country',$this->country);
		$criteria->compare('credit_limit',$this->credit_limit,true);
		$criteria->compare('phone',$this->phone,true);
		$criteria->compare('email',$this->email,true);
		$criteria->compare('contact_person',$this->contact_person,true);
		$criteria->compare('mobile',$this->mobile,true);
		$criteria->compare('payment_term',$this->payment_term);
		$criteria->compare('trn_no',$this->trn_no,true);
		$criteria->compare('gps_coordinates',$this->gps_coordinates,true);
		$criteria->compare('created_by',$this->created_by);
		$criteria->compare('created_time',$this->created_time,true);
		$criteria->compare('updated_by',$this->updated_by);
		$criteria->compare('updated_time',$this->updated_time,true);
		$criteria->compare('active',$this->active,true);
		$criteria->compare('outlet',$this->outlet);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}


	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return Vendor the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
	public static function getAllVendors(){
	    return CHtml::listData(self::model()->findAll( array( 'condition' => 'active="Y" AND outlet='.Yii::app()->user->outlet_id, 'order' => 'vendor_name ASC' ) ), 'id', 'vendor_name');
	}
}
