<?php

/**
 * Esell Sales Module
 * Author  : Jisha
 * Created :  27/07/2020
 */
class ApiController extends Controller
{

    public $method = '_stream';
    public $round_off = '2';
    public $db_main = MasCompany::MAIN_DB;
    public $outlet_id;
    public $responseBody = array(
        'status' => 'success',
        'msg' => '',
        'count' => 0,
        'data' => array(),
    );
    public $permissions = array(
        'view_cost_price' => true,
    );

    public function accessRules()
    {
        return array(
            array(
                'allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array(
                    'Have_session',
                    'Login',
                    'SalesLogin',
                    'Getusers',
                    'About',
                    'Dashboard',
                    'Version_check',
                    'Download',
                    'GetAppLogo',
                    'UpdateLocation',
                    'Reports'
                ),
                'users' => array('*'),
            ),
            array(
                'allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(),
                'expression' => '$user->isMember()',
            ),
            array(
                'deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    public function save_request($request)
    {
        // if ($request != '') {
        //     Yii::app()->response->createCommand("INSERT INTO response (app_response) VALUES('" . $request . "')")->execute();
        // }
        return true;
    }

    public function get_401()
    {
        header("HTTP/1.1 401 Unauthorized");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Unauthorized user';
        echo json_encode($this->responseBody);
    }

    public function get_400()
    {
        header("HTTP/1.1 400 Bad Request");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function get_403()
    {
        header("HTTP/1.1 403  Forbidden");
        header('Content-Type: application/json');
        $this->responseBody['status'] = 'failed';
        $this->responseBody['msg'] = 'Forbidden user';
        echo json_encode($this->responseBody);
    }

    public function get_bind()
    {
        header("HTTP/1.1 200 OK");
        header('Content-Type: application/json');
        echo json_encode($this->responseBody);
    }

    public function format_currency($value)
    {
        return number_format(round((float) $value, $this->round_off), $this->round_off, '.', '');
    }

    public function get_set_permissions()
    {
        $this->responseBody['permissions']['have_purchase'] = true;
        $this->responseBody['permissions']['consider_min_sales_price'] = true;
        $this->responseBody['permissions']['have_financial_reports'] = true;
        $this->responseBody['permissions']['have_stock_adjustment'] = true;
        $this->responseBody['permissions']['view_cost_price'] = $this->permissions['view_cost_price'];
    }

    public function convert_date($date = null)
    {

        $date_converted = date('Y-m-d');

        if ($date != null) {
            //yyyy-mm-dd
            $arr_date = explode('-', $date);
            //mm/dd/yyyy
            if (checkdate($arr_date[1], $arr_date[2], $arr_date[0])) {
                $date_converted = date('Y-m-d', strtotime(str_replace('/', '-', $date)));
            }
        }
        return $date_converted;
    }

    public function actionHave_session()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->with('outletDetails')->find(array('condition' => 't.token="' . $token . '" AND t.is_active="Y" AND t.access_type IN("SL", "CA","SM", "OM") AND outletDetails.is_active="Y"'));
        if ((int) $userModel->id > 0) {
            $this->responseBody['status'] = 'success';
            $this->responseBody['msg'] = 'Session is valid';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function connectMainDB($company_db_name)
    {
        Yii::app()->db->setActive(false);
        if (empty($company_db_name)) {
            $company_db_name = '=bluesell_new_main';
        }
        Yii::app()->db->connectionString = 'mysql:host=e-sell.me;dbname=bluesell_new_main';
        Yii::app()->db->username = 'bluesell_newadmn';
        Yii::app()->db->password = 'DKe=?ok*+KD}';
        Yii::app()->db->tablePrefix = 'erp_';
        Yii::app()->db->setActive(true);
    }

    //changed
    public function actionLogin()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $username = trim($variables['uname']);
        $password = trim($variables['pword']);
        if ($username != '' && $password != '') {
            $password = crypt($password, $password);
            $userModel = MasUsers::model()->with('outletDetails')->find(array('condition' => 't.username="' . $username . '" AND t.password="' . $password . '" AND t.is_active="Y" AND t.access_type IN("SL", "CA","SM", "OM") AND outletDetails.is_active="Y"'));
            if ($userModel->id > 0) {
                //New token
                $userModel->_generate_token;

                $this->responseBody['data']['user_id'] = $userModel->id;
                $this->responseBody['data']['access_type'] = $userModel->access_type;
                $this->responseBody['data']['fullname'] = $userModel->full_name;
                $this->responseBody['data']['phone_number'] = $userModel->phone_no;
                $this->responseBody['data']['outlet_id'] = $userModel->outlet_id;
                $this->responseBody['data']['company_id'] = $userModel->company_id;
                $this->responseBody['data']['company_name'] = $userModel->outletDetails->name;
                $this->responseBody['data']['company_name_alias'] = $userModel->companyDetails->name_other;
                $this->responseBody['data']['company_adds'] = $userModel->outletDetails->address;
                $this->responseBody['data']['outlet_trn_no'] = $userModel->outletDetails->trn_no;
                $this->responseBody['data']['subscription_end_date'] = $userModel->outletDetails->subscription_end_date;
                $this->responseBody['data']['company_logo'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . '/images/' . $userModel->companyDetails->logo;
                $this->responseBody['data']['outlet_logo'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . '/images/' . $userModel->outletDetails->logo;
                $this->responseBody['data']['token'] = $userModel->token;

                $this->_set_permissions;
                $this->_bind;
            } else {
                $this->_401;
            }
        } else {
            $this->_401;
        }
    }

    //created by jisha
    public function actionSalesLogin()
    {
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $username = trim($variables['uname']);
        $password = trim($variables['pword']);
        if ($username != '' && $password != '') {
            $password = crypt($password, $password);
            $userModel = MasUsers::model()->with('outletDetails')->find(array('condition' => 't.username="' . $username . '" AND t.password="' . $password . '" AND t.is_active="Y" AND t.access_type IN("SL", "CA","SM", "OM") AND outletDetails.is_active="Y"'));
            if ($userModel->id > 0) {

                $this->responseBody['data']['user_id'] = $userModel->id;
                $this->responseBody['data']['access_type'] = $userModel->access_type;
                $this->responseBody['data']['fullname'] = $userModel->full_name;
                $this->responseBody['data']['phone_number'] = $userModel->phone_no;
                $this->responseBody['data']['outlet_id'] = $userModel->outlet_id;
                $this->responseBody['data']['company_id'] = $userModel->company_id;
                $this->responseBody['data']['company_name'] = $userModel->outletDetails->name;
                $this->responseBody['data']['company_name_alias'] = $userModel->companyDetails->name_other;
                $this->responseBody['data']['company_adds'] = $userModel->outletDetails->address;
                $this->responseBody['data']['outlet_trn_no'] = $userModel->outletDetails->trn_no;
                $this->responseBody['data']['subscription_end_date'] = $userModel->outletDetails->subscription_end_date;
                $this->responseBody['data']['company_logo'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . '/images/' . $userModel->companyDetails->logo;
                $this->responseBody['data']['outlet_logo'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . '/images/' . $userModel->outletDetails->logo;
                $this->responseBody['data']['token'] = $userModel->token; //existing token

                $this->_set_permissions;
                $this->_bind;
            } else {
                $this->_401;
            }
        } else {
            $this->_401;
        }
    }
    //changed
    public function actionGetusers()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        $input = file_get_contents('php://input');
        $variables = json_decode($input, true);
        $term = $variables['term'];
        $term = '%' . $term . '%';


        if ($userModel->id > 0) {
            $count = 0;
            if ($userModel->access_type == 'SL') {
                $items = MasUsers::model()->findAll(array('condition' => 'outlet_id=' . (int) $userModel->outlet_id . '  AND (username LIKE \'' . $term . '\') AND access_type IN("SL", "CA") AND is_active="Y" AND id=' . $userModel->id));
            } else {
                $items = MasUsers::model()->findAll(array('condition' => 'outlet_id=' . (int) $userModel->outlet_id . '  AND (username LIKE \'' . $term . '\') AND access_type IN("SL", "CA") AND is_active="Y"'));
            }
            foreach ($items as $i => $each) {
                $this->responseBody['data'][$i]['id'] = $each->id;
                $this->responseBody['data'][$i]['name'] = $each->full_name;
                $this->responseBody['data'][$i]['outlet_id'] = $userModel->outlet_id;
                $count++;
            }
            $this->responseBody['count'] = $count;
            $this->_bind;
        } else {
            $this->_401;
        }
    }





    public function actionAbout()
    {

        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {

            $versions = $this->getVersion();
            $this->responseBody['app_base'] = $versions['app_base'];
            $this->responseBody['app_version'] = $versions['app_version'];
            $this->responseBody['licensed_to'] = $userModel->outletDetails->name;
            $this->responseBody['licensed_end_date'] = $userModel->outletDetails->subscription_end_date;
            $this->responseBody['remaining_days'] = $userModel->outletDetails->remaining_days();
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function actionDashboard()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        date_default_timezone_set("Asia/Dubai");
        if ($userModel->id > 0) {

            $userModel->db();

            $totalCash = 0;
            $totalBank = 0;
            $totalReceivable = 0;
            $totalPayable = 0;
            $sales_invoices = 0;
            $date = date('Y-m-d');
            if ($userModel->access_type == 'CA') {
                $bank = FinanceSystemAccountsMapping::getSystemdefault('bank');
                $totalBank = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.voucher_date<='" . $date . "' AND H.is_posted='Y' AND I.ledger_id=" . $bank . " AND H.outlet=" . $userModel->outlet_id)->queryScalar();
                $debtors = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                $totalReceivable = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.voucher_date<='" . $date . "' AND H.is_posted='Y' AND I.ledger_id=" . $debtors . " AND H.outlet=" . $userModel->outlet_id)->queryScalar();
                $creditors = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                $totalPayable = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.voucher_date<='" . $date . "' AND H.is_posted='Y' AND I.ledger_id=" . $creditors . " AND H.outlet=" . $userModel->outlet_id)->queryScalar();
            }
            $cash = FinanceSystemAccountsMapping::getSystemdefault('cash_account');
            $totalCash = Yii::app()->db->createCommand("select SUM(amount) AS amount from {{finance_voucher_items}} AS I LEFT JOIN {{finance_voucher_head}} AS H ON I.header_id=H.id where H.is_posted='Y' AND I.ledger_id=" . $cash . " AND H.outlet=" . $userModel->outlet_id . " AND H.created_by=" . $userModel->id)->queryScalar();
            $current_date          = date('Y-m-d H:i:s');
            $subscription_end_date = $userModel->outletDetails->subscription_end_date;
            $notify_days           = $userModel->outletDetails->notify_days;
            $first_date = strtotime($current_date);
            $second_date = strtotime($subscription_end_date);
            $datediff = $second_date - $first_date;
            $diff = round($datediff / (60 * 60 * 24));
            $this->responseBody['data'] = array(
                'server_date_time'      => $current_date,
                'subscription_end_date' => $subscription_end_date,
                'reminder_days'         => $notify_days,
                'date_difference'       => $diff,
                'cash_in_hand' => $totalCash > 0 ? $this->format_currency($totalCash) : 0,
                'cash_in_bank' => $totalBank > 0 ? $this->format_currency($totalBank) : 0,
                'total_receivable' => $totalReceivable > 0 ? $this->format_currency($totalReceivable) : 0
            );

            if ($userModel->access_type == 'CA') {
                $this->responseBody['data']['total_payable'] = $totalPayable < 0 ? $this->format_currency(abs($totalPayable)) : 0;
            } else {
                $this->responseBody['data']['no_of_sales'] = (int) $sales_invoices;
            }
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function getVersion()
    {
        return array('app_base' => 1, 'app_version' => '1.0beta');
    }

    public function actionVersion_check()
    {
        $this->responseBody['status'] = 'update';

        $versions = $this->getVersion();
        $this->responseBody['version'] = $versions['app_base'];

        $this->_bind;
    }


    public function delete_file($file_name = null, $path_user = null)
    {
        if ($file_name != null && $path_user != null) {
            if (!file_exists($path_user . $file_name)) {
                if (mkdir($path_user, 0777, false)) {
                }
            } else {
                unlink($path_user . $file_name);
            }
        }
    }






    public function actionDownload()
    {
        $filename = 'https://e-sell.me/new/app_reports/customer_statement1622956851.pdf';
        header('Content-Disposition: attachment; charset=UTF-8; filename="' . $filename . '"');
        $utf8_content = mb_convert_encoding($content, "SJIS", "UTF-8");
        echo $utf8_content;
        Yii::app()->end();
        return;
    }

    public function actionGetAppLogo()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            $userModel->db();
            $app_logo = $userModel->companyDetails->app_logo;
            $this->responseBody['status'] = 'success';
            $this->responseBody['url'] = $app_logo;
            $this->_bind;
        } else {
            $this->responseBody['msg'] = 'Unauthorized user';
            $this->responseBody['status'] = 'failed';
            $this->_400;
        }
    }
    public function actionUpdateLocation()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);

        if ($userModel->id > 0) {
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $customer_id = $variables['id'];
            if ($customer_id > 0) {
                $model = Customer::model()->findByPk($customer_id);
                $model->latitude = $variables['latitude'];
                $model->longitude = $variables['longitude'];
                $model->save();
                $this->responseBody['status'] = 'success';
                $this->_bind;
            } else {
                $this->responseBody['msg'] = 'select customer';
                $this->responseBody['status'] = 'failed';
                $this->_400;
            }
        } else {
            $this->responseBody['msg'] = 'Unauthorized user';
            $this->responseBody['status'] = 'failed';
            $this->_400;
        }
    }



    //Reports function start
    public function actionReports()
    {
        $token = filter_var($_SERVER['HTTP_TOKEN'], FILTER_SANITIZE_STRING);
        $userModel = MasUsers::model()->findBytoken($token);
        if ($userModel->id > 0) {
            //Outlet id
            $this->outlet_id = $userModel->outlet_id;
            $userModel->db();
            $input = file_get_contents('php://input');
            $variables = json_decode($input, true);
            $report_type = $variables['type'];
            $outlet = 0;
            $salesman = (int) $variables['salesman'];
            if ($userModel->access_type == 'CA') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'OM') {
                $outlet = (int) $userModel->outlet_id;
            }
            if ($userModel->access_type == 'SL') {
                $outlet = (int) $userModel->outlet_id;
                $salesman = (int) $userModel->id;
            }
            $outlet = (int) $userModel->outlet_id;

            $conditionFormats = array(
                'sales' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Sales Invoices',
                    'mainDb' => $this->db_main,
                ),
                'sales_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Sales Returns',
                    'mainDb' => $this->db_main,
                ),
                'purchase' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Purchase Invoices',
                    'mainDb' => $this->db_main,
                ),
                'purchase_return' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Purchase Returns',
                    'mainDb' => $this->db_main,
                ),
                'net_sales' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Net Sales',
                    'mainDb' => $this->db_main,
                ),
                'net_purchase' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'vendor_id' => (int) $variables['vendor'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Net Purchase',
                    'mainDb' => $this->db_main,
                ),
                'customer' => array(
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Customers',
                    'mainDb' => $this->db_main,
                ),
                'vendor' => array(
                    'vendor_id' => (int) $variables['vendor_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Vendors',
                    'mainDb' => $this->db_main,
                ),
                'current_stock' => array(
                    'product_id' => (int) $variables['product_id'],
                    'product_group_id' => (int) $variables['product_group_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Current Stock',
                    'mainDb' => $this->db_main,
                    'access_type' => $userModel->access_type
                ),
                'transaction' => array(
                    'access_type' => $userModel->access_type,
                    'user_id' => $userModel->id,
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'product_id' => (int) $variables['product_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Transaction Details',
                    'mainDb' => $this->db_main,
                ),
                'customer_statement' => array(
                    'access_type' => $userModel->access_type,
                    'user_id' => $userModel->id,
                    'outlet_id' => $outlet,
                    'customer_id' => (int) $variables['customer_id'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Customer Statement',
                    'mainDb' => $this->db_main,
                ),
                'vendor_statement' => array(
                    'vendor_id' => (int) $variables['vendor_id'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Vendor Statement',
                    'mainDb' => $this->db_main,
                ),
                'receivable_aging' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Receivable Aging',
                    'mainDb' => $this->db_main,
                ),
                'payable_aging' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Payable Aging',
                    'mainDb' => $this->db_main,
                ),
                'day_book' => array(
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Day Book',
                    'mainDb' => $this->db_main,
                ),
                'stock_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Stock Ledger',
                    'mainDb' => $this->db_main,
                ),
                'vat_return_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Return',
                    'mainDb' => $this->db_main,
                ),
                'vat_summary_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'VAT Summary',
                    'mainDb' => $this->db_main,
                ),
                'consolidated_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Consolidated Report',
                    'mainDb' => $this->db_main,
                ),
                'daily_sales_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Daily Sales Report',
                    'mainDb' => $this->db_main,
                ),
                'cost_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'brand_id' => (int) $variables['brand_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cost Report',
                    'mainDb' => $this->db_main,
                ),
                'cash_collection_report' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['report_type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman_id' => $salesman,
                    'outlet_id' => $outlet,
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cash Collection Report',
                    'mainDb' => $this->db_main,
                ),
                'costofsale' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman' => (int)$salesman,
                    'outlet_id' => $outlet,
                    'brand' => (int) $variables['brand'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Cost of Sale',
                    'mainDb' => $this->db_main,
                ),
                'brandwise' => array(
                    'from_date' => $variables['from_date'],
                    'to_date' => $variables['to_date'],
                    'type' => $variables['type'],
                    'customer_id' => (int) $variables['customer_id'],
                    'salesman' => (int)$variables['salesman'],
                    'outlet_id' => $outlet,
                    'brand' => (int) $variables['brand'],
                    'payment_term' => (int) $variables['payment_term'],
                    'view' => 'pdf_report_template_app',
                    'title' => 'Brand wise report',
                    'mainDb' => $this->db_main,
                ),
                /*
                      'tax_register' => array(
                      'from_date' => $variables['from_date'],
                      'to_date' => $variables['to_date'],
                      'outlet_id' => $outlet,
                      'view' => 'pdf_report_template_app',
                      'title' => 'Tax Register',
                      'mainDb' => $this->db_main,
                      ),
                      'vat_return' => array(
                      'from_date' => $variables['from_date'],
                      'to_date' => $variables['to_date'],
                      'outlet_id' => $outlet,
                      'view' => 'pdf_report_template_app',
                      'title' => 'VAT Region wise',
                      'mainDb' => $this->db_main,
                      ),
                      'trial_balance' => array(
                      'from_date' => $variables['from_date'],
                      'to_date' => $variables['to_date'],
                      'outlet_id' => $outlet,
                      'view' => 'pdf_report_template_app',
                      'title' => 'Trial balance',
                      'mainDb' => $this->db_main,
                      ),
                      'balance_sheet' => array(
                      'to_date' => $variables['to_date'],
                      'company_id' => $userModel->company_id,
                      'outlet_id' => $outlet,
                      'view' => 'pdf_report_template_app',
                      'title' => 'Balance Sheet',
                      'mainDb' => $this->db_main,
                      ),
                      'profit_and_loss' => array(
                      'from_date' => $variables['from_date'],
                      'to_date' => $variables['to_date'],
                      'outlet_id' => $outlet,
                      'view' => 'pdf_report_template_app',
                      'title' => 'Profit and Loss',
                      'mainDb' => $this->db_main,
                      ), */
            );

            //SQL Execution
            $this->generate_view($report_type, $conditionFormats[$report_type]);

            //Negative symbol
            function getSymbol($value = 0)
            {
                return $value >= 0 ? '' : '(-) ';
            }

            //Space Addition
            function addSpace($level = 0)
            {
                $space = '';
                for ($i = 0; $i < $level * 4; $i++) {
                    $space .= '&nbsp;';
                }
                return $space;
            }

            function get_value_label($value = 0)
            {
                return $value > 0 ? ' Dr' : ' Cr';
            }

            //HTML Data Generation
            function generate_details($report_type, $conditionFormats)
            {

                $table = '';
                $params = $conditionFormats[$report_type];

                if ($report_type == 'sales') {

                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Inv #</th>
                                            <th style="width:10%">Customer</th>
                                            <th style="width:5%">Salesman</th>
                                            <th class="bg-danger" style="width:4%">VAT</th>
                                            <th class="bg-danger" style="width:4%">Grand Total</th>
                                            <th style="width:1%">Currency</th>
                                            <th class="bg-success" style="width:4%">VAT(BC)</th>
                                            <th class="bg-success" style="width:4%">Grand Total(BC)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $salesInvoices = Yii::app()->db->createCommand('SELECT * FROM view_report_sales_invoice_list')->queryAll();
                    $total_vat = $grand_total = $total_vat_bc = $grand_total_bc = $allocated_amount = 0;

                    foreach ($salesInvoices as $each) {
                        if ($each['active'] == 'N') {
                            $each['total_vat'] = 0;
                            $each['grand_total'] = 0;
                            $each['total_vat_bc'] = 0;
                            $each['grand_total_bc'] = 0;
                        }
                        $table .= '<tr>
                                        <td>' . date('d/m/Y', strtotime($each['date'])) . '</td>
                                        <td>' . $each['voucher_no'] . '</td>
                                        <td>' . $each['customer_name'] . '</td>
                                        <td>' . $each['sales_man'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total'], " ") . '</td>
                                        <td>' . $each['currency'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat_bc'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total_bc'], " ") . '</td>
                                    </tr>';

                        $total_vat += $each['total_vat'];
                        $grand_total += $each['grand_total'];
                        $total_vat_bc += $each['total_vat_bc'];
                        $grand_total_bc += $each['grand_total_bc'];
                    }

                    $table .= '<tr>
                                    <td colspan="7" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_bc, " ") . '</b></td>
                                </tr>';

                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'sales_return') {

                    if ($each['active'] == 'N') {
                        $each['total_vat'] = 0;
                        $each['grand_total'] = 0;
                        $each['total_vat_bc'] = 0;
                        $each['grand_total_bc'] = 0;
                    }
                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Rtn #</th>
                                            <th style="width:1%">Inv #</th>
                                            <th style="width:10%">Customer</th>
                                            <th style="width:5%">Salesman</th>
                                            <th class="bg-danger" style="width:4%">VAT</th>
                                            <th class="bg-danger" style="width:4%">Grand Total</th>
                                            <th style="width:1%">Currency</th>
                                            <th class="bg-success" style="width:4%">VAT(BC)</th>
                                            <th class="bg-success" style="width:4%">Grand Total(BC)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $salesInvoices = Yii::app()->db->createCommand('SELECT * FROM view_report_creditnote_list')->queryAll();
                    $total_vat = $grand_total = $total_vat_bc = $grand_total_bc = $allocated_amount = 0;

                    foreach ($salesInvoices as $each) {

                        if ($each['active'] == 'N') {
                            $each['total_vat'] = 0;
                            $each['grand_total'] = 0;
                            $each['total_vat_bc'] = 0;
                            $each['grand_total_bc'] = 0;
                        }
                        $table .= '<tr>
                                        <td>' . date('d/m/Y', strtotime($each['date'])) . '</td>
                                        <td>' . $each['voucher_no'] . '</td>
                                        <td>' . $each['invoice_no'] . '</td>
                                        <td>' . $each['customer_name'] . '</td>
                                        <td>' . $each['sales_man'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total'], " ") . '</td>
                                        <td>' . $each['currency'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat_bc'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total_bc'], " ") . '</td>
                                    </tr>';

                        $total_vat += $each['total_vat'];
                        $grand_total += $each['grand_total'];
                        $total_vat_bc += $each['total_vat_bc'];
                        $grand_total_bc += $each['grand_total_bc'];
                    }

                    $table .= '<tr>
                                    <td colspan="8" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_bc, " ") . '</b></td>
                                </tr>';

                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'purchase') {


                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Inv #</th>
                                            <th style="width:10%">Vendor</th>
                                            <th style="width:5%">Salesman</th>
                                            <th style="width:4%">VAT</th>
                                            <th style="width:4%">Grand Total</th>
                                            <th style="width:1%">Currency</th>
                                            <th  style="width:4%">VAT(BC)</th>
                                            <th  style="width:4%">Grand Total(BC)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $salesInvoices = Yii::app()->db->createCommand('SELECT * FROM view_report_purchase_invoice_list')->queryAll();
                    $total_vat = $grand_total = $total_vat_bc = $grand_total_bc = $allocated_amount = 0;

                    foreach ($salesInvoices as $each) {
                        if ($each['active'] == 'N') {
                            $each['total_vat'] = 0;
                            $each['grand_total'] = 0;
                            $each['total_vat_bc'] = 0;
                            $each['grand_total_bc'] = 0;
                        }

                        $table .= '<tr>
                                        <td>' . date('d/m/Y', strtotime($each['date'])) . '</td>
                                        <td>' . $each['voucher_no'] . '</td>
                                        <td>' . $each['vendor_name'] . '</td>
                                        <td>' . $each['sales_man'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total'], " ") . '</td>
                                        <td>' . $each['currency'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat_bc'], " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total_bc'], " ") . '</td>
                                    </tr>';

                        $total_vat += $each['total_vat'];
                        $grand_total += $each['grand_total'];
                        $total_vat_bc += $each['total_vat_bc'];
                        $grand_total_bc += $each['grand_total_bc'];
                    }

                    $table .= '<tr>
                                    <td colspan="7" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_bc, " ") . '</b></td>
                                </tr>';

                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'purchase_return') {

                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Rtn #</th>
                                            <th style="width:1%">Inv #</th>
                                            <th style="width:10%">Vendor</th>
                                            <th style="width:5%">Salesman</th>
                                            <th style="width:4%">VAT</th>
                                            <th style="width:4%">Grand Total</th>
                                            <th style="width:1%">Currency</th>
                                            <th  style="width:4%">VAT(BC)</th>
                                            <th  style="width:4%">Grand Total(BC)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $salesInvoices = Yii::app()->db->createCommand('SELECT * FROM view_report_debitnote_list')->queryAll();
                    $total_vat = $grand_total = $total_vat_bc = $grand_total_bc = $allocated_amount = 0;

                    foreach ($salesInvoices as $each) {

                        if ($each['active'] == 'N') {
                            $each['total_vat'] = 0;
                            $each['grand_total'] = 0;
                            $each['total_vat_bc'] = 0;
                            $each['grand_total_bc'] = 0;
                        }
                        $table .= '<tr>
                                                        <td>' . date('d/m/Y', strtotime($each['date'])) . '</td>
                                                        <td>' . $each['voucher_no'] . '</td>
                                                        <td>' . $each['invoice_no'] . '</td>
                                                        <td>' . $each['vendor_name'] . '</td>
                                                        <td>' . $each['sales_man'] . '</td>
                                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat'], " ") . '</td>
                                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total'], " ") . '</td>
                                                        <td>' . $each['currency'] . '</td>
                                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['total_vat_bc'], " ") . '</td>
                                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['grand_total_bc'], " ") . '</td>
                                                    </tr>';

                        $total_vat += $each['total_vat'];
                        $grand_total += $each['grand_total'];
                        $total_vat_bc += $each['total_vat_bc'];
                        $grand_total_bc += $each['grand_total_bc'];
                    }

                    $table .= '<tr>
                                                        <td colspan="8" class="boldy text-right"><b>Total</b></td>
                                                        <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat_bc, " ") . '</b></td>
                                                        <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_bc, " ") . '</b></td>
                                                    </tr>';

                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'net_sales') {
                    $condition = '1=1 ';
                    $condition .= ' AND t.invoice_date>="' . $params['from_date'] . '"';
                    $condition .= ' AND t.invoice_date<="' . $params['to_date'] . '"';
                    if ($params['outlet_id'] > 0) {
                        $condition .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['customer_id'] > 0) {
                        $condition .= ' AND t.customer_id=' . $params['customer_id'];
                    }
                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND (t.sales_man=' . $params['salesman_id'] . ')';
                    }
                    $net_sales_sql = 'SELECT * FROM view_net_sale_report t 
                    WHERE ' . $condition . ' ORDER BY invoice_date asc,type DESC,invoice_no ASC';
                    $arr_invoices = Yii::app()->db->createCommand($net_sales_sql)->queryAll();
                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Inv #/CN. #</th>
                                            <th style="width:10%">Customer</th>
                                            <th style="width:5%">Salesman</th>
                                            <th>Sales Amt(BC)</th>
                                            <th>Inv.VAT(BC)</th>
                                            <th>Sales Amt with VAT(BC)</th>
                                            <th>CRN Amt(BC)</th>
                                            <th>CRN VAT(BC)</th>
                                            <th>CRN Amt with VAT(BC)</th>
                                            <th>Net Amt(AED)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $total_inv_amount_bc = $total_inv_vat_bc = $total_inv_grand_bc = 0;
                    $total_ret_amount_bc = $total_ret_vat_bc = $total_ret_grand_bc = 0;
                    $total_net_amount_bc = 0;
                    $sl = 1;
                    if (!empty($arr_invoices)) {
                        foreach ($arr_invoices as $each) {

                            if ($each['active'] == 'N') {
                                $each['total_amount'] = 0;
                                $each['tax_total'] = 0;
                                $each['grand_total'] = 0;
                            }

                            if ($each["type"] == 'I') {
                                $invoice_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $invoice_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $invoice_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                                $return_amount_bc = 0;
                                $return_vat_bc = 0;
                                $return_grandtotal_bc = 0;
                            } else {
                                $invoice_amount_bc = 0;
                                $invoice_vat_bc = 0;
                                $invoice_grandtotal_bc = 0;
                                $return_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $return_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $return_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                            }
                            $net_total = $invoice_grandtotal_bc - $return_grandtotal_bc;
                            $total_inv_amount_bc += $invoice_amount_bc;
                            $total_inv_vat_bc += $invoice_vat_bc;
                            $total_inv_grand_bc += $invoice_grandtotal_bc;
                            $total_ret_amount_bc += $return_amount_bc;
                            $total_ret_vat_bc += $return_vat_bc;
                            $total_ret_grand_bc += $return_grandtotal_bc;
                            $total_net_amount_bc += $net_total;
                            $table .= '<tr>
                                        <td>' . date('d/m/Y', strtotime($each['invoice_date'])) . '</td>
                                        <td>' . $each['invoice_no'] . '</td>
                                        <td>' . $each['customer_name'] . '</td>
                                        <td>' . $each['full_name'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_amount_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_vat_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_grandtotal_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_amount_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_vat_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_grandtotal_bc, " ") . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($net_total, " ") . '</td>
                                    </tr>';
                            $grand_total_bc += $each['grand_total_bc'];
                            $total_vat_bc += $each['total_vat_bc'];
                            $total_disc_bc += $each['discount_bc'];

                            $total_returns_amount_bc += $each['return_total'];
                            $total_returns_vat_bc += $each['return_vat'];
                            $total_returns_disc_bc += $each['return_disount'];
                            $totalSales += $netSale;
                        }

                        $table .= '<tr>
                                    <td colspan="4" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_net_amount_bc, " ") . '</b></td>
                                </tr>';
                        $table .= '<tr>
                                <th colspan="10" class="text-right bg-info">Net Sales Without VAT</th> 
                                <th class="text-right bg-info">' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc - $total_ret_amount_bc, "") . '</th>
                             </tr>';
                    } else {
                        $table .= '<tr>
                                <th colspan="11" class="text-center bg-info">No Details Found</th>
                             </tr>';
                    }
                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'net_purchase') {

                    $condition = '1=1 ';
                    $condition .= ' AND t.invoice_date>="' . $params['from_date'] . '"';
                    $condition .= ' AND t.invoice_date<="' . $params['to_date'] . '"';
                    if ($params['outlet_id'] > 0) {
                        $condition .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['vendor_id'] > 0) {
                        $condition .= ' AND t.vendor_id=' . $params['vendor_id'];
                    }
                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND (t.created_by=' . $params['salesman_id'] . ')';
                    }
                    $purchase_sql = 'SELECT * FROM view_net_purchase_report t 
                    WHERE ' . $condition . ' ORDER BY invoice_date asc,type DESC';
                    $arr_invoices = Yii::app()->db->createCommand($purchase_sql)->queryAll();
                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:1%">Date</th>
                                            <th style="width:1%">Inv./DN. #</th>
                                            <th style="width:10%">Vendor</th>
                                            <th style="width:5%">Salesman</th>
                                            <th  style="width:4%">Purchase Amt(BC)</th>
                                            <th  style="width:4%">Inv.VAT(BC)</th>
                                            <th class="bg-success">purchase Amt with VAT(BC)</th>
                                            <th>DN Amt(BC)</th>
                                            <th>DN VAT(BC)</th>                        
                                            <th class="bg-danger">DN Amt with VAT(BC)</th>
                                            <th>Net Amt(AED)</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $total_inv_amount_bc = $total_inv_vat_bc = $total_inv_grand_bc = 0;
                    $total_ret_amount_bc = $total_ret_vat_bc = $total_ret_grand_bc = 0;
                    $total_net_amount_bc = 0;
                    $sl = 1;
                    if (!empty($arr_invoices)) {
                        foreach ($arr_invoices as $each) {

                            if ($each['active'] == 'N') {
                                $each['total_amount'] = 0;
                                $each['tax_total'] = 0;
                                $each['grand_total'] = 0;
                            }
                            if ($each["type"] == 'I') {
                                $invoice_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $invoice_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $invoice_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                                $return_amount_bc = 0;
                                $return_vat_bc = 0;
                                $return_grandtotal_bc = 0;
                            } else {
                                $invoice_amount_bc = 0;
                                $invoice_vat_bc = 0;
                                $invoice_grandtotal_bc = 0;
                                $return_amount_bc = $each['exchange_rate'] * $each['total_amount'];
                                $return_vat_bc = $each['exchange_rate'] * $each['tax_total'];
                                $return_grandtotal_bc = $each['exchange_rate'] * $each['grand_total'];
                            }
                            $net_total = $invoice_grandtotal_bc - $return_grandtotal_bc;
                            $total_inv_amount_bc += $invoice_amount_bc;
                            $total_inv_vat_bc += $invoice_vat_bc;
                            $total_inv_grand_bc += $invoice_grandtotal_bc;
                            $total_ret_amount_bc += $return_amount_bc;
                            $total_ret_vat_bc += $return_vat_bc;
                            $total_ret_grand_bc += $return_grandtotal_bc;
                            $total_net_amount_bc += $net_total;

                            $table .= ' <tr>
                                            <td>' . date('d/m/Y', strtotime($each['invoice_date'])) . '</td>
                                            <td>' . $each['invoice_no'] . '</td>
                                            <td>' . $each['vendor_name'] . '</td>
                                            <td>' . $each['full_name'] . '</td>
                                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_amount_bc, " ") . '</td>
                                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_vat_bc, " ") . '</td>
                                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($invoice_grandtotal_bc, " ") . '</td>
                                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_amount_bc, " ") . '</td>
                                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_vat_bc, " ") . '</td>
                                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($return_grandtotal_bc, " ") . '</td>
                                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($net_total, " ") . '</td>
                                        </tr>';
                        }
                        $table .= '<tr>
                                    <td colspan="4" class="boldy text-right"><b>Total</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_inv_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_amount_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_vat_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_ret_grand_bc, " ") . '</b></td>
                                    <td class="boldy text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_net_amount_bc, " ") . '</b></td>
                                </tr>';
                        $table .= '<tr>
                                <th colspan="10" class="text-right bg-info">Net Purchase Without VAT</th> 
                                <th class="text-right bg-info">' . Yii::app()->numberFormatter->formatCurrency($total_inv_amount_bc - $total_ret_amount_bc, "") . '</th>
                             </tr>';
                    } else {
                        $table .= '<tr>
                                <th colspan="11" class="text-center bg-info">No Details Found</th>
                             </tr>';
                    }

                    $table .= '</tbody>
                            </table>
                        </div>';
                } else if ($report_type == 'current_stock') {

                    $model = new Inventory();
                    $condition = '1=1';

                    if ($params['outlet_id'] > 0) {
                        $condition .= ' AND I.outlet=' . $params['outlet_id'];
                    }

                    if ($params['product_id'] > 0) {
                        $condition .= ' AND I.id=' . $params['product_id'];
                    }

                    if ($params['product_group_id'] > 0) {
                        $condition .= ' AND I.brand=' . $params['product_group_id'];
                    }

                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND (S.created_by=' . $params['salesman_id'] . ' OR S.type="OB" )';
                    }
                    $SQL = "CREATE OR REPLACE VIEW view_report_current_stock AS
                (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,SUM(S.quantity*S.unit_exchange_rate) AS in_qty,
                0 AS out_qty,U.unit_name,CASE
                WHEN SUM(S.quantity*S.unit_exchange_rate) > 0 THEN SUM(S.quantity*S.rate)/SUM(S.quantity*S.unit_exchange_rate) ELSE 0 END AS avg_purchase_rate,0 AS avg_sales_rate,tax,O.name,O.outlet_name,I.rate 
                FROM erp_inventory I 
                LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                LEFT JOIN erp_master_brand B ON I.brand=B.id 
                LEFT JOIN erp_master_unit U ON I.unit=U.id 
                LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='IN' OR S.type='OB') group by I.id)
                UNION
                (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,0 AS in_qty,SUM(S.quantity*S.unit_exchange_rate) AS out_qty,
                U.unit_name,0 AS avg_purchase_rate,CASE
                WHEN SUM(S.quantity*S.unit_exchange_rate) > 0 THEN SUM(S.quantity*S.rate)/SUM(S.quantity*S.unit_exchange_rate) ELSE 0 END AS avg_sales_rate,0 AS tax,O.name,O.outlet_name,I.rate 
                FROM erp_inventory I 
                LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                LEFT JOIN erp_master_brand B ON I.brand=B.id 
                LEFT JOIN erp_master_unit U ON I.unit=U.id 
                LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='OUT' OR S.type='OB')  group by I.id)";
                    Yii::app()->db->createCommand($SQL)->execute();
                    $query_current_stock = "SELECT id,part_number,description,active,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,
                    unit_name,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,SUM(tax) AS tax,name,outlet_name,rate from view_report_current_stock group by id";

                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>';

                    $table .= '             <th style="width:10%">Part No</th>
                                            <th style="width:25%">Description</th>
                                            <th style="width:5%">Type</th>
                                            <th style="width:8%">Brand</th>
                                            <th style="width:10%">Quantity</th>
                                            <th style="width:10%">Unit</th>';
                    if ($params['access_type'] != 'SL') {
                        $table .= '         <th class="text-right" style="width:10%">Cost</th>';
                    }
                    $table .= '             <th>Price</th>    ';
                    if ($params['access_type'] != 'SL') {
                        $table .= '         <th class="text-right" style="width:10%">Cost Amount</th>';
                    }
                    //$table .= '             <th>Tax(%)</th>';
                    if ($params['access_type'] == 'SU') {
                        $table .= '         <th class="text-right" style="width:10%">Outlet</th>';
                    }
                    $table .= '
     
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $totalStock = 0;
                    $items = Yii::app()->db->createCommand($query_current_stock)->queryAll();
                    foreach ($items as $row) {
                        $inv_type = ($row['type'] == 'I') ? "Inventory" : "Service";
                        $avg_purchase_rate = round(StockLedger::getAvgPurchaseRate($row['id'], '', $params['outlet_id']), 2);
                        $cost_amount = $avg_purchase_rate * round($row['qty'], 3);
                        $total_amount += ($cost_amount);

                        $table .= '<tr>
                            <td>' . $row['part_number'] . '</td> 
                            <td>' . $row['description'] . '</td> 
                            <td>' . $inv_type . '</td> 
                            <td>' . $row['brand_name'] . '</td> 
                            <td class="text-right">' . round($row['qty'], 3) . '</td> 
                            <td>' . $row['unit_name'] . '</td> ';
                        if ($params['access_type'] != 'SL') {
                            $table .= ' <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($avg_purchase_rate, '') . '</td>';
                        }
                        $table .= '    <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($row['rate'], '') . '</td> ';

                        if ($params['access_type'] != 'SL') {
                            $table .= ' <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($cost_amount, '') . '</td>';
                        }
                        if ($params['access_type'] == 'SU') {
                            $table .= '         <td class="text-right">' . $row['outlet_name'] . '</td>';
                        }
                        //$table .= '    <td class="text-right">'.Yii::app()->numberFormatter->formatCurrency($row['tax'],'').' %'.'</td>   ';'                
                        $table .= '            </tr>';
                    }


                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'transaction') {
                    if ($params['product_id'] > 0) {
                        $pdt_model = Inventory::model()->findByPk($params['product_id']);
                        $default_unit = $pdt_model->Unit_symbol;
                    }
                    $model = new StockLedger();
                    $condition = '1=1';
                    if ($params['outlet_id'] > 0) {
                        $condition .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['from_date'] != '' && $params['from_date'] != '00/00/0000') {
                        $from = str_replace('/', '-', $params['from_date']);
                        $from = date("Y-m-d", strtotime($from));
                        $condition .= ' AND t.transaction_date>="' . $from . '"';
                    }
                    if ($params['to_date'] != '' && $params['to_date'] != '00/00/0000') {
                        $to = str_replace('/', '-', $params['to_date']);
                        $to = date("Y-m-d", strtotime($to));
                        $condition .= ' AND t.transaction_date<="' . $to . '"';
                    }
                    if ($params['product_id'] > 0) {
                        $condition .= ' AND t.product_id=' . $params['product_id'];
                    }
                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND (t.created_by=' . $params['salesman_id'] . ')';
                    }
                    $transaction_sql = 'SELECT * FROM view_stockledger t 
                    WHERE ' . $condition . ' ORDER BY transaction_date,in_out asc';
                    $arr_transactions = Yii::app()->db->createCommand($transaction_sql)->queryAll();
                    $open_cond = 'product_id=' . $params['product_id'];
                    if ($outlet > 0) {
                        $open_cond .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['from_date'] != '' && $params['from_date'] != '00/00/0000') {
                        $open_cond .= ' AND t.transaction_date<"' . $params['from_date'] . '"';
                    }
                    if ($params['salesman_id'] > 0) {
                        $open_cond .= ' AND (t.created_by=' . $params['salesman_id'] . ')';
                    }
                    $open_bal_in_sql = 'SELECT SUM(default_qty) AS in_qty FROM view_stockledger t 
                            WHERE ' . $open_cond . ' AND in_out="IN" ';
                    $open_bal_out_sql = 'SELECT SUM(default_qty) AS in_qty FROM view_stockledger t 
                            WHERE ' . $open_cond . ' AND in_out="OUT" ';
                    $opening_in_qty = Yii::app()->db->createCommand($open_bal_in_sql)->queryScalar();
                    $opening_out_qty = Yii::app()->db->createCommand($open_bal_out_sql)->queryScalar();
                    $balance = $opening_in_qty - $opening_out_qty;
                    //echo $condition;
                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th style="width:5%">Date</th>
                                            <th style="width:8%">Transaction No</th>
                                            <th style="width:10%">Type</th>
                                            <th style="width:20%">Customer/Vendor</th>
                                            <th style="width:5%">Cost/Item</th>
                                            <th style="width:5%">Rate/Item</th>
                                            <th style="width:5%">Quantity</th>
                                            <th style="width:5%">Unit</th>
                                            <th style="width:5%">Salesman</th>
                                            <th style="width:10%">Balance(in' . $default_unit . ')</th>                
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $open_bal_in_sql = 'SELECT SUM(default_qty) AS in_qty FROM view_stockledger t 
                    WHERE ' . $open_cond . ' AND in_out="IN" ';
                    $open_bal_out_sql = 'SELECT SUM(default_qty) AS in_qty FROM view_stockledger t 
                    WHERE ' . $open_cond . ' AND in_out="OUT" ';
                    $opening_in_qty = Yii::app()->db->createCommand($open_bal_in_sql)->queryScalar();
                    $opening_out_qty = Yii::app()->db->createCommand($open_bal_out_sql)->queryScalar();
                    $balance = $opening_in_qty - $opening_out_qty;
                    $table .= '<tr>
                                <td colspan="9" class="boldy text-right">Opening Balance</td>
                                <td class="boldy text-right">' . Yii::app()->numberFormatter->formatCurrency($balance, '') . '</td>
                            </tr>';
                    if (!empty($arr_transactions)) {
                        foreach ($arr_transactions as $row) {
                            $cost_per_item = $rate_per_item = 0;
                            if ($row['in_out'] == 'IN') {
                                $balance += $row['default_qty'];
                            } else {
                                $balance -= $row['default_qty'];
                            }
                            if ($row['type'] == 'SI' || $row['type'] == 'SR' || $row['type'] == 'GR' || $row['type'] == 'PI' || $row['type'] == 'PR') {
                                $rate_per_item = $row['rate'];
                            }
                            if ($row['type'] == 'PI' || $row['type'] == 'PR' || $row['type'] == 'MTM') {
                                $cost_per_item = $row['rate'];
                            } else {
                                $cost_per_item = StockLedger::getAvgPurchaseRate($row['product_id'], $row['transaction_date'], $params['outlet_id']);
                            }

                            $table .= '<tr>
                                <td>' . date("d/m/Y", strtotime($row['transaction_date'])) . '</td>
                                <td>' . $row['invoice_no'] . '</td>
                                <td>' . $row['transaction_type'] . '</td>
                                <td>' . $row['customer_name'] . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($cost_per_item, '') . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($rate_per_item, '') . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($row['quantity'], '') . '</td>
                                <td>' . $row['unit_label'] . '</td>
                                <td>' . $row['full_name'] . '</td>
                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($balance, '') . '</td>
                                    </tr>';
                        }
                        $table .= '<tr>
                                <th colspan="9" class="text-right bg-info">Closing Balance</th>
                                <th class="text-right bg-info">' . Yii::app()->numberFormatter->formatCurrency($balance, '') . '</th>
                            </tr>';
                    } else {
                        $table .= '<tr>
                                        <th colspan="10" class="text-center bg-info">No Details Found</th>
                                </tr>';
                    }
                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'customer') {

                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th>#</th>
                                            <th>Name</th>
                                            <th>Location</th>
                                            <th>State</th>
                                            <th>Phone</th>
                                            <th>Mobile</th>
                                            <th>TRN #</th>
                                            <th>Balance</th>
                                        </tr>
                                    </thead>
                                    <tbody>';
                    $model = new Customer();
                    $condition = '1=1';
                    if ($params['outlet_id'] > 0) {
                        $condition .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['customer_id'] > 0) {
                        $condition .= ' AND t.id=' . $params['customer_id'];
                    }
                    if ($params['salesman_id'] > 0) {
                        $condition .= " AND (t.created_by = " . $params['salesman_id'] . " OR FIND_IN_SET(" . $params['salesman_id'] . ",accessed_by))";
                    }
                    $arr_customers = $model->with('state_detail', 'outlet_detail')->findAll(array('condition' => $condition, 'order' => 'customer_name ASC'));
                    if (!empty($arr_customers)) {
                        foreach ($arr_customers as $row) {
                            $customer_subledger = FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $row->id . ' AND subledger_type_id=1'))->id;
                            $customer_ledger = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                            $current_credit = Yii::app()->db->createCommand("select SUM(I.amount) from erp_finance_voucher_items I INNER JOIN erp_finance_voucher_head H ON H.id=I.header_id where H.is_posted='Y' AND ledger_id='$customer_ledger' AND subledger_id='$customer_subledger'")->queryScalar();
                            $bal = number_format($current_credit, 2);

                            $table .= '<tr>
                                        <td>' . $row['customer_no'] . '</td>
                                        <td>' . $row['customer_name'] . '</td>
                                        <td>' . $row['location'] . '</td>
                                        <td>' . $row['state_detail']['state_name'] . '</td>
                                        <td>' . $row['phone'] . '</td>
                                        <td>' . $row['mobile'] . '</td>
                                        <td>' . $row['trn_no'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($bal, " ") . '</td>
                                    </tr>';
                        }
                    } else {
                        $table .= '<tr class="text-center"><td colspan="5">No Data Available</td></tr>';
                    }

                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'vendor') {

                    $table = '<div class="table-responsive">
                                <table class="table table-hover table-striped table-bordered ">
                                    <thead>
                                        <tr>
                                            <th>#</th>
                                            <th>Name</th>
                                            <th>Location</th>
                                            <th>State</th>
                                            <th>Phone</th>
                                            <th>Mobile</th>
                                            <th>TRN #</th>
                                            <th>Balance</th>
                                        </tr>
                                    </thead>
                                    <tbody>';

                    $model = new Vendor();
                    $condition = '1=1';
                    if ($params['outlet_id'] > 0) {
                        $condition .= " AND t.outlet=" . $params['outlet_id'];
                    }
                    if ($params['vendor_id'] > 0) {
                        $condition .= ' AND t.id=' . $params['vendor_id'];
                    }
                    if ($params['salesman_id'] > 0) {
                        $condition .= " AND (t.created_by = " . $params['salesman_id'] . " OR FIND_IN_SET(" . $params['salesman_id'] . ",accessed_by))";
                    }
                    $arr_vendors = $model->with('state_detail', 'outlet_detail')->findAll(array('condition' => $condition, 'order' => 'vendor_name ASC'));

                    if (!empty($arr_vendors)) {
                        foreach ($arr_vendors as $row) {
                            $vendor_subledger = FinanceSubledger::model()->find(array('condition' => 'parent_id=' . (int) $row->id . ' AND subledger_type_id=2'))->id;
                            $vendor_ledger = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                            $current_credit = Yii::app()->db->createCommand("select SUM(I.amount) from erp_finance_voucher_items I INNER JOIN erp_finance_voucher_head H ON H.id=I.header_id where H.is_posted='Y' AND ledger_id='$vendor_ledger' AND subledger_id='$vendor_subledger'")->queryScalar();
                            $bal = number_format($current_credit, 2);
                            $table .= '<tr>
                                        <td>' . $row['vendor_no'] . '</td>
                                        <td>' . $row['vendor_name'] . '</td>
                                        <td>' . $row['location'] . '</td>
                                        <td>' . $row['state_detail']['state_name'] . '</td>
                                        <td>' . $row['phone'] . '</td>
                                        <td>' . $row['mobile'] . '</td>
                                        <td>' . $row['trn_no'] . '</td>
                                        <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($bal, " ") . '</td>
                                    </tr>';
                        }
                    } else {
                        $table .= '<tr class="text-center"><td colspan="5">No Data Available</td></tr>';
                    }

                    $table .= '</tbody>
                                </table>
                            </div>';
                } else if ($report_type == 'customer_statement') {

                    $table = '<div class="col-md-12">
                                            <p>Dear Sir / Madam, </p>';
                    $currency = 1;
                    $to = date('Y-m-d');
                    $ledger = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                    $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $params['customer_id']))->id;
                    $table .= '<p class="text-center"><b><u>Sub: Statement of Account up to ' . date("d/m/Y", strtotime($to)) . '</u></b></p>';


                    $table .= '<p>
                                                This is to inform your Accounts Dept. that our records show an outstanding balance against the following  invoices.
                                            </p>
                                            <table id="tbl_reorder_level_table" class="table table-fixed table-bordered table-primary dataTable">
                                                <thead>
                                                    <tr>
                                                        <th class="text-center" style="width:10%">Date</th>
                                                        <th class="text-center" style="width:10%">Invoice #</th>
                                                        <th class="text-center"  style="width:10%">Due Date</th>
                                                        <th class="text-center"   style="width:10%">Invoice Value</th>
                                                        <th class="text-center"  style="width:10%">Paid</th>
                                                        <th class="text-center"  style="width:10%">Balance Amt Due</th>
                                                    </tr>
                                                </thead>';

                    $cur = MasCurrency::model()->findByPk($currency);
                    $exchange_rate = 1;
                    $currency_code = 1;

                    $amountType = 'VI.amount > 0';
                    $paidType = 'VI.amount < 0';

                    $items = array();
                    $query = 'CREATE OR REPLACE VIEW {{finance_statement}} AS (SELECT
                                    V.id AS voucher_id,
                                    V.voucher_date AS voucher_date,
                                    V.due_date AS due_date,
                                    V.reference_no AS reference_no,
                                    VI.narration AS narration,
                                    VI.id AS voucher_item_id,
                                    V.voucher_no,
                                    ROUND(ABS(VI.amount*VI.exchange_rate), 4) AS amount,
                                    ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4) AS allocated,
                                    ROUND(ABS(VI.amount*VI.exchange_rate), 4) - ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4) AS balance
                                FROM {{finance_voucher_items}} VI
                                LEFT JOIN {{finance_voucher_head}} V ON V.id = VI.header_id
                                LEFT JOIN {{finance_allocation_items}} AI ON AI.voucher_item_id = VI.id
                                WHERE V.is_posted ="Y" AND V.voucher_date<="' . $to . '"  AND ' . $amountType . ' AND VI.ledger_id = ' . $ledger . ' AND VI.subledger_id = ' . $subledger . '
                                GROUP BY VI.id)
                                UNION
                                (SELECT
                                    V.id AS voucher_id,
                                    V.voucher_date AS voucher_date,
                                    V.due_date AS due_date,
                                    V.reference_no AS reference_no,
                                    VI.narration AS narration,
                                    VI.id AS voucher_item_id,
                                    V.voucher_no,
                                    -(ROUND(ABS(VI.amount*VI.exchange_rate), 4)) AS amount,
                                    -(ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4)) AS allocated,
                                    -(ROUND(ABS(VI.amount*VI.exchange_rate), 4) - ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4)) AS balance
                                FROM {{finance_voucher_items}} VI
                                LEFT JOIN {{finance_voucher_head}} V ON V.id = VI.header_id
                                LEFT JOIN {{finance_allocation_items}} AI ON AI.voucher_item_id = VI.id
                                WHERE V.is_posted ="Y" AND V.voucher_date<="' . $to . '"  AND ' . $paidType . ' AND VI.ledger_id = ' . $ledger . ' AND VI.subledger_id = ' . $subledger . '
                                GROUP BY VI.id)';
                    if ($ledger > 0 && $subledger > 0) {
                        Yii::app()->db->createCommand($query)->execute();
                        $items = Yii::app()->db->createCommand('SELECT * FROM {{finance_statement}} order by voucher_date')->queryAll();
                    }

                    $table .= '<tbody>';
                    $totalValue = 0;
                    $balanceTotal = 0;
                    $totalPaid = 0;
                    $due_list_array = array();
                    $amount += $opening_bal;
                    $allocated += 0;
                    $totalValue += $opening_bal;
                    $balanceTotal += $opening_bal;
                    $due_array["OB"] = $opening_bal;
                    foreach ($items as $each) {
                        if (round($each['balance'], 2) != 0) :
                            $totalValue += $each['amount'];
                            $balanceTotal += $each['balance'];
                            $totalPaid += $each['allocated'];
                            $due_days = strtotime($each['voucher_date']);
                            $due = round(abs(strtotime(date('Y-m-d')) - $due_days) / 86400);
                            $array_slot = floor($due / 30);

                            if ($array_slot == 0) {
                                if (!isset($due_array["0-30"])) {
                                    $due_array["0-30"] = 0;
                                }
                                $due_array["0-30"] = $due_array["0-30"] + $each['balance'] / $exchange_rate;
                            } else if ($array_slot >= 6) {
                                if (!isset($due_array[">180"])) {
                                    $due_array[">180"] = 0;
                                }
                                $due_array[">180"] = $due_array[">180"] + $each['balance'] / $exchange_rate;
                            } else {
                                $lower_limit = ($array_slot * 30) + 1;
                                $upper_limit = $lower_limit + 29;
                                $label = "{$lower_limit}-{$upper_limit}";
                                if (!isset($due_array[$label])) {
                                    $due_array[$label] = 0;
                                }
                                $due_array[$label] = $due_array[$label] + $each['balance'] / $exchange_rate;
                            }
                            if ($_REQUEST['type'] == 'customer') {
                                $vouchreformat = $each['voucher_no'];
                            } else {
                                $head = FinanceVoucherHead::model()->findByPk($each['voucher_id']);
                                if ($head->voucher_type == 8) {
                                    $ref = PurchaseInvoice::model()->findByPk($head->voucher_no)->invoice_no;
                                } else {
                                    $ref = $each['reference_no'];
                                }
                                if (empty($ref)) {
                                    $ref = $head->voucher_no;
                                }
                                $vouchreformat = $ref;
                            }
                            $table .= '<tr>      
                                                                <td>' . date("d/m/Y", strtotime($each['voucher_date'])) . '</td>
                                                                <td>' . $vouchreformat . '</td>
                                                                <td>' . date("d/m/Y", strtotime($each['due_date'])) . '</td>
                                                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency(($each["amount"] / $exchange_rate), " ") . '</td>
                                                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency(($each['allocated'] / $exchange_rate), " ") . '</td>
                                                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency(($each['balance'] / $exchange_rate), " ") . '</td>
                                                            </tr>';
                        endif;
                    }
                    $table .= '<tr>
                                                        <td colspan="3" class="text-right"><b>Total </b></td>
                                                        <td  class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($totalValue / $exchange_rate), " ") . '</b></td>
                                                        <td  class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($totalPaid / $exchange_rate), " ") . '</b></td>
                                                        <td  class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($balanceTotal / $exchange_rate), " ") . '</b></td>
                                                    </tr>
                                                </tbody>
                                            </table>
                                        </div>';
                } else if ($report_type == 'vendor_statement') {

                    $table = '<div class="col-md-12">
                                            <p>Dear Sir / Madam, </p>';
                    $currency = 1;
                    $to = date('Y-m-d');
                    $ledger = FinanceSystemAccountsMapping::getSystemdefault('creditors');
                    $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=2 AND parent_id=' . $params['vendor_id']))->id;
                    $table .= '<p class="text-center"><b><u>Sub: Statement of Account up to ' . date("d/m/Y", strtotime($to)) . '</u></b></p>';


                    $table .= '<p>
                                                This is to inform your Accounts Dept. that our records show an outstanding balance against the following  invoices.
                                            </p>
                                            <table id="tbl_reorder_level_table" class="table table-fixed table-bordered table-primary dataTable">
                                                <thead>
                                                    <tr>
                                                        <th class="text-center" style="width:10%">Date</th>
                                                        <th class="text-center" style="width:10%">Invoice #</th>
                                                        <th class="text-center"  style="width:10%">Due Date</th>
                                                        <th class="text-center"   style="width:10%">Invoice Value</th>
                                                        <th class="text-center"  style="width:10%">Paid</th>
                                                        <th class="text-center"  style="width:10%">Balance Amt Due</th>
                                                    </tr>
                                                </thead>';

                    $cur = MasCurrency::model()->findByPk($currency);
                    $exchange_rate = 1;
                    $currency_code = 1;

                    $amountType = 'VI.amount < 0';
                    $paidType = 'VI.amount > 0';

                    $items = array();
                    $query = 'CREATE OR REPLACE VIEW {{finance_statement}} AS (SELECT
                                    V.id AS voucher_id,
                                    V.voucher_date AS voucher_date,
                                    V.due_date AS due_date,
                                    V.reference_no AS reference_no,
                                    VI.narration AS narration,
                                    VI.id AS voucher_item_id,
                                    V.voucher_no,
                                    ROUND(ABS(VI.amount*VI.exchange_rate), 4) AS amount,
                                    ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4) AS allocated,
                                    ROUND(ABS(VI.amount*VI.exchange_rate), 4) - ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4) AS balance
                                FROM {{finance_voucher_items}} VI
                                LEFT JOIN {{finance_voucher_head}} V ON V.id = VI.header_id
                                LEFT JOIN {{finance_allocation_items}} AI ON AI.voucher_item_id = VI.id
                                WHERE V.is_posted ="Y" AND V.voucher_date<="' . $to . '"  AND ' . $amountType . ' AND VI.ledger_id = ' . $ledger . ' AND VI.subledger_id = ' . $subledger . '
                                GROUP BY VI.id)
                                UNION
                                (SELECT
                                    V.id AS voucher_id,
                                    V.voucher_date AS voucher_date,
                                    V.due_date AS due_date,
                                    V.reference_no AS reference_no,
                                    VI.narration AS narration,
                                    VI.id AS voucher_item_id,
                                    V.voucher_no,
                                    -(ROUND(ABS(VI.amount*VI.exchange_rate), 4)) AS amount,
                                    -(ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4)) AS allocated,
                                    -(ROUND(ABS(VI.amount*VI.exchange_rate), 4) - ROUND(IFNULL(ABS(SUM(AI.amount)), 0), 4)) AS balance
                                FROM {{finance_voucher_items}} VI
                                LEFT JOIN {{finance_voucher_head}} V ON V.id = VI.header_id
                                LEFT JOIN {{finance_allocation_items}} AI ON AI.voucher_item_id = VI.id
                                WHERE V.is_posted ="Y" AND V.voucher_date<="' . $to . '"  AND ' . $paidType . ' AND VI.ledger_id = ' . $ledger . ' AND VI.subledger_id = ' . $subledger . '
                                GROUP BY VI.id)';
                    if ($ledger > 0 && $subledger > 0) {
                        Yii::app()->db->createCommand($query)->execute();
                        $items = Yii::app()->db->createCommand('SELECT * FROM {{finance_statement}} order by voucher_date')->queryAll();
                    }

                    $table .= '<tbody>';
                    $totalValue = 0;
                    $balanceTotal = 0;
                    $totalPaid = 0;
                    $due_list_array = array();
                    $amount += $opening_bal;
                    $allocated += 0;
                    $totalValue += $opening_bal;
                    $balanceTotal += $opening_bal;
                    $due_array["OB"] = $opening_bal;
                    foreach ($items as $each) {
                        if (round($each['balance'], 2) != 0) :
                            $totalValue += $each['amount'];
                            $balanceTotal += $each['balance'];
                            $totalPaid += $each['allocated'];
                            $due_days = strtotime($each['voucher_date']);
                            $due = round(abs(strtotime(date('Y-m-d')) - $due_days) / 86400);
                            $array_slot = floor($due / 30);

                            if ($array_slot == 0) {
                                if (!isset($due_array["0-30"])) {
                                    $due_array["0-30"] = 0;
                                }
                                $due_array["0-30"] = $due_array["0-30"] + $each['balance'] / $exchange_rate;
                            } else if ($array_slot >= 6) {
                                if (!isset($due_array[">180"])) {
                                    $due_array[">180"] = 0;
                                }
                                $due_array[">180"] = $due_array[">180"] + $each['balance'] / $exchange_rate;
                            } else {
                                $lower_limit = ($array_slot * 30) + 1;
                                $upper_limit = $lower_limit + 29;
                                $label = "{$lower_limit}-{$upper_limit}";
                                if (!isset($due_array[$label])) {
                                    $due_array[$label] = 0;
                                }
                                $due_array[$label] = $due_array[$label] + $each['balance'] / $exchange_rate;
                            }
                            if ($_REQUEST['type'] == 'customer') {
                                $vouchreformat = $each['voucher_no'];
                            } else {
                                $head = FinanceVoucherHead::model()->findByPk($each['voucher_id']);
                                if ($head->voucher_type == 8) {
                                    $ref = PurchaseInvoice::model()->findByPk($head->voucher_no)->invoice_no;
                                } else {
                                    $ref = $each['reference_no'];
                                }
                                if (empty($ref)) {
                                    $ref = $head->voucher_no;
                                }
                                $vouchreformat = $ref;
                            }
                            $table .= '<tr>      
                                                                <td>' . date("d/m/Y", strtotime($each['voucher_date'])) . '</td>
                                                                <td>' . $vouchreformat . '</td>
                                                                <td>' . date("d/m/Y", strtotime($each['due_date'])) . '</td>
                                                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency(($each["amount"] / $exchange_rate), " ") . '</td>
                                                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency(($each['allocated'] / $exchange_rate), " ") . '</td>
                                                                <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency(($each['balance'] / $exchange_rate), " ") . '</td>
                                                            </tr>';
                        endif;
                    }
                    $table .= '<tr>
                                                        <td colspan="3" class="text-right"><b>Total </b></td>
                                                        <td  class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($totalValue / $exchange_rate), " ") . '</b></td>
                                                        <td  class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($totalPaid / $exchange_rate), " ") . '</b></td>
                                                        <td  class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($balanceTotal / $exchange_rate), " ") . '</b></td>
                                                    </tr>
                                                </tbody>
                                            </table>
                                        </div>';
                } else if ($report_type == 'receivable_aging') {
                    $to_date = (count(explode('/', $variables['to_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['to_date']))) : date('Y-m-d');
                    $ledger = 2;
                    $category = 'RECEIVABLE';
                    $condition = '';
                    if ($outlet > 0) {
                        $condition .= ' AND VI.outlet=' . $outlet;
                        $condition_for_opening .= ' AND I.outlet=' . $outlet;
                    }
                    $dates_array = array(
                        '0_to_30' => array('from' => date('Y-m-d', strtotime($to_date . ' -30 days')), 'to' => $to_date),
                        '31_to_60' => array('from' => date('Y-m-d', strtotime($to_date . ' -60 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -31 days'))),
                        '61_to_90' => array('from' => date('Y-m-d', strtotime($to_date . ' -90 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -61 days'))),
                        '91_to_120' => array('from' => date('Y-m-d', strtotime($to_date . ' -120 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -91 days'))),
                        'greaterthan_120' => array('from' => '2010-01-01', 'to' => date('Y-m-d', strtotime($to_date . ' -121 days'))),
                    );

                    foreach ($dates_array as $key => $each_date_pair) {

                        $query = 'CREATE OR REPLACE VIEW {{finance_ageing_' . $key . '}} AS SELECT
                            VI.ledger_id AS ledger_id,
                            VI.subledger_id AS subledger_id,
                            IFNULL(S.description, "No Subledger") AS subledger,
                            V.id AS voucher_id,
                            V.voucher_date AS voucher_date,
                            V.due_date AS due_date,
                            V.reference_no AS reference_no,
                            VI.narration AS narration,
                            VI.id AS voucher_item_id,
                            CONCAT(VT.code,"/",V.voucher_no) AS voucher_no,
                            ROUND((VI.amount*VI.exchange_rate), 4) AS total_amount,
                            IFNULL(ABS(ROUND( SUM(AI.amount), 4)), 0) AS allocated,
                            ROUND((VI.amount*VI.exchange_rate), 4) - IFNULL(ROUND( SUM(AI.amount), 4), 0) AS amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} V ON V.id = VI.header_id
                        LEFT JOIN {{finance_allocation_items}} AI ON AI.voucher_item_id = VI.id
                        LEFT JOIN {{finance_subledger}} S ON S.id = VI.subledger_id
                        LEFT JOIN {{finance_voucher_type}} VT ON VT.id=V.voucher_type
                        WHERE V.voucher_date BETWEEN "' . $each_date_pair['from'] . '" AND "' . $each_date_pair['to'] . '" AND
                        V.is_posted="Y" AND VI.ledger_id = "' . (int) $ledger . '" ' . $condition . '
                        GROUP BY VI.id, VI.subledger_id';

                        $selectQuery = 'SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        SUM(0_30_receivable) AS 0_30_receivable,
                        SUM(0_30_unallocated) AS 0_30_unallocated,
                        SUM(31_60_receivable) AS 31_60_receivable,
                        SUM(31_60_unallocated) AS 31_60_unallocated,
                        SUM(61_90_receivable) AS 61_90_receivable,
                        SUM(61_90_unallocated) AS 61_90_unallocated,
                        SUM(91_120_receivable) AS 91_120_receivable,
                        SUM(91_120_unallocated) AS 91_120_unallocated,
                        SUM(greater_120_receivable) AS greater_120_receivable,
                        SUM(greater_120_unallocated) AS greater_120_unallocated
                    FROM ( SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 0_30_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_0_to_30}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 31_60_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_31_to_60}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 61_90_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_61_to_90}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 91_120_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_91_to_120}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS greater_120_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS greater_120_unallocated
                    FROM {{finance_ageing_greaterthan_120}} GROUP BY subledger_id ) AS A GROUP BY A.subledger_id ORDER BY A.subledger ASC';

                        Yii::app()->db->createCommand($query)->execute();
                    }

                    $aging_report = Yii::app()->db->createCommand($selectQuery)->queryAll();

                    $data = '<table class="table table-hover table-fixed table-bordered" id="tb_table">
                <thead>
                    <tr>
                        <th>#</th>
                        <th>Subledger</th>
                        <th>Type</th>
                        <th>0-30</th>
                        <th>31-60</th>
                        <th>61-90</th>
                        <th>91-120</th>
                        <th> > 120</th>
                        <th>Total</th>
                    </tr>
                </thead>
                <tbody>';
                    $grand_total = 0;
                    $one = 0;
                    $two = 0;
                    $three = 0;
                    $four = 0;
                    $five = 0;
                    $labelOne = ($category == 'RECEIVABLE') ? 'Receivable' : 'Unallocated';
                    $labelTwo = ($category == 'RECEIVABLE') ? 'Unallocated' : 'Payable';
                    foreach ($aging_report as $i => $each) :
                        $total_receivable = $each['0_30_receivable'] + $each['31_60_receivable'] + $each['61_90_receivable'] + $each['91_120_receivable'] + $each['greater_120_receivable'];
                        $total_unallocated = $each['0_30_unallocated'] + $each['31_60_unallocated'] + $each['61_90_unallocated'] + $each['91_120_unallocated'] + $each['greater_120_unallocated'];

                        $one_receivable += $each['0_30_receivable'];
                        $two_receivable += $each['31_60_receivable'];
                        $three_receivable += $each['61_90_receivable'];
                        $four_receivable += $each['91_120_receivable'];
                        $five_receivable += $each['greater_120_receivable'];
                        $grand_total_receivable += $total_receivable;

                        $one_unallocated += $each['0_30_unallocated'];
                        $two_unallocated += $each['31_60_unallocated'];
                        $three_unallocated += $each['61_90_unallocated'];
                        $four_unallocated += $each['91_120_unallocated'];
                        $five_unallocated += $each['greater_120_unallocated'];
                        $grand_total_unallocated += $total_unallocated;

                        if ($total_receivable > 0 || $total_unallocated > 0) :
                            $data .= '<tr>
																																									<td rowspan="2" >' . ($i + 1) . '</td>
																																		                            <td rowspan="2" >' . $each['subledger'] . '</td>
																																		                            <td>' . $labelOne . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['0_30_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['31_60_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['61_90_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['91_120_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['greater_120_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($total_receivable, " ") . '</td>
																																						        </tr>';
                            $data .= '<tr>
																																		                            <td>' . $labelTwo . '</td>
																																		                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['0_30_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['31_60_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['61_90_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['91_120_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['greater_120_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($total_unallocated, " ") . '</td>
																																						        </tr>';
                        endif;
                    endforeach;

                    $data .= '<tr>
                        <td colspan="2" rowspan="2" class="text-right"><b>Total</b></td>
                        <td>' . $labelOne . count($aging_report) . '</td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($one_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($two_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($three_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($four_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($five_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                    </tr>';

                    $data .= '<tr>
                        <td>' . $labelTwo . '</td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($one_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($two_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($three_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($four_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($five_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                    </tr>';
                    $data .= ' </tbody>
            </table>';
                    $table = $data;
                } else if ($report_type == 'payable_aging') {
                    $to_date = (count(explode('/', $variables['to_date'])) == 3) ? date("Y-m-d", strtotime(str_replace('/', '-', $variables['to_date']))) : date('Y-m-d');
                    $ledger = 6;
                    $category = 'PAYABLE';
                    $condition = '';
                    if ($outlet > 0) {
                        $condition .= ' AND VI.outlet=' . $outlet;
                        $condition_for_opening .= ' AND I.outlet=' . $outlet;
                    }
                    $dates_array = array(
                        '0_to_30' => array('from' => date('Y-m-d', strtotime($to_date . ' -30 days')), 'to' => $to_date),
                        '31_to_60' => array('from' => date('Y-m-d', strtotime($to_date . ' -60 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -31 days'))),
                        '61_to_90' => array('from' => date('Y-m-d', strtotime($to_date . ' -90 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -61 days'))),
                        '91_to_120' => array('from' => date('Y-m-d', strtotime($to_date . ' -120 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -91 days'))),
                        'greaterthan_120' => array('from' => '2010-01-01', 'to' => date('Y-m-d', strtotime($to_date . ' -121 days'))),
                    );

                    foreach ($dates_array as $key => $each_date_pair) {

                        $query = 'CREATE OR REPLACE VIEW {{finance_ageing_' . $key . '}} AS SELECT
                            VI.ledger_id AS ledger_id,
                            VI.subledger_id AS subledger_id,
                            IFNULL(S.description, "No Subledger") AS subledger,
                            V.id AS voucher_id,
                            V.voucher_date AS voucher_date,
                            V.due_date AS due_date,
                            V.reference_no AS reference_no,
                            VI.narration AS narration,
                            VI.id AS voucher_item_id,
                            CONCAT(VT.code,"/",V.voucher_no) AS voucher_no,
                            ROUND((VI.amount*VI.exchange_rate), 4) AS total_amount,
                            IFNULL(ABS(ROUND( SUM(AI.amount), 4)), 0) AS allocated,
                            ROUND((VI.amount*VI.exchange_rate), 4) - IFNULL(ROUND( SUM(AI.amount), 4), 0) AS amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} V ON V.id = VI.header_id
                        LEFT JOIN {{finance_allocation_items}} AI ON AI.voucher_item_id = VI.id
                        LEFT JOIN {{finance_subledger}} S ON S.id = VI.subledger_id
                        LEFT JOIN {{finance_voucher_type}} VT ON VT.id=V.voucher_type
                        WHERE V.voucher_date BETWEEN "' . $each_date_pair['from'] . '" AND "' . $each_date_pair['to'] . '" AND
                        V.is_posted="Y" AND VI.ledger_id = "' . (int) $ledger . '" ' . $condition . '
                        GROUP BY VI.id, VI.subledger_id';

                        $selectQuery = 'SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        SUM(0_30_receivable) AS 0_30_receivable,
                        SUM(0_30_unallocated) AS 0_30_unallocated,
                        SUM(31_60_receivable) AS 31_60_receivable,
                        SUM(31_60_unallocated) AS 31_60_unallocated,
                        SUM(61_90_receivable) AS 61_90_receivable,
                        SUM(61_90_unallocated) AS 61_90_unallocated,
                        SUM(91_120_receivable) AS 91_120_receivable,
                        SUM(91_120_unallocated) AS 91_120_unallocated,
                        SUM(greater_120_receivable) AS greater_120_receivable,
                        SUM(greater_120_unallocated) AS greater_120_unallocated
                    FROM ( SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 0_30_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_0_to_30}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 31_60_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_31_to_60}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 61_90_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_61_to_90}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS 91_120_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS 91_120_unallocated,
                        0 AS greater_120_receivable,
                        0 AS greater_120_unallocated
                    FROM {{finance_ageing_91_to_120}} GROUP BY subledger_id
                    UNION
                    SELECT
                        ledger_id,
                        subledger_id,
                        subledger,
                        0 AS 0_30_receivable,
                        0 AS 0_30_unallocated,
                        0 AS 31_60_receivable,
                        0 AS 31_60_unallocated,
                        0 AS 61_90_receivable,
                        0 AS 61_90_unallocated,
                        0 AS 91_120_receivable,
                        0 AS 91_120_unallocated,
                        SUM( CASE WHEN amount > 0 THEN amount ELSE 0 END) AS greater_120_receivable,
                        SUM( CASE WHEN amount < 0 THEN ABS(amount) ELSE 0 END) AS greater_120_unallocated
                    FROM {{finance_ageing_greaterthan_120}} GROUP BY subledger_id ) AS A GROUP BY A.subledger_id ORDER BY A.subledger ASC';

                        Yii::app()->db->createCommand($query)->execute();
                    }

                    $aging_report = Yii::app()->db->createCommand($selectQuery)->queryAll();

                    $data = '<table class="table table-hover table-fixed table-bordered" id="tb_table">
                <thead>
                    <tr>
                        <th>#</th>
                        <th>Subledger</th>
                        <th>Type</th>
                        <th>0-30</th>
                        <th>31-60</th>
                        <th>61-90</th>
                        <th>91-120</th>
                        <th> > 120</th>
                        <th>Total</th>
                    </tr>
                </thead>
                <tbody>';
                    $grand_total = 0;
                    $one = 0;
                    $two = 0;
                    $three = 0;
                    $four = 0;
                    $five = 0;
                    $labelOne = ($category == 'RECEIVABLE') ? 'Receivable' : 'Unallocated';
                    $labelTwo = ($category == 'RECEIVABLE') ? 'Unallocated' : 'Payable';
                    foreach ($aging_report as $i => $each) :
                        $total_receivable = $each['0_30_receivable'] + $each['31_60_receivable'] + $each['61_90_receivable'] + $each['91_120_receivable'] + $each['greater_120_receivable'];
                        $total_unallocated = $each['0_30_unallocated'] + $each['31_60_unallocated'] + $each['61_90_unallocated'] + $each['91_120_unallocated'] + $each['greater_120_unallocated'];

                        $one_receivable += $each['0_30_receivable'];
                        $two_receivable += $each['31_60_receivable'];
                        $three_receivable += $each['61_90_receivable'];
                        $four_receivable += $each['91_120_receivable'];
                        $five_receivable += $each['greater_120_receivable'];
                        $grand_total_receivable += $total_receivable;

                        $one_unallocated += $each['0_30_unallocated'];
                        $two_unallocated += $each['31_60_unallocated'];
                        $three_unallocated += $each['61_90_unallocated'];
                        $four_unallocated += $each['91_120_unallocated'];
                        $five_unallocated += $each['greater_120_unallocated'];
                        $grand_total_unallocated += $total_unallocated;

                        if ($total_receivable > 0 || $total_unallocated > 0) :
                            $data .= '<tr>
																																									<td rowspan="2" >' . ($i + 1) . '</td>
																																		                            <td rowspan="2" >' . $each['subledger'] . '</td>
																																		                            <td>' . $labelOne . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['0_30_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['31_60_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['61_90_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['91_120_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['greater_120_receivable'], " ") . '</td>
																																									<td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($total_receivable, " ") . '</td>
																																						        </tr>';
                            $data .= '<tr>
																																		                            <td>' . $labelTwo . '</td>
																																		                            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['0_30_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['31_60_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['61_90_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['91_120_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($each['greater_120_unallocated'], " ") . '</td>
																																						            <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($total_unallocated, " ") . '</td>
																																						        </tr>';
                        endif;
                    endforeach;

                    $data .= '<tr>
                        <td colspan="2" rowspan="2" class="text-right"><b>Total</b></td>
                        <td>' . $labelOne . count($aging_report) . '</td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($one_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($two_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($three_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($four_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($five_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_receivable, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                    </tr>';

                    $data .= '<tr>
                        <td>' . $labelTwo . '</td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($one_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($two_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($three_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($four_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($five_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                        <td class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total_unallocated, " ") . " " . FinanceLegder::baseCurrency() . '</b></td>
                    </tr>';
                    $data .= ' </tbody>
            </table>';
                    $table = $data;
                } else if ($report_type == 'day_book') {

                    $table = '<div class="table-responsive">';
                    $ledgerItems = array();

                    //        $ledger_id = (int) $_POST['ledger_id'];
                    //        $subledger = (int) $_POST['subledger_id'];
                    //        $brand = (int) $_POST['brand'];
                    //        $currency = 1;
                    //        $user = (int) $_POST['created_user'];
                    $cur = MasCurrency::model()->findByPk(1);
                    $exchange_rate = $cur->exchange_rate;
                    $currency_code = $cur->currency_code;
                    if (!empty($params['to_date'])) {
                        $from_date = $params['to_date'];
                        $to_date = $params['to_date'];
                    } else {
                        $from_date = date('Y-m-d');
                        $to_date = date('Y-m-d');
                    }
                    $condition = 'parent.is_posted = "Y" AND parent.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';

                    //        if ($ledger_id > 0) {
                    //            $condition .= ' AND t.ledger_id IN(' . $ledger_id . ')';
                    //        }
                    //
                    //        if ($subledger > 0) {
                    //            $condition .= ' AND t.subledger_id=' . $subledger;
                    //        }
                    //
                    //        if ($brand > 0) {
                    //            $condition .= ' AND t.brand_id=' . $brand;
                    //        }

                    if ($outlet > 0) {
                        $condition .= ' AND t.outlet=' . $outlet;
                    }

                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND parent.created_by=' . $params['salesman_id'];
                    }

                    $items = FinanceVoucherItems::model()->with('parent', 'parent.vtype')->findAll(array('condition' => $condition, 'order' => 'parent.voucher_date DESC,vtype.code ASC,parent.voucher_no ASC'));
                    $previousBalance = $openingBalance;
                    $closingBalance = $openingBalance;
                    $symbol = ($previousBalance > 0) ? 'Dr' : 'Cr';

                    $total_debit = 0;
                    $total_credit = 0;
                    $discount_allowed = FinanceSystemAccountsMapping::getSystemdefault('discount_allowed');
                    $table = '<table class="table table-hover table-fixed table-bordered" id="gl_table">
                  <thead>
                      <tr>
                        <th style="width:7%" >Salesman</th>
                        <th style="width:7%" >Date</th>
                         <th style="width:5%" >Type</th>
                        <th style="width:5%" >VO #</th>
                        <th style="width:10%" >Ledger</th>
                        <th style="width:20%" >Subledger</th>
                        <th style="width:10%" >PT</th>
                        <th style="width:20%" >Ref</th>
                        <th style="width:40%">Narration</th>
                        <th style="width:12%;text-align:right !important;" class="text-right">Value</th>
                      </tr>
                  </thead>
                  <tbody>';
                    $parent = 1;
                    $sales_total = 0;
                    $purchase_total = 0;
                    $creditnote_total = 0;
                    $debitnote_total = 0;
                    $receipt_total = 0;
                    $payment_total = 0;
                    $journal_total = 0;
                    $discount_total = 0;
                    $receipt_discount_total = 0;
                    foreach ($items as $each) {

                        $debit = ($each->amount > 0) ? round(($each->amount * $each->exchange_rate), 4) : '';
                        $credit = ($each->amount < 0) ? round(abs($each->amount * $each->exchange_rate), 4) : '';

                        $debit = ($currency == $each->currency_id) ? $debit / $each->exchange_rate : $debit / $exchange_rate;
                        $credit = ($currency == $each->currency_id) ? $credit / $each->exchange_rate : $credit / $exchange_rate;

                        $running = $debit - $credit + $previousBalance;
                        $previousBalance = $running;
                        $symbol = ($running > 0) ? 'Dr' : 'Cr';
                        $closingBalance += ($debit - $credit);

                        $total_debit += $debit;
                        $total_credit += $credit;

                        $subledgerName = ($each->subledger_id > 0) ? $each->subledger->description : '';
                        $vouchreformat = $each->parent->fullvoucher;
                        if ($each->parent->vtype->code == "SI" && $debit > 0) {
                            if ($each->parent->header_table_id > 0) {
                                $SalesInvoice = SalesInvoice::model()->findByPk($each->parent->header_table_id);
                                switch ($SalesInvoice->payment_term) {
                                    case 1:
                                        $payTerms = 'Cash';
                                        break;
                                    case 2:
                                        $payTerms = 'Credit';
                                        if ($SalesInvoice->customer->credit_days > 0) {
                                            $payTerms .= ' - ' . $SalesInvoice->customer->credit_days . ' Days ';
                                        }
                                        break;
                                    case 3:
                                        $payTerms = 'Credit 60 PDC';
                                        break;
                                    case 4:
                                        $payTerms = 'Credit 90 PDC';
                                        break;
                                    case 5:
                                        $payTerms = 'Credit 30 PDC';
                                        break;
                                    case 6:
                                        $payTerms = 'Credit 15 PDC';
                                        break;
                                }
                            }
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> SALES </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td>' . $payTerms . '</td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
                      </tr>';
                            $sales_total += $debit;
                        } else if ($each->parent->vtype->code == "PI" && $credit > 0) {
                            if ($each->parent->header_table_id > 0) {
                                $PurchaseInvoice = PurchaseInvoice::model()->findByPk($each->parent->header_table_id);
                                switch ($PurchaseInvoice->payment_term) {
                                    case 1:
                                        $payTerms = 'Cash';
                                        break;
                                    case 2:
                                        $payTerms = 'Credit';
                                        break;
                                    case 3:
                                        $payTerms = 'Credit 60 PDC';
                                        break;
                                    case 4:
                                        $payTerms = 'Credit 90 PDC';
                                        break;
                                    case 5:
                                        $payTerms = 'Credit 30 PDC';
                                        break;
                                    case 6:
                                        $payTerms = 'Credit 15 PDC';
                                        break;
                                }
                            }
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> PURCHASE </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td>' . $payTerms . '</td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
                      </tr>';
                            $purchase_total += $credit;
                        } else if ($each->parent->vtype->code == "CN" && $credit > 0) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> CREDIT NOTE </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
                      </tr>';
                            $creditnote_total += $credit;
                        } else if ($each->parent->vtype->code == "DN" && $debit > 0) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> DEBIT NOTE </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
                      </tr>';
                            $debitnote_total += $debit;
                        } else if ($each->parent->vtype->code == "PV" && $debit > 0) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> PAYMENT </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
                      </tr>';
                            $payment_total += $debit;
                        } else if ($each->parent->vtype->code == "CA" && $credit > 0) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                           <td> RECEIPT </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
                      </tr>';
                            $receipt_total += $credit;
                        } else if ($each->parent->vtype->code == "CA" && $each->ledger_id == $discount_allowed) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                           <td> DISCOUNT RECEIPT </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
                      </tr>';
                            $receipt_discount_total += $debit;
                        } else if ($each->parent->vtype->code == "CHQ" && $credit > 0) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> RECEIPT </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
                      </tr>';
                            $receipt_total += $credit;
                        } else if ($each->parent->vtype->code == "DA" && $credit > 0) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> DISCOUNT </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($credit, " ") . '</td>
                      </tr>';
                            $discount_total += $credit;
                        } else if ($each->parent->vtype->code == "JV" && $debit > 0) {
                            $table .= ' <tr  data-id="' . $parent . '" data-parent="">
                          <td>' . $each->parent->created->full_name . '</td>
                          <td>' . date("d/m/Y", strtotime($each->parent->voucher_date)) . '</td>
                          <td> JOURNAL </td>
                          <td>' . $vouchreformat . '</td>
                          <td>' . $each->ledger->description . '</td>
                          <td>' . $subledgerName . '</td>
                          <td></td>
                          <td>' . $each->parent->reference_no . '</td>
                          <td><small>' . $each->narration . '</small></td>
                          <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($debit, " ") . '</td>
                      </tr>';
                            $journal_total += $debit;
                        }
                        $parent = $child;
                    }
                    $running = $debit - $credit + $previousBalance;
                    $symbol = ($closingBalance > 0) ? 'Dr' : 'Cr';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL PURCHASE </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($purchase_total), " ") . '</b></td>
                  </tr>';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL DEBIT NOTE </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($debitnote_total), " ") . '</b></td>
                  </tr>';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL SALES </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($sales_total), " ") . '</b></td>
                  </tr>';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL CREDIT NOTE </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($creditnote_total), " ") . '</b></td>
                  </tr>';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL RECEIPT </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($receipt_total), " ") . '</b></td>
                  </tr>';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL PAYMENT </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($payment_total), " ") . '</b></td>
                  </tr>';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL JOURNAL </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($journal_total), " ") . '</b></td>
                  </tr>';
                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL SALES DISCOUNT </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($discount_total), " ") . '</b></td>
                  </tr>';

                    $table .= '<tr>
                      <td colspan="9" style="text-align:right !important;"><b>TOTAL RECEIPT DISCOUNT </b></td>
                      <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(abs($receipt_discount_total), " ") . '</b></td>
                  </tr>';

                    $table .= '</tbody>
              </table>';
                    $table .= '</div>';
                } else if ($report_type == 'stock_report') {

                    $table = '<div class="table-responsive">';

                    $model = new Inventory();
                    $condition = '1=1';
                    $outlet = $params['outlet'];
                    if ($params['outlet'] > 0) {
                        $condition .= ' AND I.outlet=' . $params['outlet'];
                    }
                    //            if($type!='')
                    //            {
                    //            $condition.=' AND I.type="'.$type.'"';    
                    //            }
                    //            if($brand>0)
                    //            {
                    //            $condition.=' AND I.brand='.$brand;    
                    //            }
                    if ($params['salesman_id'] > 0) {
                        $condition .= ' AND (S.created_by=' . $params['salesman_id'] . ' OR S.type="OB" )';
                    }
                    if (!empty($params['from_date'])) {
                        $from_date = date("Y-m-d", strtotime(str_replace('/', '-', $params['from_date'])));
                    } else {
                        $from_date = date('Y-m-d');
                    }
                    if (!empty($params['to_date'])) {
                        $to_date = date("Y-m-d", strtotime(str_replace('/', '-', $params['to_date'])));
                    } else {
                        $to_date = date('Y-m-d');
                    }

                    $colspan = 14;
                    $table = '<div class="table-responsive">
    <table class="table table-hover table-striped table-bordered ">
        <thead>
            <tr>
                <th style="width:1%">#</th>
                <th style="width:10%">Part No</th>
                <th style="width:25%">Description</th>
                <th style="width:5%">Type</th>
                <th style="width:8%">Brand</th>
                <th>Open Qty</th>
                <th>In Qty</th>
                <th>Out Qty</th>
                <th>Bal Qty</th>
                <th>Unit</th>
                <th>Cost</th>
                <th>Price</th>                        
                <th>Tax(%)</th>
                <th>Outlet</th>
            </tr>
        </thead>
        <tbody>';
                    $query_opening_stock = "SELECT id,part_number,description,active,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,
        unit_name,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,SUM(tax) AS tax,name,outlet_name,rate from view_current_stock  where transaction_date<'" . $from_date . "' group by id";
                    $query_current_stock = "SELECT id,part_number,description,active,brand_name,type,SUM(in_qty) AS in_qty,SUM(out_qty) AS out_qty,SUM(in_qty)-SUM(out_qty) AS qty,
        unit_name,SUM(avg_purchase_rate) AS avg_purchase_rate,SUM(avg_sales_rate) AS avg_sales_rate,tax,name,outlet_name,rate from view_current_stock  where transaction_date>='" . $from_date . "' AND  transaction_date<='" . $to_date . "'  group by id";
                    $SQL = "CREATE OR REPLACE VIEW view_current_stock AS
                (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,S.quantity*S.unit_exchange_rate AS in_qty,
                0 AS out_qty,U.unit_name,0 AS avg_purchase_rate,0 AS avg_sales_rate,tax,
                O.name,O.outlet_name,I.rate,S.transaction_date
                FROM erp_inventory I 
                LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                LEFT JOIN erp_master_brand B ON I.brand=B.id 
                LEFT JOIN erp_master_unit U ON I.unit=U.id 
                LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='IN' OR S.type='OB'))
                UNION
                (SELECT I.id,I.part_number,I.description,I.active,B.brand_name,I.type,0 AS in_qty,S.quantity*S.unit_exchange_rate AS out_qty,
                U.unit_name,0 AS avg_purchase_rate,0 AS avg_sales_rate,tax,O.name,O.outlet_name,I.rate,S.transaction_date
                FROM erp_inventory I 
                LEFT JOIN erp_stock_ledger S ON I.id=S.product_id 
                LEFT JOIN erp_master_brand B ON I.brand=B.id 
                LEFT JOIN erp_master_unit U ON I.unit=U.id 
                LEFT JOIN bluesell_new_main.erp_outlets O ON I.outlet=O.id where $condition AND (S.transaction_type='OUT' OR S.type='OB'))";
                    Yii::app()->db->createCommand($SQL)->execute();
                    $arr_stock = Yii::app()->db->createCommand($query_current_stock)->queryAll();
                    $sl = 1;
                    if (!empty($arr_stock)) {
                        foreach ($arr_stock as $row) {
                            $inv_type = ($row['type'] == 'I') ? "Inventory" : "Service";
                            $product_id = $row['id'];
                            $outlet = $params['outlet_id'];
                            $avg_purchase_rate = StockLedger::getAvgPurchaseRate($product_id, '', $outlet);
                            $open_in_qty = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from erp_stock_ledger where product_id=$product_id AND transaction_type='IN' AND transaction_date<'" . $from_date . "'")->queryScalar();
                            $open_out_qty = Yii::app()->db->createCommand("select SUM(quantity*unit_exchange_rate) from erp_stock_ledger where product_id=$product_id AND transaction_type='OUT' AND transaction_date<'" . $from_date . "'")->queryScalar();
                            $open = $open_in_qty - $open_out_qty;
                            if ($open > 0 || $row['in_qty'] > 0 || $row['out_qty'] > 0) {
                                $table .= '<tr>
                    <td>' . $sl . '</td> 
                    <td>' . $row['part_number'] . '</td> 
                    <td>' . $row['description'] . '</td> 
                    <td>' . $inv_type . '</td> 
                    <td>' . $row['brand_name'] . '</td> 
                    <td class="text-right">' . round(($open_in_qty - $open_out_qty), 3) . '</td>
                    <td class="text-right">' . round($row['in_qty'], 3) . '</td> 
                    <td class="text-right">' . round($row['out_qty'], 3) . '</td> 
                    <td class="text-right">' . round(($open_in_qty - $open_out_qty + $row['qty']), 3) . '</td>
                    <td>' . $row['unit_name'] . '</td> 
                    <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($avg_purchase_rate, '') . '</td> 
                    <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($row['rate'], '') . '</td> 
                    <td class="text-right">' . Yii::app()->numberFormatter->formatCurrency($row['tax'], '') . ' %' . '</td> 
                    <td class="text-right">' . $row['outlet_name'] . '</td>                    
                </tr>';
                                $sl++;
                            }
                        }
                    } else {
                        $table .= '<tr><th colspan="' . $colspan . '" class="text-center bg-info">No Details Found</th></tr>';
                    }
                    $table .= '</tbody>
              </table>';
                    $table .= '</div>';
                } else if ($report_type == 'vat_return_report') {

                    $from_date = $params['from_date'];
                    $to_date = $params['to_date'];
                    $outlet = (int) $params['outlet_id'];
                    $condition = '(t.invoice_date>="' . $from_date . '" AND t.invoice_date<="' . $to_date . '") AND t.active="Y"';
                    if ($outlet > 0) {
                        $condition .= ' AND outlet="' . $outlet . '"';
                    }
                    $sales_view_sql = 'CREATE OR REPLACE VIEW view_net_sale_report AS (SELECT "I" as type,I.id,I.customer_id,I.invoice_no,I.invoice_date,C.customer_name,I.exchange_rate,I.total_amount,0 AS total_amount1,I.grand_total,0 AS grand_total1,I.tax_total,0 AS tax_total1,I.outlet,I.active,C.state
              FROM  {{sales_invoice}} I
              LEFT JOIN  {{customer}} C
              ON C.id=I.customer_id
              )
              UNION ALL
              (SELECT "CN" as type,CN.id,CN.customer_id,CN.return_no AS invoice_no,CN.return_date AS invoice_date,C.customer_name,CN.exchange_rate,0 AS total_amount,(CN.total_amount) AS total_amount1,0 AS grand_total,(CN.grand_total) AS grand_total1,0 AS tax_total,(CN.tax_total) AS tax_total1,CN.outlet,CN.active,C.state 
              FROM  {{sales_return}} CN
              LEFT JOIN  {{customer}} C
              ON C.id=CN.customer_id)';
                    Yii::app()->db->createCommand($sales_view_sql)->execute();
                    $sales_sql = 'SELECT S.state_name,t.state,SUM(grand_total-grand_total1) AS grand,SUM(total_amount-total_amount1) AS tot,SUM(tax_total-tax_total1) AS vat FROM view_net_sale_report t INNER JOIN erp_master_state S ON t.state=S.id WHERE ' . $condition . ' GROUP BY state ORDER BY S.state_name ASC';

                    $table = '<br><table class="table table-hover table-fixed table-bordered">
                    <thead>
                        <tr>
                            <th colspan="3">VAT on Sales (Outwards)</th>
                        </tr>
                        <tr>
                            <th style="width:8%">Emirate</th>
                            <th style="width:15%">Taxable Amount</th>
                            <th style="width:15%">VAT (' . FinanceLegder::baseCurrency() . ')</th>
                        </tr>
                    </thead>
                    <tbody>';

                    $total_amount = 0;
                    $total_vat = 0;
                    $items = Yii::app()->db->createCommand($sales_sql)->queryAll();
                    foreach ($items as $each) {
                        $vat = round($each['vat'], 4);
                        $tot = round($each['tot'], 4);
                        if ($each['state'] > 0) {
                            $emirate = $each['state_name'];
                        }
                        $table .= '<tr>
                                            <td>' . $emirate . '</td>
                                            <td style="text-align:right !important;" class="text-right">' . Yii::app()->numberFormatter->formatCurrency($tot, " ") . '</td>
                                            <td style="text-align:right !important;" class="text-right">' . Yii::app()->numberFormatter->formatCurrency($vat, " ") . '</td>
                                        </tr>';

                        $total_amount += $tot;
                        $total_vat += $vat;
                    }
                    $table .= '<tr>
                                            <td style="text-align:right !important;" class="text-right"><b>TOTAL</b></td>
                                            <td style="text-align:right !important;" class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_amount, " ") . '</b></td>
                                            <td style="text-align:right !important;" class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat, " ") . '</b></td>
                                        </tr>';
                    // Start Other Income

                    $vat_sales_account = FinanceSystemAccountsMapping::getSystemdefault('sales_vat');
                    $query = 'SELECT
                        ROUND(SUM(VI.amount*VI.exchange_rate), 4) AS vat_amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} VH ON VH.id = VI.header_id
                        LEFT JOIN {{finance_legder}} L ON L.id = VI.ledger_id
                        LEFT JOIN {{finance_voucher_type}} VT ON VT.id = VH.voucher_type
                        WHERE VT.code!="SI" AND VT.code!="CN" AND L.main_account_type="A" AND L.id="' . $vat_sales_account . '" AND VH.is_posted = "Y" AND VH.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND VH.outlet="' . $outlet . '"';
                    $vat_on_other_income = -1 * Yii::app()->db->createCommand($query)->queryScalar();
                    $table .= '<tr><td colspan="2" style="text-align:right !important;"><b>VAT ON OTHER INCOME</b></td>
                                <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency(($vat_on_other_income), " ") . '</b></td></tr>';

                    // End Other Income
                    $purchase_view_sql = 'CREATE OR REPLACE VIEW view_net_purchase_report AS (SELECT "I" as type,I.id,I.invoice_date,I.exchange_rate,I.total_amount,0 AS total_amount1,I.grand_total,0 AS grand_total1,I.tax_total,0 AS tax_total1,I.outlet,I.active
              FROM  {{purchase_invoice}} I
              )
              UNION ALL
              (SELECT "CN" as type,CN.id,CN.return_date AS invoice_date,CN.exchange_rate,0 AS total_amount,(CN.total_amount) AS total_amount1,0 AS grand_total,(CN.grand_total) AS grand_total1,0 AS tax_total,(CN.tax_total) AS tax_total1,CN.outlet,CN.active
              FROM  {{purchase_return}} CN)';
                    Yii::app()->db->createCommand($purchase_view_sql)->execute();
                    $purchase_vat = Yii::app()->db->createCommand('SELECT SUM(tax_total-tax_total1) AS vat FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();

                    $table .= '<tr>
                                            <td colspan="2" style="text-align:right !important;" class="text-right"><b>VAT ON PURCHASE</b></td>
                                            <td style="text-align:right !important;" class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency($purchase_vat, " ") . '</b></td>
                                        </tr>';
                    $vat_account = FinanceSystemAccountsMapping::getSystemdefault('vat_on_expense');
                    $query = 'SELECT
                        ROUND(SUM(VI.amount*VI.exchange_rate), 4) AS vat_amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} VH ON VH.id = VI.header_id
                        LEFT JOIN {{finance_legder}} L ON L.id = VI.ledger_id
                        WHERE L.main_account_type="L" AND L.id="' . $vat_account . '" AND VH.is_posted = "Y" AND VH.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND VH.outlet="' . $outlet . '"';
                    $vat_on_expense = Yii::app()->db->createCommand($query)->queryScalar();
                    $vat_purchase_account = FinanceSystemAccountsMapping::getSystemdefault('purchase_vat');
                    $query = 'SELECT
                        ROUND(SUM(VI.amount*VI.exchange_rate), 4) AS vat_amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} VH ON VH.id = VI.header_id
                        LEFT JOIN {{finance_legder}} L ON L.id = VI.ledger_id
                        LEFT JOIN {{finance_voucher_type}} VT ON VT.id = VH.voucher_type
                        WHERE VT.code!="PI" AND VT.code!="DN" AND L.main_account_type="L" AND L.id="' . $vat_purchase_account . '" AND VH.is_posted = "Y" AND VH.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND VH.outlet="' . $outlet . '"';
                    $vat_on_other_expense = Yii::app()->db->createCommand($query)->queryScalar();
                    $table .= '<tr>
                                            <td colspan="2" style="text-align:right !important;" class="text-right"><b>VAT ON EXPENSE</b></td>
                                            <td style="text-align:right !important;" class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($vat_on_expense + $vat_on_other_expense), " ") . '</b></td>
                                        </tr>';
                    $table .= '<tr>
                                            <td colspan="2" style="text-align:right !important;" class="text-right"><b>VAT PAYABLE</b></td>
                                            <td style="text-align:right !important;" class="text-right"><b>' . Yii::app()->numberFormatter->formatCurrency(($total_vat + $vat_on_other_income - $vat_on_expense - $vat_on_other_expense - $purchase_vat), " ") . '</b></td>
                                        </tr>';
                    $table .= '</tbody>
                        </table>';
                } else if ($report_type == 'vat_summary_report') {

                    $from_date = $params['from_date'];
                    $to_date = $params['to_date'];
                    $outlet = (int) $params['outlet_id'];
                    $condition = '(t.invoice_date>="' . $from_date . '" AND t.invoice_date<="' . $to_date . '") AND t.active="Y"';
                    if ($outlet > 0) {
                        $condition .= ' AND outlet="' . $outlet . '"';
                    }
                    $sales_view_sql = 'CREATE OR REPLACE VIEW view_net_sale_report AS (SELECT "I" as type,I.id,I.customer_id,I.invoice_no,I.invoice_date,C.customer_name,I.exchange_rate,I.total_amount,0 AS total_amount1,I.grand_total,0 AS grand_total1,I.tax_total,0 AS tax_total1,I.outlet,I.active,C.state
              FROM  {{sales_invoice}} I
              LEFT JOIN  {{customer}} C
              ON C.id=I.customer_id
              )
              UNION ALL
              (SELECT "CN" as type,CN.id,CN.customer_id,CN.return_no AS invoice_no,CN.return_date AS invoice_date,C.customer_name,CN.exchange_rate,0 AS total_amount,(CN.total_amount) AS total_amount1,0 AS grand_total,(CN.grand_total) AS grand_total1,0 AS tax_total,(CN.tax_total) AS tax_total1,CN.outlet,CN.active,C.state 
              FROM  {{sales_return}} CN
              LEFT JOIN  {{customer}} C
              ON C.id=CN.customer_id)';
                    Yii::app()->db->createCommand($sales_view_sql)->execute();
                    $sales_sql = 'SELECT SUM(grand_total) AS grand,SUM(total_amount) AS tot,SUM(tax_total) AS vat FROM view_net_sale_report t WHERE ' . $condition;

                    $table = '<div class="row">
            <div class="col-sm-6">
            <div class="table-responsive">
            <table class="table table-hover table-fixed table-bordered">
                    <thead>
                        <tr>
                            <th colspan="4">VAT on Sales (Outwards)</th>
                        </tr>
                        <tr>
                            <th style="width:15%">Transaction</th>
                            <th style="width:8%">Amount</th>
                            <th style="width:8%">VAT </th>
                            <th style="width:8%">Grand Total</th>
                        </tr>
                    </thead>
                    <tbody>';

                    $total_amount = 0;
                    $total_vat = 0;
                    $total_grand = 0;
                    $grand = Yii::app()->db->createCommand('SELECT SUM(grand_total) AS grand FROM view_net_sale_report t WHERE ' . $condition)->queryScalar();
                    $tot = Yii::app()->db->createCommand('SELECT SUM(total_amount) AS tot FROM view_net_sale_report t WHERE ' . $condition)->queryScalar();
                    $vat = Yii::app()->db->createCommand('SELECT SUM(tax_total) AS vat FROM view_net_sale_report t WHERE ' . $condition)->queryScalar();

                    $table .= '<tr>
                                            <td style="text-align:left !important;"> SALES</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($tot, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($vat, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($grand, " ") . '</td>
                                        </tr>';
                    $total_grand += $grand;
                    $total_amount += $tot;
                    $total_vat += $vat;

                    $grand1 = Yii::app()->db->createCommand('SELECT SUM(grand_total1) AS grand FROM view_net_sale_report t WHERE ' . $condition)->queryScalar();
                    $tot1 = Yii::app()->db->createCommand('SELECT SUM(total_amount1) AS tot FROM view_net_sale_report t WHERE ' . $condition)->queryScalar();
                    $vat1 = Yii::app()->db->createCommand('SELECT SUM(tax_total1) AS vat FROM view_net_sale_report t WHERE ' . $condition)->queryScalar();

                    $table .= '<tr>         
                                            <td style="text-align:left !important;"> SALES RETURN</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($tot1, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($vat1, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($grand1, " ") . '</td>
                                        </tr>';
                    $total_grand -= $grand1;
                    $total_amount -= $tot1;
                    $total_vat -= $vat1;
                    $table .= '<tr>
                                            <td style="text-align:right !important;"><b>TOTAL SALES</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_amount, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_grand, " ") . '</b></td>
                                        </tr>';
                    $table .= '<tr>
                                            <td colspan="4"></td>
                                        </tr>';
                    // Start Other Income

                    $vat_sales_account = FinanceSystemAccountsMapping::getSystemdefault('sales_vat');
                    $query = 'SELECT
                        ROUND(SUM(VI.amount*VI.exchange_rate), 4) AS vat_amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} VH ON VH.id = VI.header_id
                        LEFT JOIN {{finance_legder}} L ON L.id = VI.ledger_id
                        LEFT JOIN {{finance_voucher_type}} VT ON VT.id = VH.voucher_type
                        WHERE VT.code!="SI" AND VT.code!="CN" AND L.main_account_type="A" AND L.id="' . $vat_sales_account . '" AND VH.is_posted = "Y" AND VH.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND VH.outlet="' . $outlet . '"';
                    $vat_on_other_income = -1 * Yii::app()->db->createCommand($query)->queryScalar();
                    $other_income_amount = $vat_on_other_income * 100 / 5;
                    $other_income = $other_income_amount + $vat_on_other_income;
                    $table .= '<tr>
                                            <td style="text-align:right !important;"><b>OTHER INCOME</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($other_income_amount, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($vat_on_other_income, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($other_income, " ") . '</b></td>
                                        </tr>';
                    $total_grand += $other_income;
                    $total_amount += $other_income_amount;
                    $total_vat += $vat_on_other_income;
                    $table .= '<tr>
                                            <td style="text-align:right !important;"><b>TOTAL</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_amount, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_grand, " ") . '</b></td>
                                        </tr>';

                    // End Other Income
                    $table .= '</table></div></div><div class="col-sm-6">            
               <table class="table table-hover table-fixed table-bordered">
                    <thead>
                        <tr>
                            <th colspan="4">VAT on Purchase (Inwards)</th>
                        </tr>
                        <tr>
                            <th style="width:15%">Transaction</th>
                            <th style="width:8%">Amount</th>
                            <th style="width:8%">VAT </th>
                            <th style="width:8%">Grand Total</th>
                        </tr>
                    </thead>
                    <tbody>
                <div class="table-responsive">';
                    $purchase_view_sql = 'CREATE OR REPLACE VIEW view_net_purchase_report AS (SELECT "I" as type,I.id,I.invoice_date,I.exchange_rate,I.total_amount,0 AS total_amount1,I.grand_total,0 AS grand_total1,I.tax_total,0 AS tax_total1,I.outlet,I.active
              FROM  {{purchase_invoice}} I
              )
              UNION ALL
              (SELECT "CN" as type,CN.id,CN.return_date AS invoice_date,CN.exchange_rate,0 AS total_amount,(CN.total_amount) AS total_amount1,0 AS grand_total,(CN.grand_total) AS grand_total1,0 AS tax_total,(CN.tax_total) AS tax_total1,CN.outlet,CN.active
              FROM  {{purchase_return}} CN)';
                    Yii::app()->db->createCommand($purchase_view_sql)->execute();

                    $total_amount1 = 0;
                    $total_vat1 = 0;
                    $total_grand1 = 0;
                    $grand2 = Yii::app()->db->createCommand('SELECT SUM(grand_total) AS grand FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();
                    $tot2 = Yii::app()->db->createCommand('SELECT SUM(total_amount) AS tot FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();
                    $vat2 = Yii::app()->db->createCommand('SELECT SUM(tax_total) AS vat FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();

                    $table .= '<tr>
                                            <td style="text-align:left !important;"> PURCHASE</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($tot2, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($vat2, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($grand2, " ") . '</td>
                                        </tr>';
                    $total_grand1 += $grand2;
                    $total_amount1 += $tot2;
                    $total_vat1 += $vat2;

                    $grand3 = Yii::app()->db->createCommand('SELECT SUM(grand_total1) AS grand FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();
                    $tot3 = Yii::app()->db->createCommand('SELECT SUM(total_amount1) AS tot FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();
                    $vat3 = Yii::app()->db->createCommand('SELECT SUM(tax_total1) AS vat FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();

                    $table .= '<tr>         
                                            <td style="text-align:left !important;"> PURCHASE RETURN</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($tot3, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($vat3, " ") . '</td>
                                            <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($grand3, " ") . '</td>
                                        </tr>';
                    $total_grand1 -= $grand3;
                    $total_amount1 -= $tot3;
                    $total_vat1 -= $vat3;
                    $table .= '<tr>         
                                            <td style="text-align:right !important;"><b>TOTAL PURCHASE</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_amount1, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat1, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_grand1, " ") . '</b></td>
                                        </tr>';
                    $table .= '<tr>
                                            <td colspan="4"></td>
                                        </tr>';
                    $purchase_vat = Yii::app()->db->createCommand('SELECT SUM(tax_total-tax_total1) AS vat FROM view_net_purchase_report t WHERE ' . $condition)->queryScalar();

                    $vat_account = FinanceSystemAccountsMapping::getSystemdefault('vat_on_expense');
                    $query = 'SELECT
                        ROUND(SUM(VI.amount*VI.exchange_rate), 4) AS vat_amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} VH ON VH.id = VI.header_id
                        LEFT JOIN {{finance_legder}} L ON L.id = VI.ledger_id
                        WHERE L.main_account_type="L" AND L.id="' . $vat_account . '" AND VH.is_posted = "Y" AND VH.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND VH.outlet="' . $outlet . '"';
                    $vat_on_expense = Yii::app()->db->createCommand($query)->queryScalar();
                    $vat_purchase_account = FinanceSystemAccountsMapping::getSystemdefault('purchase_vat');
                    $query = 'SELECT
                        ROUND(SUM(VI.amount*VI.exchange_rate), 4) AS vat_amount
                        FROM {{finance_voucher_items}} VI
                        LEFT JOIN {{finance_voucher_head}} VH ON VH.id = VI.header_id
                        LEFT JOIN {{finance_legder}} L ON L.id = VI.ledger_id
                        LEFT JOIN {{finance_voucher_type}} VT ON VT.id = VH.voucher_type
                        WHERE VT.code!="PI" AND VT.code!="DN" AND L.main_account_type="L" AND L.id="' . $vat_purchase_account . '" AND VH.is_posted = "Y" AND VH.voucher_date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND VH.outlet="' . $outlet . '"';
                    $vat_on_other_expense = Yii::app()->db->createCommand($query)->queryScalar();
                    $tot_vat_on_exp = $vat_on_expense + $vat_on_other_expense;
                    $other_exp_amount = $tot_vat_on_exp * 100 / 5;
                    $other_exp = $other_exp_amount + $tot_vat_on_exp;
                    $table .= '<tr>
                                            <td style="text-align:right !important;"><b>OTHER EXPENSES</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($other_exp_amount, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($tot_vat_on_exp, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($other_exp, " ") . '</b></td>
                                        </tr>';
                    $total_grand1 += $other_exp;
                    $total_amount1 += $other_exp_amount;
                    $total_vat1 += $tot_vat_on_exp;
                    $table .= '<tr>
                                            <td style="text-align:right !important;"><b>TOTAL</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_amount1, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_vat1, " ") . '</b></td>
                                            <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_grand1, " ") . '</b></td>
                                        </tr>';
                    $table .= '</tbody>
                        </table></div></div>
                        <div class="col-sm-12">
            <div class="table-responsive">
            <table class="table table-hover table-fixed table-bordered">
                    <thead>
                        <tr>
                            <td style="text-align:right !important;background-color:#66ff33;">VAT PAYABLE</td>
                            <td style="text-align:right !important;background-color:#66ff33;"><b>' . Yii::app()->numberFormatter->formatCurrency(($total_vat - $total_vat1), " ") . '</b></td>
                        </tr>
                        </table></div></div></div>';
                } else if ($report_type == 'consolidated_report') {
                    $from_date = $params['from_date'];
                    $daybook_date = $params['to_date'];
                    $outlet = (int) $params['outlet_id'];
                    $created_user = (int) $params['salesman_id'];
                    $condition_sales = "1=1";
                    $condition_sales_return = "1=1";

                    if ($outlet > 0) {
                        $condition_sales .= " AND outlet=" . $outlet;
                        $condition_sales_return .= " AND outlet=" . $outlet;
                    }
                    /* if ($created_user > 0) {
                        $condition_sales .= " AND sales_man=" . $created_user;
                        $condition_sales_return .= " AND created_by=" . $created_user;
                    }*/
                    if ($daybook_date != '' && $daybook_date != '0000-00-00') {
                        $condition_sales .= ' AND invoice_date="' . $daybook_date . '"';
                        $condition_sales_return .= ' AND return_date="' . $daybook_date . '"';
                    }


                    $sales_invoice_count = Yii::app()->db->createCommand("SELECT count(*) FROM  {{sales_invoice}} where $condition_sales")->queryScalar();
                    $sales_return_count = Yii::app()->db->createCommand("SELECT count(*) FROM  {{sales_return}} where $condition_sales_return")->queryScalar();

                    $sales_invoice_amount = Yii::app()->db->createCommand("SELECT SUM(grand_total) FROM  {{sales_invoice}} where $condition_sales")->queryScalar();
                    $sales_return_amount = Yii::app()->db->createCommand("SELECT SUM(grand_total) FROM  {{sales_return}} where $condition_sales_return")->queryScalar();

                    $sales_invoice_tax_amount = Yii::app()->db->createCommand("SELECT SUM(tax_total) FROM  {{sales_invoice}} where $condition_sales")->queryScalar();
                    $sales_return_tax_amount = Yii::app()->db->createCommand("SELECT SUM(tax_total) FROM  {{sales_return}} where $condition_sales_return")->queryScalar();



                    // -------------------------------------Sales-----------------------------------------------
                    $table = '<div class="table-responsive">
                            <table class="table table-hover table-striped table-bordered">
                                <thead>
                                    <tr>
                                        <th class="text-center" colspan="2">Sales</th> 
                                    </tr>
                                </thead>
                                <tbody>';

                    $table .= '<tr>
                            <td>Total Invoices</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($sales_invoice_count, "") . '</td>
                        </tr>';

                    $table .= '<tr>
                            <td>Total Invoices Value</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($sales_invoice_amount, "") . '</td>
                        </tr>';

                    $table .= '</tbody>
                            </table>
                        </div><br>';

                    // Second Table: Sales Return
                    $totalSalesReturnCount = 0; // Replace with your actual calculation for Sales Return
                    $totalSalesReturn = 0; // Replace with your actual calculation for Sales Return

                    $table .= '<div class="table-responsive">
                            <table class="table table-hover table-striped table-bordered">
                                <thead>
                                    <tr>
                                        <th class="text-center" colspan="2">Sales Return</th>
                                    </tr>
                                </thead>
                                <tbody>';

                    $table .= '<tr>
                            <td>No of Credit</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($sales_return_count, "") . '</td>
                        </tr>';

                    $table .= '<tr>
                            <td>Amount</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($sales_return_amount, "") . '</td>
                        </tr>';

                    $table .= '</tbody>
                            </table>
                        </div><br>';

                    // -------------------------------------Purchase-----------------------------------------------


                    $condition_purchase = "1=1";
                    $condition_purchase_return = "1=1";

                    if ($outlet > 0) {
                        $condition_purchase .= " AND outlet=" . $outlet;
                        $condition_purchase_return .= " AND outlet=" . $outlet;
                    }
                    /* if ($created_user > 0) {
                        $condition_purchase .= " AND created_by=" . $created_user;
                        $condition_purchase_return .= " AND created_by=" . $created_user;
                    }*/
                    if ($daybook_date != '' && $daybook_date != '0000-00-00') {
                        $condition_purchase .= ' AND invoice_date="' . $daybook_date . '"';
                        $condition_purchase_return .= ' AND return_date="' . $daybook_date . '"';
                    }
                    //Purchase
                    $purchase_invoice_count = Yii::app()->db->createCommand("SELECT count(*) FROM  {{purchase_invoice}} where $condition_purchase")->queryScalar();
                    $purchase_return_count = Yii::app()->db->createCommand("SELECT count(*) FROM  {{purchase_return}} where $condition_purchase_return")->queryScalar();

                    $purchase_invoice_amount = Yii::app()->db->createCommand("SELECT SUM(grand_total) FROM  {{purchase_invoice}} where $condition_purchase")->queryScalar();
                    $purchase_return_amount = Yii::app()->db->createCommand("SELECT SUM(grand_total) FROM  {{purchase_return}} where $condition_purchase_return")->queryScalar();


                    $purchase_invoice_tax_amount = Yii::app()->db->createCommand("SELECT SUM(tax_total) FROM  {{purchase_invoice}} where $condition_purchase")->queryScalar();
                    $purchase_return_tax_amount = Yii::app()->db->createCommand("SELECT SUM(tax_total) FROM  {{purchase_return}} where $condition_purchase_return")->queryScalar();


                    // First Table: Sales
                    $table .= '<div class="table-responsive">
                            <table class="table table-hover table-striped table-bordered">
                                <thead>
                                    <tr>
                                        <th class="text-center" colspan="2">Purchase</th>
                                    </tr>
                                </thead>
                                <tbody>';

                    $table .= '<tr>
                            <td>Total Invoices</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($purchase_invoice_count, "") . '</td>
                        </tr>';

                    $table .= '<tr>
                            <td>Total Invoices Value</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($purchase_invoice_amount, "") . '</td>
                        </tr>';

                    $table .= '</tbody>
                            </table>
                        </div><br>';

                    // -------------------------------------Purchase-Return----------------------------------------------



                    $table .= '<div class="table-responsive">
                            <table class="table table-hover table-striped table-bordered">
                                <thead>
                                    <tr>
                                        <th class="text-center" colspan="2">Purchase  Return</th>
                                    </tr>
                                </thead>
                                <tbody>';

                    $table .= '<tr>
                            <td>No Of Debit</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($purchase_return_count, "") . '</td>
                        </tr>';

                    $table .= '<tr>
                            <td>Amount</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($purchase_return_amount, "") . '</td>
                         </tr>';

                    $table .= '</tbody>
                            </table>
                        </div><br>';
                    // -------------------------------------Expense----------------------------------------------

                    $CashAccount = FinanceSystemAccountsMapping::getSystemdefault('cash_account');

                    $vat_account = FinanceSystemAccountsMapping::getSystemdefault('vat_on_expense');






                    $condition_expense = "1=1 AND  H.is_posted='Y' AND L.main_account_type='E'";

                    $condition_vat_expense = "1=1 AND H.is_posted='Y'  AND  ledger_id=" . $vat_account;

                    $condition_cash = "1=1 AND H.is_posted='Y'  AND amount>0 AND ledger_id=" . $CashAccount;

                    if ($outlet > 0) {
                        $condition_expense .= " AND I.outlet=" . $outlet;
                        $condition_vat_expense .= " AND I.outlet=" . $outlet;
                        $condition_cash .= " AND I.outlet=" . $outlet;
                    }
                    /*if ($created_user > 0) {
                        $condition_expense .= " AND H.created_by=" . $created_user;
                        $condition_vat_expense .= " AND H.created_by=" . $created_user;
                        $condition_cash .= " AND H.created_by=" . $created_user;
                    }*/
                    if ($daybook_date != '' && $daybook_date != '0000-00-00') {
                        $condition_expense .= ' AND H.voucher_date="' . $daybook_date . '"';
                        $condition_vat_expense .= ' AND H.voucher_date="' . $daybook_date . '"';
                        $condition_cash .= ' AND H.voucher_date="' . $daybook_date . '"';
                    }

                    $condition_expense .= ' AND I.amount >0 ';
                    $condition_expense .= ' AND H.voucher_type NOT IN (8, 9)';


                    //Expense & Cash


                    $expense_count = Yii::app()->db->createCommand("SELECT count(*) FROM {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id  LEFT join {{finance_legder}} L ON L.id = I.ledger_id where $condition_expense")->queryScalar();




                    $cash_count = Yii::app()->db->createCommand("SELECT count(*) FROM  {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id where $condition_cash")->queryScalar();

                    $expense_amount = Yii::app()->db->createCommand("SELECT SUM(amount) FROM {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id  LEFT join {{finance_legder}} L ON L.id = I.ledger_id where $condition_expense")->queryScalar();


                    $expense_vat_amount = Yii::app()->db->createCommand("SELECT SUM(amount) FROM {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id where $condition_vat_expense")->queryScalar();

                    $cash_amount = Yii::app()->db->createCommand("SELECT SUM(amount) FROM {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id where $condition_cash")->queryScalar();







                    $table .= '<div class="table-responsive">
                            <table class="table table-hover table-striped table-bordered">
                                <thead>
                                    <tr>
                                        <th class="text-center" colspan="2">Expense</th>
                                    </tr>
                                </thead>
                                <tbody>';

                    $table .= '<tr>
                            <td>No Of Expense</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($expense_count, "") . '</td>
                        </tr>';

                    $table .= '<tr>
                            <td>Amount</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($expense_amount, "") . '</td>
                       </tr>';

                    $table .= '</tbody>
                            </table>
                        </div><br>';
                    // --------------------------------------------------------------------------------------
                    $table .= '<div class="table-responsive">
                            <table class="table table-hover table-striped table-bordered">
                                <thead>
                                    <tr>
                                        <th class="text-center" colspan="2">Cash</th>
                                    </tr>
                                </thead>
                                <tbody>';

                    $table .= '<tr>
                            <td>Total Cash Collected</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($cash_amount, "") . '</td>
                        </tr>';

                    $table .= '<tr>
                            <td>Balance</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($cash_amount - $expense_amount, "") . '</td>
                        </tr>';

                    $total_vat_payable = (float)$sales_invoice_tax_amount - (float)$sales_return_tax_amount - (float)$purchase_invoice_tax_amount + (float)$purchase_return_tax_amount - (float)$expense_vat_amount;
                    $table .= '<tr>
                            <td>Total Vat Payable</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($total_vat_payable, "") . '</td>
                        </tr>';


                    $profit = ((float)$sales_invoice_amount - (float)$sales_return_amount) - ((float)$purchase_invoice_amount - (float)$purchase_return_amount) - (float)$expense_amount;

                    $table .= '<tr>
                            <td>Profit</td>
                            <td>' . Yii::app()->numberFormatter->formatCurrency($profit, "") . '</td>
                        </tr>';

                    $table .= '</tbody>
                            </table>
                        </div>';
                } else if ($report_type == 'daily_sales_report') {

                            $daybook_from_date = $params['from_date'];
                            $daybook_to_date = $params['to_date'];
                            $outlet = (int) $params['outlet_id'];
                            $created_user = (int) $params['salesman_id'];
                            $customer_id = (int) $params['customer_id'];
                            
                            $table = '<div class="table-responsive">
                            
                            
                            <b>Invoice</b>
                            
                            <table class="table table-hover table-striped table-bordered ">
                            
                                <thead>
                            
                                    <tr>
                            
                                        <th style="width:3%">Sl No.</th>
                            
                                        <th style="width:7%">Inv No.</th>
                            
                                        <th style="width:10%">Date</th>
                            
                                        <th style="width:25%">Customer</th>
                            
                                        <th style="width:10%">Invoice Amount</th>
                            
                                        <th style="width:10%">Cash Collected</th>
                            
                                        <th style="width:10%">Discount Allowed</th>
                            
                                        <th style="width:10%">Balance</th>
                            
                                    </tr>
                            
                                </thead>
                            
                                <tbody>';
                            
                            
                            
                            
                            
                            $condition_sales .= "1=1 AND active='Y'";
                            
                            $condition_sales_return .= "1=1 AND active='Y'";
                            
                            
                            
                            if ($outlet > 0) {
                            
                                $condition_sales .= " AND outlet=" . $outlet;
                            
                                $condition_sales_return .= " AND outlet=" . $outlet;
                            }
                            
                            if ($created_user > 0) {
                            
                                $condition_sales .= " AND sales_man=" . $created_user;
                            
                                $condition_sales_return .= " AND created_by=" . $created_user;
                            }
                            
                            if ($customer_id > 0) {
                            
                                $condition_sales .= " AND customer_id=" . $customer_id;
                            
                                $condition_sales_return .= " AND customer_id=" . $customer_id;
                            }
                            
                            if ($daybook_from_date != '' && $daybook_from_date != '00/00/0000') {
                            
                                $daybook_from_date = str_replace('/', '-', $daybook_from_date);
                            
                                $daybook_start_date = date("Y-m-d H:i:s", strtotime($daybook_from_date));
                            
                                $condition_sales .= ' AND (created_time>="' . $daybook_start_date . '")';
                            
                                $condition_sales_return .= ' AND (created_time>="' . $daybook_start_date . '")';
                            }
                            
                            if ($daybook_to_date != '' && $daybook_to_date != '00/00/0000') {
                            
                                $daybook_to_date = str_replace('/', '-', $daybook_to_date);
                            
                                $daybook_date = date("Y-m-d H:i:s", strtotime($daybook_to_date));
                            
                                $condition_sales .= ' AND (created_time<="' . $daybook_date . '")';
                            
                                $condition_sales_return .= ' AND (created_time<="' . $daybook_date . '")';
                            }
                            
                            //Sales
                            
                            //echo $condition_sales;
                            
                            $model = SalesInvoice::model()->findAll(array('condition' => $condition_sales));
                            
                            $grand_total = 0;
                            
                            $received_total = 0;
                            
                            foreach ($model as $i => $row) {
                            
                                if ($row['posted'] == 'Y') {
                            
                                    $outlet = $row->outlet;
                            
                                    $vouchertype = FinanceVoucherType::model()->find(array('condition' => 'code="SI" AND outlet=' . $outlet))->id;
                            
                                    $inv = FinanceVoucherHead::model()->find(array('condition' => 'voucher_type=' . $vouchertype . ' AND header_table_id=' . $row->id));
                            
                                    $debtor = FinanceSystemAccountsMapping::getSystemdefault('debtors');
                            
                                    if ($inv->id > 0) {
                            
                                        $inv_item = FinanceVoucherItems::model()->find(array('condition' => 'ledger_id=' . $debtor . ' AND header_id=' . $inv->id));
                            
                                        $inv_amount = $inv_item->amount;
                                    } else {
                            
                                        $inv_amount = 0;
                                    }
                            
                                    if ($inv_item->id > 0) {
                            
                                        $received_amount = Yii::app()->db->createCommand('select sum(amount) from erp_finance_allocation_items where voucher_item_id=' . $inv_item->id)->queryScalar();
                                    } else {
                            
                                        $received_amount = 0;
                                    }
                                }
                            
                            
                            
                                $table .= '<tr>
                            
                                            <td style="text-align:center !important;">' .  ($i + 1) . '</td>
                            
                                            <td style="text-align:left !important;">' .  $row->invoice_no . '</td>
                            
                                            <td style="text-align:left !important;">' .  date('d/m/Y h:i a', strtotime($row->created_time)) . '</td>
                            
                                            <td style="text-align:left !important;">' .  $row->customer->customer_name . '</td>
                            
                                            <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row->grand_total, "") . '</td>
                            
                                            <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row->received, "") . '</td>
                            
                                            <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row->discount, "") . '</td>
                            
                                            <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row->grand_total - $received_amount, "") . '</td>
                            
                                        </tr>';
                            
                            
                                $grand_total += $row->grand_total;
                            
                                $received_total += $row->received;
                            
                                $discount += $row->discount;
                            
                                $balance = $row->grand_total - $received_amount;
                            
                                $tot_balance += $balance;
                            }
                            
                            $total_sales = $grand_total;
                            
                            
                            
                            $table .= '<tr>
                            
                                        <td style="text-align:right !important;" colspan="4"><b>Total</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($grand_total, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($received_total, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($discount, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($tot_balance, "") . '</b></td>
                            
                                    </tr>
                            
                                </tbody>
                            
                            </table>
                            
                            <b>Creditnote</b>';
                            
                            $table .= '<table class="table table-hover table-striped table-bordered ">
                            
                                <thead>
                            
                                    <tr>
                            
                                        <th style="width:3%">Sl No.</th>
                            
                                        <th style="width:7%">Cn. No.</th>
                            
                                        <th style="width:10%">Cn. Date</th>
                            
                                        <th style="width:25%">Customer</th>
                            
                                        <th style="width:7%">Inv. No.</th>
                            
                                        <th style="width:10%">Inv. Date</th>
                            
                                        <th style="width:10%">Inv. Amt.</th>
                            
                                        <th style="width:10%">Cn. Amt.</th>
                            
                                        <th style="width:10%">Balance</th>
                            
                                    </tr>
                            
                                </thead>
                            
                                <tbody>';
                            
                            
                            //Sales Return
                            
                            $model = SalesReturn::model()->findAll(array('condition' => $condition_sales_return));
                            
                            $grand_total = 0;
                            
                            $cn_total = 0;
                            
                            foreach ($model as $i => $row) {
                            
                            
                            
                                $table .= '<tr>
                            
                                            <td style="text-align:center !important;">' .  ($i + 1) . '</td>
                            
                                            <td style="text-align:left !important;">' .  $row->return_no . '</td>
                            
                                            <td style="text-align:left !important;">' .  date('d/m/Y h:i a', strtotime($row->created_time)) . '</td>
                            
                                            <td style="text-align:left !important;">' .  $row->sr_customer->customer_name . '</td>
                            
                                            <td style="text-align:left !important;">' .  $row->sr_invoice->invoice_no . '</td>
                            
                                            <td style="text-align:left !important;">' .  date('d/m/Y', strtotime($row->sr_invoice->invoice_date)) . '</td>
                            
                                            <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row->grand_total, "") . '</td>
                            
                                            <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row->sr_invoice->grand_total, "") . '</td>
                            
                                            <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row->grand_total - $row->sr_invoice->grand_total, "") . '</td>
                            
                                        </tr>';
                            
                            
                            
                                $grand_total += $row->sr_invoice->grand_total;
                            
                                $cn_total += $row->grand_total;
                            }
                            
                            $total_sales_return = $cn_total;
                            
                            
                            $table .= '<tr>
                            
                                        <td style="text-align:right !important;" colspan="6"><b>Total</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($grand_total, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($cn_total, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($grand_total - $cn_total, "") . '</b></td>
                            
                                    </tr>
                            
                                </tbody>
                            
                            </table>
                            
                            <b>Receipt</b>';
                            
                            $table .= '<table class="table table-hover table-striped table-bordered ">
                            
                                <thead>
                            
                                    <tr>
                            
                                        <th style="width:3%">Sl No.</th>
                            
                                        <th style="width:7%">Voucher No.</th>
                            
                                        <th style="width:10%">Voucher Date</th>
                            
                                        <th style="width:25%">Customer</th>
                            
                                        <!--                <th style="width:7%">Inv No.</th>
                            
                                        <th style="width:10%">Inv. Date</th>
                            
                                        <th style="width:10%">Inv. Amount</th>-->
                            
                                        <th style="width:10%">Cash Collected</th>
                            
                                        <th style="width:10%">Discount Allowed</th>
                            
                                        <th style="width:10%">Receipt Amount</th>
                            
                                        <!--                <th style="width:10%">Discount</th>
                            
                                        <th style="width:10%">Balance</th>-->
                            
                                    </tr>
                            
                                </thead>
                            
                                <tbody>';
                            
                            
                            
                            $CashAccount = FinanceSystemAccountsMapping::getSystemdefault('cash_account');
                            
                            $condition_expense = "1=1 AND H.is_posted='Y' AND H.is_posted='Y' AND amount<0 AND ledger_id=" . $CashAccount;
                            
                            $condition_cash = "1=1 AND H.is_posted='Y' AND H.is_posted='Y' AND amount>0 AND ledger_id=" . $CashAccount;
                            
                            
                            
                            if ($outlet > 0) {
                            
                                $condition_expense .= " AND I.outlet=" . $outlet;
                            
                                $condition_cash .= " AND I.outlet=" . $outlet;
                            }
                            
                            /* if($created_user>0){
                            
                                        $condition_expense.=" AND H.created_by=".$created_user;
                            
                                        $condition_cash.=" AND H.created_by=".$created_user;
                            
                                    }*/
                            
                            if ($created_user > 0) {
                                $subledger_id = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=3 AND parent_id=' . $created_user))->id;
                                if ((int)$subledger_id > 0) {
                                    $condition_expense .= " AND I.subledger_id=" . $subledger_id;
                                    $condition_cash .= " AND I.subledger_id=" . $subledger_id;
                                }
                            }
                            
                            if ($daybook_from_date != '' && $daybook_from_date != '00/00/0000') {
                            
                                $daybook_from_date = str_replace('/', '-', $daybook_from_date);
                            
                                $daybook_start_date = date("Y-m-d H:i:s", strtotime($daybook_from_date));
                            
                                $condition_expense .= ' AND (H.created_datetime>="' . $daybook_start_date . '")';
                            
                                $condition_cash .= ' AND (H.created_datetime>="' . $daybook_start_date . '")';
                            }
                            
                            if ($daybook_to_date != '' && $daybook_to_date != '00/00/0000') {
                            
                                $daybook_to_date = str_replace('/', '-', $daybook_to_date);
                            
                                $daybook_date = date("Y-m-d H:i:s", strtotime($daybook_to_date));
                            
                                $condition_expense .= ' AND (H.created_datetime<="' . $daybook_date . '")';
                            
                                $condition_cash .= ' AND (H.created_datetime<="' . $daybook_date . '")';
                            }
                            
                            //Expense & Cash
                            
                            $grand_total = 0;
                            
                            $received_total = 0;
                            
                            $discount_tot = 0;
                            
                            $expenses = Yii::app()->db->createCommand("SELECT H.voucher_no,H.id,I.id AS voucher_item_id,H.created_datetime,I.amount FROM {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id where $condition_cash")->queryAll();
                            
                            foreach ($expenses as $i => $row) {
                            
                                if ($customer_id > 0) {
                            
                                    $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $customer_id));
                                }
                            
                                $voucher = FinanceVoucherItems::model()->find(array('condition' => 'amount<0 AND header_id=' . $row['id']));
                            
                                if ($voucher->id > 0) {
                            
                                    $id = $voucher->id;
                            
                                    $invoice = SalesInvoice::model()->findByPk($voucher->parent->header_table_id);
                            
                                    //                    $received_amount=-1*Yii::app()->db->createCommand('select sum(amount) from erp_finance_allocation_items where voucher_item_id='.$voucher->id)->queryScalar();
                            
                                } else {
                            
                                    $id = 0;
                            
                                    //                    $received_amount=0;
                            
                                }
                            
                            
                            
                                if ($customer_id > 0) {
                            
                                    if ($voucher->subledger_id == $subledger->id) {
                            
                                        $allocations = FinanceAllocationItems::model()->findAll(array('condition' => 'voucher_item_id=' . $id));
                            
                                        $tot_alloc_amount = 0;
                            
                                        foreach ($allocations as $eachAllocations) {
                            
                                            $allocs = FinanceAllocationItems::model()->findAll(array('condition' => 'header_id=' . $eachAllocations->header_id));
                            
                                            foreach ($allocs as $eachAllocs) {
                            
                                                if ($eachAllocs->amount > 0) {
                            
                                                    $tot_alloc_amount += $eachAllocs->amount;
                                                }
                                            }
                                        }
                            
                                        if ($invoice->invoice_no > 0) {
                            
                                            $grand_total1 = $invoice->grand_total;
                                        } else {
                            
                                            $grand_total1 = $tot_alloc_amount;
                                        }
                            
                            
                            
                                        $table .= '<tr>
                            
                                                    <td style="text-align:center !important;">' .  ($i + 1) . '</td>
                            
                                                    <td style="text-align:left !important;">' .  $row['voucher_no'] . '</a></td>
                            
                                                    <td style="text-align:left !important;">' .  date('d/m/Y', strtotime($row['created_datetime'])) . '</td>
                            
                                                    <td style="text-align:left !important;">' .  $voucher->subledger->description . '</td>                    
                            
                                                    <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row['amount'], "") . '</td>
                            
                                                    <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($invoice->discount, "") . '</td>
                            
                                                    <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row['amount'] + $invoice->discount, "") . '</td>
                                                </tr>';
                            
                            
                            
                                        $grand_total += $grand_total1;
                            
                                        $received_total += 1 * $row['amount'];
                            
                                        $discount_tot += $invoice->discount;
                                    }
                                } else {
                            
                                    if ($voucher->subledger->description != '') {
                            
                                        $allocations = FinanceAllocationItems::model()->findAll(array('condition' => 'voucher_item_id=' . $id));
                            
                                        $tot_alloc_amount = 0;
                            
                                        foreach ($allocations as $eachAllocations) {
                            
                                            $allocs = FinanceAllocationItems::model()->findAll(array('condition' => 'header_id=' . $eachAllocations->header_id));
                            
                                            foreach ($allocs as $eachAllocs) {
                            
                                                if ($eachAllocs->amount > 0) {
                            
                                                    $tot_alloc_amount += $eachAllocs->amount;
                                                }
                                            }
                                        }
                            
                                        if ($invoice->invoice_no > 0) {
                            
                                            $grand_total1 = $invoice->grand_total;
                                        } else {
                            
                                            $grand_total1 = $tot_alloc_amount;
                                        }
                            
                            
                            
                                        $table .= '<tr>
                            
                                                    <td style="text-align:center !important;">' .  ($i + 1) . '</td>
                            
                                                    <td style="text-align:left !important;">' .  $row['voucher_no'] . '</a></td>
                            
                                                    <td style="text-align:left !important;">' .  date('d/m/Y', strtotime($row['created_datetime'])) . '</td>
                            
                                                    <td style="text-align:left !important;">' .  $voucher->subledger->description . '</td>                     
                            
                                                    <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row['amount'], "") . '</td>
                            
                                                    <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($invoice->discount, "") . '</td>
                            
                                                    <td style="text-align:right !important;">' .  Yii::app()->numberFormatter->formatCurrency($row['amount'] + $invoice->discount, "") . '</td>
                            
                                                    
                                                </tr>';
                            
                            
                            
                                        $tot_amt += $row['amount'];
                            
                                        $grand_total += $grand_total1;
                            
                                        $received_total += 1 * ($row['amount'] + $invoice->discount);
                            
                                        $discount_tot += $invoice->discount;
                                    }
                                }
                            }
                            
                            $cash_collected = $received_total;
                            
                            
                            
                            $table .= '<tr>
                            
                                        <td style="text-align:right !important;" colspan="4"><b>Total</b></td>           
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($tot_amt, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($discount_tot, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($received_total, "") . '</b></td>
                            
                                    </tr>
                            
                                </tbody>
                            
                            </table>
                            
                            <b>Summary</b>';
                            
                            $table .= '<table class="table table-hover table-striped table-bordered ">
                            
                                <thead>
                            
                                    <tr>
                            
                                        <th style="width:25%">Total Sales</th>
                            
                                        <th style="width:25%">Total Creditnote</th>
                            
                                        <th style="width:25%">Total Cash Collected</th>
                            
                                        <th style="width:25%">Total Discount Allowed</th>
                            
                                    </tr>
                            
                                </thead>
                            
                                <tbody>
                            
                                    <tr>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($total_sales, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($total_sales_return, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($cash_collected, "") . '</b></td>
                            
                                        <td style="text-align:right !important;"><b>' .  Yii::app()->numberFormatter->formatCurrency($discount_tot, "") . '</b></td>
                            
                                    </tr>
                            
                                </tbody>
                            
                            </table>
                            
                            </div>';


                } else if ($report_type == 'cost_report') {

                    $from_date = $params['from_date'];
                    $to_date = $params['to_date'];
                    $outlet = (int) $params['outlet_id'];

                    $condition = ' where H.active="Y" AND H.invoice_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';
                    if ($outlet > 0) {
                        $condition .= ' AND H.outlet=' . $outlet;
                        $conditions .= ' AND outlet=' . $outlet;
                    }
                    $SQL = ' Select H.invoice_no,C.customer_name,I.description,I.net_amount,I.quantity,H.invoice_date,I.product_id,I.tax_amount,H.id AS invoice_id,I.id AS item_id from erp_sales_invoice H INNER JOIN erp_sales_invoice_items I ON H.id=I.invoice_id INNER JOIN erp_customer C ON H.customer_id=C.id ' . $condition . ' order by H.id';

                    $all = Yii::app()->db->createCommand($SQL)->queryAll();
                    $table = '<table class="table table-hover table-fixed table-bordered">
                    <thead>
                        <tr>
                            <th style="width:10%">Date</th>
                            <th style="width:10%">Invoice No</th>
                            <th style="width:25%">Customer</th>
                            <th style="width:25%">Part No.</th>
                            <th style="width:5%">Net Total</th>
                            <th style="width:5%">Tax</th>
                            <th style="width:5%">Qty</th>
                            <th style="width:5%">Cost</th>
                            <th style="width:5%">Total Cost</th>
                            <th style="width:5%">Profit</th>
                            <th style="width:5%">Profit(%)</th>
                        </tr>
                    </thead>
                    <tbody>';
                    $total_cost = 0;
                    $total_profit = 0;
                    $total_sales = 0;
                    $total_tax = 0;
                    foreach ($all as $each) {
                        $qty = Yii::app()->db->createCommand('SELECT quantity*unit_exchange_rate AS qty from erp_stock_ledger where  type="SI" AND header_id=' . $each["invoice_id"] . ' AND item_id=' . $each["item_id"] . ' AND product_id=' . $each["product_id"] . ' AND transaction_date <= "' . $each['invoice_date'] . '" ' . $conditions)->queryScalar();
                        $qty = round($qty, 2);
                        $current_in_rate = Yii::app()->db->createCommand('SELECT CASE
                                WHEN SUM(quantity*unit_exchange_rate) > 0 THEN SUM(quantity*rate)/SUM(quantity*unit_exchange_rate) ELSE 0 END AS avg_purchase_rate 
                                FROM {{stock_ledger}}  WHERE transaction_type = "IN" AND type!= "GT" AND product_id=' . $each["product_id"] . ' AND transaction_date <= "' . $each['invoice_date'] . '" ' . $conditions)->queryScalar();
                        $profit = $each['net_amount'] - $each['tax_amount'] - ($qty * $current_in_rate);
                        $cost = ($qty * $current_in_rate);
                        if ($cost > 0) {
                            $profit_percent = $profit / ($cost) * 100;
                        }
                        $table .= '<tr><td>' . date("d-m-Y", strtotime($each['invoice_date'])) . '</td><td>' . $each['invoice_no'] . '</td><td>' . $each['customer_name'] . '</td><td>' . $each["product_id"] . '/' . $each['description'] . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['net_amount'], "") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['tax_amount'], "") . '</td><td>' . $qty . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($current_in_rate, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($qty * $current_in_rate, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($profit, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($profit_percent, " ") . '%</td></tr>';
                        $total_cost += $qty * $current_in_rate;
                        $total_profit += $profit;
                        $total_sales += $each['net_amount'];
                        $total_tax += $each['tax_amount'];
                    }
                    if ($total_cost > 0) {
                        $percent = ($total_sales - $total_cost) / $total_cost * 100;
                    } else {
                        $percent = 0;
                    }
                    $table .= '<tr><td colspan="4" align="right"><b>Total</b></td><td style="text-align:right !important;" align="right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_sales, " ") . '</b></td><td align="right" style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_tax, " ") . '</b></td><td></td><td></td><td style="text-align:right !important;" align="right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_cost, " ") . '</b></td><td style="text-align:right !important;" align="right"><b>' . Yii::app()->numberFormatter->formatCurrency($total_profit, " ") . '</b></td><td style="text-align:right !important;" align="right"><b>' . Yii::app()->numberFormatter->formatCurrency($percent, " ") . '%</b></td></tr>';
                    $table .= '</tbody>
                    </table>';
                } else if ($report_type == 'cash_collection_report') {
                    $from_date = $params['from_date'];
                    $to_date = $params['to_date'];
                    $outlet = (int) $params['outlet_id'];
                    $from = $params['from_date'];
                    $to = $params['to_date'];
                    $created_user = (int) $params['salesman_id'];
                    $customer_id = (int) $params['customer_id'];
                    $salesman = (int) $params['salesman_id'];

                    $table = '<table class="table table-hover table-striped table-bordered ">
                            <thead>
                            <tr>
                            <th style="width:3%">Sl No.</th>
                            <th style="width:7%">Voucher No.</th>
                            <th style="width:7%">Inv No.</th>
                            <th style="width:10%">Inv. Date</th>
                            <th style="width:25%">Customer</th>
                            <th style="width:10%">Inv. Amount</th>
                            <th style="width:10%">Voucher Date</th>
                            <th style="width:10%">Receipt Amount</th>
                            <th style="width:10%">Discount</th>
                            <th style="width:10%">Balance</th>
                            </tr>
                            </thead>
                            <tbody>';
                    $CashAccount = FinanceSystemAccountsMapping::getSystemdefault('cash_account');
                    $condition_expense = "1=1 AND H.is_posted='Y' AND H.is_posted='Y' AND amount<0 AND ledger_id=" . $CashAccount;
                    $condition_cash = "1=1 AND H.is_posted='Y' AND H.is_posted='Y' AND amount>0 AND ledger_id=" . $CashAccount;

                    if ($outlet > 0) {
                        $condition_expense .= " AND I.outlet=" . $outlet;
                        $condition_cash .= " AND I.outlet=" . $outlet;
                    }
                    if ($salesman > 0) {
                        $condition_expense .= " AND H.created_by=" . $salesman;
                        $condition_cash .= " AND H.created_by=" . $salesman;
                    }
                    if ($from != '' && $to != '') {
                        $condition_cash .= ' AND (H.voucher_date>="' . $from . '" AND H.voucher_date<="' . $to . '")';
                    }
                    //Expense & Cash
                    $grand_total = 0;
                    $received_total = 0;
                    $discount_tot = 0;
                    $expenses = Yii::app()->db->createCommand("SELECT H.voucher_no,H.id,I.id AS voucher_item_id,H.voucher_date,I.amount FROM {{finance_voucher_items}} I INNER JOIN {{finance_voucher_head}} H ON H.id=I.header_id where $condition_cash")->queryAll();
                    foreach ($expenses as $i => $row) {
                        if ($customer_id > 0) {
                            $subledger = FinanceSubledger::model()->find(array('condition' => 'subledger_type_id=1 AND parent_id=' . $customer_id));
                        }
                        $voucher = FinanceVoucherItems::model()->find(array('condition' => 'amount<0 AND header_id=' . $row['id']));
                        if ($voucher->id > 0) {
                            $invoice = SalesInvoice::model()->findByPk($voucher->parent->header_table_id);
                            //                    $received_amount=-1*Yii::app()->db->createCommand('select sum(amount) from erp_finance_allocation_items where voucher_item_id='.$voucher->id)->queryScalar();
                        } else {
                            //                    $received_amount=0;
                        }
                        if ($customer_id > 0) {
                            if ($voucher->subledger_id == $subledger->id) {
                                if ($invoice->invoice_no > 0) {
                                    $dd = date('d/m/Y', strtotime($invoice->invoice_date));
                                }
                                $table .= '<tr>
                    <td style="text-align:center !important;">' . ($i + 1) . '</td> 
                    <td style="text-align:left !important;">' . $row["voucher_no"] . '</td> 
                    <td style="text-align:left !important;">' . $invoice->invoice_no . '</td> 
                    <td style="text-align:left !important;">' . $dd . '</td> 
                    <td style="text-align:left !important;">' . $voucher->subledger->description . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total, "") . '</td>
                    <td style="text-align:left !important;">' . date("d/m/Y", strtotime($row["voucher_date"])) . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row["amount"], "") . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->discount, "") . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total - $row["amount"] - $invoice->discount, "") . '</td>
                </tr>';

                                $grand_total += $invoice->grand_total;
                                $received_total += 1 * $row['amount'];
                                $discount_tot += $invoice->discount;
                            }
                        } else {
                            if ($voucher->subledger->description != '') {
                                if ($invoice->invoice_no > 0) {
                                    $dt = date('d/m/Y', strtotime($invoice->invoice_date));
                                }
                                $table .= '<tr><td style="text-align:center !important;">' . ($i + 1) . '</td> 
                    <td style="text-align:left !important;">' . $row["voucher_no"] . '</td> 
                    <td style="text-align:left !important;">' . $invoice->invoice_no . '</td> 
                    <td style="text-align:left !important;">' . $dt . '</td> 
                    <td style="text-align:left !important;">' . $voucher->subledger->description . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total, "") . '</td>
                    <td style="text-align:left !important;">' . date("d/m/Y", strtotime($row["voucher_date"])) . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($row["amount"], "") . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->discount, "") . '</td> 
                    <td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($invoice->grand_total - $row["amount"] - $invoice->discount, "") . '</td>
                </tr>';

                                $grand_total += $invoice->grand_total;
                                $received_total += 1 * $row['amount'];
                                $discount_tot += $invoice->discount;
                            }
                        }
                    }
                    $cash_collected = $received_total;

                    $table .= '<tr>
                    <td style="text-align:right !important;" colspan="5"><b>Total</b></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total, "") . '</b></td>
                    <td style="text-align:right !important;"></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($received_total, "") . '</b></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($discount_tot, "") . '</b></td> 
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($grand_total - $received_total - $discount_tot, "") . '</b></td>
                </tr>
            </tbody>
            </table>';
                } else if ($report_type == 'costofsale') {
                    $from_date = $params['from_date'];
                    $to_date = $params['to_date'];
                    $customer_id = (int) $params['customer_id'];
                    $salesman = (int) $params['salesman'];
                    $brand = (int) $params['brand'];
                    $outlet = (int) $params['outlet_id'];
                    $condition = ' WHERE H.active="Y" AND H.invoice_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';
                    if ($outlet > 0) {
                        $condition .= ' AND H.outlet=' . $outlet;
                        $conditions .= ' AND outlet=' . $outlet;
                    }
                    if ($customer_id) {
                        $condition .= ' AND H.customer_id=' . $customer_id;
                    }
                    if ($salesman) {
                        $condition .= ' AND H.sales_man=' . $salesman;
                    }
                    if ((int) $brand > 0) {
                        $condition .= ' AND S.brand=' . (int) $brand;
                    }
                    $SQL = ' Select H.invoice_no,C.customer_name,I.description,I.net_amount,I.quantity,I.rate,H.invoice_date,I.product_id,I.tax_amount,H.id AS invoice_id,I.id AS item_id,B.brand_name 
                       from erp_sales_invoice H 
                       INNER JOIN erp_sales_invoice_items I ON H.id=I.invoice_id 
                       INNER JOIN erp_inventory S ON S.id=I.product_id 
                       INNER JOIN erp_master_brand B ON B.id=S.brand 
                       INNER JOIN erp_customer C ON H.customer_id=C.id ' . $condition . ' order by H.id';

                    $all = Yii::app()->db->createCommand($SQL)->queryAll();
                    $table = '<table class="table table-hover table-fixed table-bordered">
                        <thead>
                            <tr>
                                <th style="width:6%">Date</th>
                                <th style="width:6%">Invoice No</th>
                                <th style="width:20%">Customer</th>
                                <th style="width:15%">Part No.</th>
                                <th style="width:10%">Brand</th>
                                <th style="width:5%">Qty</th>
                                <th style="width:8%">Rate</th>
                                <th style="width:5%">Tax</th>
                                <th style="width:5%">Net Total</th>
                                <th style="width:5%">Cost</th>
                                <th style="width:5%">Total Cost</th>
                                <th style="width:5%">Profit</th>
                                <th style="width:5%">Profit(%)</th>
                            </tr>
                        </thead>
                        <tbody>';

                    $total_cost = 0;
                    $total_profit = 0;
                    $total_sales = 0;
                    $total_tax = 0;
                    foreach ($all as $each) {
                        $qty = Yii::app()->db->createCommand('SELECT quantity*unit_exchange_rate AS qty from erp_stock_ledger where  type="SI" AND header_id=' . $each["invoice_id"] . ' AND item_id=' . $each["item_id"] . ' AND product_id=' . $each["product_id"] . ' AND transaction_date <= "' . $each['invoice_date'] . '" ' . $conditions)->queryScalar();
                        $qty = round($qty, 2);
                        $current_in_rate = Yii::app()->db->createCommand('SELECT CASE
                            WHEN SUM(quantity*unit_exchange_rate) > 0 THEN SUM(quantity*rate)/SUM(quantity*unit_exchange_rate) ELSE 0 END AS avg_purchase_rate 
                            FROM {{stock_ledger}}  WHERE transaction_type = "IN" AND type!= "GT" AND product_id=' . $each["product_id"] . ' AND transaction_date <= "' . $each['invoice_date'] . '" ' . $conditions)->queryScalar();
                        $profit = $each['net_amount'] - $each['tax_amount'] - ($qty * $current_in_rate);
                        $cost = ($qty * $current_in_rate);
                        if ($cost > 0) {
                            $profit_percent = $profit / ($cost) * 100;
                        }

                        $table .= '<tr><td>' . date("d-m-Y", strtotime($each['invoice_date'])) . '</td><td>' . $each['invoice_no'] . '</td><td>' . $each['customer_name'] . '</td><td>' . $each["product_id"] . '/' . $each['description'] . '</td><td>' . $each['brand_name'] . '</td><td>' . $qty . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['rate'], "") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['tax_amount'], "") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['net_amount'], "") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($current_in_rate, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($qty * $current_in_rate, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($profit, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($profit_percent, " ") . '%</td></tr>';
                        $total_cost += $qty * $current_in_rate;
                        $total_profit += $profit;
                        $total_sales += $each['net_amount'];
                        $total_tax += $each['tax_amount'];
                    }
                    if ($total_cost > 0) {
                        $percent = ($total_sales - $total_cost) / $total_cost * 100;
                    } else {
                        $percent = 0;
                    }
                    $table .= '<tr><td colspan="7">Total</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($total_tax, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($total_sales, " ") . '</td><td></td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($total_cost, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($total_profit, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($percent, " ") . '%</td></tr>';
                    $table .= '</tbody>
                        </table>';
                } else if ($report_type == 'brandwise') {
                    $from_date = $params['from_date'];
                    $to_date = $params['to_date'];
                    $customer_id = (int) $params['customer_id'];
                    $salesman = (int) $params['salesman'];
                    $brand = (int) $params['brand'];
                    $payment_term = (int) $params['payment_term'];
                    $outlet = (int) $params['outlet_id'];
                    $condition = ' WHERE H.active="Y" AND H.invoice_date BETWEEN "' . $from_date . '" AND "' . $to_date . '"';
                    if ($outlet > 0) {
                        $condition .= ' AND H.outlet=' . $outlet;
                        $conditions .= ' AND outlet=' . $outlet;
                    }
                    if ($customer_id) {
                        $condition .= ' AND H.customer_id=' . $customer_id;
                    }
                    if ($salesman) {
                        $condition .= ' AND H.sales_man=' . $salesman;
                    }
                    if ($payment_term) {
                        $condition .= ' AND H.payment_term=' . $payment_term;
                    }
                    if ((int) $brand > 0) {
                        $condition .= ' AND S.brand=' . (int) $brand;
                    }
                    $SQL = ' Select H.invoice_no,C.customer_name,I.description,I.net_amount,P.payment_term_name,I.quantity,I.rate,H.invoice_date,I.product_id,I.tax_amount,H.id AS invoice_id,I.id AS item_id,B.brand_name 
                    from erp_sales_invoice H 
                    INNER JOIN erp_sales_invoice_items I ON H.id=I.invoice_id 
                    INNER JOIN erp_inventory S ON S.id=I.product_id 
                    INNER JOIN erp_master_payment_terms P ON H.payment_term =P.id
                    INNER JOIN erp_master_brand B ON B.id=S.brand
                    INNER JOIN erp_customer C ON H.customer_id=C.id ' . $condition . ' order by H.id';

                    $all = Yii::app()->db->createCommand($SQL)->queryAll();
                    $table = '<table class="table table-hover table-fixed table-bordered">
                        <thead>
                            <tr>
                                <th style="width:6%">Date</th>
                                <th style="width:6%">Invoice No</th>
                                <th style="width:20%">Customer</th>
                                <th style="width:15%">Part No.</th>
                                <th style="width:10%">Brand</th>
                                <th style="width:10%">Payment</th>
                                <th style="width:5%">Qty</th>
                                <th style="width:8%">Rate</th>
                                <th style="width:8%">Total</th>
                                <th style="width:5%">Tax</th>
                                <th style="width:5%">Net Total</th>
                            </tr>
                        </thead>
                        <tbody>';

                    $total_cost = 0;
                    $total_profit = 0;
                    $net = 0;
                    $total_tax = 0;
                    $total = 0;
                    foreach ($all as $each) {
                        $qty = Yii::app()->db->createCommand('SELECT quantity AS qty from erp_stock_ledger where  type="SI" AND header_id=' . $each["invoice_id"] . ' AND item_id=' . $each["item_id"] . ' AND product_id=' . $each["product_id"] . ' AND transaction_date <= "' . $each['invoice_date'] . '" ' . $conditions)->queryScalar();
                        $qty = round($qty, 2);
                        $current_in_rate = Yii::app()->db->createCommand('SELECT CASE WHEN SUM(quantity*unit_exchange_rate) > THEN SUM(quantity*rate)/SUM(quantity*unit_exchange_rate) ELSE 0 END AS avg_purchase_rate 
                        FROM {{stock_ledger}}  WHERE transaction_type = "IN" AND type!= "GT" AND product_id=' . $each["product_id"] . ' AND transaction_date <= "' . $each['invoice_date'] . '" ' . $conditions)->queryScalar();
                        $profit = $each['net_amount'] - $each['tax_amount'] - ($qty * $current_in_rate);
                        $cost = ($qty * $rate);
                        $total = ($qty * $each['rate']);
                        if ($cost > 0) {
                            $profit_percent = $profit / ($cost) * 100;
                        }

                        $table .= '<tr><td>' . date("d-m-Y", strtotime($each['invoice_date'])) . '</td><td>' . $each['invoice_no'] . '</td><td>' . $each['customer_name'] . '</td><td>' . $each["product_id"] . '/' . $each['description'] . '</td><td>' . $each['brand_name'] . '</td><td>' . $each['payment_term_name'] . '</td><td>' . $qty . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['rate'], "") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($total, " ") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['tax_amount'], "") . '</td><td style="text-align:right !important;">' . Yii::app()->numberFormatter->formatCurrency($each['net_amount'], "") . '</td></tr>';
                        $tot += $total;
                        $total_cost += $qty * $current_in_rate;
                        $total_profit += $profit;
                        $net += $each['net_amount'];
                        $total_tax += $each['tax_amount'];
                    }
                    if ($total_cost > 0) {
                        $percent = ($total_sales - $total_cost) / $total_cost * 100;
                    } else {
                        $percent = 0;
                    }
                    $table .= '<tr>
                    <td colspan="8" style="text-align:right !important;"><b>Total</b></td>
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($tot, " ") . '</b></td>
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($total_tax, " ") . '</b></td>
                    <td style="text-align:right !important;"><b>' . Yii::app()->numberFormatter->formatCurrency($net, " ") . '</b></td></tr>';
                    $table .= '</tbody>
                            </table>';
                }
                return $table;
            }

            $path = "app_reports/";
            $filename = $report_type . time();
            $mpdf = Yii::app()->ePdf->mpdf('', 'A4');
            $stylesheet = file_get_contents(Yii::getPathOfAlias('webroot.css') . '/pdf_style.css');
            $mpdf->WriteHTML($stylesheet, 1);
            $mpdf->WriteHTML($this->renderPartial('application.views.front.pdf.' . $conditionFormats[$report_type]['view'], array('table' => generate_details($report_type, $conditionFormats), 'title' => $conditionFormats[$report_type]['title'], 'logo' => $userModel->companyDetails->pdf_logo, 'params' => $conditionFormats[$report_type], 'userModel' => $userModel), true));
            $mpdf->Output($path . $filename . '.pdf', 'F');
            $this->responseBody['file'] = str_replace('index.php', '', $_SERVER['SCRIPT_URI']) . $path . $filename . '.pdf';
            $this->_bind;
        } else {
            $this->_401;
        }
    }

    public function generate_view($view_type, $params)
    {

        /*$sales_voucher_id = 4;
        $creditnote = 5;
        $purchase_voucher_id = 1;
        $debitnote = 2;*/

        if ($view_type == 'sales') {

            $condition = 'V.invoice_date >= "' . $params['from_date'] . '" AND V.invoice_date <= "' . $params['to_date'] . '" ';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }
            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.sales_man=' . $params['salesman_id'];
            }
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }
            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_sales_invoice_list AS SELECT
                                                V.id,
                                                V.invoice_date AS date,
                                                V.invoice_no AS voucher_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                                O.name AS outlet_name,
						MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                                FROM {{sales_invoice}} V
                                            LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                            LEFT JOIN {{customer}} C ON C.id = V.customer_id
                                            LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = V.sales_man
                                            LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC')->execute();
        } else if ($view_type == 'sales_return') {

            $condition = 'V.return_date >= "' . $params['from_date'] . '" AND V.return_date <="' . $params['to_date'] . '"';
            if ($params['customer_id'] > 0) {
                $condition .= ' AND V.customer_id=' . $params['customer_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND SI.sales_man=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_creditnote_list AS SELECT
                                                V.id,
                                                V.return_date AS date,
                                                V.return_no AS voucher_no,
                                                SI.invoice_no,
                                                C.customer_name AS customer_name,
                                                U.full_name AS sales_man,
                                               	O.name AS outlet_name,
                                                MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active
                                        FROM {{sales_return}} V
                                        LEFT JOIN {{sales_invoice}} SI ON SI.id = V.invoice_id
                                        LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                        LEFT JOIN {{customer}} C ON C.id = V.customer_id
                                        LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = SI.sales_man
                                        LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                        WHERE ' . $condition . ' ORDER BY V.return_date ASC,V.id ASC')->execute();
        } else if ($view_type == 'purchase') {
            //$params['vendor_id']=$params['ledger_id'];
            $condition = 'V.invoice_date >= "' . $params['from_date'] . '" AND V.invoice_date <= "' . $params['to_date'] . '" ';
            if ($params['vendor_id'] > 0) {
                $condition .= ' AND V.vendor_id=' . $params['vendor_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND V.created_by=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_purchase_invoice_list AS SELECT
                                            V.id,
                                            V.invoice_date AS date,
                                            V.invoice_no AS voucher_no,
                                            C.vendor_name AS vendor_name,
                                            U.full_name AS sales_man,
                                            O.name AS outlet_name,
                                            MC.currency_code AS currency,
                                            V.tax_total AS total_vat,
                                            V.grand_total,
                                            (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                            (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                            V.active
                                            FROM {{purchase_invoice}} V
                                            LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                            LEFT JOIN {{vendor}} C ON C.id = V.vendor_id                                                
                                            LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = V.created_by
                                            LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                            WHERE ' . $condition . ' ORDER BY V.invoice_date ASC,V.id ASC ')->execute();
        } else if ($view_type == 'purchase_return') {
            $condition = 'V.return_date >= "' . $params['from_date'] . '" AND V.return_date <="' . $params['to_date'] . '"';
            if ($params['vendor_id'] > 0) {
                $condition .= ' AND V.vendor_id=' . $params['vendor_id'];
            }

            if ($params['salesman_id'] > 0) {
                $condition .= ' AND PI.created_by=' . $params['salesman_id'];
            }

            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_report_debitnote_list AS SELECT
                                                V.id,
                                                V.return_date AS date,
                                                V.return_no AS voucher_no,
                                                PI.invoice_no,
                                                C.vendor_name AS vendor_name,
                                                U.full_name AS sales_man,
                                               	O.name AS outlet_name,
                                                MC.currency_code AS currency,
                                                V.tax_total AS total_vat,
                                                V.grand_total,
                                                (MC.exchange_rate *V.tax_total) AS total_vat_bc,
                                                (MC.exchange_rate *V.grand_total) AS grand_total_bc,
                                                V.active                                                
                                        FROM {{purchase_return}} V
                                        LEFT JOIN {{purchase_invoice}} PI ON PI.id = V.invoice_id
                                        LEFT JOIN {{master_currency}} MC ON MC.id = V.currency
                                        LEFT JOIN {{vendor}} C ON C.id = V.vendor_id
                                        LEFT JOIN ' . $this->db_main . '.{{users}} U ON U.id = PI.created_by
                                        LEFT JOIN ' . $this->db_main . '.{{outlets}} O ON O.id = V.outlet
                                        WHERE ' . $condition . ' ORDER BY V.return_date ASC,V.id ASC')->execute();
        } else if ($view_type == 'net_purchase') {

            /*Yii::app()->db->createCommand('CREATE OR REPLACE VIEW view_net_purchase_report AS (SELECT "I" as type,I.id,I.vendor_id,I.created_by,I.invoice_no,I.invoice_date,V.vendor_name,U.full_name,I.exchange_rate,I.total_amount,I.grand_total,I.tax_total,I.outlet,I.active
              FROM  {{purchase_invoice}} I
              LEFT JOIN  {{vendor}} V
              ON V.id=I.vendor_id
              LEFT JOIN   bluesell_new_main.erp_users U
              ON U.id=I.created_by
              )
              UNION ALL
              (SELECT "CN" as type,CN.id,I.vendor_id,I.created_by,CN.return_no AS invoice_no,CN.return_date AS invoice_date,V.vendor_name,U.full_name,I.exchange_rate,CN.total_amount,CN.grand_total ,CN.tax_total,CN.outlet,CN.active 
              FROM  {{purchase_return}} CN
              LEFT JOIN  {{vendor}} V
              ON V.id=CN.vendor_id
              LEFT JOIN {{purchase_invoice}} I
              ON I.id=CN.invoice_id
              LEFT JOIN   bluesell_new_main.erp_users U
              ON U.id=I.created_by)')->execute();*/
        } /* else if ($view_type == 'aging') {

            $ledger_id = (int) $params['ledger_id'];
            $outlet = (int) $params['outlet_id'];
            $to_date = $params['to_date'];
            $type = $params['type'] == 'payable' ? 'payable' : 'receivable';

            $dates_array = array(
                '0_to_30' => array('from' => date('Y-m-d', strtotime($to_date . ' -30 days')), 'to' => $to_date),
                '31_to_60' => array('from' => date('Y-m-d', strtotime($to_date . ' -60 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -31 days'))),
                '61_to_90' => array('from' => date('Y-m-d', strtotime($to_date . ' -90 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -61 days'))),
                '91_to_120' => array('from' => date('Y-m-d', strtotime($to_date . ' -120 days')), 'to' => date('Y-m-d', strtotime($to_date . ' -91 days'))),
                'greaterthan_120' => array('from' => '2010-01-01', 'to' => date('Y-m-d', strtotime($to_date . ' -121 days'))),
            );

            foreach ($dates_array as $key => $each_date_pair) {

                $condition = 'V.date BETWEEN "' . $each_date_pair['from'] . '" AND "' . $each_date_pair['to'] . '" AND V.is_posted="Y"';
                if ($ledger_id > 0) {
                    $condition .= ' AND T.ledger_id = ' . (int) $ledger;
                }

                if ($outlet > 0) {
                    $condition .= ' AND L.outlet_id=' . (int) $outlet;
                }

                if ($type == 'receivable') {
                    $condition .= '  AND L.classification="' . VanLedgers::CUSTOMER . '"';
                } else {
                    $condition .= '  AND L.classification="' . VanLedgers::VENDOR . '"';
                }

                Yii::app()->db->createCommand('CREATE OR REPLACE VIEW {{ageing_' . $key . '}} AS SELECT
                                                    L.id AS ledger_id,
                                                    L.name AS ledger_name,
                                                    IFNULL(ROUND(SUM(T.amount*T.exchange_rate), 6), 0) AS voucher_value
                                                FROM {{transactions}} T
                                                LEFT JOIN {{vouchers}} V ON V.id = T.voucher_id
                                                LEFT JOIN {{ledgers}} L ON L.id = T.ledger_id
                                                LEFT JOIN {{allocation}} A ON A.child_voucher_id = V.id
                                                WHERE ' . $condition . '
                                                GROUP BY L.id')->execute();
            }
        } else if ($view_type == 'trial_balance') {

            $condition = 'V.date  BETWEEN "' . $params['from_date'] . '" AND "' . $params['to_date'] . '" AND  V.is_posted="Y"';
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet_id=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW trial_balance AS
                                            SELECT
                                                L.outlet_id,
                                                L.is_expandable,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND((SUM(CASE WHEN TR.amount > 0 THEN TR.amount*TR.exchange_rate ELSE 0 END)), 6) AS debit,
                                                ABS(ROUND((SUM(CASE WHEN TR.amount < 0 THEN TR.amount*TR.exchange_rate ELSE 0 END)), 6)) AS credit,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        } else if ($view_type == 'balance_sheet') {

            $condition = 'V.date <= "' . $params['to_date'] . '" AND L.type IN ("A", "L") AND V.is_posted="Y"';
            if ($params['outlet_id'] > 0) {
                $condition .= ' AND V.outlet_id=' . $params['outlet_id'];
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW balance_sheet AS
                                            SELECT
                                                L.outlet_id,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        } else if ($view_type == 'profit_and_loss') {

            $from_date = $params['from_date'];
            $to_date = $params['to_date'];
            $outlet = $params['outlet_id'];

            $condition = 'V.date BETWEEN "' . $from_date . '" AND "' . $to_date . '" AND L.type IN ("I", "E") AND V.is_posted="Y"';
            if ($outlet > 0) {
                $condition .= ' AND V.outlet_id=' . $outlet;
            }

            Yii::app()->db->createCommand('CREATE OR REPLACE VIEW profit_and_loss AS
                                            SELECT
                                                L.outlet_id,
                                                L.system_generated,
                                                L.is_parent,
                                                L.level,
                                                L.parent_id,
                                                L.type,
                                                L.id AS ledger_id,
                                                L.name AS ledger_name,
                                                ROUND(SUM(TR.amount*TR.exchange_rate), 6) AS balance
                                            FROM vs_transactions TR
                                            LEFT JOIN vs_vouchers V ON V.id = TR.voucher_id
                                            LEFT JOIN vs_ledgers L ON TR.ledger_id = L.id
                                            WHERE ' . $condition . ' GROUP BY L.id ORDER BY L.type ASC,L.name ASC ')->execute();
        }*/
    }
}
